"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ActivityTableService = exports.NEW_ACTIVITY_ADD = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const activity_table_entity_1 = require("./entities/activity-table.entity");
const GlobalConstants_1 = require("../helper/GlobalConstants");
const graphql_redis_subscriptions_1 = require("graphql-redis-subscriptions");
const typeorm_2 = require("typeorm");
const theme_table_service_1 = require("../theme-table/theme-table.service");
exports.NEW_ACTIVITY_ADD = "NEW_ACTIVITY_ADD";
let ActivityTableService = exports.ActivityTableService = class ActivityTableService {
    constructor(activityTableRepository, pubSub, themeTableService) {
        this.activityTableRepository = activityTableRepository;
        this.pubSub = pubSub;
        this.themeTableService = themeTableService;
    }
    async create(createActivityTableInput) {
        let checkIfExists = await this.checkIfExists(createActivityTableInput.activityName, createActivityTableInput.themeId);
        if (checkIfExists) {
            return {
                status: false,
                status_text: "Already Exists",
                data: null
            };
        }
        else {
            let dd = new activity_table_entity_1.ActivityTable();
            dd.activityName = createActivityTableInput.activityName;
            dd.unitOfMeasurement = createActivityTableInput.unitOfMeasurement;
            let theme = await this.themeTableService.findOne(createActivityTableInput.themeId);
            dd.ThemeTable = theme;
            let result = await this.activityTableRepository.save(dd);
            this.pubSub.publish(exports.NEW_ACTIVITY_ADD, 1);
            return {
                status: true,
                status_text: "Successfully Created",
                data: result
            };
        }
    }
    async createorElseReturnObject(createActivityTableInput) {
        let checkIfAlreadyExists = await this.activityTableRepository.findOne({
            where: {
                activityName: createActivityTableInput.activityName.trim(),
                ThemeTable: {
                    themeId: createActivityTableInput.themeId
                }
            }
        });
        if (!checkIfAlreadyExists) {
            let dd = new activity_table_entity_1.ActivityTable();
            dd.activityName = createActivityTableInput.activityName.trim();
            dd.unitOfMeasurement = createActivityTableInput.unitOfMeasurement.trim();
            let theme = await this.themeTableService.findOne(createActivityTableInput.themeId);
            dd.ThemeTable = theme;
            let result = await this.activityTableRepository.save(dd);
            return result;
        }
        else {
            return checkIfAlreadyExists;
        }
    }
    async checkIfExists(activityName, themeId) {
        let result = await this.activityTableRepository.count({
            where: {
                activityName: activityName,
                ThemeTable: {
                    themeId
                }
            }
        });
        if (result == 0) {
            return false;
        }
        else {
            return true;
        }
    }
    async findAll() {
        return await this.activityTableRepository.find({
            relations: {
                ThemeTable: true
            }
        });
    }
    async getCount() {
        return await this.activityTableRepository.count();
    }
    async findAllByThemeId(themeId) {
        return await this.activityTableRepository.find({
            where: {
                ThemeTable: {
                    themeId
                }
            },
            relations: {
                ThemeTable: true
            }
        });
    }
    async findOne(activityId) {
        return await this.activityTableRepository.findOne({
            where: {
                activityId: activityId
            },
            relations: {
                ThemeTable: true
            }
        });
    }
    async update(activityId, updateActivityTableInput) {
        let result = await this.activityTableRepository.findOne({
            where: {
                activityId: activityId
            }
        });
        if (result) {
            result.activityName = updateActivityTableInput.activityName;
            result.unitOfMeasurement = updateActivityTableInput.unitOfMeasurement;
            let theme = await this.themeTableService.findOne(updateActivityTableInput.themeId);
            result.ThemeTable = theme;
            let r = await this.activityTableRepository.save(result);
            this.pubSub.publish(exports.NEW_ACTIVITY_ADD, 1);
            return {
                status: true,
                status_text: "Successfully Updated",
                data: r
            };
        }
        else {
            return {
                status: false,
                status_text: "Not found",
                data: null
            };
        }
    }
    async remove(activityId) {
        let result = await this.activityTableRepository.findOne({
            where: {
                activityId: activityId
            }
        });
        if (result) {
            await this.activityTableRepository.remove(result);
            this.pubSub.publish(exports.NEW_ACTIVITY_ADD, 1);
            return {
                status: true,
                status_text: "Successfully Removed",
                data: null
            };
        }
        else {
            return {
                status: false,
                status_text: "Not Found",
                data: null
            };
        }
    }
};
exports.ActivityTableService = ActivityTableService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(activity_table_entity_1.ActivityTable)),
    __param(1, (0, common_1.Inject)(GlobalConstants_1.PUB_SUB)),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        graphql_redis_subscriptions_1.RedisPubSub,
        theme_table_service_1.ThemeTableService])
], ActivityTableService);
