"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AdvertisementTableService = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const advertisement_type_table_service_1 = require("../advertisement-type-table/advertisement-type-table.service");
const advertisement_type_table_entity_1 = require("../advertisement-type-table/entities/advertisement-type-table.entity");
const farmers_table_entity_1 = require("../farmers-table/entities/farmers-table.entity");
const farmers_table_service_1 = require("../farmers-table/farmers-table.service");
const user_collection_service_1 = require("../user-collection/user-collection.service");
const typeorm_2 = require("typeorm");
const advertisement_table_entity_1 = require("./entities/advertisement-table.entity");
let AdvertisementTableService = class AdvertisementTableService {
    constructor(advertisementTableRepository, advertisementTypeTableService, userCollectionService, farmersTableService) {
        this.advertisementTableRepository = advertisementTableRepository;
        this.advertisementTypeTableService = advertisementTypeTableService;
        this.userCollectionService = userCollectionService;
        this.farmersTableService = farmersTableService;
    }
    async create(createAdvertisementTableInput, user) {
        let checkIfExists = await this.checkIfExists(createAdvertisementTableInput);
        if (checkIfExists) {
            return {
                status: false,
                status_text: "Already Exists",
                data: null
            };
        }
        let ht = new advertisement_table_entity_1.AdvertisementTable();
        let userTable = await this.userCollectionService.getOneByUserId(user.userId);
        let farmerTable = await this.farmersTableService.findOne(createAdvertisementTableInput.farmerId);
        let advertisementTypeTable = await this.advertisementTypeTableService.findOne(createAdvertisementTableInput.advertisementTypeId);
        ht.UserTable = userTable;
        ht.FarmersTable = farmerTable;
        ht.AdvertisementTypeTable = advertisementTypeTable;
        ht.title = createAdvertisementTableInput.title;
        ht.description = createAdvertisementTableInput.description;
        let result = await this.advertisementTableRepository.save(ht);
        return {
            status: true,
            status_text: "Successfully Submitted",
            data: result
        };
    }
    async checkIfExists(createAdvertisementTableInput) {
        let result = await this.advertisementTableRepository.count({
            where: {
                title: createAdvertisementTableInput.title,
                description: createAdvertisementTableInput.description,
            }
        });
        if (result == 0) {
            return false;
        }
        else {
            return true;
        }
    }
    async findByFarmerId(farmerId) {
        let farmer = await this.farmersTableService.findOne(farmerId);
        return await this.advertisementTableRepository.find({
            where: {
                FarmersTable: farmer
            },
            relations: ["UserTable"]
        });
    }
    async findAll() {
        return await this.advertisementTableRepository.find({
            relations: ["UserTable", "FarmersTable", "AdvertisementTypeTable"]
        });
    }
    async findOne(advertisementId) {
        return await this.advertisementTableRepository.findOne({
            where: {
                advertisementId
            },
            relations: ["UserTable", "FarmersTable"]
        });
    }
    async findOneByFarmerId(farmerId) {
        let farmerTable = await this.farmersTableService.findOne(farmerId);
        let result = await this.advertisementTableRepository.createQueryBuilder("r")
            .where("r.farmerId=:farmerId", { farmerId: farmerTable.farmerId })
            .leftJoinAndMapOne("r.FarmersTable", farmers_table_entity_1.FarmersTable, "ml1", "ml1.farmerId=r.farmerId")
            .leftJoinAndMapOne("r.AdvertisementTypeTable", advertisement_type_table_entity_1.AdvertisementTypeTable, "ml2", "ml2.advertisementTypeId=r.advertisementTypeId")
            .getMany();
        return result;
    }
    async update(advertisementId, updateAdvertisementTableInput, user) {
        let ht = await this.advertisementTableRepository.findOne({
            where: {
                advertisementId
            }
        });
        if (!ht) {
            return {
                status: false,
                status_text: "Not found",
                data: null
            };
        }
        else {
            let userTable = await this.userCollectionService.getOneByUserId(user.userId);
            let farmerTable = await this.farmersTableService.findOne(updateAdvertisementTableInput.farmerId);
            let advertisementTypeTable = await this.advertisementTypeTableService.findOne(updateAdvertisementTableInput.advertisementTypeId);
            ht.UserTable = userTable;
            ht.FarmersTable = farmerTable;
            ht.AdvertisementTypeTable = advertisementTypeTable;
            ht.title = updateAdvertisementTableInput.title;
            ht.description = updateAdvertisementTableInput.description;
            let result = await this.advertisementTableRepository.save(ht);
            return {
                status: true,
                status_text: "Successfully Updated",
                data: result
            };
        }
    }
    async remove(advertisementId) {
        let ht = await this.advertisementTableRepository.findOne({
            where: {
                advertisementId
            }
        });
        if (!ht) {
            return {
                status: false,
                status_text: "Not found",
                data: null
            };
        }
        else {
            await this.advertisementTableRepository.remove(ht);
            return {
                status: true,
                status_text: "Successfully Deleted",
                data: null
            };
        }
    }
    async toggleActiveStatus(id) {
        let u = await this.findOne(id);
        if (!!u) {
            let oldStatus = u.status;
            u.status = !oldStatus;
            let newStatus = !oldStatus ? 'Activated' : 'De-Activated';
            let u1 = await this.advertisementTableRepository.save(u);
            return {
                status: true,
                status_text: "Status Changed",
                data: await this.findOne(id)
            };
        }
        else {
            return {
                status: false,
                status_text: "User not found",
                data: null
            };
        }
    }
};
AdvertisementTableService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(advertisement_table_entity_1.AdvertisementTable)),
    __param(1, (0, common_1.Inject)((0, common_1.forwardRef)(() => advertisement_type_table_service_1.AdvertisementTypeTableService))),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        advertisement_type_table_service_1.AdvertisementTypeTableService,
        user_collection_service_1.UserCollectionService,
        farmers_table_service_1.FarmersTableService])
], AdvertisementTableService);
exports.AdvertisementTableService = AdvertisementTableService;
