"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AnimalHealthCoverageService = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const moment_1 = __importDefault(require("moment"));
const animal_entry_table_service_1 = require("../animal-table/animal-entry-table.service");
const animal_entry_table_entity_1 = require("../animal-table/entities/animal-entry-table.entity");
const farmers_table_entity_1 = require("../farmers-table/entities/farmers-table.entity");
const health_coverage_table_entity_1 = require("../health-coverage-table/entities/health-coverage-table.entity");
const health_coverage_table_service_1 = require("../health-coverage-table/health-coverage-table.service");
const housing_table_entity_1 = require("../housing-table/entities/housing-table.entity");
const typeorm_2 = require("typeorm");
const animal_health_coverage_entity_1 = require("./entities/animal-health-coverage.entity");
const data_entry_table_service_1 = require("../data-entry-table/data-entry-table.service");
const GlobalConstants_1 = require("../helper/GlobalConstants");
let AnimalHealthCoverageService = class AnimalHealthCoverageService {
    constructor(animalHealthCoverageTableRepository, dataEntryTableService, animalEntryTableService, healthCoverageTableService) {
        this.animalHealthCoverageTableRepository = animalHealthCoverageTableRepository;
        this.dataEntryTableService = dataEntryTableService;
        this.animalEntryTableService = animalEntryTableService;
        this.healthCoverageTableService = healthCoverageTableService;
    }
    async create(createAnimalHealthCoverageInput, user) {
        if (Array.isArray(createAnimalHealthCoverageInput.animalData) &&
            createAnimalHealthCoverageInput.animalData.length > 0) {
            for (const el of createAnimalHealthCoverageInput.animalData) {
                let checkIfExists = await this.checkIfExists(el.animalId, createAnimalHealthCoverageInput.dateOfCoverage);
                if (!checkIfExists) {
                    let dd = new animal_health_coverage_entity_1.AnimalHealthCoverage();
                    dd.dateOfCoverage = createAnimalHealthCoverageInput.dateOfCoverage;
                    dd.details = createAnimalHealthCoverageInput.details;
                    dd.isNextRequired = createAnimalHealthCoverageInput.isNextRequired;
                    if (createAnimalHealthCoverageInput.isNextRequired == "YES") {
                        dd.nextDate = createAnimalHealthCoverageInput.nextDate;
                    }
                    else {
                        dd.nextDate = null;
                    }
                    dd.HealthCoverageTable =
                        await this.healthCoverageTableService.findOne(createAnimalHealthCoverageInput.coverageId);
                    dd.AnimalTable = await this.animalEntryTableService.findByAnimalId(el.animalId);
                    await this.animalHealthCoverageTableRepository.save(dd);
                }
            }
            this.dataEntryTableService.create({ type: GlobalConstants_1.CREATE, typeOfEntry: GlobalConstants_1.COVERAGE_ADD, remarks: "Health Coverage Added" }, user);
            return {
                status: true,
                status_text: 'Successfully Created',
                data: null,
            };
        }
        else if (createAnimalHealthCoverageInput.currentStockId != null &&
            createAnimalHealthCoverageInput.currentStockId != undefined) {
            let animals = await this.animalEntryTableService.findByCurrentStock(createAnimalHealthCoverageInput.currentStockId, createAnimalHealthCoverageInput.qty);
            for (const el of animals) {
                let checkIfExists = await this.checkIfExists(el.animalId, createAnimalHealthCoverageInput.dateOfCoverage);
                if (!checkIfExists) {
                    let dd = new animal_health_coverage_entity_1.AnimalHealthCoverage();
                    dd.dateOfCoverage = createAnimalHealthCoverageInput.dateOfCoverage;
                    dd.details = createAnimalHealthCoverageInput.details;
                    dd.isNextRequired = createAnimalHealthCoverageInput.isNextRequired;
                    dd.nextDate = createAnimalHealthCoverageInput.nextDate;
                    dd.HealthCoverageTable =
                        await this.healthCoverageTableService.findOne(createAnimalHealthCoverageInput.coverageId);
                    dd.AnimalTable = await this.animalEntryTableService.findByAnimalId(el.animalId);
                    await this.animalHealthCoverageTableRepository.save(dd);
                }
            }
            this.dataEntryTableService.create({ type: GlobalConstants_1.CREATE, typeOfEntry: GlobalConstants_1.COVERAGE_ADD, remarks: "Health Coverage Added" }, user);
            return {
                status: true,
                status_text: 'Successfully Created',
                data: null,
            };
        }
    }
    async checkIfExists(animalId, dateOfCoverage) {
        let rr = await this.animalHealthCoverageTableRepository.count({
            where: {
                dateOfCoverage: dateOfCoverage,
                AnimalTable: {
                    animalId: animalId,
                },
            },
        });
        if (rr == 0) {
            return false;
        }
        else {
            return true;
        }
    }
    async findAll() {
        return await this.animalHealthCoverageTableRepository.find({
            relations: {
                AnimalTable: {
                    HousingTable: {
                        FarmersTable: true,
                    },
                },
                HealthCoverageTable: true,
            },
        });
    }
    async findOne(healthCoverageId) {
        return await this.animalHealthCoverageTableRepository.findOne({
            where: {
                healthCoverageId: healthCoverageId,
            },
            relations: {
                AnimalTable: {
                    HousingTable: {
                        FarmersTable: true,
                    },
                },
                HealthCoverageTable: true,
            },
        });
    }
    async findAnimalHealthCoverageByFarmerId(farmerId) {
        return await this.animalHealthCoverageTableRepository.find({
            where: {
                AnimalTable: {
                    HousingTable: {
                        FarmersTable: {
                            farmerId,
                        },
                    },
                },
            },
            relations: {
                AnimalTable: {
                    HousingTable: {
                        FarmersTable: true,
                    },
                },
                HealthCoverageTable: true,
            },
        });
    }
    async findAnimalHealthCoverageByHousingd(housingId) {
        return await this.animalHealthCoverageTableRepository.find({
            where: {
                AnimalTable: {
                    HousingTable: {
                        housingId,
                    },
                },
            },
            relations: {
                AnimalTable: {
                    HousingTable: true,
                },
                HealthCoverageTable: true,
            },
        });
    }
    async update(healthCoverageId, updateAnimalHealthCoverageInput, user) {
        let ht = await this.animalHealthCoverageTableRepository.findOne({
            where: {
                healthCoverageId,
            },
        });
        if (!ht) {
            return {
                status: false,
                status_text: 'Not found',
                data: null,
            };
        }
        else {
            ht.dateOfCoverage = updateAnimalHealthCoverageInput.dateOfCoverage;
            ht.details = updateAnimalHealthCoverageInput.details;
            ht.isNextRequired = updateAnimalHealthCoverageInput.isNextRequired;
            if (updateAnimalHealthCoverageInput.isNextRequired == "YES") {
                ht.nextDate = updateAnimalHealthCoverageInput.nextDate;
            }
            else {
                ht.nextDate = null;
            }
            ht.HealthCoverageTable = await this.healthCoverageTableService.findOne(updateAnimalHealthCoverageInput.coverageId);
            ht.AnimalTable = await this.animalEntryTableService.findByAnimalId(updateAnimalHealthCoverageInput.animalId);
            let result = await this.animalHealthCoverageTableRepository.save(ht);
            this.dataEntryTableService.create({ type: GlobalConstants_1.UPDATE, typeOfEntry: GlobalConstants_1.COVERAGE_UPDATE, remarks: "Health Coverage Updated" }, user);
            return {
                status: true,
                status_text: 'Successfully Updated',
                data: result,
            };
        }
    }
    async remove(healthCoverageId) {
        let ht = await this.animalHealthCoverageTableRepository.findOne({
            where: {
                healthCoverageId,
            },
        });
        if (!ht) {
            return {
                status: false,
                status_text: 'Not found',
                data: null,
            };
        }
        else {
            await this.animalHealthCoverageTableRepository.remove(ht);
            return {
                status: true,
                status_text: 'Successfully Deleted',
                data: null,
            };
        }
    }
    async toggleNextDoseStatus(healthCoverageId, user) {
        let ht = await this.animalHealthCoverageTableRepository.findOne({
            where: {
                healthCoverageId
            }
        });
        if (ht) {
            let newStatus = 0;
            if (ht.nextStatus == 1) {
                newStatus = 0;
            }
            else {
                newStatus = 1;
            }
            ht.nextStatus = newStatus;
            await this.animalHealthCoverageTableRepository.save(ht);
            this.dataEntryTableService.create({ type: GlobalConstants_1.UPDATE, typeOfEntry: GlobalConstants_1.COVERAGE_UPDATE, remarks: "Health Coverage Status Updated" }, user);
            return {
                status: true,
                status_text: "Successfully Updated",
                data: null
            };
        }
        else {
            return {
                status: false,
                status_text: "Not found",
                data: null
            };
        }
    }
    async getHealthCoverageDetailsOfOneAnimal(animalId) {
        let result = this.animalHealthCoverageTableRepository.createQueryBuilder("r")
            .leftJoinAndMapOne("r.AnimalTable", animal_entry_table_entity_1.AnimalEntryTable, "ml1", "ml1.animalId=r.animalId")
            .leftJoinAndMapOne("r.HealthCoverageTable", health_coverage_table_entity_1.HealthCoverageTable, "ml2", "ml2.coverageId=r.coverageId")
            .where("r.animalId=:animalId", { animalId: animalId })
            .getMany();
        return result;
    }
    checkDaysLeft(date) {
        let d = (0, moment_1.default)(date, 'YYYY-MM-DD').diff((0, moment_1.default)(), 'days');
        if (d < 0) {
            return `has expired ${Math.abs(d)} days ago.`;
        }
        else {
            return `is approaching. you have ${d} days left.`;
        }
    }
    async getExpiredOrNearlyExpiredData(farmerId, days) {
        let today = (0, moment_1.default)().format("YYYY-MM-DD");
        let result = await this.animalHealthCoverageTableRepository.createQueryBuilder("r")
            .leftJoinAndMapOne("r.AnimalTable", animal_entry_table_entity_1.AnimalEntryTable, "ml3", "ml3.animalId=r.animalId")
            .leftJoinAndMapOne("ml3.HousingTable", housing_table_entity_1.HousingTable, "ml1", "ml1.housingId=ml3.housingId")
            .leftJoinAndMapOne("ml1.FarmersTable", farmers_table_entity_1.FarmersTable, "ml2", "ml2.farmerId=ml1.farmerId")
            .where("ml2.farmerId=:farmerId", { farmerId: farmerId })
            .andWhere("DATEDIFF(STR_TO_DATE(r.nextDate, '%Y-%m-%d'),STR_TO_DATE(:today, '%Y-%m-%d')) < :days", { days: days, today: today })
            .getMany();
        let d = [];
        result.map((el) => {
            let d1 = {
                description: `Next Vaccination date for health coverage of Animal#${el.AnimalTable.animalId} ${this.checkDaysLeft(el.nextDate)}`,
                link: `/admin/farmers/single/details?farmerId=${farmerId}&step=health_coverage`
            };
            d.push(d1);
        });
        return d;
    }
};
AnimalHealthCoverageService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(animal_health_coverage_entity_1.AnimalHealthCoverage)),
    __param(2, (0, common_1.Inject)((0, common_1.forwardRef)(() => animal_entry_table_service_1.AnimalEntryTableService))),
    __param(3, (0, common_1.Inject)((0, common_1.forwardRef)(() => health_coverage_table_service_1.HealthCoverageTableService))),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        data_entry_table_service_1.DataEntryTableService,
        animal_entry_table_service_1.AnimalEntryTableService,
        health_coverage_table_service_1.HealthCoverageTableService])
], AnimalHealthCoverageService);
exports.AnimalHealthCoverageService = AnimalHealthCoverageService;
