import { UserCollectionService } from 'src/user-collection/user-collection.service';
import { Repository } from 'typeorm';
import { HousingTableService } from 'src/housing-table/housing-table.service';
import { FarmersTableService } from 'src/farmers-table/farmers-table.service';
import { BreedTableService } from 'src/breed-table/breed-table.service';
import { CurrentStockTableService } from 'src/current-stock/current-stock-table.service';
import { AnimalEntryTable } from './entities/animal-entry-table.entity';
import { CreateAnimalTableInput } from './dto/create-animal-table.input';
import { UpdateAnimalTableInput } from './dto/update-animal-table.input';
import { RedisPubSub } from 'graphql-redis-subscriptions';
import { StockBalanceTableService } from 'src/stock-balance/stock-balance-table.service';
import { LedgerEntryTableService } from 'src/ledger-table/ledger-entry-table.service';
import { ParentTableService } from 'src/parent-table/parent-table.service';
import { CreateBoarTableInput } from './dto/create-boar-table.input';
export declare const NEW_ANIMAL_ADD = "NEW_ANIMAL_ADD";
export declare const ANIMAL_UPDATE = "ANIMAL_UPDATE";
export declare class AnimalEntryTableService {
    private readonly animalTableRepository;
    private pubSub;
    private readonly userCollectionService;
    private readonly farmersTableService;
    private readonly housingTableService;
    private readonly breedTableService;
    private readonly stockBalanceTableService;
    private readonly currentStockTableService;
    private readonly ledgerEntryTableService;
    private readonly parentTableService;
    constructor(animalTableRepository: Repository<AnimalEntryTable>, pubSub: RedisPubSub, userCollectionService: UserCollectionService, farmersTableService: FarmersTableService, housingTableService: HousingTableService, breedTableService: BreedTableService, stockBalanceTableService: StockBalanceTableService, currentStockTableService: CurrentStockTableService, ledgerEntryTableService: LedgerEntryTableService, parentTableService: ParentTableService);
    create(createAnimalTableInput: CreateAnimalTableInput, user: any): Promise<{
        status: boolean;
        status_text: string;
        data: AnimalEntryTable;
    }>;
    createBoar(createBoarTableInput: CreateBoarTableInput, user: any): Promise<{
        status: boolean;
        status_text: string;
        data: AnimalEntryTable;
    }>;
    multipleEntry(createAnimalTableInput: CreateAnimalTableInput, user: any, qtyMale: number, qtyFemale: number, qtyTotal: number, sowId: any, boarId: any, currentStockId: number): Promise<{
        status: boolean;
        status_text: string;
        data: any;
    }>;
    findByHousingId(housingId: number): Promise<AnimalEntryTable[]>;
    findByAnimalId(animalId: number): Promise<AnimalEntryTable>;
    findByStockId(stockId: number): Promise<AnimalEntryTable[]>;
    findByAnimalAge(age: string, limit: number): Promise<AnimalEntryTable[]>;
    findByCurrentStock(currentStockId: number, limit: number): Promise<AnimalEntryTable[]>;
    findByAnimalForBreeding(housingId: number): Promise<AnimalEntryTable[]>;
    findAll(): Promise<AnimalEntryTable[]>;
    findOne(animalId: number): Promise<AnimalEntryTable[]>;
    findAllBoarDetached(): Promise<AnimalEntryTable[]>;
    findOneForReport(animalId: number): Promise<AnimalEntryTable>;
    findAllAnimalByDistAndDivAndFarmer(districtId: number, subDivisionId: number, farmerId: number): Promise<{
        data: any;
        summary: any[];
    }>;
    update(animalId: number, updateAnimalTableInput: UpdateAnimalTableInput, user: any): Promise<{
        status: boolean;
        status_text: string;
        data: AnimalEntryTable;
    }>;
    deceaseEntry(animalId: number, status: number): Promise<boolean>;
    ageUpdate(): Promise<{
        status: boolean;
        status_text: string;
    }>;
    updateBoarSow(animalId: number): Promise<{
        status: boolean;
        status_text: string;
    }>;
    updateCastratedStatus(animalId: number): Promise<{
        status: boolean;
        status_text: string;
    }>;
    findIndividualFarmerReport(districtId: number, subDivisionId: number, farmerId: number): Promise<{
        data: AnimalEntryTable[];
        summary: any[];
    }>;
}
