"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AnimalVaccinationTableService = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const moment_1 = __importDefault(require("moment"));
const animal_entry_table_service_1 = require("../animal-table/animal-entry-table.service");
const animal_entry_table_entity_1 = require("../animal-table/entities/animal-entry-table.entity");
const disease_control_table_service_1 = require("../disease-control/disease-control-table.service");
const farmers_table_entity_1 = require("../farmers-table/entities/farmers-table.entity");
const housing_table_entity_1 = require("../housing-table/entities/housing-table.entity");
const vaccine_table_service_1 = require("../vaccine-table/vaccine-table.service");
const typeorm_2 = require("typeorm");
const animal_vaccination_table_entity_1 = require("./entities/animal-vaccination-table.entity");
const data_entry_table_service_1 = require("../data-entry-table/data-entry-table.service");
const GlobalConstants_1 = require("../helper/GlobalConstants");
let AnimalVaccinationTableService = class AnimalVaccinationTableService {
    constructor(animalVaccinationTableRepository, dataEntryTableService, animalEntryTableService, vaccineTableService, diseaseControlTableService) {
        this.animalVaccinationTableRepository = animalVaccinationTableRepository;
        this.dataEntryTableService = dataEntryTableService;
        this.animalEntryTableService = animalEntryTableService;
        this.vaccineTableService = vaccineTableService;
        this.diseaseControlTableService = diseaseControlTableService;
    }
    async create(createAnimalVaccinationTableInput, user) {
        if (Array.isArray(createAnimalVaccinationTableInput.animalData) && createAnimalVaccinationTableInput.animalData.length > 0) {
            for (const el of createAnimalVaccinationTableInput.animalData) {
                let checkIfExists = await this.checkIfExists(el.animalId, createAnimalVaccinationTableInput.dateOfVaccination);
                if (!checkIfExists) {
                    let dd = new animal_vaccination_table_entity_1.AnimalVaccinationTable();
                    dd.dateOfVaccination = createAnimalVaccinationTableInput.dateOfVaccination;
                    dd.details = createAnimalVaccinationTableInput.details;
                    dd.nextDateOfVaccination = createAnimalVaccinationTableInput.nextDateOfVaccination;
                    dd.VaccineTable = await this.vaccineTableService.findOne(createAnimalVaccinationTableInput.vaccineId);
                    dd.AnimalTable = await this.animalEntryTableService.findByAnimalId(el.animalId);
                    dd.DiseaseControlTable = await this.diseaseControlTableService.findOne(createAnimalVaccinationTableInput.diseaseControlId);
                    await this.animalVaccinationTableRepository.save(dd);
                }
            }
        }
        else if (createAnimalVaccinationTableInput.currentStockId != null && createAnimalVaccinationTableInput.currentStockId != undefined) {
            let animals = await this.animalEntryTableService.findByCurrentStock(createAnimalVaccinationTableInput.currentStockId, createAnimalVaccinationTableInput.qty);
            for (const el of animals) {
                let checkIfExists = await this.checkIfExists(el.animalId, createAnimalVaccinationTableInput.dateOfVaccination);
                if (!checkIfExists) {
                    let dd = new animal_vaccination_table_entity_1.AnimalVaccinationTable();
                    dd.dateOfVaccination = createAnimalVaccinationTableInput.dateOfVaccination;
                    dd.details = createAnimalVaccinationTableInput.details;
                    dd.nextDateOfVaccination = createAnimalVaccinationTableInput.nextDateOfVaccination;
                    dd.VaccineTable = await this.vaccineTableService.findOne(createAnimalVaccinationTableInput.vaccineId);
                    dd.AnimalTable = await this.animalEntryTableService.findByAnimalId(el.animalId);
                    dd.DiseaseControlTable = await this.diseaseControlTableService.findOne(createAnimalVaccinationTableInput.diseaseControlId);
                    await this.animalVaccinationTableRepository.save(dd);
                }
            }
        }
        this.dataEntryTableService.create({ type: GlobalConstants_1.CREATE, typeOfEntry: GlobalConstants_1.VACCINATION_ADD, remarks: "Vaccination Added" }, user);
        return {
            status: true,
            status_text: "Successfully Created",
            data: null
        };
    }
    async checkIfExists(animalId, dateOfVaccination) {
        let rr = await this.animalVaccinationTableRepository.count({
            where: {
                dateOfVaccination: dateOfVaccination,
                AnimalTable: {
                    animalId: animalId
                }
            }
        });
        if (rr == 0) {
            return false;
        }
        else {
            return true;
        }
    }
    async findAll() {
        return await this.animalVaccinationTableRepository.find({
            relations: {
                AnimalTable: {
                    HousingTable: {
                        FarmersTable: true
                    }
                },
                VaccineTable: true,
                DiseaseControlTable: true
            }
        });
    }
    async findOne(vaccinationId) {
        return await this.animalVaccinationTableRepository.findOne({
            where: {
                vaccinationId: vaccinationId
            },
            relations: {
                AnimalTable: {
                    HousingTable: {
                        FarmersTable: true
                    }
                },
                VaccineTable: true,
                DiseaseControlTable: true
            }
        });
    }
    async findVaccinationByFarmerId(farmerId) {
        return await this.animalVaccinationTableRepository.find({
            where: {
                AnimalTable: {
                    HousingTable: {
                        FarmersTable: {
                            farmerId
                        }
                    }
                },
            },
            relations: {
                AnimalTable: {
                    HousingTable: {
                        FarmersTable: true
                    }
                },
                VaccineTable: true,
                DiseaseControlTable: true
            }
        });
    }
    async findVaccinationByHousingId(housingId) {
        let result = await this.animalVaccinationTableRepository.find({
            where: {
                AnimalTable: {
                    HousingTable: {
                        housingId
                    }
                },
            },
            relations: {
                AnimalTable: {
                    HousingTable: true
                },
                VaccineTable: true,
                DiseaseControlTable: true
            }
        });
        console.log("🚀 ~ file: animal-vaccination-table.service.ts ~ line 204 ~ AnimalVaccinationTableService ~ findVaccinationByHousingId ~ result", result);
        return result;
    }
    async update(vaccinationId, updateAnimalVaccinationTableInput, user) {
        let ht = await this.animalVaccinationTableRepository.findOne({
            where: {
                vaccinationId
            }
        });
        if (!ht) {
            return {
                status: false,
                status_text: "Not found",
                data: null
            };
        }
        else {
            ht.dateOfVaccination = updateAnimalVaccinationTableInput.dateOfVaccination;
            ht.details = updateAnimalVaccinationTableInput.details;
            ht.nextDateOfVaccination = updateAnimalVaccinationTableInput.nextDateOfVaccination;
            ht.VaccineTable = await this.vaccineTableService.findOne(updateAnimalVaccinationTableInput.vaccineId);
            ht.AnimalTable = await this.animalEntryTableService.findByAnimalId(updateAnimalVaccinationTableInput.animalId);
            ht.DiseaseControlTable = await this.diseaseControlTableService.findOne(updateAnimalVaccinationTableInput.diseaseControlId);
            let result = await this.animalVaccinationTableRepository.save(ht);
            this.dataEntryTableService.create({ type: GlobalConstants_1.UPDATE, typeOfEntry: GlobalConstants_1.VACCINATION_UPDATE, remarks: "Vaccination Updated" }, user);
            return {
                status: true,
                status_text: "Successfully Updated",
                data: result
            };
        }
    }
    async remove(vaccinationId) {
        let ht = await this.animalVaccinationTableRepository.findOne({
            where: {
                vaccinationId
            }
        });
        if (!ht) {
            return {
                status: false,
                status_text: "Not found",
                data: null
            };
        }
        else {
            await this.animalVaccinationTableRepository.remove(ht);
            return {
                status: true,
                status_text: "Successfully Deleted",
                data: null
            };
        }
    }
    async getVaccineDetailsOfOneAnimal(animalId) {
        let result = await this.animalVaccinationTableRepository.createQueryBuilder("r")
            .leftJoinAndMapOne("r.AnimalTable", animal_entry_table_entity_1.AnimalEntryTable, "ml1", "ml1.animalId=r.animalId")
            .where("r.animalId=:animalId", { animalId: animalId })
            .getMany();
        return result;
    }
    async toggleNextDoseStatus(vaccinationId, user) {
        let ht = await this.animalVaccinationTableRepository.findOne({
            where: {
                vaccinationId
            }
        });
        if (ht) {
            let newStatus = 0;
            if (ht.isNextDoseGiven == 1) {
                newStatus = 0;
            }
            else {
                newStatus = 1;
            }
            ht.isNextDoseGiven = newStatus;
            await this.animalVaccinationTableRepository.save(ht);
            this.dataEntryTableService.create({ type: GlobalConstants_1.UPDATE, typeOfEntry: GlobalConstants_1.VACCINATION_UPDATE, remarks: "Vaccination Status Updated" }, user);
            return {
                status: true,
                status_text: "Successfully Updated",
            };
        }
        else {
            return {
                status: false,
                status_text: "Not found",
            };
        }
    }
    checkDaysLeft(date) {
        let d = (0, moment_1.default)(date, 'YYYY-MM-DD').diff((0, moment_1.default)(), 'days');
        if (d < 0) {
            return `has expired ${Math.abs(d)} days ago.`;
        }
        else {
            return `is approaching. you have ${d} days left.`;
        }
    }
    async getExpiredOrNearlyExpiredData(farmerId, days) {
        let today = (0, moment_1.default)().format("YYYY-MM-DD");
        let result = await this.animalVaccinationTableRepository.createQueryBuilder("r")
            .leftJoinAndMapOne("r.AnimalTable", animal_entry_table_entity_1.AnimalEntryTable, "ml3", "ml3.animalId=r.animalId")
            .leftJoinAndMapOne("ml3.HousingTable", housing_table_entity_1.HousingTable, "ml1", "ml1.housingId=ml3.housingId")
            .leftJoinAndMapOne("ml1.FarmersTable", farmers_table_entity_1.FarmersTable, "ml2", "ml2.farmerId=ml1.farmerId")
            .where("ml2.farmerId=:farmerId", { farmerId: farmerId })
            .andWhere("DATEDIFF(STR_TO_DATE(r.nextDateOfVaccination, '%Y-%m-%d'),STR_TO_DATE(:today, '%Y-%m-%d')) < :days", { days: days, today: today })
            .getMany();
        let d = [];
        result.map((el) => {
            let d1 = {
                description: `Next Vaccination for disease control date of Animal#${el.AnimalTable.animalId} ${this.checkDaysLeft(el.nextDateOfVaccination)}`,
                link: `/admin/farmers/single/details?farmerId=${farmerId}&step=disease_control`
            };
            d.push(d1);
        });
        return d;
    }
};
AnimalVaccinationTableService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(animal_vaccination_table_entity_1.AnimalVaccinationTable)),
    __param(2, (0, common_1.Inject)((0, common_1.forwardRef)(() => animal_entry_table_service_1.AnimalEntryTableService))),
    __param(3, (0, common_1.Inject)((0, common_1.forwardRef)(() => vaccine_table_service_1.VaccineTableService))),
    __param(4, (0, common_1.Inject)((0, common_1.forwardRef)(() => disease_control_table_service_1.DiseaseControlTableService))),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        data_entry_table_service_1.DataEntryTableService,
        animal_entry_table_service_1.AnimalEntryTableService,
        vaccine_table_service_1.VaccineTableService,
        disease_control_table_service_1.DiseaseControlTableService])
], AnimalVaccinationTableService);
exports.AnimalVaccinationTableService = AnimalVaccinationTableService;
