"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AppGateway = void 0;
const websockets_1 = require("@nestjs/websockets");
const common_1 = require("@nestjs/common");
const socket_io_1 = require("socket.io");
const ws_auth_guard_1 = require("./auth/guards/ws-auth.guard");
const auth_service_1 = require("./auth/auth.service");
const GlobalConstants_1 = require("./helper/GlobalConstants");
const graphql_redis_subscriptions_1 = require("graphql-redis-subscriptions");
let AppGateway = exports.AppGateway = class AppGateway {
    constructor(authService, pubSub) {
        this.authService = authService;
        this.pubSub = pubSub;
        this.logger = new common_1.Logger('AppGateway');
    }
    afterInit(server) {
        this.logger.log('Init');
    }
    async getActiveInstancesCount() {
        let count = await this.server.of("/").sockets.size;
        this.pubSub.publish('ADMIN_HEAD_COUNT', count);
        return { count: count };
    }
    handleDisconnect(client) {
        this.logger.log(`Client disconnected: ${client.id}`);
        this.getActiveInstancesCount();
    }
    async handleConnection(client, data) {
        var _a, _b;
        let token = (_b = (_a = client.handshake) === null || _a === void 0 ? void 0 : _a.auth) === null || _b === void 0 ? void 0 : _b.Authorization;
        if (token === null) {
            client.disconnect();
            this.logger.log(`Client Disconnect by Auth: ${client.id}`);
        }
        else {
            let decoded = await this.authService.isValidJwt(token);
            if (decoded) {
                this.logger.log(`Client connected: ${client.id}`);
                this.logger.log(`User connected: ${decoded.username}`);
                this.getActiveInstancesCount();
            }
            else {
                client.disconnect();
                this.logger.log(`Client Disconnect by Auth: ${client.id}`);
            }
        }
    }
    emitToClient(socket, type, data) {
        this.server.to(socket).emit(type, data);
    }
};
__decorate([
    (0, websockets_1.WebSocketServer)(),
    __metadata("design:type", socket_io_1.Server)
], AppGateway.prototype, "server", void 0);
__decorate([
    (0, common_1.UseGuards)(ws_auth_guard_1.WsJwtGuard),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [socket_io_1.Socket, Object]),
    __metadata("design:returntype", Promise)
], AppGateway.prototype, "handleConnection", null);
exports.AppGateway = AppGateway = __decorate([
    (0, websockets_1.WebSocketGateway)({
        cors: {
            origin: "*",
            methods: ["GET", "POST"],
            transports: ['websocket', 'polling'],
            credentials: true
        },
        allowEIO3: true
    }),
    __param(1, (0, common_1.Inject)(GlobalConstants_1.PUB_SUB)),
    __metadata("design:paramtypes", [auth_service_1.AuthService,
        graphql_redis_subscriptions_1.RedisPubSub])
], AppGateway);
