"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AppResolver = void 0;
const common_1 = require("@nestjs/common");
const graphql_1 = require("@nestjs/graphql");
const apollo_server_errors_1 = require("apollo-server-errors");
const graphql_redis_subscriptions_1 = require("graphql-redis-subscriptions");
const app_gateway_1 = require("./app.gateway");
const auth_service_1 = require("./auth/auth.service");
const gql_auth_guard_1 = require("./auth/guards/gql-auth.guard");
const roles_guard_1 = require("./auth/guards/roles.guard");
const GlobalConstants_1 = require("./helper/GlobalConstants");
const Count_type_1 = require("./types/Count.type");
const loginSuccess_type_1 = require("./types/loginSuccess.type");
const roles_decorator_1 = require("./user-collection/decorators/roles.decorator");
const role_enum_1 = require("./user-collection/roles/role.enum");
const app_service_1 = require("./app.service");
const dashboard_graph_mk_count_1 = require("./types/dashboard-graph-mk-count");
let AppResolver = exports.AppResolver = class AppResolver {
    constructor(authService, appGateway, appService, pubSub) {
        this.authService = authService;
        this.appGateway = appGateway;
        this.appService = appService;
        this.pubSub = pubSub;
    }
    async login(username, password) {
        let cr = await this.authService.login({
            username,
            password,
        });
        if (cr.status) {
            return cr;
        }
        else {
            throw new apollo_server_errors_1.UserInputError('Username or Password not matched', {
                username: username,
            });
        }
    }
    socketCount() {
        return this.appGateway.getActiveInstancesCount();
    }
    socketCountSub() {
        return this.pubSub.asyncIterator('ADMIN_HEAD_COUNT');
    }
    getCountOfMkRelatedDataForGraph(year) {
        return this.appService.getCountOfMkRelatedDataForGraph(year);
    }
};
__decorate([
    (0, graphql_1.Mutation)((returns) => loginSuccess_type_1.LoginSuccessType),
    __param(0, (0, graphql_1.Args)('username')),
    __param(1, (0, graphql_1.Args)('password')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String]),
    __metadata("design:returntype", Promise)
], AppResolver.prototype, "login", null);
__decorate([
    (0, graphql_1.Query)((returns) => Count_type_1.SocketCount),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], AppResolver.prototype, "socketCount", null);
__decorate([
    (0, graphql_1.Subscription)(() => Count_type_1.SocketCountSub, {
        resolve: (payload) => {
            return { count: payload };
        }
    }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], AppResolver.prototype, "socketCountSub", null);
__decorate([
    (0, roles_decorator_1.Roles)(role_enum_1.Role.SUPERADMIN),
    (0, common_1.UseGuards)(gql_auth_guard_1.GqlAuthGuard, roles_guard_1.RolesGuard),
    (0, graphql_1.Query)(() => [dashboard_graph_mk_count_1.DashboardGraphMkCountType]),
    __param(0, (0, graphql_1.Args)('year')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", void 0)
], AppResolver.prototype, "getCountOfMkRelatedDataForGraph", null);
exports.AppResolver = AppResolver = __decorate([
    (0, graphql_1.Resolver)(),
    __param(3, (0, common_1.Inject)(GlobalConstants_1.PUB_SUB)),
    __metadata("design:paramtypes", [auth_service_1.AuthService,
        app_gateway_1.AppGateway,
        app_service_1.AppService,
        graphql_redis_subscriptions_1.RedisPubSub])
], AppResolver);
