"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AssessmentAnswerTableService = exports.NEW_ANSWER_ADD = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const assessment_answer_table_entity_1 = require("./entities/assessment-answer-table.entity");
const typeorm_2 = require("typeorm");
const GlobalConstants_1 = require("../helper/GlobalConstants");
const graphql_redis_subscriptions_1 = require("graphql-redis-subscriptions");
const assessment_master_table_service_1 = require("../assessment-master-table/assessment-master-table.service");
const user_collection_service_1 = require("../user-collection/user-collection.service");
const assessment_questions_table_service_1 = require("../assessment-questions-table/assessment-questions-table.service");
const moment_1 = __importDefault(require("moment"));
const user_collection_entity_1 = require("../user-collection/entities/user-collection.entity");
const assessment_master_table_entity_1 = require("../assessment-master-table/entities/assessment-master-table.entity");
const course_progression_table_service_1 = require("../course-progression-table/course-progression-table.service");
exports.NEW_ANSWER_ADD = "NEW_ANSWER_ADD";
let AssessmentAnswerTableService = exports.AssessmentAnswerTableService = class AssessmentAnswerTableService {
    constructor(assessmentAnswersTableRepository, pubSub, assessmentMasterTableService, userCollectionService, assessmentQuestionsTableService, courseProgressionTableService) {
        this.assessmentAnswersTableRepository = assessmentAnswersTableRepository;
        this.pubSub = pubSub;
        this.assessmentMasterTableService = assessmentMasterTableService;
        this.userCollectionService = userCollectionService;
        this.assessmentQuestionsTableService = assessmentQuestionsTableService;
        this.courseProgressionTableService = courseProgressionTableService;
    }
    async create(createAssessmentAnswerTableInput, user) {
        let checkIfExistsCount = await this.checkIfExists(createAssessmentAnswerTableInput, user);
        let attempt = 1;
        if (checkIfExistsCount > 0) {
            attempt = checkIfExistsCount + 1;
        }
        let dd = new assessment_answer_table_entity_1.AssessmentAnswerTable();
        let assessmentMaster = await this.assessmentMasterTableService.findOne(createAssessmentAnswerTableInput.assessmentId);
        dd.AssessmentMasterTable = assessmentMaster;
        dd.answers = JSON.parse(createAssessmentAnswerTableInput.answers);
        let today = (0, moment_1.default)().format("DD-MM-YYYY");
        dd.assessmentDate = today;
        let questions = await this.assessmentQuestionsTableService.findOneByAssessmentMasterId(createAssessmentAnswerTableInput.assessmentId);
        let score = 0;
        for (const el of JSON.parse(createAssessmentAnswerTableInput.answers)) {
            let selectedQuestion = questions.find(el1 => el1.assessmentQuestionId === el.assessmentQuestionId);
            if (selectedQuestion) {
                if (el.value === selectedQuestion.correctAnswer) {
                    score = score + selectedQuestion.marks;
                }
            }
        }
        dd.score = score.toFixed(0);
        dd.attempt = attempt;
        if (score >= parseFloat(assessmentMaster.passingScore)) {
            await this.courseProgressionTableService.updateCourseProgressionAssessment(assessmentMaster.CourseTable.courseId, user, 1, (0, moment_1.default)().format("DD-MM-YYYY"), score, 1);
        }
        else {
            await this.courseProgressionTableService.updateCourseProgressionAssessment(assessmentMaster.CourseTable.courseId, user, 1, (0, moment_1.default)().format("DD-MM-YYYY"), score, 0);
        }
        let userData = await this.userCollectionService.findOneByUsername(user.username);
        dd.UserTable = userData;
        let result = await this.assessmentAnswersTableRepository.save(dd);
        this.pubSub.publish(exports.NEW_ANSWER_ADD, 1);
        return {
            status: true,
            status_text: "Successfully Created",
            data: result
        };
    }
    convertedArray(array) {
        return array.map(innerArray => {
            const obj = {};
            innerArray.forEach(([key, value]) => {
                obj[key] = value;
            });
            return obj;
        });
    }
    async recalculateScore(answerId) {
        let dd = await this.assessmentAnswersTableRepository.findOne({
            where: {
                answerId
            },
            relations: {
                AssessmentMasterTable: true
            }
        });
        if (dd) {
            let questions = await this.assessmentQuestionsTableService.findOneByAssessmentMasterId(dd.AssessmentMasterTable.assessmentId);
            let score = 0;
            let answers = dd.answers;
            for (const el of answers) {
                let selectedQuestion = questions.find(el1 => el1.assessmentQuestionId === el.assessmentQuestionId);
                if (selectedQuestion) {
                    let selectedAnswers = selectedQuestion.options;
                    let selectedOption = selectedAnswers.find(el3 => el3.optionLabel === el.value);
                    if (selectedOption) {
                        score = score + parseFloat(selectedOption.optionValue);
                    }
                }
            }
            dd.score = score.toFixed(0);
            let result = await this.assessmentAnswersTableRepository.save(dd);
            return result;
        }
        else {
            return false;
        }
    }
    async checkIfExists(createAssessmentAnswerTableInput, user) {
        let result = await this.assessmentAnswersTableRepository.count({
            where: {
                AssessmentMasterTable: {
                    assessmentId: createAssessmentAnswerTableInput.assessmentId
                },
                UserTable: {
                    id: user.id
                }
            }
        });
        return result;
    }
    async findAll() {
        return await this.assessmentAnswersTableRepository.find();
    }
    async findOne(answerId) {
        return await this.assessmentAnswersTableRepository.findOne({
            where: {
                answerId: answerId
            },
            relations: {
                AssessmentMasterTable: true,
                UserTable: {
                    DesignationTable: true
                }
            }
        });
    }
    async findMyLatestCourse(assessmentId, userId) {
        let result = await this.assessmentAnswersTableRepository.findOne({
            where: {
                AssessmentMasterTable: {
                    assessmentId
                },
                UserTable: {
                    id: userId
                }
            },
            relations: {
                AssessmentMasterTable: true,
                UserTable: {
                    DesignationTable: true
                }
            },
            order: {
                answerId: 'DESC'
            }
        });
        return result.score;
    }
    async findAllByCourseIdAndUserId(courseId, userId) {
        return await this.assessmentAnswersTableRepository.find({
            where: {
                AssessmentMasterTable: {
                    CourseTable: {
                        courseId
                    }
                },
                UserTable: {
                    id: userId
                }
            },
            relations: {
                AssessmentMasterTable: {
                    CourseTable: true
                }
            }
        });
    }
    async getListOfAnswersByUser(limit, page, type, search, user) {
        let data = await this.assessmentAnswersTableRepository
            .createQueryBuilder('r')
            .where('r.userId=:userId', { userId: user.id })
            .andWhere('r.type=:type', { type: type })
            .leftJoinAndMapOne('r.UserTable', user_collection_entity_1.UserTable, 'ml1', 'ml1.id=r.userId')
            .leftJoinAndMapOne('r.assessmentMasterTable', assessment_master_table_entity_1.AssessmentMasterTable, 'ml3', 'ml3.assessmentId=r.assessmentId')
            .take(limit)
            .skip(page);
        if (search != '') {
            let s = `${search}%`;
        }
        data = await data.getManyAndCount();
        let [result, count] = data;
        return {
            data: result,
            total: count,
        };
    }
    async remove(answerId) {
        let result = await this.assessmentAnswersTableRepository.findOne({
            where: {
                answerId: answerId
            }
        });
        if (result) {
            await this.assessmentAnswersTableRepository.remove(result);
            this.pubSub.publish(exports.NEW_ANSWER_ADD, 1);
            return {
                status: true,
                status_text: "Successfully Removed",
                data: null
            };
        }
        else {
            return {
                status: false,
                status_text: "Not Found",
                data: null
            };
        }
    }
};
exports.AssessmentAnswerTableService = AssessmentAnswerTableService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(assessment_answer_table_entity_1.AssessmentAnswerTable)),
    __param(1, (0, common_1.Inject)(GlobalConstants_1.PUB_SUB)),
    __param(5, (0, common_1.Inject)((0, common_1.forwardRef)(() => course_progression_table_service_1.CourseProgressionTableService))),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        graphql_redis_subscriptions_1.RedisPubSub,
        assessment_master_table_service_1.AssessmentMasterTableService,
        user_collection_service_1.UserCollectionService,
        assessment_questions_table_service_1.AssessmentQuestionsTableService,
        course_progression_table_service_1.CourseProgressionTableService])
], AssessmentAnswerTableService);
