"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AssessmentInterpretationTableService = exports.NEW_INTERPRETATION_ADD = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const assessment_interpretation_table_entity_1 = require("./entities/assessment-interpretation-table.entity");
const typeorm_2 = require("typeorm");
const GlobalConstants_1 = require("../helper/GlobalConstants");
const graphql_redis_subscriptions_1 = require("graphql-redis-subscriptions");
const assessment_master_table_service_1 = require("../assessment-master-table/assessment-master-table.service");
exports.NEW_INTERPRETATION_ADD = "NEW_INTERPRETATION_ADD";
let AssessmentInterpretationTableService = exports.AssessmentInterpretationTableService = class AssessmentInterpretationTableService {
    constructor(assessmentInterpretationTableRepository, pubSub, assessmentMasterTableService) {
        this.assessmentInterpretationTableRepository = assessmentInterpretationTableRepository;
        this.pubSub = pubSub;
        this.assessmentMasterTableService = assessmentMasterTableService;
    }
    async create(createAssessmentInterpretationTableInput) {
        let checkIfExists = await this.checkIfExists(createAssessmentInterpretationTableInput.interpretation, createAssessmentInterpretationTableInput.type, createAssessmentInterpretationTableInput.classification, createAssessmentInterpretationTableInput.maxScore, createAssessmentInterpretationTableInput.minScore);
        if (checkIfExists) {
            return {
                status: false,
                status_text: "Already Exists",
                data: null
            };
        }
        else {
            let dd = new assessment_interpretation_table_entity_1.AssessmentInterpretationTable();
            dd.interpretation = createAssessmentInterpretationTableInput.interpretation;
            dd.type = createAssessmentInterpretationTableInput.type;
            dd.classification = createAssessmentInterpretationTableInput.classification;
            dd.maxScore = createAssessmentInterpretationTableInput.maxScore;
            dd.minScore = createAssessmentInterpretationTableInput.minScore;
            let assessmentMaster = await this.assessmentMasterTableService.findOne(createAssessmentInterpretationTableInput.assessmentId);
            dd.AssessmentMasterTable = assessmentMaster;
            let result = await this.assessmentInterpretationTableRepository.save(dd);
            this.pubSub.publish(exports.NEW_INTERPRETATION_ADD, 1);
            return {
                status: true,
                status_text: "Successfully Created",
                data: result
            };
        }
    }
    async checkIfExists(interpretation, type, classification, maxScore, minScore) {
        let result = await this.assessmentInterpretationTableRepository.count({
            where: {
                interpretation: interpretation,
                type: type,
                maxScore: maxScore,
                minScore: minScore,
                classification: classification,
            }
        });
        if (result == 0) {
            return false;
        }
        else {
            return true;
        }
    }
    async findAll() {
        return await this.assessmentInterpretationTableRepository.find({
            relations: {
                AssessmentMasterTable: true
            }
        });
    }
    async findInterpretationByScore(assessmentId, score) {
        let result = await this.assessmentInterpretationTableRepository.find({
            where: {
                AssessmentMasterTable: {
                    assessmentId: assessmentId
                },
            }
        });
        for (const el of result) {
            if (parseFloat(el.minScore) <= parseFloat(score) && parseFloat(score) <= parseFloat(el.maxScore)) {
                return el;
            }
        }
        return false;
    }
    async findOne(interpretationId) {
        return await this.assessmentInterpretationTableRepository.findOne({
            where: {
                interpretationId: interpretationId
            }
        });
    }
    async update(interpretationId, updateAssessmentInterpretationTableInput) {
        let result = await this.assessmentInterpretationTableRepository.findOne({
            where: {
                interpretationId: interpretationId
            }
        });
        if (result) {
            let dd = result;
            dd.interpretation = updateAssessmentInterpretationTableInput.interpretation;
            dd.type = updateAssessmentInterpretationTableInput.type;
            dd.classification = updateAssessmentInterpretationTableInput.classification;
            dd.maxScore = updateAssessmentInterpretationTableInput.maxScore;
            dd.minScore = updateAssessmentInterpretationTableInput.minScore;
            let assessmentMaster = await this.assessmentMasterTableService.findOne(updateAssessmentInterpretationTableInput.assessmentId);
            dd.AssessmentMasterTable = assessmentMaster;
            let result1 = await this.assessmentInterpretationTableRepository.save(dd);
            this.pubSub.publish(exports.NEW_INTERPRETATION_ADD, 1);
            return {
                status: true,
                status_text: "Successfully Updated",
                data: result1
            };
        }
        else {
            return {
                status: false,
                status_text: "Not Found",
                data: null
            };
        }
    }
    async remove(interpretationId) {
        let result = await this.assessmentInterpretationTableRepository.findOne({
            where: {
                interpretationId: interpretationId
            }
        });
        if (result) {
            await this.assessmentInterpretationTableRepository.remove(result);
            this.pubSub.publish(exports.NEW_INTERPRETATION_ADD, 1);
            return {
                status: true,
                status_text: "Successfully Removed",
                data: null
            };
        }
        else {
            return {
                status: false,
                status_text: "Not Found",
                data: null
            };
        }
    }
};
exports.AssessmentInterpretationTableService = AssessmentInterpretationTableService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(assessment_interpretation_table_entity_1.AssessmentInterpretationTable)),
    __param(1, (0, common_1.Inject)(GlobalConstants_1.PUB_SUB)),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        graphql_redis_subscriptions_1.RedisPubSub,
        assessment_master_table_service_1.AssessmentMasterTableService])
], AssessmentInterpretationTableService);
