"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AssessmentMasterTableService = exports.NEW_ASSESSMENT_ADD = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const assessment_master_table_entity_1 = require("./entities/assessment-master-table.entity");
const typeorm_2 = require("typeorm");
const GlobalConstants_1 = require("../helper/GlobalConstants");
const graphql_redis_subscriptions_1 = require("graphql-redis-subscriptions");
const course_table_service_1 = require("../course-table/course-table.service");
const course_progression_table_service_1 = require("../course-progression-table/course-progression-table.service");
const assessment_answer_table_service_1 = require("../assessment-answer-table/assessment-answer-table.service");
exports.NEW_ASSESSMENT_ADD = "NEW_ASSESSMENT_ADD";
let AssessmentMasterTableService = exports.AssessmentMasterTableService = class AssessmentMasterTableService {
    constructor(assessmentMasterTableRepository, pubSub, courseTableService, assessmentAnswerTableService, courseProgressionTableService) {
        this.assessmentMasterTableRepository = assessmentMasterTableRepository;
        this.pubSub = pubSub;
        this.courseTableService = courseTableService;
        this.assessmentAnswerTableService = assessmentAnswerTableService;
        this.courseProgressionTableService = courseProgressionTableService;
    }
    async create(createAssessmentMasterTableInput) {
        let checkIfExists = await this.checkIfExists(createAssessmentMasterTableInput);
        if (checkIfExists) {
            return {
                status: false,
                status_text: "Already Exists",
                data: null
            };
        }
        else {
            let dd = new assessment_master_table_entity_1.AssessmentMasterTable();
            dd.assessmentName = createAssessmentMasterTableInput.assessmentName;
            dd.type = createAssessmentMasterTableInput.type;
            dd.totalScore = createAssessmentMasterTableInput.totalScore;
            dd.passingScore = createAssessmentMasterTableInput.passingScore;
            dd.isActive = createAssessmentMasterTableInput.isActive;
            dd.duration = createAssessmentMasterTableInput.duration;
            dd.week = createAssessmentMasterTableInput.week;
            dd.CourseTable = await this.courseTableService.findOne(createAssessmentMasterTableInput.courseId);
            let result = await this.assessmentMasterTableRepository.save(dd);
            this.pubSub.publish(exports.NEW_ASSESSMENT_ADD, 1);
            return {
                status: true,
                status_text: "Successfully Created",
                data: result
            };
        }
    }
    async checkIfExists(createAssessmentMasterTableInput) {
        let result = await this.assessmentMasterTableRepository.count({
            where: {
                assessmentName: createAssessmentMasterTableInput.assessmentName,
                type: createAssessmentMasterTableInput.type,
                week: createAssessmentMasterTableInput.week,
                totalScore: createAssessmentMasterTableInput.totalScore,
                passingScore: createAssessmentMasterTableInput.passingScore,
                CourseTable: {
                    courseId: createAssessmentMasterTableInput.courseId
                }
            }
        });
        if (result == 0) {
            return false;
        }
        else {
            return true;
        }
    }
    async findAll() {
        return await this.assessmentMasterTableRepository.find();
    }
    async findOne(assessmentId) {
        return await this.assessmentMasterTableRepository.findOne({
            where: {
                assessmentId: assessmentId
            },
            relations: {
                CourseTable: true
            }
        });
    }
    async findOneAssessmentByCourseUnitId(courseId) {
        return await this.assessmentMasterTableRepository.find({
            where: {
                CourseTable: {
                    courseId
                }
            }
        });
    }
    async findOneAssessmentByCourseUnitIdActive(courseId, user) {
        let result = await this.assessmentMasterTableRepository.find({
            where: {
                CourseTable: {
                    courseId
                },
                isActive: true
            },
            relations: {
                CourseTable: true
            }
        });
        let index = 0;
        for (const el of result) {
            let score = await this.assessmentAnswerTableService.findMyLatestCourse(el.assessmentId, user.id);
            result[index]['isAllowed'] = true;
            result[index]['score'] = score;
            index++;
        }
        return result;
    }
    async update(assessmentId, updateAssessmentMasterTableInput) {
        let result = await this.assessmentMasterTableRepository.findOne({
            where: {
                assessmentId: assessmentId
            }
        });
        if (result) {
            result.assessmentName = updateAssessmentMasterTableInput.assessmentName;
            result.type = updateAssessmentMasterTableInput.type;
            result.week = updateAssessmentMasterTableInput.week;
            result.totalScore = updateAssessmentMasterTableInput.totalScore;
            result.passingScore = updateAssessmentMasterTableInput.passingScore;
            result.isActive = updateAssessmentMasterTableInput.isActive;
            result.duration = updateAssessmentMasterTableInput.duration;
            result.CourseTable = await this.courseTableService.findOne(updateAssessmentMasterTableInput.courseId);
            let r = await this.assessmentMasterTableRepository.save(result);
            this.pubSub.publish(exports.NEW_ASSESSMENT_ADD, 1);
            return {
                status: true,
                status_text: "Successfully Updated",
                data: r
            };
        }
        else {
            return {
                status: false,
                status_text: "Not found",
                data: null
            };
        }
    }
    async toggleActiveStatus(assessmentId) {
        let u = await this.findOne(assessmentId);
        if (!!u) {
            let oldStatus = u.isActive;
            u.isActive = !oldStatus;
            let newStatus = !oldStatus ? 'Activated' : 'De-Activated';
            let u1 = await this.assessmentMasterTableRepository.save(u);
            return {
                status: true,
                status_text: "Status Changed",
                data: await this.findOne(assessmentId)
            };
        }
        else {
            return {
                status: false,
                status_text: "Survey not found",
                data: null
            };
        }
    }
    async remove(assessmentId) {
        let result = await this.assessmentMasterTableRepository.findOne({
            where: {
                assessmentId: assessmentId
            }
        });
        if (result) {
            await this.assessmentMasterTableRepository.remove(result);
            this.pubSub.publish(exports.NEW_ASSESSMENT_ADD, 1);
            return {
                status: true,
                status_text: "Successfully Removed",
                data: null
            };
        }
        else {
            return {
                status: false,
                status_text: "Not Found",
                data: null
            };
        }
    }
};
exports.AssessmentMasterTableService = AssessmentMasterTableService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(assessment_master_table_entity_1.AssessmentMasterTable)),
    __param(1, (0, common_1.Inject)(GlobalConstants_1.PUB_SUB)),
    __param(2, (0, common_1.Inject)((0, common_1.forwardRef)(() => course_table_service_1.CourseTableService))),
    __param(3, (0, common_1.Inject)((0, common_1.forwardRef)(() => assessment_answer_table_service_1.AssessmentAnswerTableService))),
    __param(4, (0, common_1.Inject)((0, common_1.forwardRef)(() => course_progression_table_service_1.CourseProgressionTableService))),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        graphql_redis_subscriptions_1.RedisPubSub,
        course_table_service_1.CourseTableService,
        assessment_answer_table_service_1.AssessmentAnswerTableService,
        course_progression_table_service_1.CourseProgressionTableService])
], AssessmentMasterTableService);
