"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AssessmentQuestionsTableService = exports.NEW_QUESTION_ADD = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const assessment_questions_table_entity_1 = require("./entities/assessment-questions-table.entity");
const typeorm_2 = require("typeorm");
const GlobalConstants_1 = require("../helper/GlobalConstants");
const graphql_redis_subscriptions_1 = require("graphql-redis-subscriptions");
const assessment_master_table_service_1 = require("../assessment-master-table/assessment-master-table.service");
exports.NEW_QUESTION_ADD = "NEW_QUESTION_ADD";
let AssessmentQuestionsTableService = exports.AssessmentQuestionsTableService = class AssessmentQuestionsTableService {
    constructor(assessmentQuestionsTableRepository, pubSub, assessmentMasterTableService) {
        this.assessmentQuestionsTableRepository = assessmentQuestionsTableRepository;
        this.pubSub = pubSub;
        this.assessmentMasterTableService = assessmentMasterTableService;
    }
    async create(createAssessmentQuestionsTableInput) {
        let checkIfAlreadyExists = await this.checkIfAlreadyExists(createAssessmentQuestionsTableInput);
        if (checkIfAlreadyExists == 0) {
            let st = new assessment_questions_table_entity_1.AssessmentQuestionsTable();
            st.assessmentQuestionEn = createAssessmentQuestionsTableInput.assessmentQuestionEn;
            st.assessmentQuestionAs = createAssessmentQuestionsTableInput.assessmentQuestionAs;
            st.assessmentQuestionLabel = createAssessmentQuestionsTableInput.assessmentQuestionLabel;
            st.correctAnswer = createAssessmentQuestionsTableInput.correctAnswer;
            st.marks = createAssessmentQuestionsTableInput.marks;
            let assessmentMaster = await this.assessmentMasterTableService.findOne(createAssessmentQuestionsTableInput.assessmentId);
            st.AssessmentMasterTable = assessmentMaster;
            st.options = createAssessmentQuestionsTableInput.options.map((option) => ({
                optionLabel: option.optionLabel,
                optionText: option.optionText,
            }));
            let result = await this.assessmentQuestionsTableRepository.save(st);
            this.pubSub.publish(exports.NEW_QUESTION_ADD, 1);
            return {
                status: true,
                status_text: "Successfully Added",
                data: result
            };
        }
        else {
            return {
                status: false,
                status_text: "Already Exists"
            };
        }
    }
    async checkIfAlreadyExists(createAssessmentQuestionsTableInput) {
        let result = await this.assessmentQuestionsTableRepository.count({
            where: {
                assessmentQuestionEn: createAssessmentQuestionsTableInput.assessmentQuestionEn,
                assessmentQuestionAs: createAssessmentQuestionsTableInput.assessmentQuestionAs,
                assessmentQuestionLabel: createAssessmentQuestionsTableInput.assessmentQuestionLabel,
                AssessmentMasterTable: {
                    assessmentId: createAssessmentQuestionsTableInput.assessmentId
                }
            }
        });
        return result;
    }
    async findAll() {
        return await this.assessmentQuestionsTableRepository.find({
            relations: {
                AssessmentMasterTable: true
            }
        });
    }
    async findOne(assessmentQuestionId) {
        return await this.assessmentQuestionsTableRepository.findOne({
            where: {
                assessmentQuestionId: assessmentQuestionId
            },
            relations: {
                AssessmentMasterTable: true
            }
        });
    }
    async findOneByAssessmentMasterId(assessmentId) {
        return await this.assessmentQuestionsTableRepository.find({
            where: {
                AssessmentMasterTable: {
                    assessmentId: assessmentId
                }
            },
            relations: {
                AssessmentMasterTable: true
            }
        });
    }
    async findOneAssessmentByType(type) {
        let result = await this.assessmentQuestionsTableRepository.find({
            where: {
                AssessmentMasterTable: {
                    type: type
                }
            },
            relations: {
                AssessmentMasterTable: true
            }
        });
        let assessmentId = 0;
        if (result.length > 0) {
            assessmentId = result[0].AssessmentMasterTable.assessmentId;
        }
        let labelArray = result.map(el => el.assessmentQuestionLabel);
        return {
            data: this.shuffleArray(result),
            length: this.findLargest(labelArray),
            assessmentId: assessmentId
        };
    }
    async findOneAssessmentByIdForApp(assessmentId) {
        let result = await this.assessmentQuestionsTableRepository.find({
            where: {
                AssessmentMasterTable: {
                    assessmentId: assessmentId
                }
            },
            relations: {
                AssessmentMasterTable: true
            }
        });
        let duration = 0;
        if (result.length > 0) {
            duration = result[0].AssessmentMasterTable.duration;
        }
        let labelArray = result.map(el => el.assessmentQuestionLabel);
        return {
            data: this.shuffleArray(result),
            length: this.findLargest(labelArray),
            assessmentId: assessmentId,
            duration
        };
    }
    findLargest(arr) {
        if (arr.length === 0) {
            return null;
        }
        let maxNum = arr[0];
        for (let i = 1; i < arr.length; i++) {
            if (arr[i] > maxNum) {
                maxNum = arr[i];
            }
        }
        return maxNum;
    }
    shuffleArray(array) {
        for (let i = array.length - 1; i > 0; i--) {
            const j = Math.floor(Math.random() * (i + 1));
            [array[i], array[j]] = [array[j], array[i]];
        }
        return array;
    }
    async update(assessmentQuestionId, updateAssessmentQuestionsTableInput) {
        let result = await this.assessmentQuestionsTableRepository.findOne({
            where: {
                assessmentQuestionId: assessmentQuestionId
            }
        });
        if (result) {
            let st = result;
            st.assessmentQuestionEn = updateAssessmentQuestionsTableInput.assessmentQuestionEn;
            st.assessmentQuestionAs = updateAssessmentQuestionsTableInput.assessmentQuestionAs;
            st.assessmentQuestionLabel = updateAssessmentQuestionsTableInput.assessmentQuestionLabel;
            st.correctAnswer = updateAssessmentQuestionsTableInput.correctAnswer;
            st.marks = updateAssessmentQuestionsTableInput.marks;
            let assessmentMaster = await this.assessmentMasterTableService.findOne(updateAssessmentQuestionsTableInput.assessmentId);
            st.AssessmentMasterTable = assessmentMaster;
            st.options = updateAssessmentQuestionsTableInput.options.map((option) => ({
                optionLabel: option.optionLabel,
                optionText: option.optionText,
            }));
            let result1 = await this.assessmentQuestionsTableRepository.save(st);
            this.pubSub.publish(exports.NEW_QUESTION_ADD, 1);
            return {
                status: true,
                status_text: "Successfully Updated",
                data: result1
            };
        }
        else {
            return {
                status: false,
                status_text: "Not Found",
                data: null
            };
        }
    }
    async remove(assessmentQuestionId) {
        let result = await this.assessmentQuestionsTableRepository.findOne({
            where: {
                assessmentQuestionId: assessmentQuestionId
            }
        });
        if (result) {
            await this.assessmentQuestionsTableRepository.remove(result);
            this.pubSub.publish(exports.NEW_QUESTION_ADD, 1);
            return {
                status: true,
                status_text: "Successfully Removed",
                data: null
            };
        }
        else {
            return {
                status: false,
                status_text: "Not Found",
                data: null
            };
        }
    }
};
exports.AssessmentQuestionsTableService = AssessmentQuestionsTableService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(assessment_questions_table_entity_1.AssessmentQuestionsTable)),
    __param(1, (0, common_1.Inject)(GlobalConstants_1.PUB_SUB)),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        graphql_redis_subscriptions_1.RedisPubSub,
        assessment_master_table_service_1.AssessmentMasterTableService])
], AssessmentQuestionsTableService);
