"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BankLoanTableService = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const farmers_table_entity_1 = require("../farmers-table/entities/farmers-table.entity");
const farmers_table_service_1 = require("../farmers-table/farmers-table.service");
const sub_division_table_service_1 = require("../sub-division-table/sub-division-table.service");
const user_collection_service_1 = require("../user-collection/user-collection.service");
const typeorm_2 = require("typeorm");
const bank_loan_table_entity_1 = require("./entities/bank-loan-table.entity");
const data_entry_table_service_1 = require("../data-entry-table/data-entry-table.service");
const GlobalConstants_1 = require("../helper/GlobalConstants");
let BankLoanTableService = class BankLoanTableService {
    constructor(bankLoanTableRepository, userCollectionService, farmersTableService, subDivisionTableService, dataEntryTableService) {
        this.bankLoanTableRepository = bankLoanTableRepository;
        this.userCollectionService = userCollectionService;
        this.farmersTableService = farmersTableService;
        this.subDivisionTableService = subDivisionTableService;
        this.dataEntryTableService = dataEntryTableService;
    }
    async create(createBankLoanTableInput, user) {
        let checkIfExists = await this.checkIfExists(createBankLoanTableInput);
        if (checkIfExists) {
            return {
                status: false,
                status_text: "Already Exists",
                data: null
            };
        }
        let ht = new bank_loan_table_entity_1.BankLoanTable();
        let userTable = await this.userCollectionService.getOneByUserId(user.userId);
        let farmerTable = await this.farmersTableService.findOne(createBankLoanTableInput.farmerId);
        ht.UserTable = userTable;
        ht.FarmersTable = farmerTable;
        ht.hasLoan = createBankLoanTableInput.hasLoan;
        if (createBankLoanTableInput.hasLoan === "YES") {
            ht.nameOfBank = createBankLoanTableInput.nameOfBank;
            ht.loanAmount = createBankLoanTableInput.loanAmount;
            ht.repaymentAmount = createBankLoanTableInput.repaymentAmount;
        }
        else {
            ht.nameOfBank = null;
            ht.loanAmount = null;
            ht.repaymentAmount = null;
        }
        ht.furtherRequirement = createBankLoanTableInput.furtherRequirement;
        if (createBankLoanTableInput.furtherRequirement === "YES") {
            ht.requirementAmount = createBankLoanTableInput.requirementAmount;
        }
        else {
            ht.requirementAmount = null;
        }
        ht.difficultyToAccess = createBankLoanTableInput.difficultyToAccess;
        if (createBankLoanTableInput.difficultyToAccess === "YES") {
            ht.difficulty = createBankLoanTableInput.difficulty;
        }
        else {
            ht.difficulty = null;
        }
        let result = await this.bankLoanTableRepository.save(ht);
        this.dataEntryTableService.create({ type: GlobalConstants_1.CREATE, typeOfEntry: GlobalConstants_1.BANK_LOAN_ADD, remarks: "New bank loan added" }, user);
        return {
            status: true,
            status_text: "Successfully Submitted",
            data: result
        };
    }
    async checkIfExists(createBankLoanTableInput) {
        let result = await this.bankLoanTableRepository.count({
            where: {
                hasLoan: createBankLoanTableInput.hasLoan,
                nameOfBank: createBankLoanTableInput.nameOfBank,
                loanAmount: createBankLoanTableInput.loanAmount,
                repaymentAmount: createBankLoanTableInput.repaymentAmount,
                furtherRequirement: createBankLoanTableInput.furtherRequirement,
                difficultyToAccess: createBankLoanTableInput.difficultyToAccess,
                difficulty: createBankLoanTableInput.difficulty,
            }
        });
        if (result == 0) {
            return false;
        }
        else {
            return true;
        }
    }
    async findByFarmerId(farmerId) {
        let farmer = await this.farmersTableService.findOne(farmerId);
        return await this.bankLoanTableRepository.find({
            where: {
                FarmersTable: farmer
            },
            relations: ["UserTable"]
        });
    }
    async findAll() {
        return await this.bankLoanTableRepository.find({
            relations: ["UserTable", "FarmersTable"]
        });
    }
    async findOne(bankLoanId) {
        return await this.bankLoanTableRepository.findOne({
            where: {
                bankLoanId
            },
            relations: ["UserTable", "FarmersTable"]
        });
    }
    async findOneByFarmerId(farmerId) {
        let farmerTable = await this.farmersTableService.findOne(farmerId);
        let result = await this.bankLoanTableRepository.createQueryBuilder("r")
            .where("r.farmerId=:farmerId", { farmerId: farmerTable.farmerId })
            .leftJoinAndMapOne("r.FarmersTable", farmers_table_entity_1.FarmersTable, "ml1", "ml1.farmerId=r.farmerId")
            .getMany();
        return result;
    }
    async update(bankLoanId, updateBankLoanTableInput, user) {
        let ht = await this.bankLoanTableRepository.findOne({
            where: {
                bankLoanId
            }
        });
        if (!ht) {
            return {
                status: false,
                status_text: "Not found",
                data: null
            };
        }
        else {
            let userTable = await this.userCollectionService.getOneByUserId(user.userId);
            let farmerTable = await this.farmersTableService.findOne(updateBankLoanTableInput.farmerId);
            ht.UserTable = userTable;
            ht.FarmersTable = farmerTable;
            ht.hasLoan = updateBankLoanTableInput.hasLoan;
            if (updateBankLoanTableInput.hasLoan === "YES") {
                ht.nameOfBank = updateBankLoanTableInput.nameOfBank;
                ht.loanAmount = updateBankLoanTableInput.loanAmount;
                ht.repaymentAmount = updateBankLoanTableInput.repaymentAmount;
            }
            else {
                ht.nameOfBank = null;
                ht.loanAmount = null;
                ht.repaymentAmount = null;
            }
            ht.furtherRequirement = updateBankLoanTableInput.furtherRequirement;
            if (updateBankLoanTableInput.furtherRequirement === "YES") {
                ht.requirementAmount = updateBankLoanTableInput.requirementAmount;
            }
            else {
                ht.requirementAmount = null;
            }
            ht.difficultyToAccess = updateBankLoanTableInput.difficultyToAccess;
            if (updateBankLoanTableInput.difficultyToAccess === "YES") {
                ht.difficulty = updateBankLoanTableInput.difficulty;
            }
            else {
                ht.difficulty = null;
            }
            let result = await this.bankLoanTableRepository.save(ht);
            this.dataEntryTableService.create({ type: GlobalConstants_1.UPDATE, typeOfEntry: GlobalConstants_1.BANK_LOAN_UPDATE, remarks: "bank loan UPDATED" }, user);
            return {
                status: true,
                status_text: "Successfully Updated",
                data: result
            };
        }
    }
    async remove(bankLoanId, user) {
        let ht = await this.bankLoanTableRepository.findOne({
            where: {
                bankLoanId
            }
        });
        if (!ht) {
            return {
                status: false,
                status_text: "Not found",
                data: null
            };
        }
        else {
            await this.bankLoanTableRepository.remove(ht);
            this.dataEntryTableService.create({ type: GlobalConstants_1.DELETE, typeOfEntry: GlobalConstants_1.BANK_LOAN_DELETE, remarks: "bank loan DELETED" }, user);
            return {
                status: true,
                status_text: "Successfully Deleted",
                data: null
            };
        }
    }
    async countBankLoanDetails(districtId) {
        const subdivisions = await this.subDivisionTableService.fetchSubDivisionByDistrictId(districtId);
        let data = [];
        for (const e of subdivisions) {
            let d1 = await this.bankLoanTableRepository.createQueryBuilder("r")
                .leftJoinAndMapOne("r.FarmersTable", farmers_table_entity_1.FarmersTable, "ml1", "ml1.farmerId=r.farmerId")
                .where("ml1.subDivisionId=:subDivisionId", { subDivisionId: e.subDivisionId })
                .getMany();
            console.log(d1);
            let d2 = {
                SubDivisionTable: e,
                noOfAvailedLoans: d1 ? d1.filter((el) => el.hasLoan === "YES").length : 0,
                noOfRequiredLoans: d1 ? d1.filter((el) => el.furtherRequirement === "YES").length : 0
            };
            data.push(d2);
        }
        return data;
    }
};
BankLoanTableService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(bank_loan_table_entity_1.BankLoanTable)),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        user_collection_service_1.UserCollectionService,
        farmers_table_service_1.FarmersTableService,
        sub_division_table_service_1.SubDivisionTableService,
        data_entry_table_service_1.DataEntryTableService])
], BankLoanTableService);
exports.BankLoanTableService = BankLoanTableService;
