"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BlockTableService = exports.NEW_BLOCK_ADD = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const graphql_redis_subscriptions_1 = require("graphql-redis-subscriptions");
const GlobalConstants_1 = require("../helper/GlobalConstants");
const typeorm_2 = require("typeorm");
const block_table_entity_1 = require("./entities/block-table.entity");
const district_table_service_1 = require("../district-table/district-table.service");
exports.NEW_BLOCK_ADD = "NEW_BLOCK_ADD";
let BlockTableService = exports.BlockTableService = class BlockTableService {
    constructor(blockTableRepository, pubSub, districtTableService) {
        this.blockTableRepository = blockTableRepository;
        this.pubSub = pubSub;
        this.districtTableService = districtTableService;
    }
    async create(createBlockTableInput) {
        let checkIfExists = await this.checkIfExists(createBlockTableInput.blockName, createBlockTableInput.districtId);
        if (checkIfExists) {
            return {
                status: false,
                status_text: "Already Exists",
                data: null
            };
        }
        else {
            let dd = new block_table_entity_1.BlockTable();
            dd.blockName = createBlockTableInput.blockName;
            let district = await this.districtTableService.findOne(createBlockTableInput.districtId);
            dd.DistrictTable = district;
            let result = await this.blockTableRepository.save(dd);
            this.pubSub.publish(exports.NEW_BLOCK_ADD, 1);
            return {
                status: true,
                status_text: "Successfully Created",
                data: result
            };
        }
    }
    async createorElseReturnObject(createBlockTableInput) {
        let checkIfAlreadyExists = await this.blockTableRepository.findOne({
            where: {
                blockName: createBlockTableInput.blockName.trim(),
                DistrictTable: {
                    districtId: createBlockTableInput.districtId
                }
            }
        });
        if (!checkIfAlreadyExists) {
            let dd = new block_table_entity_1.BlockTable();
            dd.blockName = createBlockTableInput.blockName.trim();
            let district = await this.districtTableService.findOne(createBlockTableInput.districtId);
            dd.DistrictTable = district;
            let result = await this.blockTableRepository.save(dd);
            return result;
        }
        else {
            return checkIfAlreadyExists;
        }
    }
    async checkIfExists(blockName, districtId) {
        let result = await this.blockTableRepository.count({
            where: {
                blockName: blockName,
                DistrictTable: {
                    districtId
                }
            }
        });
        if (result == 0) {
            return false;
        }
        else {
            return true;
        }
    }
    async findAll() {
        return await this.blockTableRepository.find({
            relations: {
                DistrictTable: {
                    StateTable: true
                }
            }
        });
    }
    async findAllByIds(Ids) {
        return await this.blockTableRepository.find({
            where: {
                blockId: (0, typeorm_2.In)(Ids)
            },
            relations: {
                DistrictTable: {
                    StateTable: true
                }
            }
        });
    }
    async getCount() {
        return await this.blockTableRepository.count();
    }
    async getCountUnderAState(stateId) {
        let result = await this.blockTableRepository.find({
            where: {
                DistrictTable: {
                    StateTable: {
                        stateId
                    }
                }
            }
        });
        return result.map((el) => el.blockId);
    }
    async getCountUnderADistrict(districtId) {
        let result = await this.blockTableRepository.find({
            where: {
                DistrictTable: {
                    districtId
                }
            }
        });
        return result.map(el => el.blockId);
    }
    async findAllByDistrictId(districtId) {
        return await this.blockTableRepository.find({
            where: {
                DistrictTable: {
                    districtId
                }
            },
            relations: {
                DistrictTable: true
            }
        });
    }
    async findOne(blockId) {
        return await this.blockTableRepository.findOne({
            where: {
                blockId: blockId
            },
            relations: {
                DistrictTable: {
                    StateTable: true
                }
            }
        });
    }
    async update(blockId, updateBlockTableInput) {
        let result = await this.blockTableRepository.findOne({
            where: {
                blockId: blockId
            }
        });
        if (result) {
            result.blockName = updateBlockTableInput.blockName;
            let district = await this.districtTableService.findOne(updateBlockTableInput.districtId);
            result.DistrictTable = district;
            let r = await this.blockTableRepository.save(result);
            this.pubSub.publish(exports.NEW_BLOCK_ADD, 1);
            return {
                status: true,
                status_text: "Successfully Updated",
                data: r
            };
        }
        else {
            return {
                status: false,
                status_text: "Not found",
                data: null
            };
        }
    }
    async remove(blockId) {
        let result = await this.blockTableRepository.findOne({
            where: {
                blockId: blockId
            }
        });
        if (result) {
            await this.blockTableRepository.remove(result);
            this.pubSub.publish(exports.NEW_BLOCK_ADD, 1);
            return {
                status: true,
                status_text: "Successfully Removed",
                data: null
            };
        }
        else {
            return {
                status: false,
                status_text: "Not Found",
                data: null
            };
        }
    }
};
exports.BlockTableService = BlockTableService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(block_table_entity_1.BlockTable)),
    __param(1, (0, common_1.Inject)(GlobalConstants_1.PUB_SUB)),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        graphql_redis_subscriptions_1.RedisPubSub,
        district_table_service_1.DistrictTableService])
], BlockTableService);
