"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BreedingSingleTableService = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const animal_entry_table_service_1 = require("../animal-table/animal-entry-table.service");
const animal_entry_table_entity_1 = require("../animal-table/entities/animal-entry-table.entity");
const farmers_table_entity_1 = require("../farmers-table/entities/farmers-table.entity");
const housing_table_entity_1 = require("../housing-table/entities/housing-table.entity");
const housing_table_service_1 = require("../housing-table/housing-table.service");
const parent_table_service_1 = require("../parent-table/parent-table.service");
const sub_division_table_service_1 = require("../sub-division-table/sub-division-table.service");
const user_collection_service_1 = require("../user-collection/user-collection.service");
const typeorm_2 = require("typeorm");
const breeding_single_table_entity_1 = require("./entities/breeding-single-table.entity");
const data_entry_table_service_1 = require("../data-entry-table/data-entry-table.service");
const GlobalConstants_1 = require("../helper/GlobalConstants");
let BreedingSingleTableService = class BreedingSingleTableService {
    constructor(breedingSingleTableRepository, dataEntryTableService, userCollectionService, housingTableService, animalEntryTableService, parentTableService, subDivisionTableService) {
        this.breedingSingleTableRepository = breedingSingleTableRepository;
        this.dataEntryTableService = dataEntryTableService;
        this.userCollectionService = userCollectionService;
        this.housingTableService = housingTableService;
        this.animalEntryTableService = animalEntryTableService;
        this.parentTableService = parentTableService;
        this.subDivisionTableService = subDivisionTableService;
    }
    async create(createBreedingSingleTableInput, user) {
        let checkIfExists = await this.checkIfExists(createBreedingSingleTableInput);
        if (checkIfExists) {
            return {
                status: false,
                status_text: "Already Exists",
                data: null
            };
        }
        else {
            let ht = new breeding_single_table_entity_1.BreedingSingleTable();
            let userTable = await this.userCollectionService.getOneByUserId(user.userId);
            let housingTable = await this.housingTableService.findOne(createBreedingSingleTableInput.housingId);
            let sowTable = await this.animalEntryTableService.findByAnimalId(createBreedingSingleTableInput.sowId);
            let boarTable = await this.animalEntryTableService.findByAnimalId(createBreedingSingleTableInput.boarId);
            ht.dateOfInsemination = createBreedingSingleTableInput.dateOfInsemination;
            ht.UserTable = userTable;
            ht.HousingTable = housingTable;
            ht.SowTable = sowTable;
            ht.ageOfSowAtBegining = sowTable.age;
            ht.weightOfSowAtBegining = sowTable.weight;
            if (boarTable) {
                ht.ageOfBoarAtBegining = boarTable.age;
                ht.weightOfBoarAtBegining = boarTable.weight;
                ht.BoarTable = boarTable;
            }
            ht.typeOfInsemination = createBreedingSingleTableInput.typeOfInsemination;
            ht.sourceOfSemen = createBreedingSingleTableInput.sourceOfSemen;
            let result = await this.breedingSingleTableRepository.save(ht);
            this.dataEntryTableService.create({ type: GlobalConstants_1.CREATE, typeOfEntry: GlobalConstants_1.BREEDING_ADD, remarks: "New breeding added" }, user);
            return {
                status: true,
                status_text: "Successfully Created",
                data: result
            };
        }
    }
    async checkIfExists(createBreedingSingleTableInput) {
        let rr = await this.breedingSingleTableRepository.count({
            where: {
                dateOfInsemination: createBreedingSingleTableInput.dateOfInsemination,
                BoarTable: {
                    animalId: createBreedingSingleTableInput.boarId
                },
                SowTable: {
                    animalId: createBreedingSingleTableInput.sowId
                }
            }
        });
        if (rr == 0) {
            return false;
        }
        else {
            return true;
        }
    }
    async fetchBreedingDetailsSingle(animalId) {
        let result = await this.breedingSingleTableRepository.createQueryBuilder("r")
            .leftJoinAndMapOne("r.SowTable", animal_entry_table_entity_1.AnimalEntryTable, "ml1", "ml1.animalId=r.sowId")
            .leftJoinAndMapOne("r.BoarTable", animal_entry_table_entity_1.AnimalEntryTable, "ml2", "ml2.animalId=r.boarId")
            .where("r.boarId=:boarId OR r.sowId=:sowId", { boarId: animalId, sowId: animalId })
            .getMany();
        return result;
    }
    async checkBreedingPossibility(sowId, boarId) {
        return await this.parentTableService.checkBreedingPossibility(sowId, boarId);
    }
    async findByHousingId(housingId) {
        return await this.breedingSingleTableRepository.find({
            where: {
                HousingTable: {
                    housingId
                }
            },
            relations: {
                UserTable: true,
                HousingTable: {
                    FarmersTable: true,
                },
                SowTable: true,
                BoarTable: true
            }
        });
    }
    async updateDetailsOfInsemination(breedingSingleId, ageOfSowAtFarrowing, totalNoOfPiglets, totalMale, totalFemale, status, weightOfEach, dateOfFarrowing, litterSize, user) {
        let result = await this.breedingSingleTableRepository.findOne({
            where: {
                breedingSingleId
            }
        });
        if (result) {
            result.ageOfSowAtFarrowing = ageOfSowAtFarrowing;
            result.totalNoOfPiglets = totalNoOfPiglets;
            result.totalMale = totalMale;
            result.totalFemale = totalFemale;
            result.status = status;
            result.weightOfEach = weightOfEach;
            result.litterSize = litterSize;
            result.dateOfFarrowing = dateOfFarrowing;
            await this.breedingSingleTableRepository.save(result);
            this.dataEntryTableService.create({ type: GlobalConstants_1.UPDATE, typeOfEntry: GlobalConstants_1.BREEDING_UPDATE, remarks: "New breeding updated" }, user);
            return {
                status: true,
                status_text: "Successfully Updated"
            };
        }
        else {
            return {
                status: false,
                status_text: "Details not found"
            };
        }
    }
    async findAll() {
        return await this.breedingSingleTableRepository.find({
            relations: {
                UserTable: true,
                HousingTable: {
                    FarmersTable: true,
                },
                SowTable: true,
                BoarTable: true
            }
        });
    }
    async findOne(breedingSingleId) {
        return await this.breedingSingleTableRepository.findOne({
            where: {
                breedingSingleId: breedingSingleId,
            },
            relations: {
                UserTable: true,
                HousingTable: {
                    FarmersTable: true,
                },
                SowTable: true,
                BoarTable: true
            }
        });
    }
    async countBreedingDetailsReport(districtId, startingDate, endingDate) {
        const subdivisions = await this.subDivisionTableService.fetchSubDivisionByDistrictId(districtId);
        let data = [];
        for (const e of subdivisions) {
            let d1 = await this.breedingSingleTableRepository.createQueryBuilder("r")
                .leftJoinAndMapOne("r.HousingTable", housing_table_entity_1.HousingTable, "ml1", "ml1.housingId=r.housingId")
                .leftJoinAndMapOne("ml1.FarmersTable", farmers_table_entity_1.FarmersTable, "ml2", "ml1.farmerId=ml2.farmerId")
                .where("ml2.subDivisionId=:subDivisionId", { subDivisionId: e.subDivisionId })
                .andWhere("STR_TO_DATE(r.dateOfInsemination,'%Y-%m-%d') BETWEEN STR_TO_DATE(:startingDate,'%Y-%m-%d') AND STR_TO_DATE(:endingDate,'%Y-%m-%d')", {
                startingDate: startingDate,
                endingDate: endingDate,
            })
                .getMany();
            console.log(d1);
            let d2 = {
                SubDivisionTable: e,
                noOfBreeding: d1 ? d1.length : 0,
                noOfSuccess: d1 ? d1.filter((el) => el.status == 1).length : 0,
                noOfFailed: d1 ? d1.filter((el) => el.status == 2).length : 0,
            };
            data.push(d2);
        }
        return data;
    }
    update(id, updateBreedingSingleTableInput) {
        return `This action updates a #${id} breedingSingleTable`;
    }
    remove(id) {
        return `This action removes a #${id} breedingSingleTable`;
    }
};
BreedingSingleTableService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(breeding_single_table_entity_1.BreedingSingleTable)),
    __param(2, (0, common_1.Inject)((0, common_1.forwardRef)(() => user_collection_service_1.UserCollectionService))),
    __param(3, (0, common_1.Inject)((0, common_1.forwardRef)(() => housing_table_service_1.HousingTableService))),
    __param(4, (0, common_1.Inject)((0, common_1.forwardRef)(() => animal_entry_table_service_1.AnimalEntryTableService))),
    __param(5, (0, common_1.Inject)((0, common_1.forwardRef)(() => parent_table_service_1.ParentTableService))),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        data_entry_table_service_1.DataEntryTableService,
        user_collection_service_1.UserCollectionService,
        housing_table_service_1.HousingTableService,
        animal_entry_table_service_1.AnimalEntryTableService,
        parent_table_service_1.ParentTableService,
        sub_division_table_service_1.SubDivisionTableService])
], BreedingSingleTableService);
exports.BreedingSingleTableService = BreedingSingleTableService;
