"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ChallengesFacedService = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const difficulties_table_service_1 = require("../difficulties-table/difficulties-table.service");
const difficulties_table_entity_1 = require("../difficulties-table/entities/difficulties-table.entity");
const farmers_table_entity_1 = require("../farmers-table/entities/farmers-table.entity");
const farmers_table_service_1 = require("../farmers-table/farmers-table.service");
const typeorm_2 = require("typeorm");
const challenges_faced_entity_1 = require("./entities/challenges-faced.entity");
const moment_1 = __importDefault(require("moment"));
const district_table_entity_1 = require("../district-table/entities/district-table.entity");
const sub_division_table_entity_1 = require("../sub-division-table/entities/sub-division-table.entity");
let ChallengesFacedService = class ChallengesFacedService {
    constructor(challengesFacedRepository, difficultiesTableService, farmersTableService) {
        this.challengesFacedRepository = challengesFacedRepository;
        this.difficultiesTableService = difficultiesTableService;
        this.farmersTableService = farmersTableService;
    }
    async create(createChallengesFacedInput, user) {
        let checkIfExists = await this.checkIfExists(createChallengesFacedInput);
        if (checkIfExists) {
            return {
                status: false,
                status_text: "Already Exists",
                data: null
            };
        }
        let ht = new challenges_faced_entity_1.ChallengesFaced();
        let farmerTable = await this.farmersTableService.findOne(createChallengesFacedInput.farmerId);
        let difficultyTable = await this.difficultiesTableService.findOne(createChallengesFacedInput.difficultyId);
        ht.FarmersTable = farmerTable;
        ht.DifficultiesTable = difficultyTable;
        ht.remarks = createChallengesFacedInput.remarks;
        let result = await this.challengesFacedRepository.save(ht);
        return {
            status: true,
            status_text: "Successfully Submitted",
            data: result
        };
    }
    async checkIfExists(createChallengesFacedInput) {
        let result = await this.challengesFacedRepository.count({
            where: {
                remarks: createChallengesFacedInput.remarks,
            }
        });
        if (result == 0) {
            return false;
        }
        else {
            return true;
        }
    }
    async findAll() {
        return await this.challengesFacedRepository.find({
            relations: ["FarmersTable", "DifficultiesTable"]
        });
    }
    async findOne(challengeFacedId) {
        return await this.challengesFacedRepository.findOne({
            where: {
                challengeFacedId
            },
            relations: ["FarmersTable", "DifficultiesTable"]
        });
    }
    async findByFarmerId(farmerId) {
        let farmer = await this.farmersTableService.findOne(farmerId);
        return await this.challengesFacedRepository.find({
            where: {
                FarmersTable: farmer
            },
            relations: ["DifficultiesTable"]
        });
    }
    async findOneByFarmerId(farmerId) {
        let farmerTable = await this.farmersTableService.findOne(farmerId);
        let result = await this.challengesFacedRepository.createQueryBuilder("r")
            .where("r.farmerId=:farmerId", { farmerId: farmerTable.farmerId })
            .leftJoinAndMapOne("r.FarmersTable", farmers_table_entity_1.FarmersTable, "ml1", "ml1.farmerId=r.farmerId")
            .leftJoinAndMapOne("r.DifficultiesTable", difficulties_table_entity_1.DifficultiesTable, "ml2", "ml2.difficultyId=r.difficultyId")
            .getMany();
        return result;
    }
    async update(challengeFacedId, updateChallengesFacedInput, user) {
        let ht = await this.challengesFacedRepository.findOne({
            where: {
                challengeFacedId
            }
        });
        if (!ht) {
            return {
                status: false,
                status_text: "Not found",
                data: null
            };
        }
        else {
            let farmerTable = await this.farmersTableService.findOne(updateChallengesFacedInput.farmerId);
            let difficultyTable = await this.difficultiesTableService.findOne(updateChallengesFacedInput.difficultyId);
            ht.FarmersTable = farmerTable;
            ht.DifficultiesTable = difficultyTable;
            ht.remarks = updateChallengesFacedInput.remarks;
            let result = await this.challengesFacedRepository.save(ht);
            return {
                status: true,
                status_text: "Successfully Updated",
                data: result
            };
        }
    }
    async remove(challengeFacedId) {
        let ht = await this.challengesFacedRepository.findOne({
            where: {
                challengeFacedId
            }
        });
        if (!ht) {
            return {
                status: false,
                status_text: "Not found",
                data: null
            };
        }
        else {
            await this.challengesFacedRepository.remove(ht);
            return {
                status: true,
                status_text: "Successfully Deleted",
                data: null
            };
        }
    }
    async toggleActiveStatus(id) {
        let u = await this.findOne(id);
        if (!!u) {
            let oldStatus = u.status;
            u.status = !oldStatus;
            if (!oldStatus == true) {
                u.dateOfResolution = (0, moment_1.default)().format("DD-MM-YYYY");
            }
            else {
                u.dateOfResolution = "";
            }
            let newStatus = !oldStatus ? 'Activated' : 'De-Activated';
            let u1 = await this.challengesFacedRepository.save(u);
            return {
                status: true,
                status_text: "Status Changed",
                data: await this.findOne(id)
            };
        }
        else {
            return {
                status: false,
                status_text: "User not found",
                data: null
            };
        }
    }
    async getDifficultyReportDistrictWise(districtId, difficultyId) {
        let d1 = await this.challengesFacedRepository.createQueryBuilder("r")
            .leftJoinAndMapOne("r.FarmersTable", farmers_table_entity_1.FarmersTable, "ml1", "ml1.farmerId=r.farmerId")
            .leftJoinAndMapOne("ml1.DistrictTable", district_table_entity_1.DistrictTable, "ml2", "ml2.districtId=ml1.districtId")
            .leftJoinAndMapOne("ml1.SubDivisionTable", sub_division_table_entity_1.SubDivisionTable, "ml4", "ml4.subDivisionId=ml1.subDivisionId")
            .leftJoinAndMapOne("r.DifficultiesTable", difficulties_table_entity_1.DifficultiesTable, "ml3", "ml3.difficultyId=r.difficultyId")
            .where("ml2.districtId=:districtId", { districtId: districtId })
            .andWhere("r.difficultyId=:difficultyId", { difficultyId: difficultyId })
            .getMany();
        return d1;
    }
};
ChallengesFacedService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(challenges_faced_entity_1.ChallengesFaced)),
    __param(1, (0, common_1.Inject)((0, common_1.forwardRef)(() => difficulties_table_service_1.DifficultiesTableService))),
    __param(2, (0, common_1.Inject)((0, common_1.forwardRef)(() => farmers_table_service_1.FarmersTableService))),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        difficulties_table_service_1.DifficultiesTableService,
        farmers_table_service_1.FarmersTableService])
], ChallengesFacedService);
exports.ChallengesFacedService = ChallengesFacedService;
