"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ClientCollectionService = exports.NEW_CLIENT_ADD = void 0;
const common_1 = require("@nestjs/common");
const mongoose_1 = require("@nestjs/mongoose");
const mongoose_2 = require("mongoose");
const GlobalConstants_1 = require("../helper/GlobalConstants");
const graphql_redis_subscriptions_1 = require("graphql-redis-subscriptions");
const client_collection_entity_1 = require("./entities/client-collection.entity");
const client_category_collection_service_1 = require("../client-category-collection/client-category-collection.service");
const mail_service_1 = require("../mail/mail.service");
const user_collection_service_1 = require("../user-collection/user-collection.service");
const authentication_collection_service_1 = require("../authentication-collection/authentication-collection.service");
exports.NEW_CLIENT_ADD = 'NEW_CLIENT_ADD';
let ClientCollectionService = class ClientCollectionService {
    constructor(clientRepository, clientCategoryService, pubSub, mailService, userCollectionService, authenticationCollectionService) {
        this.clientRepository = clientRepository;
        this.clientCategoryService = clientCategoryService;
        this.pubSub = pubSub;
        this.mailService = mailService;
        this.userCollectionService = userCollectionService;
        this.authenticationCollectionService = authenticationCollectionService;
    }
    async create(createClientCollectionInput) {
        let checkIfExists = await this.checkIfExists(createClientCollectionInput.dlNumber);
        if (!checkIfExists) {
            let cc = await this.clientCategoryService.findOneById(createClientCollectionInput.clientCategoryId);
            let cm = new client_collection_entity_1.ClientCollection();
            cm.organizationName = createClientCollectionInput.organizationName;
            cm.contactPerson = createClientCollectionInput.contactPerson;
            cm.dlNumber = createClientCollectionInput.dlNumber;
            cm.BillingAddress = createClientCollectionInput.BillingAddress;
            cm.ClientCategory = cc;
            let c = await this.clientRepository.create(cm);
            this.pubSub.publish(exports.NEW_CLIENT_ADD, 1);
            return {
                status: true,
                status_text: 'Successfully Added',
                data: c,
            };
        }
        else {
            return {
                status: false,
                status_text: 'Already exists',
                data: null,
            };
        }
    }
    async signup(createSignupInput) {
        let checkIfExists = await this.checkIfExistsEmail(createSignupInput.email);
        if (!checkIfExists) {
            let cc = await this.clientCategoryService.findOneById(createSignupInput.clientCategoryId);
            let cm = new client_collection_entity_1.ClientCollection();
            cm.organizationName = createSignupInput.organizationName;
            cm.contactPerson = createSignupInput.contactPerson;
            cm.email = createSignupInput.email;
            cm.phone = createSignupInput.phone;
            cm.ClientCategory = cc;
            let c = await this.clientRepository.create(cm);
            this.pubSub.publish(exports.NEW_CLIENT_ADD, 1);
            let result = await this.userCollectionService.signupUser({
                name: createSignupInput.organizationName,
                phone: createSignupInput.phone,
                email: createSignupInput.email,
                superType: 'CLIENT',
                password: createSignupInput.password,
                Client: c,
                role: ['ADMIN'],
            });
            this.mailService.welcomeEmail({
                toEmail: createSignupInput.email,
                data: {
                    name: createSignupInput.organizationName,
                    link: await this.authenticationCollectionService.createAuthentication(createSignupInput.email, 'CLIENT_EMAIL_VERIFICATION'),
                },
            });
            return {
                status: true,
                status_text: 'Successfully Added',
                data: c,
            };
        }
        else {
            return {
                status: false,
                status_text: 'Already exists',
                data: null,
            };
        }
    }
    async checkIfExists(dlNumber) {
        let r = await this.clientRepository
            .count({
            dlNumber: dlNumber,
        })
            .exec();
        if (r > 0) {
            return true;
        }
        else {
            return false;
        }
    }
    async markAsEmailVerified(email) {
        let r = await this.clientRepository
            .findOne({
            email: email,
        })
            .exec();
        if (r) {
            r.isEmailVerified = true;
            let cc1 = await this.clientRepository.updateOne({ _id: r.id }, r);
            return true;
        }
        else {
            return false;
        }
    }
    async checkIfExistsEmail(email) {
        let r = await this.clientRepository
            .count({
            email: email,
        })
            .exec();
        if (r > 0) {
            return true;
        }
        else {
            return false;
        }
    }
    async findAll(limit, page, search) {
        const offset = limit * (page - 1);
        const filters = {};
        if (search) {
            filters.$text = {
                $search: search,
            };
        }
        let count = await this.clientRepository.count();
        let result = await this.clientRepository
            .find(filters)
            .limit(limit)
            .skip(offset)
            .populate('ClientCategory')
            .exec();
        return { result: result, count: count, page: page };
    }
    async findOneById(id) {
        return await this.clientRepository
            .findOne({ _id: id })
            .populate('ClientCategory')
            .exec();
    }
    async findOneUsersById(id) { }
    async updateOrAddAddress(id, billingAddress, edit, index) {
        if (edit) {
            let d = await this.findOneById(id);
            let newD = d.BillingAddress;
            newD.splice(index, 1, billingAddress);
            let res = await this.clientRepository.updateOne({ _id: id }, { BillingAddress: newD });
            return {
                status: true,
                status_text: 'Address Updated',
                data: await this.findOneById(id),
            };
        }
        else {
            let res = await this.clientRepository.updateOne({ _id: id }, { $push: { BillingAddress: billingAddress } });
            return {
                status: true,
                status_text: 'Address Updated',
                data: await this.findOneById(id),
            };
        }
    }
    async deleteBillingAddress(id, index) {
        let d = await this.findOneById(id);
        let newD = d.BillingAddress;
        newD.splice(index, 1);
        let res = await this.clientRepository.updateOne({ _id: id }, { BillingAddress: newD });
        return {
            status: true,
            status_text: 'Address Deleted',
            data: await this.findOneById(id),
        };
    }
    async remove(id) {
        await this.clientRepository.deleteOne({ _id: id });
        this.pubSub.publish(exports.NEW_CLIENT_ADD, 1);
        return {
            status: true,
            status_text: 'Successfully deleted',
            data: null,
        };
    }
    async update(id, updateClientCollectionInput) {
        let cm = await this.clientRepository.findOne({ _id: id }).exec();
        if (cm) {
            let ccc = await this.clientCategoryService.findOneById(updateClientCollectionInput.clientCategoryId);
            if (updateClientCollectionInput.organizationName) {
                cm.organizationName = updateClientCollectionInput.organizationName;
            }
            if (updateClientCollectionInput.contactPerson) {
                cm.contactPerson = updateClientCollectionInput.contactPerson;
            }
            if (updateClientCollectionInput.dlNumber) {
                cm.dlNumber = updateClientCollectionInput.dlNumber;
            }
            if (updateClientCollectionInput.email) {
                cm.email = updateClientCollectionInput.email;
            }
            if (updateClientCollectionInput.phone) {
                cm.phone = updateClientCollectionInput.phone;
            }
            if (updateClientCollectionInput.markupPercentage) {
                cm.markupPercentage = updateClientCollectionInput.markupPercentage;
            }
            if (updateClientCollectionInput.discountPercentage) {
                cm.discountPercentage = updateClientCollectionInput.discountPercentage;
            }
            if (updateClientCollectionInput.BillingAddress) {
                cm.BillingAddress = updateClientCollectionInput.BillingAddress;
            }
            cm.ClientCategory = ccc;
            let cc1 = await this.clientRepository.updateOne({ _id: id }, cm);
            this.pubSub.publish(exports.NEW_CLIENT_ADD, 1);
            return {
                status: true,
                status_text: 'Successfully Updated',
                data: await this.findOneById(id),
            };
        }
        else {
            return {
                status: false,
                status_text: 'Not found',
                data: null,
            };
        }
    }
};
ClientCollectionService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, mongoose_1.InjectModel)(client_collection_entity_1.ClientCollection.name)),
    __param(2, (0, common_1.Inject)(GlobalConstants_1.PUB_SUB)),
    __param(3, (0, common_1.Inject)((0, common_1.forwardRef)(() => mail_service_1.MailService))),
    __param(4, (0, common_1.Inject)((0, common_1.forwardRef)(() => user_collection_service_1.UserCollectionService))),
    __param(5, (0, common_1.Inject)((0, common_1.forwardRef)(() => authentication_collection_service_1.AuthenticationCollectionService))),
    __metadata("design:paramtypes", [mongoose_2.Model,
        client_category_collection_service_1.ClientCategoryCollectionService,
        graphql_redis_subscriptions_1.RedisPubSub,
        mail_service_1.MailService,
        user_collection_service_1.UserCollectionService,
        authentication_collection_service_1.AuthenticationCollectionService])
], ClientCollectionService);
exports.ClientCollectionService = ClientCollectionService;
