"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CollectionTableService = exports.DASHBOARD_COUNT_CHANGE = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const user_collection_service_1 = require("../user-collection/user-collection.service");
const typeorm_2 = require("typeorm");
const collection_table_entity_1 = require("./entities/collection-table.entity");
const moment_1 = __importDefault(require("moment"));
const user_collection_entity_1 = require("../user-collection/entities/user-collection.entity");
const graphql_redis_subscriptions_1 = require("graphql-redis-subscriptions");
const GlobalConstants_1 = require("../helper/GlobalConstants");
exports.DASHBOARD_COUNT_CHANGE = 'DASHBOARD_COUNT_CHANGE';
let CollectionTableService = class CollectionTableService {
    constructor(collectionTableRepository, userCollectionService, pubSub, connection) {
        this.collectionTableRepository = collectionTableRepository;
        this.userCollectionService = userCollectionService;
        this.pubSub = pubSub;
        this.connection = connection;
    }
    async create(createCollectionTableInput, username) {
        let us = await this.userCollectionService.findOne(createCollectionTableInput.userId);
        if (createCollectionTableInput.collectionType == 'MONTHLY') {
            if (createCollectionTableInput.date.length > 0) {
                let firstDay = createCollectionTableInput.date[0];
                let month = (0, moment_1.default)(firstDay, 'YYYY-MM-DD').format('YYYY-MM');
                const startOfMonth = (0, moment_1.default)(month, 'YYYY-MM')
                    .startOf('month')
                    .format('YYYY-MM-DD');
                const endOfMonth = (0, moment_1.default)(month, 'YYYY-MM')
                    .endOf('month')
                    .format('YYYY-MM-DD');
                const queryRunner = await this.connection.createQueryRunner();
                await queryRunner.connect();
                await queryRunner.startTransaction();
                try {
                    let checkIfAlreadyExistsMonthly = await this.checkIfAlreadyExistsMonthly(queryRunner, us, month, createCollectionTableInput.amount, createCollectionTableInput.collectionType, startOfMonth, endOfMonth);
                    if (!checkIfAlreadyExistsMonthly) {
                        let ct = new collection_table_entity_1.CollectionTable();
                        ct.collectionDate = (0, moment_1.default)().format('YYYY-MM-DD');
                        ct.collectionTime = (0, moment_1.default)().format('HH:mm:ss');
                        ct.collectedBy = username;
                        ct.amount = createCollectionTableInput.amount;
                        ct.month = month;
                        ct.noOfDays = createCollectionTableInput.date.length;
                        ct.date = startOfMonth;
                        ct.endDate = endOfMonth;
                        ct.collectionType = createCollectionTableInput.collectionType;
                        ct.UserTable = us;
                        let result = await queryRunner.manager.save(ct);
                        await queryRunner.commitTransaction();
                    }
                }
                catch (err) {
                    await queryRunner.rollbackTransaction();
                }
                finally {
                    await queryRunner.release();
                }
            }
        }
        else {
            for (const el of createCollectionTableInput.date) {
                let ifExists = await this.ifExists(createCollectionTableInput, el);
                if (!ifExists) {
                    let ct = new collection_table_entity_1.CollectionTable();
                    ct.collectionDate = (0, moment_1.default)().format('YYYY-MM-DD');
                    ct.collectionTime = (0, moment_1.default)().format('HH:mm:ss');
                    ct.collectedBy = username;
                    ct.amount = us.prefixAmount;
                    ct.month = (0, moment_1.default)().format('YYYY-MM');
                    if (createCollectionTableInput.collectionType == 'DAILY') {
                        ct.noOfDays = 1;
                    }
                    else {
                        ct.noOfDays = createCollectionTableInput.date.length;
                    }
                    ct.date = el;
                    ct.collectionType = createCollectionTableInput.collectionType;
                    ct.UserTable = us;
                    let result = await this.collectionTableRepository.save(ct);
                }
            }
        }
        this.pubSub.publish(exports.DASHBOARD_COUNT_CHANGE, 1);
        return {
            status: true,
            status_text: 'Payment successfully Collected',
            data: null,
        };
    }
    async ifExists(createCollectionTableInput, date) {
        let us = await this.userCollectionService.findOne(createCollectionTableInput.userId);
        let res = await this.collectionTableRepository.count({
            where: {
                UserTable: us,
                date: date,
            },
        });
        if (res > 0) {
            return true;
        }
        else {
            return false;
        }
    }
    async checkIfAlreadyExistsMonthly(queryRunner, user, month, amount, collectionType, startOfMonth, endOfMonth) {
        let res = await queryRunner.manager.count(collection_table_entity_1.CollectionTable, {
            where: {
                UserTable: user,
                date: startOfMonth,
                endDate: endOfMonth,
                month: month,
                amount: amount,
                collectionType: collectionType
            },
        });
        if (res > 0) {
            return true;
        }
        else {
            return false;
        }
    }
    async findAllCollectionOfUserByMonth(username, month) {
        let mn = (0, moment_1.default)().format('YYYY-MM');
        if (month && month != '') {
            mn = month;
        }
        const startOfMonth = (0, moment_1.default)(mn, 'YYYY-MM')
            .startOf('month')
            .format('YYYY-MM-DD');
        const endOfMonth = (0, moment_1.default)(mn, 'YYYY-MM')
            .endOf('month')
            .format('YYYY-MM-DD');
        let us = await this.userCollectionService.findOneByUsername(username);
        if (us.collectionType == 'MONTHLY') {
            let result = await this.collectionTableRepository
                .createQueryBuilder('r')
                .andWhere('STR_TO_DATE(r.date, "%Y-%m-%d") >= STR_TO_DATE(:entryDate,"%Y-%m-%d")', { entryDate: startOfMonth })
                .andWhere('STR_TO_DATE(r.endDate, "%Y-%m-%d") <= STR_TO_DATE(:endDate,"%Y-%m-%d")', { endDate: endOfMonth })
                .andWhere('r.userId=:userId', { userId: us.id })
                .getOne();
            if (result) {
                let ddd = this.enumerateDaysBetweenDates(result.date, result.endDate);
                let returnObj = [];
                ddd.map((el) => {
                    let d = {
                        key: el,
                        value: {
                            selected: true,
                            selectedColor: 'green',
                            disableTouchEvent: true,
                        },
                    };
                    returnObj.push(d);
                });
                return {
                    status: true,
                    data: JSON.stringify(returnObj),
                };
            }
            else {
                return {
                    status: false,
                    data: null,
                };
            }
        }
        else {
            let result = await this.collectionTableRepository
                .createQueryBuilder('r')
                .andWhere('STR_TO_DATE(r.date, "%Y-%m-%d") >= STR_TO_DATE(:entryDate,"%Y-%m-%d")', { entryDate: startOfMonth })
                .andWhere('STR_TO_DATE(r.date, "%Y-%m-%d") <= STR_TO_DATE(:endDate,"%Y-%m-%d")', { endDate: endOfMonth })
                .andWhere('r.userId=:userId', { userId: us.id })
                .getMany();
            let returnObj = [];
            result.map((el) => {
                let d = {
                    key: el.date,
                    value: {
                        selected: true,
                        selectedColor: 'green',
                        disableTouchEvent: true,
                    },
                };
                returnObj.push(d);
            });
            return {
                status: true,
                data: JSON.stringify(returnObj),
            };
        }
    }
    enumerateDaysBetweenDates(startDate, endDate) {
        var dates = [];
        var currDate = (0, moment_1.default)(startDate, 'YYYY-MM-DD').startOf('day');
        var lastDate = (0, moment_1.default)(endDate, 'YYYY-MM-DD').startOf('day');
        dates.push((0, moment_1.default)(startDate, 'YYYY-MM-DD').format('YYYY-MM-DD'));
        while (currDate.add(1, 'days').diff(lastDate) <= 0) {
            console.log(currDate.toDate());
            dates.push((0, moment_1.default)(currDate.clone().toDate()).format('YYYY-MM-DD'));
        }
        return dates;
    }
    async getReportsByMonth(startDate, endDate, type) {
        const startDate1 = (0, moment_1.default)(startDate, 'DD-MM-YYYY').format('YYYY-MM-DD');
        const endDate1 = (0, moment_1.default)(endDate, 'DD-MM-YYYY').format('YYYY-MM-DD');
        let result = await this.collectionTableRepository
            .createQueryBuilder('r')
            .andWhere('STR_TO_DATE(r.collectionDate, "%Y-%m-%d") BETWEEN STR_TO_DATE(:entryDate,"%Y-%m-%d") AND STR_TO_DATE(:endDate,"%Y-%m-%d")', { entryDate: startDate1, endDate: endDate1 })
            .select('MAX(r.collectedBy)', 'collectedBy')
            .addSelect('SUM(r.amount)', 'amount')
            .groupBy('r.collectedBy')
            .getRawMany();
        let htTable = [];
        let total = 0;
        result.map((el, index) => {
            let ht11 = ['<tr>'];
            ht11.push(`<td>${index + 1}</td>`);
            ht11.push(`<td>${el.collectedBy}</td>`);
            ht11.push(`<td style="text-align: right">${parseFloat(el.amount).toFixed(2)}</td>`);
            ht11.push('</tr>');
            total = total + parseFloat(el.amount || 0);
            htTable.push(ht11.join(''));
        });
        let ht = `<table width="100%" style="border-collapse: collapse" border="1px">
        <thead>
        <tr>
          <td colspan="3" style="text-align: center">Collector Wise Report for ${startDate1} to ${endDate1}</td>
        </tr>
          <tr>
            <td style="font-weight: bold">#</td>
            <td style="font-weight: bold">Collector</td>
            <td style="font-weight: bold; text-align: right">Amount Collected</td>
          </tr>
        </thead>
        <tbody>

        ${htTable.join('')}

        </tbody>
        <tbody>
        <tr>
        <td style="font-weight: bold" colspan="2">TOTAL</td>
        <td style="font-weight: bold; text-align: right">${total.toFixed(2)}</td>
        </tr>
        </tbody>

      </table>`;
        return {
            status: true,
            data: result,
            html: ht,
        };
    }
    async getReportsByMonthOfUser(startDate, endDate, type) {
        const startDate1 = (0, moment_1.default)(startDate, 'DD-MM-YYYY').format('YYYY-MM-DD');
        const endDate1 = (0, moment_1.default)(endDate, 'DD-MM-YYYY').format('YYYY-MM-DD');
        let result = await this.collectionTableRepository
            .createQueryBuilder('r')
            .andWhere('STR_TO_DATE(r.collectionDate, "%Y-%m-%d") BETWEEN STR_TO_DATE(:entryDate,"%Y-%m-%d") AND STR_TO_DATE(:endDate,"%Y-%m-%d")', { entryDate: startDate1, endDate: endDate1 })
            .leftJoinAndMapOne('r.UserTable', user_collection_entity_1.UserTable, 'ml', 'ml.id=r.userId')
            .getMany();
        let htTable = [];
        let total = 0;
        result.map((el, index) => {
            let ht11 = ['<tr>'];
            ht11.push(`<td>${index + 1}</td>`);
            ht11.push(`<td>${el.UserTable.username}</td>`);
            ht11.push(`<td>${el.UserTable.name}</td>`);
            ht11.push(`<td>${el.collectedBy}</td>`);
            ht11.push(`<td>${el.collectionDate} ${el.collectionTime}</td>`);
            ht11.push(`<td>${(0, moment_1.default)(el.month, 'YYYY-MM').format('YYYY-MMM')}</td>`);
            ht11.push(`<td>${el.date}</td>`);
            ht11.push(`<td>${el.endDate}</td>`);
            ht11.push(`<td>${el.collectionType}</td>`);
            ht11.push(`<td style="text-align: right">${parseFloat(el.amount).toFixed(2)}</td>`);
            ht11.push('</tr>');
            total = total + parseFloat(el.amount || 0);
            htTable.push(ht11.join(''));
        });
        let ht = `<table width="100%" style="border-collapse: collapse" border="1px">
        <thead>
        <tr>
          <td colspan="10" style="text-align: center">User Wise Report for ${startDate1} to ${endDate1}</td>
        </tr>
          <tr>
            <td style="font-weight: bold">#</td>
            <td style="font-weight: bold">Reg. Id</td>
            <td style="font-weight: bold">Name</td>
            <td style="font-weight: bold">Collected By</td>
            <td style="font-weight: bold">Collected On</td>
            <td style="font-weight: bold">Month</td>
            <td style="font-weight: bold">Date</td>
            <td style="font-weight: bold">End Date</td>
            <td style="font-weight: bold">Collection Type</td>
            <td style="font-weight: bold; text-align: right">Amount Collected</td>
            
          </tr>
        </thead>
        <tbody>

        ${htTable.join('')}

        </tbody>
        <tbody>
        <tr>
        <td style="font-weight: bold" colspan="9">TOTAL</td>
        <td style="font-weight: bold; text-align: right">${total.toFixed(2)}</td>
        </tr>
        </tbody>

      </table>`;
        return {
            status: true,
            data: result,
            html: ht,
        };
    }
    async getReportsOfUserByDate(startDate, endDate, username) {
        const startDate1 = (0, moment_1.default)(startDate, 'DD-MM-YYYY').format('YYYY-MM-DD');
        const endDate1 = (0, moment_1.default)(endDate, 'DD-MM-YYYY').format('YYYY-MM-DD');
        let us = await this.userCollectionService.findOneByUsername(username);
        let result = await this.collectionTableRepository
            .createQueryBuilder('r')
            .andWhere('STR_TO_DATE(r.collectionDate, "%Y-%m-%d") BETWEEN STR_TO_DATE(:entryDate,"%Y-%m-%d") AND STR_TO_DATE(:endDate,"%Y-%m-%d")', { entryDate: startDate1, endDate: endDate1 })
            .leftJoinAndMapOne('r.UserTable', user_collection_entity_1.UserTable, 'ml', 'ml.id=r.userId')
            .andWhere('r.userId=:userId', { userId: us.id })
            .getMany();
        let htTable = [];
        let total = 0;
        result.map((el, index) => {
            let ht11 = ['<tr>'];
            ht11.push(`<td>${index + 1}</td>`);
            ht11.push(`<td>${el.UserTable.username}</td>`);
            ht11.push(`<td>${el.UserTable.name}</td>`);
            ht11.push(`<td>${el.collectedBy}</td>`);
            ht11.push(`<td>${el.collectionDate} ${el.collectionTime}</td>`);
            ht11.push(`<td>${(0, moment_1.default)(el.month, 'YYYY-MM').format('YYYY-MMM')}</td>`);
            ht11.push(`<td>${el.date}</td>`);
            ht11.push(`<td>${el.endDate}</td>`);
            ht11.push(`<td>${el.collectionType}</td>`);
            ht11.push(`<td style="text-align: right">${parseFloat(el.amount).toFixed(2)}</td>`);
            ht11.push('</tr>');
            total = total + parseFloat(el.amount || 0);
            htTable.push(ht11.join(''));
        });
        let ht = `<table width="100%" style="border-collapse: collapse" border="1px">
        <thead>
        <tr>
          <td colspan="10" style="text-align: center">User Wise Report for ${startDate1} to ${endDate1}</td>
        </tr>
          <tr>
            <td style="font-weight: bold">#</td>
            <td style="font-weight: bold">Reg. Id</td>
            <td style="font-weight: bold">Name</td>
            <td style="font-weight: bold">Collected By</td>
            <td style="font-weight: bold">Collected On</td>
            <td style="font-weight: bold">Month</td>
            <td style="font-weight: bold">Date</td>
            <td style="font-weight: bold">End Date</td>
            <td style="font-weight: bold">Collection Type</td>
            <td style="font-weight: bold; text-align: right">Amount Collected</td>
            
          </tr>
        </thead>
        <tbody>

        ${htTable.join('')}

        </tbody>
        <tbody>
        <tr>
        <td style="font-weight: bold" colspan="9">TOTAL</td>
        <td style="font-weight: bold; text-align: right">${total.toFixed(2)}</td>
        </tr>
        </tbody>

      </table>`;
        return {
            status: true,
            data: result,
            html: ht,
        };
    }
    async findDashboardCount(username) {
        let totalCollection = await this.collectionTableRepository
            .createQueryBuilder('r')
            .select('SUM(amount)', 'amount')
            .getRawOne();
        let totalCollectionByMe = await this.collectionTableRepository
            .createQueryBuilder('r')
            .leftJoinAndMapOne('r.UserTable', user_collection_entity_1.UserTable, 'ml', 'ml.id=r.userId')
            .andWhere('ml.username=:username', { username: username })
            .select('SUM(amount)', 'amount')
            .getRawOne();
        let d = {
            totalCollection: parseFloat(totalCollection.amount || 0).toFixed(2),
            totalCollectionByMe: parseFloat(totalCollectionByMe.amount || 0).toFixed(2),
        };
        return d;
    }
    findAll() {
        return `This action returns all collectionTable`;
    }
    findOne(collectionId) {
        return `This action returns a #${collectionId} collectionTable`;
    }
    update(collectionId, updateCollectionTableInput) {
        return `This action updates a #${collectionId} collectionTable`;
    }
    remove(collectionId) {
        return `This action removes a #${collectionId} collectionTable`;
    }
};
CollectionTableService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(collection_table_entity_1.CollectionTable)),
    __param(1, (0, common_1.Inject)((0, common_1.forwardRef)(() => user_collection_service_1.UserCollectionService))),
    __param(2, (0, common_1.Inject)(GlobalConstants_1.PUB_SUB)),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        user_collection_service_1.UserCollectionService,
        graphql_redis_subscriptions_1.RedisPubSub,
        typeorm_2.Connection])
], CollectionTableService);
exports.CollectionTableService = CollectionTableService;
