"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const graphql_relay_1 = require("graphql-relay");
const graphql_1 = require("@nestjs/graphql");
function checkPagingSanity(args) {
    const { first = 0, last = 0, after, before, } = args;
    const isForwardPaging = !!first || !!after;
    const isBackwardPaging = !!last || !!before;
    if (isForwardPaging && isBackwardPaging) {
        throw new Error('Relay pagination cannot be forwards AND backwards!');
    }
    if ((isForwardPaging && before) || (isBackwardPaging && after)) {
        throw new Error('Paging must use either first/after or last/before!');
    }
    if ((isForwardPaging && first < 0) || (isBackwardPaging && last < 0)) {
        throw new Error('Paging limit must be positive!');
    }
    if (last && !before) {
        throw new Error("When paging backwards, a 'before' argument is required!");
    }
    return isForwardPaging
        ? { pagingType: 'forward', after, first }
        : isBackwardPaging
            ? { pagingType: 'backward', before, last }
            : { pagingType: 'none' };
}
const getId = (cursor) => parseInt((0, graphql_relay_1.fromGlobalId)(cursor).id, 10);
const nextId = (cursor) => getId(cursor) + 1;
function getPagingParameters(args) {
    const meta = checkPagingSanity(args);
    switch (meta.pagingType) {
        case 'forward': {
            return {
                limit: meta.first,
                offset: meta.after ? nextId(meta.after) : 0,
            };
        }
        case 'backward': {
            const { last, before } = meta;
            let limit = last;
            let offset = getId(before) - last;
            if (offset < 0) {
                limit = Math.max(last + offset, 0);
                offset = 0;
            }
            return { offset, limit };
        }
        default:
            return {};
    }
}
let ConnectionArgs = class ConnectionArgs {
    pagingParams() {
        return getPagingParameters(this);
    }
};
__decorate([
    (0, graphql_1.Field)({ nullable: true, description: 'Paginate before opaque cursor' }),
    __metadata("design:type", String)
], ConnectionArgs.prototype, "before", void 0);
__decorate([
    (0, graphql_1.Field)({ nullable: true, description: 'Paginate after opaque cursor' }),
    __metadata("design:type", String)
], ConnectionArgs.prototype, "after", void 0);
__decorate([
    (0, graphql_1.Field)({ nullable: true, description: 'Paginate first' }),
    __metadata("design:type", Number)
], ConnectionArgs.prototype, "first", void 0);
__decorate([
    (0, graphql_1.Field)({ nullable: true, description: 'Paginate last' }),
    __metadata("design:type", Number)
], ConnectionArgs.prototype, "last", void 0);
ConnectionArgs = __decorate([
    (0, graphql_1.ArgsType)()
], ConnectionArgs);
exports.default = ConnectionArgs;
