"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConsumptionSubTableService = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const graphql_redis_subscriptions_1 = require("graphql-redis-subscriptions");
const animal_entry_table_service_1 = require("../animal-table/animal-entry-table.service");
const animal_entry_table_entity_1 = require("../animal-table/entities/animal-entry-table.entity");
const consumption_table_service_1 = require("../consumption-table/consumption-table.service");
const consumption_table_entity_1 = require("../consumption-table/entities/consumption-table.entity");
const district_table_service_1 = require("../district-table/district-table.service");
const farmers_table_entity_1 = require("../farmers-table/entities/farmers-table.entity");
const GlobalConstants_1 = require("../helper/GlobalConstants");
const housing_table_entity_1 = require("../housing-table/entities/housing-table.entity");
const ingredient_table_entity_1 = require("../ingredient-table/entities/ingredient-table.entity");
const ration_sub_table_entity_1 = require("../ration-sub-table/entity/ration-sub-table.entity");
const ration_table_entity_1 = require("../ration-table/entities/ration-table.entity");
const ration_table_service_1 = require("../ration-table/ration-table.service");
const sub_division_table_service_1 = require("../sub-division-table/sub-division-table.service");
const typeorm_2 = require("typeorm");
const consumption_sub_table_entity_1 = require("./entities/consumption-sub-table.entity");
let ConsumptionSubTableService = class ConsumptionSubTableService {
    constructor(consumptionSubTableRepository, pubSub, consumptionTableService, rationTableService, animalEntryTableService, subDivisionTableService, districtTableService) {
        this.consumptionSubTableRepository = consumptionSubTableRepository;
        this.pubSub = pubSub;
        this.consumptionTableService = consumptionTableService;
        this.rationTableService = rationTableService;
        this.animalEntryTableService = animalEntryTableService;
        this.subDivisionTableService = subDivisionTableService;
        this.districtTableService = districtTableService;
    }
    async create(consumptionId, createConsumptionTableInput) {
        if (Array.isArray(createConsumptionTableInput.rationSubData) && createConsumptionTableInput.rationSubData.length > 0) {
            if (Array.isArray(createConsumptionTableInput.animalSubData) && createConsumptionTableInput.animalSubData.length > 0) {
                for (const el of createConsumptionTableInput.animalSubData) {
                    for (const el1 of createConsumptionTableInput.rationSubData) {
                        let dd = new consumption_sub_table_entity_1.ConsumptionSubTable();
                        dd.ConsumptionTable = await this.consumptionTableService.findOne(consumptionId);
                        dd.AnimalTable = await this.animalEntryTableService.findByAnimalId(el.animalId);
                        dd.RationTable = await this.rationTableService.findOne(el1.rationId);
                        let calculatedPerHead = el1.rationPerHead / createConsumptionTableInput.animalSubData.length;
                        dd.rationPerHead = calculatedPerHead;
                        await this.consumptionSubTableRepository.save(dd);
                    }
                }
            }
            else {
                let qty = createConsumptionTableInput.qty;
                if (qty > 0) {
                    let animals = await this.animalEntryTableService.findByCurrentStock(createConsumptionTableInput.currentStockId, qty);
                    if (animals.length > 0) {
                        for (const el of animals) {
                            for (const el1 of createConsumptionTableInput.rationSubData) {
                                let dd = new consumption_sub_table_entity_1.ConsumptionSubTable();
                                dd.ConsumptionTable = await this.consumptionTableService.findOne(consumptionId);
                                dd.AnimalTable = await this.animalEntryTableService.findByAnimalId(el.animalId);
                                dd.RationTable = await this.rationTableService.findOne(el1.rationId);
                                let calculatedPerHead = el1.rationPerHead / animals.length;
                                dd.rationPerHead = calculatedPerHead;
                                await this.consumptionSubTableRepository.save(dd);
                            }
                        }
                    }
                }
            }
            return {
                status: true,
                status_text: "Successfully Created",
                data: null
            };
        }
        else
            return {
                status: false,
                status_text: "Invalid Data",
                data: null
            };
    }
    async findAll() {
        return await this.consumptionSubTableRepository.find({
            relations: {
                ConsumptionTable: true,
                AnimalTable: true,
                RationTable: true
            }
        });
    }
    async findOne(consumptionSubId) {
        return await this.consumptionSubTableRepository.findOne({
            where: {
                consumptionSubId: consumptionSubId
            },
            relations: ["ConsumptionTable"]
        });
    }
    async remove(consumptionId) {
        let result = await this.consumptionSubTableRepository.findOne({
            where: {
                ConsumptionTable: {
                    consumptionId: consumptionId
                }
            }
        });
        if (result) {
            await this.consumptionSubTableRepository.remove(result);
            return {
                status: true,
                status_text: "Successfully Removed",
                data: null
            };
        }
        else {
            return {
                status: false,
                status_text: "Not Found",
                data: null
            };
        }
    }
    async countFeedingDetailsClusterWise(districtId, startingDate, endingDate) {
        const subdivisions = await this.subDivisionTableService.fetchSubDivisionByDistrictId(districtId);
        let data = [];
        for (const e of subdivisions) {
            let d1 = await this.consumptionSubTableRepository.createQueryBuilder("r")
                .leftJoinAndMapOne("r.ConsumptionTable", consumption_table_entity_1.ConsumptionTable, "ml1", "ml1.consumptionId=r.consumptionId")
                .leftJoinAndMapOne("ml1.HousingTable", housing_table_entity_1.HousingTable, "ml2", "ml1.housingId=ml2.housingId")
                .leftJoinAndMapOne("ml2.FarmersTable", farmers_table_entity_1.FarmersTable, "ml3", "ml2.farmerId=ml3.farmerId")
                .where("ml3.subDivisionId=:subDivisionId", { subDivisionId: e.subDivisionId })
                .andWhere("STR_TO_DATE(ml1.dateOfEntry,'%Y-%m-%d') BETWEEN STR_TO_DATE(:startingDate,'%Y-%m-%d') AND STR_TO_DATE(:endingDate,'%Y-%m-%d')", {
                startingDate: startingDate,
                endingDate: endingDate
            })
                .select("SUM(r.rationPerHead)", "totalFeed")
                .addSelect("COUNT(r.animalId)", "totalPigs")
                .groupBy("ml3.subDivisionId")
                .getRawOne();
            let d2 = {
                SubDivisionTable: e,
                totalFeed: d1 ? d1.totalFeed : 0,
                totalPigs: d1 ? d1.totalPigs : 0,
            };
            data.push(d2);
        }
        console.log(data);
        return data;
    }
    async getFeedingDetailsDistrictWise() {
        let districts = await this.districtTableService.findAll();
        let data = [];
        for (const e of districts) {
            let d1 = await this.consumptionSubTableRepository.createQueryBuilder("r")
                .leftJoinAndMapOne("r.ConsumptionTable", consumption_table_entity_1.ConsumptionTable, "ml1", "ml1.consumptionId=r.consumptionId")
                .leftJoinAndMapOne("ml1.HousingTable", housing_table_entity_1.HousingTable, "ml2", "ml2.housingId=ml1.housingId")
                .leftJoinAndMapOne("ml2.FarmersTable", farmers_table_entity_1.FarmersTable, "ml3", "ml3.farmerId=ml2.farmerId")
                .where("ml3.districtId=:districtId", { districtId: e.districtId })
                .select("SUM(r.rationPerHead)", "totalFeed")
                .addSelect("COUNT(r.animalId)", "totalPigs")
                .groupBy("ml3.districtId")
                .getRawOne();
            let d2 = {
                DistrictTable: e,
                totalFeed: d1 ? d1.totalFeed : 0,
                totalPigs: d1 ? d1.totalPigs : 0,
            };
            data.push(d2);
        }
        return data;
    }
    async getFeedingDetailsOfOneAnimal(animalId) {
        let result = await this.consumptionSubTableRepository.createQueryBuilder("r")
            .leftJoinAndMapOne("r.AnimalTable", animal_entry_table_entity_1.AnimalEntryTable, "ml1", "ml1.animalId=r.animalId")
            .leftJoinAndMapOne("r.RationTable", ration_table_entity_1.RationTable, "ml2", "ml2.rationId=r.rationId")
            .leftJoinAndMapMany("ml2.RationSubTable", ration_sub_table_entity_1.RationSubTable, "ml4", "ml4.rationId=ml2.rationId")
            .leftJoinAndMapOne("ml4.IngredientTable", ingredient_table_entity_1.IngredientTable, "ml5", "ml5.ingredientId=ml4.ingredientId")
            .leftJoinAndMapOne("r.ConsumptionTable", consumption_table_entity_1.ConsumptionTable, "ml3", "ml3.consumptionId=r.consumptionId")
            .where("r.animalId=:animalId", { animalId: animalId })
            .getMany();
        console.log(result);
        return result;
    }
};
ConsumptionSubTableService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(consumption_sub_table_entity_1.ConsumptionSubTable)),
    __param(1, (0, common_1.Inject)(GlobalConstants_1.PUB_SUB)),
    __param(2, (0, common_1.Inject)((0, common_1.forwardRef)(() => consumption_table_service_1.ConsumptionTableService))),
    __param(3, (0, common_1.Inject)((0, common_1.forwardRef)(() => ration_table_service_1.RationTableService))),
    __param(4, (0, common_1.Inject)((0, common_1.forwardRef)(() => animal_entry_table_service_1.AnimalEntryTableService))),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        graphql_redis_subscriptions_1.RedisPubSub,
        consumption_table_service_1.ConsumptionTableService,
        ration_table_service_1.RationTableService,
        animal_entry_table_service_1.AnimalEntryTableService,
        sub_division_table_service_1.SubDivisionTableService,
        district_table_service_1.DistrictTableService])
], ConsumptionSubTableService);
exports.ConsumptionSubTableService = ConsumptionSubTableService;
