"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConsumptionTableService = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const graphql_redis_subscriptions_1 = require("graphql-redis-subscriptions");
const consumption_sub_table_service_1 = require("../consumption-sub-table/consumption-sub-table.service");
const consumption_sub_total_table_service_1 = require("../consumption-sub-total-table/consumption-sub-total-table.service");
const GlobalConstants_1 = require("../helper/GlobalConstants");
const housing_table_service_1 = require("../housing-table/housing-table.service");
const sub_division_table_service_1 = require("../sub-division-table/sub-division-table.service");
const typeorm_2 = require("typeorm");
const consumption_table_entity_1 = require("./entities/consumption-table.entity");
const data_entry_table_service_1 = require("../data-entry-table/data-entry-table.service");
const GlobalConstants_2 = require("../helper/GlobalConstants");
const user_collection_service_1 = require("../user-collection/user-collection.service");
let ConsumptionTableService = class ConsumptionTableService {
    constructor(consumptionTableRepository, userCollectionService, dataEntryTableService, pubSub, housingTableService, consumptionSubTableService, consumptionSubTotalTableService, subDivisionTableService) {
        this.consumptionTableRepository = consumptionTableRepository;
        this.userCollectionService = userCollectionService;
        this.dataEntryTableService = dataEntryTableService;
        this.pubSub = pubSub;
        this.housingTableService = housingTableService;
        this.consumptionSubTableService = consumptionSubTableService;
        this.consumptionSubTotalTableService = consumptionSubTotalTableService;
        this.subDivisionTableService = subDivisionTableService;
    }
    async create(createConsumptionTableInput, user) {
        let dd = new consumption_table_entity_1.ConsumptionTable();
        dd.dateOfEntry = createConsumptionTableInput.dateOfEntry;
        dd.remarks = createConsumptionTableInput.remarks;
        dd.frequency = createConsumptionTableInput.frequency;
        let housing = await this.housingTableService.findOne(createConsumptionTableInput.housingId);
        dd.HousingTable = housing;
        let result = await this.consumptionTableRepository.save(dd);
        let temp = {
            animalSubData: createConsumptionTableInput.animalData,
            rationSubData: createConsumptionTableInput.rationData,
            qty: createConsumptionTableInput.qty,
            currentStockId: createConsumptionTableInput.currentStockId
        };
        await this.consumptionSubTableService.create(result.consumptionId, temp);
        await this.consumptionSubTotalTableService.create(result.consumptionId, createConsumptionTableInput.rationData);
        this.dataEntryTableService.create({ type: GlobalConstants_2.CREATE, typeOfEntry: GlobalConstants_2.FEEDING_ADD, remarks: "New feeding added" }, user);
        return {
            status: true,
            status_text: "Successfully Created",
            data: result
        };
    }
    async findAll() {
        return await this.consumptionTableRepository.find({
            relations: {
                ConsumptionSubTable: {
                    AnimalTable: {
                        HousingTable: {
                            FarmersTable: true
                        }
                    },
                    RationTable: {
                        BrandTable: true
                    }
                }
            }
        });
    }
    async findOne(consumptionId) {
        return await this.consumptionTableRepository.findOne({
            where: {
                consumptionId: consumptionId
            }
        });
    }
    async findOneConsumption(consumptionId) {
        return await this.consumptionTableRepository.findOne({
            where: {
                consumptionId: consumptionId
            },
            relations: {
                ConsumptionSubTable: {
                    AnimalTable: {
                        HousingTable: {
                            FarmersTable: true
                        }
                    },
                    RationTable: {
                        BrandTable: true
                    }
                }
            }
        });
    }
    async findConsumptionByFarmerId(farmerId) {
        return await this.consumptionTableRepository.find({
            where: {
                ConsumptionSubTable: {
                    AnimalTable: {
                        HousingTable: {
                            FarmersTable: {
                                farmerId
                            }
                        }
                    },
                    RationTable: {
                        BrandTable: true
                    }
                }
            },
            relations: {
                ConsumptionSubTable: {
                    AnimalTable: {
                        HousingTable: {
                            FarmersTable: true
                        }
                    },
                    RationTable: {
                        BrandTable: true
                    }
                }
            }
        });
    }
    async findConsumptionByHousingId(housingId) {
        let result = await this.consumptionTableRepository.find({
            where: {
                HousingTable: {
                    housingId
                }
            },
            relations: {
                ConsumptionSubTable: {
                    AnimalTable: {
                        HousingTable: {
                            FarmersTable: true
                        }
                    },
                    RationTable: {
                        BrandTable: true
                    }
                },
                ConsumptionSubTotalTable: {
                    RationTable: {
                        BrandTable: true
                    }
                }
            }
        });
        let index = 0;
        for (const el of result) {
            let ageGroup = "";
            let totalAge = 0;
            let count = 0;
            let rcount = 0;
            el.ConsumptionSubTable.map((el1) => {
                totalAge = totalAge + parseFloat(el1.AnimalTable.age);
                count++;
            });
            if (count > 0) {
                ageGroup = totalAge / count;
            }
            rcount = el.ConsumptionSubTotalTable.length;
            count = count / rcount;
            result[index]['ageGroup'] = ageGroup;
            result[index]['animalCount'] = count;
            index++;
        }
        return result;
    }
    async remove(consumptionId, user) {
        let ht = await this.consumptionTableRepository.findOne({
            where: {
                consumptionId: consumptionId
            }
        });
        if (!ht) {
            return {
                status: false,
                status_text: "Not Found",
                data: null
            };
        }
        else {
            await this.consumptionTableRepository.remove(ht);
            this.dataEntryTableService.create({ type: GlobalConstants_2.DELETE, typeOfEntry: GlobalConstants_2.FEEDING_DELETE, remarks: "feeding added" }, user);
            return {
                status: true,
                status_text: "Successfully Deleted",
                data: null
            };
        }
    }
};
ConsumptionTableService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(consumption_table_entity_1.ConsumptionTable)),
    __param(3, (0, common_1.Inject)(GlobalConstants_1.PUB_SUB)),
    __param(4, (0, common_1.Inject)((0, common_1.forwardRef)(() => housing_table_service_1.HousingTableService))),
    __param(5, (0, common_1.Inject)((0, common_1.forwardRef)(() => consumption_sub_table_service_1.ConsumptionSubTableService))),
    __param(6, (0, common_1.Inject)((0, common_1.forwardRef)(() => consumption_sub_total_table_service_1.ConsumptionSubTotalTableService))),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        user_collection_service_1.UserCollectionService,
        data_entry_table_service_1.DataEntryTableService,
        graphql_redis_subscriptions_1.RedisPubSub,
        housing_table_service_1.HousingTableService,
        consumption_sub_table_service_1.ConsumptionSubTableService,
        consumption_sub_total_table_service_1.ConsumptionSubTotalTableService,
        sub_division_table_service_1.SubDivisionTableService])
], ConsumptionTableService);
exports.ConsumptionTableService = ConsumptionTableService;
