"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CourseProgressionTableService = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const course_progression_table_entity_1 = require("./entities/course-progression-table.entity");
const typeorm_2 = require("typeorm");
const GlobalConstants_1 = require("../helper/GlobalConstants");
const graphql_redis_subscriptions_1 = require("graphql-redis-subscriptions");
const course_unit_table_service_1 = require("../course-unit-table/course-unit-table.service");
const user_collection_service_1 = require("../user-collection/user-collection.service");
const assessment_answer_table_service_1 = require("../assessment-answer-table/assessment-answer-table.service");
let CourseProgressionTableService = exports.CourseProgressionTableService = class CourseProgressionTableService {
    constructor(courseProgressionTableRepository, pubSub, courseUnitTableService, userCollectionService, assessmentAnswerTableService) {
        this.courseProgressionTableRepository = courseProgressionTableRepository;
        this.pubSub = pubSub;
        this.courseUnitTableService = courseUnitTableService;
        this.userCollectionService = userCollectionService;
        this.assessmentAnswerTableService = assessmentAnswerTableService;
    }
    async updateCourseProgression(courseUnitId, type, user) {
        let result = await this.courseProgressionTableRepository.findOne({
            where: {
                CourseUnitTable: {
                    courseUnitId
                },
                UserTable: {
                    id: user.id
                },
                type
            }
        });
        if (result) {
            result.isComplete = 1;
            result.canTakeAssesment = 1;
            await this.courseProgressionTableRepository.save(result);
            return {
                status: true,
                status_text: "Successfully Updated"
            };
        }
        else {
            let rs = new course_progression_table_entity_1.CourseProgressionTable();
            rs.CourseUnitTable = await this.courseUnitTableService.findOne(courseUnitId);
            rs.UserTable = await this.userCollectionService.findOne(user.id);
            rs.isComplete = 1;
            rs.canTakeAssesment = 1;
            rs.type = type;
            await this.courseProgressionTableRepository.save(rs);
            return {
                status: true,
                status_text: "Successfully Updated"
            };
        }
    }
    async getProgressionReport(courseUnitId, user) {
        return await this.courseProgressionTableRepository.findOne({
            where: {
                CourseUnitTable: {
                    courseUnitId
                },
                UserTable: {
                    id: user.id
                }
            }
        });
    }
    async checkIfAllowedForAssessment(courseUnitId, user) {
        let result = await this.courseProgressionTableRepository.findOne({
            where: {
                CourseUnitTable: {
                    courseUnitId
                },
                UserTable: {
                    id: user.id
                }
            }
        });
        if (result) {
            if (result.assesmentStatus == 0) {
                return true;
            }
            else {
                return false;
            }
        }
        return false;
    }
    async getScoreForAssessment(courseUnitId, user) {
        let result = await this.courseProgressionTableRepository.findOne({
            where: {
                CourseUnitTable: {
                    courseUnitId
                },
                UserTable: {
                    id: user.id
                }
            }
        });
        if (result) {
            if (result.assesmentStatus == 1) {
                return result.assesmentScore;
            }
            else {
                return false;
            }
        }
        return false;
    }
    async updateCourseProgressionAssessment(courseUnitId, user, assesmentTaken, assesmentTakenOn, assesmentScore, assesmentStatus) {
        let result = await this.courseProgressionTableRepository.findOne({
            where: {
                CourseUnitTable: {
                    courseUnitId
                },
                UserTable: {
                    id: user.id
                }
            }
        });
        if (result) {
            result.assesmentTaken = assesmentTaken;
            result.assesmentTakenOn = assesmentTakenOn;
            result.assesmentScore = assesmentScore;
            result.assesmentStatus = assesmentStatus;
            await this.courseProgressionTableRepository.save(result);
            return true;
        }
        else {
            return false;
        }
    }
    async findAllCourseProgression(courseId) {
        let result = await this.courseProgressionTableRepository.find({
            where: {
                CourseUnitTable: {
                    CourseTable: {
                        courseId
                    }
                }
            },
            relations: {
                CourseUnitTable: true,
                UserTable: true
            }
        });
        let index = 0;
        for (const el of result) {
            result[index]['AssessmentAnswersTable'] = await this.assessmentAnswerTableService.findAllByCourseIdAndUserId(el.CourseUnitTable.courseUnitId, el.UserTable.id);
            index++;
        }
        return result;
    }
    async toggleStatusOfTable(courseProgressionId, type, currentValue) {
        let result = await this.courseProgressionTableRepository.findOne({
            where: {
                courseProgressionId
            }
        });
        if (result) {
            let newValue = 0;
            if (currentValue == 1) {
                newValue = 0;
            }
            if (currentValue == 0) {
                newValue = 1;
            }
            if (type == "isComplete") {
                if (newValue == 0) {
                    result.isComplete = 0;
                    result.canTakeAssesment = 0;
                    result.assesmentScore = 0;
                    result.assesmentTaken = 0;
                    result.assesmentTakenOn = null;
                    result.assesmentStatus = 0;
                }
                if (newValue == 1) {
                    result.isComplete = 1;
                }
            }
            if (type == "canTakeAssesment") {
                if (newValue == 0) {
                    result.canTakeAssesment = 0;
                }
                if (newValue == 1) {
                    result.canTakeAssesment = 1;
                }
            }
            if (type == "assesmentTaken") {
                if (newValue == 0) {
                    result.assesmentTaken = 0;
                }
                if (newValue == 1) {
                    result.assesmentTaken = 1;
                }
            }
            if (type == "assesmentStatus") {
                if (newValue == 0) {
                    result.assesmentStatus = 0;
                }
                if (newValue == 1) {
                    result.assesmentStatus = 1;
                }
            }
            await this.courseProgressionTableRepository.save(result);
            return {
                status: true,
                status_text: "Successfully Updated"
            };
        }
        else {
            return {
                status: false,
                status_text: "Not found"
            };
        }
    }
};
exports.CourseProgressionTableService = CourseProgressionTableService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(course_progression_table_entity_1.CourseProgressionTable)),
    __param(1, (0, common_1.Inject)(GlobalConstants_1.PUB_SUB)),
    __param(2, (0, common_1.Inject)((0, common_1.forwardRef)(() => course_unit_table_service_1.CourseUnitTableService))),
    __param(3, (0, common_1.Inject)((0, common_1.forwardRef)(() => user_collection_service_1.UserCollectionService))),
    __param(4, (0, common_1.Inject)((0, common_1.forwardRef)(() => assessment_answer_table_service_1.AssessmentAnswerTableService))),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        graphql_redis_subscriptions_1.RedisPubSub,
        course_unit_table_service_1.CourseUnitTableService,
        user_collection_service_1.UserCollectionService,
        assessment_answer_table_service_1.AssessmentAnswerTableService])
], CourseProgressionTableService);
