"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CourseSessionTableService = void 0;
const common_1 = require("@nestjs/common");
const course_session_table_entity_1 = require("./entities/course-session-table.entity");
const typeorm_1 = require("typeorm");
const typeorm_2 = require("@nestjs/typeorm");
const course_table_service_1 = require("../course-table/course-table.service");
const moment_1 = __importDefault(require("moment"));
let CourseSessionTableService = exports.CourseSessionTableService = class CourseSessionTableService {
    constructor(courseSessionTableRepository, courseTableService) {
        this.courseSessionTableRepository = courseSessionTableRepository;
        this.courseTableService = courseTableService;
    }
    async create(createCourseSessionTableInput) {
        const checkIfExists = await this.checkIfExists(createCourseSessionTableInput);
        if (!checkIfExists) {
            let cst = new course_session_table_entity_1.CourseSessionTable();
            cst.sessionName = createCourseSessionTableInput.sessionName;
            cst.capacity = createCourseSessionTableInput.capacity;
            cst.sessionStartingDate = (0, moment_1.default)(createCourseSessionTableInput.sessionStartingDate, "YYYY-MM-DD").format("DD-MM-YYYY");
            cst.sessionEndingDate = (0, moment_1.default)(createCourseSessionTableInput.sessionEndingDate, "YYYY-MM-DD").format("DD-MM-YYYY");
            cst.CourseTable = await this.courseTableService.findOne(createCourseSessionTableInput.courseId);
            await this.courseSessionTableRepository.save(cst);
            return {
                status: true,
                status_text: "Succesfully Created"
            };
        }
        else {
            return {
                status: false,
                status_text: "Already Exists"
            };
        }
    }
    async checkIfExists(createCourseSessionTableInput) {
        let result = await this.courseSessionTableRepository.findOne({
            where: {
                sessionName: createCourseSessionTableInput.sessionName,
                sessionStartingDate: (0, moment_1.default)(createCourseSessionTableInput.sessionStartingDate, "YYYY-MM-DD").format("DD-MM-YYYY"),
                sessionEndingDate: (0, moment_1.default)(createCourseSessionTableInput.sessionEndingDate, "YYYY-MM-DD").format("DD-MM-YYYY"),
                CourseTable: {
                    courseId: createCourseSessionTableInput.courseId
                }
            }
        });
        return result;
    }
    async findAll() {
        return await this.courseSessionTableRepository.find({
            relations: {
                CourseTable: true
            }
        });
    }
    async findAllCourseByCourseId(courseId) {
        return await this.courseSessionTableRepository.find({
            where: {
                CourseTable: {
                    courseId
                },
                isEnrollmentOpen: 1
            }
        });
    }
    async findOne(courseSessionId) {
        return await this.courseSessionTableRepository.findOne({
            where: {
                courseSessionId
            },
            relations: {
                CourseTable: true
            }
        });
    }
    async findOneByCourseId(courseId) {
        return await this.courseSessionTableRepository.find({
            where: {
                CourseTable: {
                    courseId
                }
            },
            relations: {
                CourseTable: true
            }
        });
    }
    async update(courseSessionId, updateCourseSessionTableInput) {
        let result = await this.courseSessionTableRepository.findOne({
            where: {
                courseSessionId
            }
        });
        if (result) {
            result.sessionName = updateCourseSessionTableInput.sessionName;
            result.capacity = updateCourseSessionTableInput.capacity;
            result.sessionStartingDate = (0, moment_1.default)(updateCourseSessionTableInput.sessionStartingDate, "YYYY-MM-DD").format("DD-MM-YYYY");
            result.sessionEndingDate = (0, moment_1.default)(updateCourseSessionTableInput.sessionEndingDate, "YYYY-MM-DD").format("DD-MM-YYYY");
            result.CourseTable = await this.courseTableService.findOne(updateCourseSessionTableInput.courseId);
            await this.courseSessionTableRepository.save(result);
            return {
                status: true,
                status_text: "Successfully Updated"
            };
        }
        else {
            return {
                status: false,
                status_text: "Not found"
            };
        }
    }
    async remove(courseSessionId) {
        let result = await this.courseSessionTableRepository.findOne({
            where: {
                courseSessionId
            }
        });
        if (result) {
            await this.courseSessionTableRepository.remove(result);
            return {
                status: true,
                status_text: "Successfully Deleted"
            };
        }
        else {
            return {
                status: false,
                status_text: "Not found"
            };
        }
    }
    async toggleStatusofCourseSession(courseSessionId) {
        let res = await this.courseSessionTableRepository.findOne({
            where: {
                courseSessionId
            }
        });
        if (res) {
            let oldStatus = res.isEnrollmentOpen;
            let newStatus = 1;
            if (oldStatus == 0) {
                newStatus = 1;
            }
            if (oldStatus == 1) {
                newStatus = 0;
            }
            res.isEnrollmentOpen = newStatus;
            let result = await this.courseSessionTableRepository.save(res);
            return {
                status: true,
                status_text: "Successfully Updated",
                data: result
            };
        }
        else {
            return {
                status: false,
                status_text: "Not found"
            };
        }
    }
};
exports.CourseSessionTableService = CourseSessionTableService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_2.InjectRepository)(course_session_table_entity_1.CourseSessionTable)),
    __param(1, (0, common_1.Inject)((0, common_1.forwardRef)(() => course_table_service_1.CourseTableService))),
    __metadata("design:paramtypes", [typeorm_1.Repository,
        course_table_service_1.CourseTableService])
], CourseSessionTableService);
