"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CourseTableService = exports.NEW_COURSE_ADD = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const course_table_entity_1 = require("./entities/course-table.entity");
const typeorm_2 = require("typeorm");
const GlobalConstants_1 = require("../helper/GlobalConstants");
const graphql_redis_subscriptions_1 = require("graphql-redis-subscriptions");
const file_table_service_1 = require("../file-table/file-table.service");
const subscription_table_service_1 = require("../subscription-table/subscription-table.service");
const course_unit_table_service_1 = require("../course-unit-table/course-unit-table.service");
const course_progression_table_service_1 = require("../course-progression-table/course-progression-table.service");
const user_collection_service_1 = require("../user-collection/user-collection.service");
const course_session_table_service_1 = require("../course-session-table/course-session-table.service");
exports.NEW_COURSE_ADD = "NEW_COURSE_ADD";
let CourseTableService = exports.CourseTableService = class CourseTableService {
    constructor(courseTableRepository, pubSub, fileTableService, subscriptionTableService, courseUnitTableService, courseProgressionTableService, userCollectionService, courseSessionTableService) {
        this.courseTableRepository = courseTableRepository;
        this.pubSub = pubSub;
        this.fileTableService = fileTableService;
        this.subscriptionTableService = subscriptionTableService;
        this.courseUnitTableService = courseUnitTableService;
        this.courseProgressionTableService = courseProgressionTableService;
        this.userCollectionService = userCollectionService;
        this.courseSessionTableService = courseSessionTableService;
    }
    async create(createCourseTableInput) {
        let checkIfAlreadyExists = await this.checkIfAlreadyExists(createCourseTableInput);
        if (checkIfAlreadyExists == 0) {
            let st = new course_table_entity_1.CourseTable();
            st.courseName = createCourseTableInput.courseName;
            st.courseCode = createCourseTableInput.courseCode;
            st.courseDescription = createCourseTableInput.courseDescription;
            st.courseDuration = createCourseTableInput.courseDuration;
            st.courseType = createCourseTableInput.courseType;
            st.hasAssesment = createCourseTableInput.hasAssesment;
            st.isActive = createCourseTableInput.isActive;
            st.UserTable = await this.userCollectionService.getOneByUserId(createCourseTableInput.userId);
            let result = await this.courseTableRepository.save(st);
            this.pubSub.publish(exports.NEW_COURSE_ADD, 1);
            return {
                status: true,
                status_text: "Successfully Added",
                data: result
            };
        }
        else {
            return {
                status: false,
                status_text: "Already Exists"
            };
        }
    }
    async checkIfAlreadyExists(createCourseTableInput) {
        let result = await this.courseTableRepository.count({
            where: {
                courseCode: createCourseTableInput.courseCode,
            }
        });
        return result;
    }
    async findAll() {
        let result = await this.courseTableRepository.find({
            relations: {
                UserTable: true
            }
        });
        let index = 0;
        for (const r of result) {
            let url = await this.fileTableService.getUrlOfFileId(r.courseBanner);
            result[index]['courseBannerUrl'] = url;
            index++;
        }
        return result;
    }
    async findAllActiveCourses() {
        let result = await this.courseTableRepository.find({
            where: {
                isActive: "YES"
            },
            relations: {
                CourseUnitTable: true
            }
        });
        let index = 0;
        for (const r of result) {
            let url = await this.fileTableService.getUrlOfFileId(r.courseBanner);
            result[index]['courseBannerUrl'] = url;
            result[index]['courseUnitCount'] = r.CourseUnitTable.length;
            let totalPages = 0;
            r.CourseUnitTable.map(el => {
                totalPages += el.numberOfPages;
            });
            result[index]['courseUnitPages'] = totalPages;
            index++;
        }
        return result;
    }
    async findMyActiveCourses(user) {
        let result = await this.courseTableRepository.find({
            where: {
                isActive: "YES",
                SubscriptionTable: {
                    UserTable: {
                        id: user.id
                    },
                    status: "YES"
                }
            },
            relations: {
                CourseUnitTable: true
            }
        });
        let index = 0;
        for (const r of result) {
            let url = await this.fileTableService.getUrlOfFileId(r.courseBanner);
            result[index]['courseBannerUrl'] = url;
            result[index]['courseUnitCount'] = r.CourseUnitTable.length;
            let totalPages = 0;
            r.CourseUnitTable.map(el => {
                totalPages += el.numberOfPages;
            });
            result[index]['courseUnitPages'] = totalPages;
            index++;
        }
        return result;
    }
    async searchActiveCourses(search) {
        let result = await this.courseTableRepository.createQueryBuilder('r')
            .andWhere("r.isActive='YES'");
        if (search != '') {
            let s = `${search}%`;
            result = await result.andWhere('r.courseName LIKE :courseName', { courseName: s });
        }
        result = await result.getMany();
        console.log("🚀 ~ CourseTableService ~ searchActiveCourses ~ result:", result);
        let index = 0;
        for (const r of result) {
            let url = await this.fileTableService.getUrlOfFileId(r.courseBanner);
            result[index]['courseBannerUrl'] = url;
            let totalPages = 0;
            result[index]['courseUnitPages'] = totalPages;
            index++;
        }
        return result;
    }
    async findOne(courseId) {
        let result = await this.courseTableRepository.findOne({
            where: {
                courseId
            },
            relations: {
                CourseUnitTable: true
            }
        });
        if (result) {
            let url = await this.fileTableService.getUrlOfFileId(result.courseBanner);
            result.courseBannerUrl = url;
            result.courseUnitCount = result.CourseUnitTable.length;
            let totalPages = 0;
            result.CourseUnitTable.map(el => {
                totalPages += el.numberOfPages;
            });
            result.courseUnitPages = totalPages;
        }
        return result;
    }
    async findOneRestricted(courseId, user) {
        let isAllowed = await this.subscriptionTableService.checkIfAllowed(courseId, user.id);
        let result = await this.courseTableRepository.findOne({
            where: {
                courseId
            }
        });
        if (result) {
            let CourseUnitTable = await this.courseUnitTableService.findOneCourseUnitByCourseId(result.courseId);
            result.CourseUnitTable = CourseUnitTable;
            let url = await this.fileTableService.getUrlOfFileId(result.courseBanner);
            result.courseBannerUrl = url;
            result.courseUnitCount = result.CourseUnitTable.length;
            let totalPages = 0;
            result.CourseUnitTable.map(el => {
                totalPages += el.numberOfPages;
            });
            result.courseUnitPages = totalPages;
            result.isAllowed = isAllowed;
            let sessions = await this.courseSessionTableService.findAllCourseByCourseId(result.courseId);
            result.CourseSessionTable = sessions;
            let index = 0;
            let previousUnitId = 0;
            for (const unit of result.CourseUnitTable) {
                let progression = await this.courseProgressionTableService.getProgressionReport(unit.courseUnitId, user);
                result.CourseUnitTable[index]['CourseProgressionTable'] = progression;
                let isAllowedUnit = false;
                if (previousUnitId == 0) {
                    isAllowedUnit = true;
                }
                else {
                    let previousProgression = await this.courseProgressionTableService.getProgressionReport(previousUnitId, user);
                    if (previousProgression && previousProgression.assesmentStatus == 1) {
                        isAllowedUnit = true;
                    }
                }
                result.CourseUnitTable[index]['isAllowedUnit'] = isAllowedUnit;
                previousUnitId = unit.courseUnitId;
                index++;
            }
        }
        return result;
    }
    async findOneRestrictedNew(courseId, user) {
        let isAllowed = await this.subscriptionTableService.checkIfAllowed(courseId, user.id);
        let subscriptionDetails = await this.subscriptionTableService.getDetails(courseId, user.id);
        let result = await this.courseTableRepository.findOne({
            where: {
                courseId
            },
            relations: {
                AssessmentMasterTable: true
            }
        });
        if (result) {
            let CourseUnitTable = await this.courseUnitTableService.findOneCourseUnitByCourseId(result.courseId);
            result.CourseUnitTable = CourseUnitTable;
            let url = await this.fileTableService.getUrlOfFileId(result.courseBanner);
            result.courseBannerUrl = url;
            result.courseUnitCount = result.CourseUnitTable.length;
            let totalPages = 0;
            result.CourseUnitTable.map(el => {
                totalPages += el.numberOfPages;
            });
            result.courseUnitPages = totalPages;
            result.isAllowed = isAllowed;
            let sessions = await this.courseSessionTableService.findAllCourseByCourseId(result.courseId);
            result.CourseSessionTable = sessions;
            let index = 0;
            let previousUnitId = 0;
            for (const unit of result.CourseUnitTable) {
                let progression = await this.courseProgressionTableService.getProgressionReport(unit.courseUnitId, user);
                result.CourseUnitTable[index]['CourseProgressionTable'] = progression;
                let isAllowedUnit = false;
                if (subscriptionDetails) {
                    if (subscriptionDetails.currentWeek >= unit.week) {
                        isAllowedUnit = true;
                    }
                }
                result.CourseUnitTable[index]['isAllowedUnit'] = isAllowedUnit;
                previousUnitId = unit.courseUnitId;
                index++;
            }
        }
        return result;
    }
    async updateCourseBanner(courseCode, fileId) {
        let result = await this.courseTableRepository.findOne({
            where: {
                courseCode
            }
        });
        if (result) {
            result.courseBanner = fileId;
            await this.courseTableRepository.save(result);
            return true;
        }
        else {
            return false;
        }
    }
    async update(courseId, updateCourseTableInput) {
        let result = await this.findOne(courseId);
        if (result) {
            result.courseName = updateCourseTableInput.courseName;
            result.courseCode = updateCourseTableInput.courseCode;
            result.courseDescription = updateCourseTableInput.courseDescription;
            result.courseDuration = updateCourseTableInput.courseDuration;
            result.courseType = updateCourseTableInput.courseType;
            result.hasAssesment = updateCourseTableInput.hasAssesment;
            result.isActive = updateCourseTableInput.isActive;
            result.UserTable = await this.userCollectionService.getOneByUserId(updateCourseTableInput.userId);
            let updatedResult = await this.courseTableRepository.save(result);
            this.pubSub.publish(exports.NEW_COURSE_ADD, 1);
            return {
                status: true,
                status_text: "Successfully Updated",
                data: updatedResult
            };
        }
        else {
            return {
                status: false,
                status_text: "Not Found"
            };
        }
    }
    async remove(courseId) {
        let result = await this.findOne(courseId);
        if (result) {
            await this.courseTableRepository.remove(result);
            this.pubSub.publish(exports.NEW_COURSE_ADD, 1);
            return {
                status: true,
                status_text: "Successfully Deleted"
            };
        }
        else {
            return {
                status: false,
                status_text: "Not Found"
            };
        }
    }
    async toggleStatusofCourse(courseId) {
        let res = await this.courseTableRepository.findOne({
            where: {
                courseId
            }
        });
        if (res) {
            let oldStatus = res.isActive;
            let newStatus = "YES";
            if (oldStatus == "YES") {
                newStatus = "NO";
            }
            if (oldStatus == "NO") {
                newStatus = "YES";
            }
            res.isActive = newStatus;
            let result = await this.courseTableRepository.save(res);
            return {
                status: true,
                status_text: "Successfully Updated",
                data: result
            };
        }
        else {
            return {
                status: false,
                status_text: "Not found"
            };
        }
    }
    async toggleHasAssesmentofCourse(courseId) {
        let res = await this.courseTableRepository.findOne({
            where: {
                courseId
            }
        });
        if (res) {
            let oldStatus = res.hasAssesment;
            let newStatus = "YES";
            if (oldStatus == "YES") {
                newStatus = "NO";
            }
            if (oldStatus == "NO") {
                newStatus = "YES";
            }
            res.hasAssesment = newStatus;
            let result = await this.courseTableRepository.save(res);
            return {
                status: true,
                status_text: "Successfully Updated",
                data: result
            };
        }
        else {
            return {
                status: false,
                status_text: "Not found"
            };
        }
    }
};
exports.CourseTableService = CourseTableService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(course_table_entity_1.CourseTable)),
    __param(1, (0, common_1.Inject)(GlobalConstants_1.PUB_SUB)),
    __param(2, (0, common_1.Inject)((0, common_1.forwardRef)(() => file_table_service_1.FileTableService))),
    __param(3, (0, common_1.Inject)((0, common_1.forwardRef)(() => subscription_table_service_1.SubscriptionTableService))),
    __param(4, (0, common_1.Inject)((0, common_1.forwardRef)(() => course_unit_table_service_1.CourseUnitTableService))),
    __param(5, (0, common_1.Inject)((0, common_1.forwardRef)(() => course_progression_table_service_1.CourseProgressionTableService))),
    __param(6, (0, common_1.Inject)((0, common_1.forwardRef)(() => user_collection_service_1.UserCollectionService))),
    __param(7, (0, common_1.Inject)((0, common_1.forwardRef)(() => course_session_table_service_1.CourseSessionTableService))),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        graphql_redis_subscriptions_1.RedisPubSub,
        file_table_service_1.FileTableService,
        subscription_table_service_1.SubscriptionTableService,
        course_unit_table_service_1.CourseUnitTableService,
        course_progression_table_service_1.CourseProgressionTableService,
        user_collection_service_1.UserCollectionService,
        course_session_table_service_1.CourseSessionTableService])
], CourseTableService);
