"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CourseUnitTableService = exports.NEW_COURSE_UNIT_ADD = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const course_unit_table_entity_1 = require("./entities/course-unit-table.entity");
const typeorm_2 = require("typeorm");
const GlobalConstants_1 = require("../helper/GlobalConstants");
const graphql_redis_subscriptions_1 = require("graphql-redis-subscriptions");
const course_table_service_1 = require("../course-table/course-table.service");
const file_table_service_1 = require("../file-table/file-table.service");
exports.NEW_COURSE_UNIT_ADD = "NEW_COURSE_UNIT_ADD";
let CourseUnitTableService = exports.CourseUnitTableService = class CourseUnitTableService {
    constructor(courseUnitTableRepository, pubSub, courseTableService, fileTableService) {
        this.courseUnitTableRepository = courseUnitTableRepository;
        this.pubSub = pubSub;
        this.courseTableService = courseTableService;
        this.fileTableService = fileTableService;
    }
    async create(createCourseUnitTableInput) {
        let checkIfAlreadyExists = await this.checkIfAlreadyExists(createCourseUnitTableInput);
        if (checkIfAlreadyExists == 0) {
            let st = new course_unit_table_entity_1.CourseUnitTable();
            let courseTable = await this.courseTableService.findOne(createCourseUnitTableInput.courseId);
            st.CourseTable = courseTable;
            st.week = createCourseUnitTableInput.week;
            st.courseUnitName = createCourseUnitTableInput.courseUnitName;
            st.courseUnitDescription = createCourseUnitTableInput.courseUnitDescription;
            st.courseUnitDuration = createCourseUnitTableInput.courseUnitDuration;
            st.courseUnitBanner = createCourseUnitTableInput.courseUnitBanner;
            st.isLink = createCourseUnitTableInput.isLink;
            st.link = createCourseUnitTableInput.link;
            let result = await this.courseUnitTableRepository.save(st);
            this.pubSub.publish(exports.NEW_COURSE_UNIT_ADD, 1);
            return {
                status: true,
                status_text: "Successfully Added",
                data: result
            };
        }
        else {
            return {
                status: false,
                status_text: "Already Exists"
            };
        }
    }
    async checkIfAlreadyExists(createCourseUnitTableInput) {
        let result = await this.courseUnitTableRepository.count({
            where: {
                courseUnitName: createCourseUnitTableInput.courseUnitName,
                CourseTable: {
                    courseId: createCourseUnitTableInput.courseId
                },
                week: createCourseUnitTableInput.week
            }
        });
        return result;
    }
    async findAll() {
        return await this.courseUnitTableRepository.find();
    }
    async findOne(courseUnitId) {
        let result = await this.courseUnitTableRepository.findOne({
            where: {
                courseUnitId
            },
            relations: {
                CourseTable: true
            }
        });
        if (result) {
            let bannerUrl = await this.fileTableService.getUrlOfFileId(result.courseUnitBanner);
            result.courseUnitBannerUrl = bannerUrl;
        }
        return result;
    }
    async findOneCourseUnitByCourseId(courseId) {
        let result = await this.courseUnitTableRepository.find({
            where: {
                CourseTable: {
                    courseId
                }
            },
            order: {
                week: "ASC"
            }
        });
        let index = 0;
        for (const r of result) {
            let bannerurl = await this.fileTableService.getUrlOfFileId(r.courseUnitBanner);
            result[index]['courseUnitBannerUrl'] = bannerurl;
            index++;
        }
        return result;
    }
    async update(courseUnitId, updateCourseUnitTableInput) {
        let result = await this.findOne(courseUnitId);
        if (result) {
            let courseTable = await this.courseTableService.findOne(updateCourseUnitTableInput.courseId);
            result.CourseTable = courseTable;
            result.courseUnitName = updateCourseUnitTableInput.courseUnitName;
            result.courseUnitDescription = updateCourseUnitTableInput.courseUnitDescription;
            result.courseUnitDuration = updateCourseUnitTableInput.courseUnitDuration;
            result.courseUnitBanner = updateCourseUnitTableInput.courseUnitBanner;
            result.week = updateCourseUnitTableInput.week;
            result.isLink = updateCourseUnitTableInput.isLink;
            result.link = updateCourseUnitTableInput.link;
            let updatedResult = await this.courseUnitTableRepository.save(result);
            this.pubSub.publish(exports.NEW_COURSE_UNIT_ADD, 1);
            return {
                status: true,
                status_text: "Successfully Updated",
                data: updatedResult
            };
        }
        else {
            return {
                status: false,
                status_text: "Not Found"
            };
        }
    }
    async remove(courseUnitId) {
        let result = await this.findOne(courseUnitId);
        if (result) {
            await this.courseUnitTableRepository.remove(result);
            this.pubSub.publish(exports.NEW_COURSE_UNIT_ADD, 1);
            return {
                status: true,
                status_text: "Successfully Deleted"
            };
        }
        else {
            return {
                status: false,
                status_text: "Not Found"
            };
        }
    }
    async toggleHasAssesmentofCourseUnit(courseUnitId) {
        let res = await this.courseUnitTableRepository.findOne({
            where: {
                courseUnitId
            }
        });
        if (res) {
            let oldStatus = res.hasAssesment;
            let newStatus = "YES";
            if (oldStatus == "YES") {
                newStatus = "NO";
            }
            if (oldStatus == "NO") {
                newStatus = "YES";
            }
            res.hasAssesment = newStatus;
            let result = await this.courseUnitTableRepository.save(res);
            return {
                status: true,
                status_text: "Successfully Updated",
                data: result
            };
        }
        else {
            return {
                status: false,
                status_text: "Not found"
            };
        }
    }
};
exports.CourseUnitTableService = CourseUnitTableService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(course_unit_table_entity_1.CourseUnitTable)),
    __param(1, (0, common_1.Inject)(GlobalConstants_1.PUB_SUB)),
    __param(2, (0, common_1.Inject)((0, common_1.forwardRef)(() => course_table_service_1.CourseTableService))),
    __param(3, (0, common_1.Inject)((0, common_1.forwardRef)(() => file_table_service_1.FileTableService))),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        graphql_redis_subscriptions_1.RedisPubSub,
        course_table_service_1.CourseTableService,
        file_table_service_1.FileTableService])
], CourseUnitTableService);
