"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CurrentStockCollectionService = void 0;
const common_1 = require("@nestjs/common");
const mongoose_1 = require("@nestjs/mongoose");
const graphql_redis_subscriptions_1 = require("graphql-redis-subscriptions");
const mongoose_2 = require("mongoose");
const GlobalConstants_1 = require("../helper/GlobalConstants");
const product_collection_service_1 = require("../product-collection/product-collection.service");
const current_stock_collection_entity_1 = require("./entities/current-stock-collection.entity");
const user_collection_service_1 = require("../user-collection/user-collection.service");
const client_collection_service_1 = require("../client-collection/client-collection.service");
let CurrentStockCollectionService = class CurrentStockCollectionService {
    constructor(currentStockCollectionRepository, productService, clientCollectionService, userCollectionService, pubSub) {
        this.currentStockCollectionRepository = currentStockCollectionRepository;
        this.productService = productService;
        this.clientCollectionService = clientCollectionService;
        this.userCollectionService = userCollectionService;
        this.pubSub = pubSub;
    }
    async updateCurrentStock(productId, size, qty, basePrice, mrp, barcode, mfgDate, expiryDate, batchNo, factor) {
        let pc = await this.productService.findOneById(productId);
        let checkIfExists = await this.checkIfExists(pc, size, basePrice, expiryDate, batchNo);
        if (checkIfExists == null || checkIfExists == undefined) {
            let cm = new current_stock_collection_entity_1.CurrentStockCollection();
            cm.qty = qty * factor;
            cm.basePrice = basePrice;
            cm.size = size;
            cm.mrp = mrp;
            cm.batchNo = batchNo;
            cm.mfgDate = mfgDate;
            cm.expiryDate = expiryDate;
            cm.Product = pc;
            let index = await this.getLatestIndex();
            cm.itemCodeIndex = index;
            let itemCode = this.padWithZero(index, 5);
            cm.itemCode = itemCode;
            if (barcode == "") {
                cm.barcode = itemCode;
            }
            else {
                cm.barcode = barcode;
            }
            let c = await this.currentStockCollectionRepository.create(cm);
            return {
                status: true,
                status_text: "Successfully Added",
                id: c._id,
            };
        }
        else {
            checkIfExists.qty = checkIfExists.qty + qty * factor;
            let cc1 = await this.currentStockCollectionRepository.updateOne({ _id: checkIfExists.id }, checkIfExists);
            return {
                status: true,
                status_text: "Successfully Added",
                id: checkIfExists.id,
            };
        }
        return null;
    }
    padWithZero(num, targetLength) {
        return String(num).padStart(targetLength, "0");
    }
    async getLatestIndex() {
        let r = await this.currentStockCollectionRepository
            .find()
            .sort({ itemCodeIndex: -1 })
            .limit(1);
        if (r.length == 0) {
            return 1;
        }
        else {
            return r[0].itemCodeIndex + 1;
        }
    }
    async checkIfExists(pc, size, basePrice, expiryDate, batchNo) {
        let result = await this.currentStockCollectionRepository
            .findOne({
            Product: pc,
            size: size,
            basePrice: basePrice,
            batchNo: batchNo,
            expiryDate: expiryDate,
        })
            .exec();
        return result;
    }
    async getStockWithBarcode(barcode, clientId) {
        if (clientId != null) {
            let Client = await this.clientCollectionService.findOneById(clientId);
            let result = await this.currentStockCollectionRepository
                .findOne({
                barcode: barcode,
            })
                .populate("Product")
                .sort({
                _id: 1,
            })
                .exec();
            if (result) {
                result.price = await this.priceCalculator(Client.discountPercentage, Client.markupPercentage, result.Product.pricingMethod, result.basePrice, result.mrp, result.Product.taxRate);
                return result;
            }
            else {
                return null;
            }
        }
        return null;
    }
    async getStockWithPriceSpecific(pc, size, qty, username) {
        let user = await this.userCollectionService.getUserDetailsWithUsername(username);
        let result = await this.currentStockCollectionRepository
            .findOne({
            Product: pc,
            size: size,
        })
            .sort({
            _id: 1,
        })
            .exec();
        if (result) {
            return await this.priceCalculator(user.Client.discountPercentage, user.ClientmarkupPercentage, pc.pricingMethod, result.basePrice, result.mrp, pc.taxRate);
        }
        else {
            return null;
        }
    }
    async getStockWithPrice(pc, discountPercentage, markupPercentage) {
        let result = await this.currentStockCollectionRepository
            .find({
            Product: pc,
        })
            .exec();
        await Promise.all(result.map(async (el, index) => {
            result[index]["price"] = await this.priceCalculator(discountPercentage, markupPercentage, pc.pricingMethod, el.basePrice, el.mrp, pc.taxRate);
        }));
        return result;
    }
    async priceCalculator(discountPercentage, markupPercentage, pricingMethod, basePrice, mrp, taxRate) {
        if (pricingMethod == "DISCOUNT_METHOD") {
            let newP = parseFloat(mrp) -
                parseFloat(mrp) * (parseFloat(discountPercentage) / 100);
            let newPt = newP + newP * (parseFloat(taxRate) / 100);
            if (newPt >= parseFloat(mrp)) {
                return mrp;
            }
            else {
                return newPt.toFixed(2);
            }
        }
        if (pricingMethod == "MARKUP_METHOD") {
            let newP = parseFloat(basePrice) +
                (parseFloat(basePrice) + parseFloat(markupPercentage) / 100);
            let newPt = newP + newP * (parseFloat(taxRate) / 100);
            if (newPt >= parseFloat(mrp)) {
                return mrp;
            }
            else {
                return newPt.toFixed(2);
            }
        }
    }
    async findAll() {
        let result = await this.currentStockCollectionRepository
            .find()
            .populate("Product")
            .exec();
        return result;
    }
    async findOneById(id) {
        return await this.currentStockCollectionRepository
            .findOne({ _id: id })
            .populate("Product")
            .exec();
    }
    async remove(id) {
        await this.currentStockCollectionRepository.deleteOne({ _id: id });
        return {
            status: true,
            status_text: "Successfully deleted",
            data: null,
        };
    }
    async update(id, updateCurrentStockCollectionInput) {
        let cc = await this.currentStockCollectionRepository
            .findOne({ _id: id })
            .exec();
        if (cc) {
            let pc = await this.productService.findOneById(updateCurrentStockCollectionInput.productId);
            cc.Product = pc;
            cc.basePrice = updateCurrentStockCollectionInput.basePrice;
            cc.qty = updateCurrentStockCollectionInput.qty;
            cc.size = updateCurrentStockCollectionInput.size;
            cc.mrp = updateCurrentStockCollectionInput.mrp;
            cc.batchNo = updateCurrentStockCollectionInput.batchNo;
            cc.expiryDate = updateCurrentStockCollectionInput.expiryDate;
            cc.mfgDate = updateCurrentStockCollectionInput.mfgDate;
            let cc1 = await this.currentStockCollectionRepository.updateOne({ _id: id }, cc);
            return {
                status: true,
                status_text: "Successfully Updated",
                data: await this.findOneById(id),
            };
        }
        else {
            return {
                status: false,
                status_text: "Not found",
                data: null,
            };
        }
    }
    async updateStock(id, qty) {
        let cc = await this.currentStockCollectionRepository
            .findOne({ _id: id })
            .exec();
        if (cc) {
            if (qty < 0 && cc.qty < -qty) {
                return {
                    status: false,
                    status_text: "Insufficient stock",
                    data: null,
                };
            }
            else {
                let newQty = cc.qty + qty;
                let cc1 = await this.currentStockCollectionRepository
                    .updateOne({ _id: id }, { $set: { qty: newQty } })
                    .exec();
                return {
                    status: true,
                    status_text: "Successfully Updated",
                    data: await this.findOneById(id),
                };
            }
        }
        else {
            return {
                status: false,
                status_text: "Not found",
                data: null,
            };
        }
    }
};
CurrentStockCollectionService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, mongoose_1.InjectModel)(current_stock_collection_entity_1.CurrentStockCollection.name)),
    __param(1, (0, common_1.Inject)((0, common_1.forwardRef)(() => product_collection_service_1.ProductCollectionService))),
    __param(2, (0, common_1.Inject)((0, common_1.forwardRef)(() => client_collection_service_1.ClientCollectionService))),
    __param(3, (0, common_1.Inject)((0, common_1.forwardRef)(() => user_collection_service_1.UserCollectionService))),
    __param(4, (0, common_1.Inject)(GlobalConstants_1.PUB_SUB)),
    __metadata("design:paramtypes", [mongoose_2.Model,
        product_collection_service_1.ProductCollectionService,
        client_collection_service_1.ClientCollectionService,
        user_collection_service_1.UserCollectionService,
        graphql_redis_subscriptions_1.RedisPubSub])
], CurrentStockCollectionService);
exports.CurrentStockCollectionService = CurrentStockCollectionService;
