"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CurrentStockTableService = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const typeorm_2 = require("typeorm");
const moment_1 = __importDefault(require("moment"));
const housing_table_service_1 = require("../housing-table/housing-table.service");
const breed_table_service_1 = require("../breed-table/breed-table.service");
const current_stock_table_entity_1 = require("./entities/current-stock-table.entity");
const sub_division_table_service_1 = require("../sub-division-table/sub-division-table.service");
const farmers_table_entity_1 = require("../farmers-table/entities/farmers-table.entity");
const housing_table_entity_1 = require("../housing-table/entities/housing-table.entity");
const district_table_service_1 = require("../district-table/district-table.service");
const breed_table_entity_1 = require("../breed-table/entities/breed-table.entity");
const access_table_service_1 = require("../access-table/access-table.service");
let CurrentStockTableService = class CurrentStockTableService {
    constructor(currentStockTableRepository, housingTableService, breedTableService, accessTableService, districtTableService, subDivisionTableService) {
        this.currentStockTableRepository = currentStockTableRepository;
        this.housingTableService = housingTableService;
        this.breedTableService = breedTableService;
        this.accessTableService = accessTableService;
        this.districtTableService = districtTableService;
        this.subDivisionTableService = subDivisionTableService;
    }
    async create(createCurrentStockTableInput) {
        let checkIfExists = await this.checkIfExists(createCurrentStockTableInput);
        if (checkIfExists) {
            checkIfExists.qtyMale += createCurrentStockTableInput.qtyMale;
            checkIfExists.qtyFemale += createCurrentStockTableInput.qtyFemale;
            checkIfExists.qtyTotal += createCurrentStockTableInput.qtyTotal;
            let totalWeight = parseFloat(checkIfExists.totalWeight.toString()) + createCurrentStockTableInput.totalWeight;
            checkIfExists.totalWeight = totalWeight;
            await this.currentStockTableRepository.save(checkIfExists);
            return {
                status: true,
                status_text: "Successfully Submitted",
                data: checkIfExists
            };
        }
        else {
            let ht = new current_stock_table_entity_1.CurrentStockTable();
            let housingTable = await this.housingTableService.findOne(createCurrentStockTableInput.housingId);
            let breedTable = await this.breedTableService.findOne(createCurrentStockTableInput.breedId);
            ht.HousingTable = housingTable;
            ht.BreedTable = breedTable;
            ht.ageGroup = createCurrentStockTableInput.ageGroup;
            ht.qtyMale = createCurrentStockTableInput.qtyMale;
            ht.qtyFemale = createCurrentStockTableInput.qtyFemale;
            ht.qtyTotal = createCurrentStockTableInput.qtyTotal;
            ht.totalWeight = createCurrentStockTableInput.totalWeight;
            ht.dateOfAgeUpdate = (0, moment_1.default)().format('YYYY-MM-DD');
            let result = await this.currentStockTableRepository.save(ht);
            return {
                status: true,
                status_text: "Successfully Submitted",
                data: result,
            };
        }
    }
    async checkIfExists(createCurrentStockTableInput) {
        let result = await this.currentStockTableRepository.find({
            where: {
                HousingTable: {
                    housingId: createCurrentStockTableInput.housingId
                },
                BreedTable: {
                    breedId: createCurrentStockTableInput.breedId
                },
                ageGroup: createCurrentStockTableInput.ageGroup
            }
        });
        if (!result) {
            return false;
        }
        else {
            return result[0];
        }
    }
    async findByHousingId(housingId) {
        return await this.currentStockTableRepository.find({
            where: {
                HousingTable: {
                    housingId
                }
            },
            relations: ["HousingTable.FarmersTable", "BreedTable"]
        });
    }
    async findOne(currentStockId) {
        return await this.currentStockTableRepository.findOne({
            where: {
                stockId: currentStockId
            },
            relations: ["HousingTable", "HousingTable.FarmersTable", "BreedTable"]
        });
    }
    async findAll() {
        return await this.currentStockTableRepository.find({
            relations: ["BreedTable", "HousingTable", "HousingTable.FarmersTable", "AnimalTable"]
        });
    }
    async getForDiseaseAndSale(housingId) {
        return await this.currentStockTableRepository.find({
            relations: ["BreedTable", "HousingTable", "HousingTable.FarmersTable"],
            where: {
                qtyTotal: (0, typeorm_2.MoreThan)(0),
                HousingTable: {
                    housingId: housingId
                }
            }
        });
    }
    async getAllCount(farmerId) {
        let result = await this.currentStockTableRepository.createQueryBuilder("r");
        if (farmerId) {
            result = await result.leftJoinAndMapOne("r.HousingTable", housing_table_entity_1.HousingTable, "ml1", "ml1.housingId=r.housingId");
            result = await result.where("ml1.farmerId=:farmerId", { farmerId: farmerId });
        }
        result = await result.select('SUM(r.qtyMale) as maleCount').addSelect('SUM(r.qtyFemale) as femaleCount');
        result = await result.getRawOne();
        if (result) {
            return result;
        }
        else {
            return {
                maleCount: 0,
                femaleCount: 0
            };
        }
    }
    async getAllCountByDistAndDiv(districtId, subDivisionId) {
        let result = await this.currentStockTableRepository.createQueryBuilder("r");
        result = await result.leftJoinAndMapOne("r.HousingTable", housing_table_entity_1.HousingTable, "ml1", "ml1.housingId=r.housingId");
        result = await result.leftJoinAndMapOne("ml1.FarmersTable", farmers_table_entity_1.FarmersTable, "ml2", "ml2.farmerId=ml1.farmerId");
        if (subDivisionId) {
            result = await result.where("ml2.subDivisionId=:subDivisionId", { subDivisionId: subDivisionId });
        }
        else {
            result = await result.where("ml2.districtId=:districtId", { districtId: districtId });
        }
        result = await result.select('SUM(r.qtyMale) as maleCount').addSelect('SUM(r.qtyFemale) as femaleCount');
        result = await result.getRawOne();
        if (result) {
            return result;
        }
        else {
            return {
                maleCount: 0,
                femaleCount: 0
            };
        }
    }
    async getAgeWiseCount(farmerId) {
        let pigletCount = 0;
        let nurseryCount = 0;
        let growerCount = 0;
        let finisherCount = 0;
        let result = await this.currentStockTableRepository.createQueryBuilder("r");
        if (farmerId) {
            result = await result.leftJoinAndMapOne("r.HousingTable", housing_table_entity_1.HousingTable, "ml1", "ml1.housingId=r.housingId");
            result = await result.where("ml1.farmerId=:farmerId", { farmerId: farmerId });
        }
        result = await result.getMany();
        if (result.length > 0) {
            result.forEach((el) => {
                let ag = parseFloat(el.ageGroup) / 4;
                if (ag >= 0 && ag < 3) {
                    pigletCount = pigletCount + el.qtyTotal;
                }
                if (ag >= 3 && ag < 8) {
                    nurseryCount = nurseryCount + el.qtyTotal;
                }
                if (ag >= 8 && ag < 16) {
                    growerCount = growerCount + el.qtyTotal;
                }
                if (ag >= 16) {
                    finisherCount = finisherCount + el.qtyTotal;
                }
            });
        }
        return {
            pigletCount,
            nurseryCount,
            growerCount,
            finisherCount
        };
    }
    async getAgeWiseCountBYDistAndDiv(districtId, subDivisionId) {
        let pigletCount = 0;
        let nurseryCount = 0;
        let growerCount = 0;
        let finisherCount = 0;
        let result = await this.currentStockTableRepository.createQueryBuilder("r");
        result = await result.leftJoinAndMapOne("r.HousingTable", housing_table_entity_1.HousingTable, "ml1", "ml1.housingId=r.housingId");
        result = await result.leftJoinAndMapOne("ml1.FarmersTable", farmers_table_entity_1.FarmersTable, "ml2", "ml2.farmerId=ml1.farmerId");
        if (subDivisionId) {
            result = await result.where("ml2.subDivisionId=:subDivisionId", { subDivisionId: subDivisionId });
        }
        else {
            result = await result.where("ml2.districtId=:districtId", { districtId: districtId });
        }
        result = await result.getMany();
        if (result.length > 0) {
            result.forEach((el) => {
                let ag = parseFloat(el.ageGroup) / 4;
                if (ag >= 0 && ag < 3) {
                    pigletCount = pigletCount + el.qtyTotal;
                }
                if (ag >= 3 && ag < 8) {
                    nurseryCount = nurseryCount + el.qtyTotal;
                }
                if (ag >= 8 && ag < 16) {
                    growerCount = growerCount + el.qtyTotal;
                }
                if (ag >= 16) {
                    finisherCount = finisherCount + el.qtyTotal;
                }
            });
        }
        return {
            pigletCount,
            nurseryCount,
            growerCount,
            finisherCount
        };
    }
    async ageUpdate() {
        let result = await this.currentStockTableRepository.find();
        for (const el of result) {
            let current = (0, moment_1.default)((0, moment_1.default)().format('YYYY-MM-DD'));
            let prev = (0, moment_1.default)(el.dateOfAgeUpdate);
            let days = current.diff(prev, "days");
            let weeks = parseFloat(el.ageGroup) + parseFloat((days / 30).toFixed(2));
            el.ageGroup = weeks.toString();
            el.dateOfAgeUpdate = (0, moment_1.default)().format('YYYY-MM-DD');
            await this.currentStockTableRepository.save(el);
        }
        return {
            status: true,
            status_text: 'Successfully Updated'
        };
    }
    async fetchDistrictWiseTotalStock() {
        const districts = await this.districtTableService.findAll();
        let data = [];
        for (const d of districts) {
            let d3 = await this.currentStockTableRepository.createQueryBuilder("r")
                .leftJoinAndMapOne("r.HousingTable", housing_table_entity_1.HousingTable, "ml1", "ml1.housingId=r.housingId")
                .leftJoinAndMapOne("ml1.FarmersTable", farmers_table_entity_1.FarmersTable, "ml2", "ml1.farmerId=ml2.farmerId")
                .where("ml2.districtId=:districtId", { districtId: d.districtId })
                .select("SUM(r.qtyMale)", "totalMale")
                .addSelect("SUM(r.qtyFemale)", "totalFemale")
                .addSelect("SUM(r.qtyTotal)", "qtyTotal")
                .addSelect("SUM(r.totalWeight)", "totalWeight")
                .groupBy('ml2.districtId')
                .getRawOne();
            let d1 = {
                DistrictTable: d,
                totalMale: d3 ? d3.totalMale : 0,
                totalFemale: d3 ? d3.totalFemale : 0,
                qtyTotal: d3 ? d3.qtyTotal : 0,
                totalWeight: d3 ? d3.totalWeight : 0
            };
            data.push(d1);
        }
        return data;
    }
    async countAllStockDistrictWise() {
        const districts = await this.districtTableService.findAll();
        let data = [];
        for (const d of districts) {
            let count = await this.currentStockTableRepository.createQueryBuilder("r")
                .leftJoinAndMapOne("r.HousingTable", housing_table_entity_1.HousingTable, "ml1", "ml1.housingId=r.housingId")
                .leftJoinAndMapOne("ml1.FarmersTable", farmers_table_entity_1.FarmersTable, "ml2", "ml1.farmerId=ml2.farmerId")
                .andWhere("ml2.districtId=:districtId", { districtId: d.districtId })
                .select("SUM(r.qtyMale)", "qtyMale")
                .addSelect("SUM(r.qtyFemale)", "qtyFemale")
                .addSelect("SUM(r.qtyTotal)", "qtyTotal")
                .addSelect("SUM(r.totalWeight)", "totalWeight")
                .groupBy('ml2.districtId')
                .getRawOne();
            let d1 = {
                DistrictTable: d,
                qtyMale: count ? count.qtyMale : 0,
                qtyFemale: count ? count.qtyFemale : 0,
                qtyTotal: count ? count.qtyTotal : 0,
                totalWeight: count ? count.totalWeight : 0,
            };
            data.push(d1);
        }
        return data;
    }
    async countAllStockAgeWise() {
        const breeds = await this.breedTableService.findAll();
        let data = [];
        for (const d of breeds) {
            let count = await this.currentStockTableRepository.createQueryBuilder("r")
                .leftJoinAndMapOne("r.BreedTable", breed_table_entity_1.BreedTable, "ml1", "ml1.breedId=r.breedId")
                .andWhere("ml1.breedId=:breedId", { breedId: d.breedId })
                .select("SUM(r.qtyMale)", "qtyMale")
                .addSelect("SUM(r.qtyFemale)", "qtyFemale")
                .addSelect("SUM(r.qtyTotal)", "qtyTotal")
                .addSelect("SUM(r.totalWeight)", "totalWeight")
                .groupBy('ml1.breedId')
                .getRawOne();
            let d1 = {
                BreedTable: d,
                ageGroup: count ? count.ageGroup : "",
                qtyMale: count ? count.qtyMale : 0,
                qtyFemale: count ? count.qtyFemale : 0,
                qtyTotal: count ? count.qtyTotal : 0,
                totalWeight: count ? count.totalWeight : 0,
            };
            data.push(d1);
        }
        return data;
    }
    async countAllStockClusterWise(username) {
        const access = await this.accessTableService.findAllByUserName(username);
        if (access.DistrictTable) {
            const subdivisions = await this.subDivisionTableService.fetchSubDivisionByDistrictId(access.DistrictTable.districtId);
            let data = [];
            for (const d of subdivisions) {
                let d1 = await this.currentStockTableRepository.createQueryBuilder("r")
                    .leftJoinAndMapOne("r.HousingTable", housing_table_entity_1.HousingTable, "ml1", "ml1.housingId=r.housingId")
                    .leftJoinAndMapOne("ml1.FarmersTable", farmers_table_entity_1.FarmersTable, "ml2", "ml2.farmerId=ml1.farmerId")
                    .where("ml2.subDivisionId=:subDivisionId", { subDivisionId: d.subDivisionId })
                    .select("SUM(r.qtyMale)", "qtyMale")
                    .addSelect("SUM(r.qtyFemale)", "qtyFemale")
                    .addSelect("SUM(r.qtyTotal)", "qtyTotal")
                    .addSelect("SUM(r.totalWeight)", "totalWeight")
                    .groupBy('ml2.subDivisionId')
                    .getRawOne();
                let d2 = {
                    SubDivisionTable: d,
                    qtyMale: d1 ? d1.qtyMale : 0,
                    qtyFemale: d1 ? d1.qtyFemale : 0,
                    qtyTotal: d1 ? d1.qtyTotal : 0,
                    totalWeight: d1 ? d1.totalWeight : 0,
                };
                data.push(d2);
            }
            return data;
        }
        else {
            return null;
        }
    }
    async countAllStockBreedWise() {
        const breeds = await this.breedTableService.findAll();
        let data = [];
        for (const d of breeds) {
            let count = await this.currentStockTableRepository.createQueryBuilder("r")
                .leftJoinAndMapOne("r.BreedTable", breed_table_entity_1.BreedTable, "ml1", "ml1.breedId=r.breedId")
                .andWhere("ml1.breedId=:breedId", { breedId: d.breedId })
                .select("SUM(r.qtyMale)", "qtyMale")
                .addSelect("SUM(r.qtyFemale)", "qtyFemale")
                .addSelect("SUM(r.qtyTotal)", "qtyTotal")
                .addSelect("SUM(r.totalWeight)", "totalWeight")
                .groupBy('ml1.breedId')
                .getRawOne();
            let d1 = {
                BreedTable: d,
                qtyMale: count ? count.qtyMale : 0,
                qtyFemale: count ? count.qtyFemale : 0,
                qtyTotal: count ? count.qtyTotal : 0,
                totalWeight: count ? count.totalWeight : 0,
            };
            data.push(d1);
        }
        return data;
    }
};
CurrentStockTableService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(current_stock_table_entity_1.CurrentStockTable)),
    __param(1, (0, common_1.Inject)((0, common_1.forwardRef)(() => housing_table_service_1.HousingTableService))),
    __param(2, (0, common_1.Inject)((0, common_1.forwardRef)(() => breed_table_service_1.BreedTableService))),
    __param(3, (0, common_1.Inject)((0, common_1.forwardRef)(() => access_table_service_1.AccessTableService))),
    __param(5, (0, common_1.Inject)((0, common_1.forwardRef)(() => sub_division_table_service_1.SubDivisionTableService))),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        housing_table_service_1.HousingTableService,
        breed_table_service_1.BreedTableService,
        access_table_service_1.AccessTableService,
        district_table_service_1.DistrictTableService,
        sub_division_table_service_1.SubDivisionTableService])
], CurrentStockTableService);
exports.CurrentStockTableService = CurrentStockTableService;
