"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DistrictTableService = exports.NEW_DISTRICT_ADD = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const graphql_redis_subscriptions_1 = require("graphql-redis-subscriptions");
const GlobalConstants_1 = require("../helper/GlobalConstants");
const typeorm_2 = require("typeorm");
const district_table_entity_1 = require("./entities/district-table.entity");
const state_table_service_1 = require("../state-table/state-table.service");
exports.NEW_DISTRICT_ADD = "NEW_DISTRICT_ADD";
let DistrictTableService = exports.DistrictTableService = class DistrictTableService {
    constructor(districtTableRepository, pubSub, stateTableService) {
        this.districtTableRepository = districtTableRepository;
        this.pubSub = pubSub;
        this.stateTableService = stateTableService;
    }
    async create(createDistrictTableInput) {
        let checkIfExists = await this.checkIfExists(createDistrictTableInput.districtName, createDistrictTableInput.stateId);
        if (checkIfExists) {
            return {
                status: false,
                status_text: "Already Exists",
                data: null
            };
        }
        else {
            let dd = new district_table_entity_1.DistrictTable();
            dd.districtName = createDistrictTableInput.districtName;
            let state = await this.stateTableService.findOne(createDistrictTableInput.stateId);
            dd.StateTable = state;
            let result = await this.districtTableRepository.save(dd);
            this.pubSub.publish(exports.NEW_DISTRICT_ADD, 1);
            return {
                status: true,
                status_text: "Successfully Created",
                data: result
            };
        }
    }
    async createorElseReturnObject(createDistrictTableInput) {
        let checkIfAlreadyExists = await this.districtTableRepository.findOne({
            where: {
                districtName: createDistrictTableInput.districtName.trim(),
                StateTable: {
                    stateId: createDistrictTableInput.stateId
                }
            }
        });
        if (!checkIfAlreadyExists) {
            let dd = new district_table_entity_1.DistrictTable();
            dd.districtName = createDistrictTableInput.districtName.trim();
            let state = await this.stateTableService.findOne(createDistrictTableInput.stateId);
            dd.StateTable = state;
            let result = await this.districtTableRepository.save(dd);
            return result;
        }
        else {
            return checkIfAlreadyExists;
        }
    }
    async getCount() {
        return await this.districtTableRepository.count();
    }
    async getCountUnderAState(stateId) {
        let result = await this.districtTableRepository.find({
            where: {
                StateTable: {
                    stateId
                }
            }
        });
        return result.map((el) => el.districtId);
    }
    async checkIfExists(districtName, stateId) {
        let result = await this.districtTableRepository.count({
            where: {
                districtName: districtName,
                StateTable: {
                    stateId
                }
            }
        });
        if (result == 0) {
            return false;
        }
        else {
            return true;
        }
    }
    async findAll() {
        return await this.districtTableRepository.find({
            relations: {
                StateTable: true
            }
        });
    }
    async findAllByIds(ids) {
        return await this.districtTableRepository.find({
            where: {
                districtId: (0, typeorm_2.In)(ids)
            },
            relations: {
                StateTable: true
            }
        });
    }
    async findAllByStateId(stateId) {
        return await this.districtTableRepository.find({
            where: {
                StateTable: {
                    stateId
                }
            },
            relations: {
                StateTable: true
            }
        });
    }
    async findOne(districtId) {
        return await this.districtTableRepository.findOne({
            where: {
                districtId: districtId
            },
            relations: {
                StateTable: true
            }
        });
    }
    async update(districtId, updateDistrictTableInput) {
        let result = await this.districtTableRepository.findOne({
            where: {
                districtId: districtId
            }
        });
        if (result) {
            result.districtName = updateDistrictTableInput.districtName;
            let state = await this.stateTableService.findOne(updateDistrictTableInput.stateId);
            result.StateTable = state;
            let r = await this.districtTableRepository.save(result);
            this.pubSub.publish(exports.NEW_DISTRICT_ADD, 1);
            return {
                status: true,
                status_text: "Successfully Updated",
                data: r
            };
        }
        else {
            return {
                status: false,
                status_text: "Not found",
                data: null
            };
        }
    }
    async remove(districtId) {
        let result = await this.districtTableRepository.findOne({
            where: {
                districtId: districtId
            }
        });
        if (result) {
            await this.districtTableRepository.remove(result);
            this.pubSub.publish(exports.NEW_DISTRICT_ADD, 1);
            return {
                status: true,
                status_text: "Successfully Removed",
                data: null
            };
        }
        else {
            return {
                status: false,
                status_text: "Not Found",
                data: null
            };
        }
    }
};
exports.DistrictTableService = DistrictTableService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(district_table_entity_1.DistrictTable)),
    __param(1, (0, common_1.Inject)(GlobalConstants_1.PUB_SUB)),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        graphql_redis_subscriptions_1.RedisPubSub,
        state_table_service_1.StateTableService])
], DistrictTableService);
