"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FarmersTableService = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const access_table_service_1 = require("../access-table/access-table.service");
const current_stock_table_service_1 = require("../current-stock/current-stock-table.service");
const district_table_service_1 = require("../district-table/district-table.service");
const housing_table_service_1 = require("../housing-table/housing-table.service");
const sub_division_table_service_1 = require("../sub-division-table/sub-division-table.service");
const user_collection_service_1 = require("../user-collection/user-collection.service");
const typeorm_2 = require("typeorm");
const farmers_table_entity_1 = require("./entities/farmers-table.entity");
const data_entry_table_service_1 = require("../data-entry-table/data-entry-table.service");
const GlobalConstants_1 = require("../helper/GlobalConstants");
let FarmersTableService = class FarmersTableService {
    constructor(farmersTableRepository, districtTableService, subDivisionTableService, dataEntryTableService, userCollectionService, accessTableService, housingTableService, currentStockTableService) {
        this.farmersTableRepository = farmersTableRepository;
        this.districtTableService = districtTableService;
        this.subDivisionTableService = subDivisionTableService;
        this.dataEntryTableService = dataEntryTableService;
        this.userCollectionService = userCollectionService;
        this.accessTableService = accessTableService;
        this.housingTableService = housingTableService;
        this.currentStockTableService = currentStockTableService;
    }
    async create(createFarmersTableInput, user) {
        let checkIfExists = await this.checkIfExists(createFarmersTableInput);
        if (checkIfExists) {
            return {
                status: true,
                status_text: "Already Exists",
                data: null
            };
        }
        else {
            let userTable = await this.userCollectionService.getOneByUserId(user.userId);
            let district = await this.districtTableService.findOne(createFarmersTableInput.districtId);
            let subDivision = await this.subDivisionTableService.findOne(createFarmersTableInput.subDivisionId);
            let farmerCode = await this.getFarmerCode();
            console.log("🚀 ~ file: farmers-table.service.ts ~ line 44 ~ FarmersTableService ~ create ~ farmerCode", farmerCode);
            let ft = new farmers_table_entity_1.FarmersTable();
            ft.DistrictTable = district;
            ft.SubDivisionTable = subDivision;
            ft.farmerCodeIndex = farmerCode;
            ft.farmerCode = this.padWithZero(farmerCode, 6);
            ft.farmerVillage = createFarmersTableInput.farmerVillage;
            ft.farmerAadhar = createFarmersTableInput.farmerAadhar;
            ft.gender = createFarmersTableInput.gender;
            ft.caste = createFarmersTableInput.caste;
            ft.gramPanchayat = createFarmersTableInput.gramPanchayat;
            ft.circleOffice = createFarmersTableInput.circleOffice;
            ft.wardNo = createFarmersTableInput.wardNo;
            ft.villageDispensary = createFarmersTableInput.villageDispensary;
            ft.farmerState = createFarmersTableInput.farmerState;
            ft.farmerPostOffice = createFarmersTableInput.farmerPostOffice;
            ft.farmerName = createFarmersTableInput.farmerName;
            ft.farmerFatherName = createFarmersTableInput.farmerFatherName;
            ft.farmerPin = createFarmersTableInput.farmerPin;
            ft.farmerPhone = createFarmersTableInput.farmerPhone;
            ft.UserTable = userTable;
            let result = await this.farmersTableRepository.save(ft);
            this.dataEntryTableService.create({ type: GlobalConstants_1.CREATE, typeOfEntry: GlobalConstants_1.FARMER_ADD, remarks: "New farmer added" }, user);
            return {
                status: true,
                status_text: "Successfully Added",
                data: result
            };
        }
    }
    async countAllFarmers(farmerId) {
        let farmers = await this.farmersTableRepository.count();
        let { backyard, organized } = await this.housingTableService.countAllHousing(farmerId);
        return {
            farmers, backyard, organized
        };
    }
    async countAllFarmersByDistAndDiv(districtId, subDivisionId) {
        if (!subDivisionId) {
            let farmers = await this.farmersTableRepository.count({
                where: {
                    DistrictTable: {
                        districtId: districtId
                    }
                }
            });
            let { backyard, organized } = await this.housingTableService.countAllHousingByDistAndDiv(districtId, subDivisionId);
            return {
                farmers, backyard, organized
            };
        }
        else {
            let farmers = await this.farmersTableRepository.count({
                where: {
                    DistrictTable: {
                        districtId: districtId
                    },
                    SubDivisionTable: {
                        subDivisionId: subDivisionId
                    }
                }
            });
            let { backyard, organized } = await this.housingTableService.countAllHousingByDistAndDiv(districtId, subDivisionId);
            return {
                farmers, backyard, organized
            };
        }
    }
    async checkIfExists(createFarmersTableInput) {
        let rr = await this.farmersTableRepository.count({
            where: {
                farmerAadhar: createFarmersTableInput.farmerAadhar
            }
        });
        if (rr == 0) {
            return false;
        }
        else {
            return true;
        }
    }
    async getFarmerCode() {
        const query = this.farmersTableRepository.createQueryBuilder("r");
        query.select("MAX(r.farmerCodeIndex)", "farmerCodeIndex");
        let data = await query.getRawOne();
        if (data.farmerCodeIndex) {
            return parseFloat(data.farmerCodeIndex) + 1;
        }
        else {
            return 1;
        }
    }
    async findAll() {
        return await this.farmersTableRepository.find({
            relations: ["DistrictTable", "SubDivisionTable", "UserTable"]
        });
    }
    async findAllWithAccess(username) {
        let ac = await this.accessTableService.findAllByUserName(username);
        if (ac) {
            let filter = {};
            if (ac.DistrictTable) {
                filter = Object.assign(Object.assign({}, filter), { DistrictTable: {
                        districtId: ac.DistrictTable.districtId
                    } });
            }
            if (ac.SubDivisionTable) {
                filter = Object.assign(Object.assign({}, filter), { SubDivisionTable: {
                        subDivisionId: ac.SubDivisionTable.subDivisionId
                    } });
            }
            return await this.farmersTableRepository.find({
                where: filter,
                relations: ["DistrictTable", "SubDivisionTable", "UserTable"]
            });
        }
    }
    async findAllFarmerReport() {
        let result = await this.farmersTableRepository.find({
            relations: {
                DistrictTable: true,
                SubDivisionTable: true,
                UserTable: true
            }
        });
        let index = 0;
        for (const r of result) {
            const { organized, backyard } = await this.housingTableService.countAllHousingArea(r.farmerId);
            const { maleCount, femaleCount } = await this.currentStockTableService.getAllCount(r.farmerId);
            result[index]['OtherData'] = {
                organized,
                backyard,
                maleCount: maleCount || 0,
                femaleCount: femaleCount || 0,
                totalCount: parseFloat(maleCount || 0) + parseFloat(femaleCount || 0)
            };
            index++;
        }
        return result;
    }
    async findAllFarmerReportSingle(farmerId) {
        let result = await this.farmersTableRepository.find({
            where: {
                farmerId
            },
            relations: {
                DistrictTable: true,
                SubDivisionTable: true,
                UserTable: true
            }
        });
        let index = 0;
        for (const r of result) {
            const { organized, backyard } = await this.housingTableService.countAllHousingArea(r.farmerId);
            const { maleCount, femaleCount } = await this.currentStockTableService.getAllCount(r.farmerId);
            result[index]['OtherData'] = {
                organized,
                backyard,
                maleCount: maleCount || 0,
                femaleCount: femaleCount || 0,
                totalCount: parseFloat(maleCount || 0) + parseFloat(femaleCount || 0)
            };
            index++;
        }
        return result;
    }
    padWithZero(num, targetLength) {
        return String(num).padStart(targetLength, '0');
    }
    async findOne(farmerId) {
        return await this.farmersTableRepository.findOne({
            where: {
                farmerId: farmerId
            },
            relations: ["DistrictTable", "SubDivisionTable", "UserTable"]
        });
    }
    async findOneByCode(farmerCode) {
        return await this.farmersTableRepository.findOne({
            where: {
                farmerCode: farmerCode
            },
            relations: ["DistrictTable", "SubDivisionTable", "UserTable"]
        });
    }
    async findOneById(farmerId) {
        return await this.farmersTableRepository.findOne({
            where: {
                farmerId: farmerId
            },
            relations: ["DistrictTable", "SubDivisionTable", "UserTable"]
        });
    }
    async update(farmerId, updateFarmersTableInput, user) {
        let ft = await this.findOne(farmerId);
        if (ft) {
            let district = await this.districtTableService.findOne(updateFarmersTableInput.districtId);
            let subDivision = await this.subDivisionTableService.findOne(updateFarmersTableInput.subDivisionId);
            let userTable = await this.userCollectionService.getOneByUserId(user.userId);
            ft.DistrictTable = district;
            ft.SubDivisionTable = subDivision;
            ft.farmerVillage = updateFarmersTableInput.farmerVillage;
            ft.gender = updateFarmersTableInput.gender;
            ft.caste = updateFarmersTableInput.caste;
            ft.gramPanchayat = updateFarmersTableInput.gramPanchayat;
            ft.circleOffice = updateFarmersTableInput.circleOffice;
            ft.wardNo = updateFarmersTableInput.wardNo;
            ft.villageDispensary = updateFarmersTableInput.villageDispensary;
            ft.farmerState = updateFarmersTableInput.farmerState;
            ft.farmerPostOffice = updateFarmersTableInput.farmerPostOffice;
            ft.farmerName = updateFarmersTableInput.farmerName;
            ft.farmerFatherName = updateFarmersTableInput.farmerFatherName;
            ft.farmerPin = updateFarmersTableInput.farmerPin;
            ft.farmerPhone = updateFarmersTableInput.farmerPhone;
            ft.UserTable = userTable;
            let result = await this.farmersTableRepository.save(ft);
            this.dataEntryTableService.create({ type: GlobalConstants_1.UPDATE, typeOfEntry: GlobalConstants_1.FARMER_UPDATE, remarks: "farmer UPDATED" }, user);
            return {
                status: true,
                status_text: "Successfully Updated",
                data: result
            };
        }
        else {
            return {
                status: false,
                status_text: "Not found",
                data: null
            };
        }
    }
    async remove(farmerId, user) {
        let ft = await this.findOne(farmerId);
        if (ft) {
            await this.farmersTableRepository.remove(ft);
            this.dataEntryTableService.create({ type: GlobalConstants_1.DELETE, typeOfEntry: GlobalConstants_1.FARMER_DELETED, remarks: "farmer DELETED" }, user);
            return {
                status: true,
                status_text: "Successfully Deleted",
                data: null
            };
        }
        else {
            return {
                status: false,
                status_text: "Not found",
                data: null
            };
        }
    }
    async fetchDistrictWiseFarmingArea() {
        let districts = await this.districtTableService.findAll();
        let data = [];
        for (const d of districts) {
            const { organized, backyard } = await this.housingTableService.countAllHousingAreaDistrictWise(d.districtId);
            let d1 = {
                DistrictTable: d,
                organized,
                backyard
            };
            data.push(d1);
        }
        return data;
    }
};
FarmersTableService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(farmers_table_entity_1.FarmersTable)),
    __param(4, (0, common_1.Inject)((0, common_1.forwardRef)(() => user_collection_service_1.UserCollectionService))),
    __param(5, (0, common_1.Inject)((0, common_1.forwardRef)(() => access_table_service_1.AccessTableService))),
    __param(6, (0, common_1.Inject)((0, common_1.forwardRef)(() => housing_table_service_1.HousingTableService))),
    __param(7, (0, common_1.Inject)((0, common_1.forwardRef)(() => current_stock_table_service_1.CurrentStockTableService))),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        district_table_service_1.DistrictTableService,
        sub_division_table_service_1.SubDivisionTableService,
        data_entry_table_service_1.DataEntryTableService,
        user_collection_service_1.UserCollectionService,
        access_table_service_1.AccessTableService,
        housing_table_service_1.HousingTableService,
        current_stock_table_service_1.CurrentStockTableService])
], FarmersTableService);
exports.FarmersTableService = FarmersTableService;
