"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FblpSubTableService = void 0;
const common_1 = require("@nestjs/common");
const fblp_sub_table_entity_1 = require("./entities/fblp-sub-table.entity");
const typeorm_1 = require("typeorm");
const typeorm_2 = require("@nestjs/typeorm");
const income_source_table_service_1 = require("../income-source-table/income-source-table.service");
const HelperFunctions_1 = require("../helper/HelperFunctions");
const livelihood_source_table_service_1 = require("../livelihood-source-table/livelihood-source-table.service");
let FblpSubTableService = exports.FblpSubTableService = class FblpSubTableService {
    constructor(fblpSubTableRepository, livelihoodSourceTableService, incomeSourceTableService) {
        this.fblpSubTableRepository = fblpSubTableRepository;
        this.livelihoodSourceTableService = livelihoodSourceTableService;
        this.incomeSourceTableService = incomeSourceTableService;
    }
    async create(createFblpSubTableInput, fblpTable) {
        let results = [];
        await this.removeByFblpId(fblpTable.fblpId);
        for (const el of createFblpSubTableInput) {
            let fblpSubTable = new fblp_sub_table_entity_1.FblpSubTable();
            let incomeData = await this.incomeSourceTableService.findOne(el.incomeSourceId);
            fblpSubTable.IncomeSourceTable = incomeData;
            let lsData = await this.livelihoodSourceTableService.findOne(el.lsId);
            fblpSubTable.LivelihoodSourceTable = lsData;
            fblpSubTable.description = el.description;
            fblpSubTable.estimatedExpenditure = el.estimatedExpenditure;
            fblpSubTable.ownInvestment = el.ownInvestment;
            fblpSubTable.projectContribution = el.projectContribution;
            fblpSubTable.loanRequired = el.loanRequired;
            fblpSubTable.FblpTable = fblpTable;
            if (incomeData.incomeSourceCode == "AGRICULTURE") {
                fblpSubTable.farmEquipmentAvailable = el.farmEquipmentAvailable;
                fblpSubTable.insuranceSupportNeeded = el.insuranceSupportNeeded;
                fblpSubTable.areaOfLand = el.areaOfLand;
                fblpSubTable.areaOfLandUnit = el.areaOfLandUnit;
                fblpSubTable.conversionToBigha = (0, HelperFunctions_1.convertToBigha)(parseFloat(el.areaOfLand), el.areaOfLandUnit);
            }
            if (incomeData.incomeSourceCode == "LIVESTOCK") {
                fblpSubTable.shedSupportNeeded = el.shedSupportNeeded;
                fblpSubTable.insuranceSupportNeeded = el.insuranceSupportNeeded;
            }
            if (incomeData.incomeSourceCode == "FISHERY") {
                fblpSubTable.pondConstructionNeeded = el.pondConstructionNeeded;
                fblpSubTable.pondRenevationNeeded = el.pondRenevationNeeded;
                fblpSubTable.breedingUnitNeeded = el.breedingUnitNeeded;
            }
            let result = await this.fblpSubTableRepository.save(fblpSubTable);
            results.push(result);
        }
        return results;
    }
    async findByFblpId(fblpId) {
        let results = await this.fblpSubTableRepository.find({
            where: {
                FblpTable: {
                    fblpId
                }
            },
            relations: {
                IncomeSourceTable: true,
                LivelihoodSourceTable: true
            }
        });
        return results;
    }
    async getFblpConsolidatedReportList(shgId, lsCode) {
        let results = await this.fblpSubTableRepository.find({
            where: {
                FblpTable: {
                    MkTable: {
                        ShgTable: {
                            shgId
                        }
                    }
                },
                LivelihoodSourceTable: {
                    lsCode
                }
            },
            relations: {
                FblpTable: {
                    MkTable: {
                        ShgTable: {
                            VillageTable: {
                                GrampanchayatTable: true
                            }
                        }
                    }
                }
            }
        });
        return results;
    }
    findOne(id) {
        return `This action returns a #${id} fblpSubTable`;
    }
    update(id, updateFblpSubTableInput) {
        return `This action updates a #${id} fblpSubTable`;
    }
    remove(id) {
        return `This action removes a #${id} fblpSubTable`;
    }
    async removeByFblpId(fblpId) {
        let results = await this.fblpSubTableRepository.find({
            where: {
                FblpTable: {
                    fblpId
                }
            }
        });
        await this.fblpSubTableRepository.remove(results);
        return true;
    }
};
exports.FblpSubTableService = FblpSubTableService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_2.InjectRepository)(fblp_sub_table_entity_1.FblpSubTable)),
    __param(1, (0, common_1.Inject)((0, common_1.forwardRef)(() => livelihood_source_table_service_1.LivelihoodSourceTableService))),
    __param(2, (0, common_1.Inject)((0, common_1.forwardRef)(() => income_source_table_service_1.IncomeSourceTableService))),
    __metadata("design:paramtypes", [typeorm_1.Repository,
        livelihood_source_table_service_1.LivelihoodSourceTableService,
        income_source_table_service_1.IncomeSourceTableService])
], FblpSubTableService);
