"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FblpTableService = void 0;
const common_1 = require("@nestjs/common");
const fblp_table_entity_1 = require("./entities/fblp-table.entity");
const typeorm_1 = require("typeorm");
const typeorm_2 = require("@nestjs/typeorm");
const mk_table_service_1 = require("../mk-table/mk-table.service");
const user_collection_service_1 = require("../user-collection/user-collection.service");
const fblp_sub_table_service_1 = require("../fblp-sub-table/fblp-sub-table.service");
const moment_1 = __importDefault(require("moment"));
const mk_entitlements_table_service_1 = require("../mk-entitlements-table/mk-entitlements-table.service");
const livelihood_source_table_service_1 = require("../livelihood-source-table/livelihood-source-table.service");
const sessionmaster_table_service_1 = require("../sessionmaster-table/sessionmaster-table.service");
let FblpTableService = exports.FblpTableService = class FblpTableService {
    constructor(fblpTableRepository, userCollectionService, mkTableService, fblpSubTableService, mkEntitlementsTableService, livelihoodSourceTableService, sessionmasterTableService) {
        this.fblpTableRepository = fblpTableRepository;
        this.userCollectionService = userCollectionService;
        this.mkTableService = mkTableService;
        this.fblpSubTableService = fblpSubTableService;
        this.mkEntitlementsTableService = mkEntitlementsTableService;
        this.livelihoodSourceTableService = livelihoodSourceTableService;
        this.sessionmasterTableService = sessionmasterTableService;
    }
    async create(createFblpTableInput, user) {
        const checkIfAlreadyDoneRecently = await this.checkIfAlreadyDoneRecently(createFblpTableInput.mkId);
        if (!checkIfAlreadyDoneRecently) {
            let fblp = new fblp_table_entity_1.FblpTable();
            let mkData = await this.mkTableService.findOne(createFblpTableInput.mkId);
            fblp.MkTable = mkData;
            let currentDate = (0, moment_1.default)();
            fblp.year = parseFloat(currentDate.format("YYYY"));
            fblp.entryDate = (0, moment_1.default)().format("DD-MM-YYYY");
            let session = await this.sessionmasterTableService.findActiveSession();
            fblp.SessionmasterTable = session;
            fblp.editableTill = currentDate.add(7, 'days').format("DD-MM-YYYY");
            fblp.performAfter = currentDate.add(12, 'months').format("DD-MM-YYYY");
            let userData = await this.userCollectionService.findOneByUsername(user.username);
            fblp.UserTable = userData;
            let result = await this.fblpTableRepository.save(fblp);
            if (createFblpTableInput.data.length > 0) {
                await this.fblpSubTableService.create(createFblpTableInput.data, result);
            }
            if (createFblpTableInput.entitlements.length > 0) {
                await this.mkEntitlementsTableService.createFromFblp(createFblpTableInput.entitlements, user, mkData.mkId);
            }
            return {
                status: true,
                status_text: "FBLP Successfully Submitted",
                data: result
            };
        }
        else {
            return {
                status: false,
                status_text: "You cannot perform FBLP for this Mahila Kisan Now"
            };
        }
    }
    async checkIfAlreadyDoneRecently(mkId) {
        let result = await this.fblpTableRepository.findOne({
            where: {
                MkTable: {
                    mkId
                },
            },
            order: {
                fblpId: "DESC"
            }
        });
        if (result) {
            let performAfter = (0, moment_1.default)(result.performAfter, "DD-MM-YYYY");
            let currentDate = (0, moment_1.default)();
            if (currentDate.isAfter(performAfter)) {
                return false;
            }
            else {
                return true;
            }
        }
        else {
            return false;
        }
    }
    async findAll() {
        return await this.fblpTableRepository.find({
            relations: {
                MkTable: {
                    ShgTable: {
                        VillageTable: {
                            GrampanchayatTable: {
                                BlockTable: {
                                    DistrictTable: {
                                        StateTable: true
                                    }
                                }
                            }
                        }
                    }
                },
                UserTable: true
            }
        });
    }
    async countByShgId(shgId) {
        return await this.fblpTableRepository.count({
            where: {
                MkTable: {
                    ShgTable: {
                        shgId
                    }
                }
            }
        });
    }
    async findOneByMkId(mkId) {
        let results = await this.fblpTableRepository.findOne({
            where: {
                MkTable: {
                    mkId
                }
            },
            relations: {
                UserTable: true,
                FblpSubTable: {
                    IncomeSourceTable: true,
                    LivelihoodSourceTable: true
                },
                MkTable: {
                    ShgTable: {
                        VoTable: true
                    },
                    VillageTable: {
                        GrampanchayatTable: {
                            BlockTable: {
                                DistrictTable: {
                                    StateTable: true
                                }
                            }
                        }
                    },
                    MkEntitlementsTable: {
                        SchemeMasterSubTable: {
                            SchemeMasterTable: true
                        }
                    }
                }
            },
            order: {
                fblpId: "DESC"
            }
        });
        return results;
    }
    async getListOfLsFromLatestFblp(mkId) {
        let lsCodes = [];
        let latestFblp = await this.fblpTableRepository.findOne({
            where: {
                MkTable: {
                    mkId
                }
            },
            order: {
                fblpId: "DESC"
            },
            relations: {
                FblpSubTable: {
                    LivelihoodSourceTable: true
                }
            }
        });
        if (latestFblp) {
            latestFblp.FblpSubTable.map((el) => {
                if (!lsCodes.includes(el.LivelihoodSourceTable.lsCode)) {
                    lsCodes.push(el.LivelihoodSourceTable.lsCode);
                }
            });
            return {
                lsCodes: lsCodes
            };
        }
        else {
            return {
                lsCodes: []
            };
        }
    }
    async update(fblpId, updateFblpTableInput, user) {
        let fblp = await this.fblpTableRepository.findOne({
            where: {
                fblpId
            },
            relations: {
                MkTable: true
            }
        });
        if (fblp) {
            if (updateFblpTableInput.data.length > 0) {
                await this.fblpSubTableService.create(updateFblpTableInput.data, fblp);
            }
            if (updateFblpTableInput.entitlements.length > 0) {
                await this.mkEntitlementsTableService.createFromFblp(updateFblpTableInput.entitlements, user, fblp.MkTable.mkId);
            }
            return {
                status: true,
                status_text: "Successfully Updated"
            };
        }
        else {
            return {
                status: false,
                status_text: "Not Found"
            };
        }
    }
    async updateFblpEditableDate(fblpId, editableTill) {
        let fblp = await this.fblpTableRepository.findOne({
            where: {
                fblpId
            }
        });
        if (fblp) {
            fblp.editableTill = (0, moment_1.default)(editableTill, "YYYY-MM-DD").format("DD-MM-YYYY");
            await this.fblpTableRepository.save(fblp);
            return {
                status: true,
                status_text: "Successfully Updated"
            };
        }
        else {
            return {
                status: false,
                status_text: "Not Found"
            };
        }
    }
    async remove(fblpId) {
        let res = await this.fblpTableRepository.findOne({
            where: {
                fblpId
            }
        });
        if (res) {
            await this.fblpTableRepository.remove(res);
            return {
                status: true,
                status_text: "Successfully Deleted"
            };
        }
        else {
            return {
                status: false,
                status_text: "Not found"
            };
        }
    }
    async getCount() {
        return await this.fblpTableRepository.count();
    }
    async getCountByMonth(month) {
        const startOfMonth = (0, moment_1.default)(month).startOf('month').format('YYYY-MM-DD');
        const endOfMonth = (0, moment_1.default)(month).endOf('month').format('YYYY-MM-DD');
        let result = await this.fblpTableRepository
            .createQueryBuilder('fl')
            .andWhere("STR_TO_DATE(fl.entryDate, '%d-%m-%Y') BETWEEN :startDate AND :endDate", {
            endDate: endOfMonth,
            startDate: startOfMonth,
        })
            .getCount();
        return result;
    }
    async getCountByUsername(username) {
        return await this.fblpTableRepository.count({
            where: {
                UserTable: {
                    username
                }
            }
        });
    }
    async getFblpTotalReport(stateId, districtId, blockId, type) {
        const livelihoodData = await this.livelihoodSourceTableService.findAll();
        let ffData = [];
        if (stateId > 0) {
            ffData = await this.fblpTableRepository.find({
                where: {
                    MkTable: {
                        ShgTable: {
                            VillageTable: {
                                GrampanchayatTable: {
                                    BlockTable: {
                                        DistrictTable: {
                                            StateTable: {
                                                stateId,
                                            },
                                        },
                                    },
                                },
                            },
                        },
                    }
                },
                relations: {
                    MkTable: {
                        VillageTable: {
                            GrampanchayatTable: true
                        }
                    },
                    FblpSubTable: {
                        LivelihoodSourceTable: true
                    },
                },
            });
        }
        if (districtId > 0) {
            ffData = await this.fblpTableRepository.find({
                where: {
                    MkTable: {
                        ShgTable: {
                            VillageTable: {
                                GrampanchayatTable: {
                                    BlockTable: {
                                        DistrictTable: {
                                            districtId: districtId,
                                        },
                                    },
                                },
                            },
                        },
                    }
                },
                relations: {
                    MkTable: {
                        VillageTable: {
                            GrampanchayatTable: true
                        }
                    },
                    FblpSubTable: {
                        LivelihoodSourceTable: true
                    },
                },
            });
        }
        if (blockId > 0) {
            ffData = await this.fblpTableRepository.find({
                where: {
                    MkTable: {
                        ShgTable: {
                            VillageTable: {
                                GrampanchayatTable: {
                                    BlockTable: {
                                        blockId,
                                    },
                                },
                            },
                        },
                    }
                },
                relations: {
                    MkTable: {
                        VillageTable: {
                            GrampanchayatTable: true
                        }
                    },
                    FblpSubTable: {
                        LivelihoodSourceTable: true
                    },
                },
            });
        }
        if (stateId == 0 && districtId == 0 && blockId == 0) {
            ffData = await this.fblpTableRepository.find({
                relations: {
                    MkTable: {
                        VillageTable: {
                            GrampanchayatTable: true
                        }
                    },
                    FblpSubTable: {
                        LivelihoodSourceTable: true
                    },
                },
            });
        }
        let data = [];
        let filteredLsData = livelihoodData.filter((el) => el.incomeActivityCode == type);
        for (const lsd of filteredLsData) {
            let totalExpenditure = 0;
            let totalOwnInvestment = 0;
            let totalProjectContribution = 0;
            let totalLoanRequired = 0;
            let count = 0;
            let totalLand = 0;
            let farmEquipment = 0;
            let insuranceSupport = 0;
            let shedConstruction = 0;
            let breedingUnit = 0;
            let pondRenovation = 0;
            let pondContruction = 0;
            let mkData = [];
            let grandTotalLoan = 0;
            for (const fd of ffData) {
                fd.FblpSubTable.map((el, index) => {
                    if (el.LivelihoodSourceTable.lsCode == lsd.lsCode) {
                        count++;
                        totalExpenditure = totalExpenditure + parseFloat(el.estimatedExpenditure);
                        totalOwnInvestment = totalOwnInvestment + parseFloat(el.ownInvestment);
                        totalLoanRequired = totalLoanRequired + parseFloat(el.loanRequired);
                        totalProjectContribution = totalProjectContribution + parseFloat(el.projectContribution);
                        mkData.push({
                            mkName: fd.MkTable.mkName,
                            villageName: fd.MkTable.VillageTable.villageName,
                            gpName: fd.MkTable.VillageTable.GrampanchayatTable.gpName,
                            estimatedExpenditure: el.estimatedExpenditure,
                            ownInvestment: el.ownInvestment,
                            loanRequired: el.loanRequired,
                            projectContribution: el.projectContribution,
                            landSize: el.conversionToBigha,
                            farmEquipmentAvailable: el.farmEquipmentAvailable,
                            insuranceSupportNeeded: el.insuranceSupportNeeded,
                            shedSupportNeeded: el.shedSupportNeeded,
                            breedingUnitNeeded: el.breedingUnitNeeded,
                            pondConstructionNeeded: el.pondConstructionNeeded,
                            pondRenevationNeeded: el.pondRenevationNeeded
                        });
                        totalLand = totalLand + parseFloat(el.conversionToBigha);
                        if (el.farmEquipmentAvailable == "YES") {
                            farmEquipment++;
                        }
                        if (el.insuranceSupportNeeded == "YES") {
                            insuranceSupport++;
                        }
                        if (el.shedSupportNeeded == "YES") {
                            shedConstruction++;
                        }
                        if (el.breedingUnitNeeded == "YES") {
                            breedingUnit++;
                        }
                        if (el.pondConstructionNeeded == "YES") {
                            pondContruction++;
                        }
                        if (el.pondRenevationNeeded == "YES") {
                            pondRenovation++;
                        }
                    }
                });
            }
            data.push({
                lsCode: lsd.lsCode,
                count: count,
                totalExpenditure,
                totalOwnInvestment,
                totalProjectContribution,
                totalLoanRequired,
                totalLand,
                farmEquipment,
                insuranceSupport,
                shedConstruction,
                breedingUnit,
                pondRenovation,
                pondContruction,
                mkData
            });
        }
        return data;
    }
    async getFblpListReport(stateId, districtId, blockId, sessionId) {
        let ffData = [];
        if (stateId > 0) {
            ffData = await this.fblpTableRepository.find({
                where: {
                    MkTable: {
                        ShgTable: {
                            VillageTable: {
                                GrampanchayatTable: {
                                    BlockTable: {
                                        DistrictTable: {
                                            StateTable: {
                                                stateId,
                                            },
                                        },
                                    },
                                },
                            },
                        },
                    },
                    SessionmasterTable: {
                        sessionId
                    }
                },
                relations: {
                    FblpSubTable: {
                        LivelihoodSourceTable: true
                    },
                    MkTable: {
                        ShgTable: {
                            VillageTable: {
                                GrampanchayatTable: {
                                    BlockTable: {
                                        DistrictTable: {
                                            StateTable: true
                                        }
                                    }
                                }
                            }
                        }
                    },
                    UserTable: true
                },
            });
        }
        if (districtId > 0) {
            ffData = await this.fblpTableRepository.find({
                where: {
                    MkTable: {
                        ShgTable: {
                            VillageTable: {
                                GrampanchayatTable: {
                                    BlockTable: {
                                        DistrictTable: {
                                            districtId: districtId,
                                        },
                                    },
                                },
                            },
                        },
                    },
                    SessionmasterTable: {
                        sessionId
                    }
                },
                relations: {
                    FblpSubTable: {
                        LivelihoodSourceTable: true
                    },
                    MkTable: {
                        ShgTable: {
                            VillageTable: {
                                GrampanchayatTable: {
                                    BlockTable: {
                                        DistrictTable: {
                                            StateTable: true
                                        }
                                    }
                                }
                            }
                        }
                    },
                    UserTable: true
                },
            });
        }
        if (blockId > 0) {
            ffData = await this.fblpTableRepository.find({
                where: {
                    MkTable: {
                        ShgTable: {
                            VillageTable: {
                                GrampanchayatTable: {
                                    BlockTable: {
                                        blockId,
                                    },
                                },
                            },
                        },
                    },
                    SessionmasterTable: {
                        sessionId
                    }
                },
                relations: {
                    FblpSubTable: {
                        LivelihoodSourceTable: true
                    },
                    MkTable: {
                        ShgTable: {
                            VillageTable: {
                                GrampanchayatTable: {
                                    BlockTable: {
                                        DistrictTable: {
                                            StateTable: true
                                        }
                                    }
                                }
                            }
                        }
                    },
                    UserTable: true
                },
            });
        }
        if (stateId == 0 && districtId == 0 && blockId == 0) {
            ffData = await this.fblpTableRepository.find({
                where: {
                    SessionmasterTable: {
                        sessionId
                    }
                },
                relations: {
                    FblpSubTable: {
                        LivelihoodSourceTable: true
                    },
                    MkTable: {
                        ShgTable: {
                            VillageTable: {
                                GrampanchayatTable: {
                                    BlockTable: {
                                        DistrictTable: {
                                            StateTable: true
                                        }
                                    }
                                }
                            }
                        }
                    },
                    UserTable: true
                },
            });
        }
        let index = 0;
        for (const fd of ffData) {
            let totalExpenditure = 0;
            let totalOwnInvestment = 0;
            let totalProjectContribution = 0;
            let totalLoanRequired = 0;
            let agriculture = 0;
            let livestock = 0;
            let fishery = 0;
            let vermicompost = 0;
            let totalLand = 0;
            let farmEquipment = 0;
            let insuranceSupport = 0;
            let shedConstruction = 0;
            let breedingUnit = 0;
            let pondRenovation = 0;
            let pondContruction = 0;
            fd.FblpSubTable.map((el) => {
                if (el.LivelihoodSourceTable.incomeActivityCode == "AGRICULTURE") {
                    agriculture++;
                }
                if (el.LivelihoodSourceTable.incomeActivityCode == "FISHERY") {
                    fishery++;
                }
                if (el.LivelihoodSourceTable.incomeActivityCode == "LIVESTOCK") {
                    livestock++;
                }
                if (el.LivelihoodSourceTable.incomeActivityCode == "VERMICOMPOST") {
                    vermicompost++;
                }
                totalExpenditure = totalExpenditure + parseFloat(el.estimatedExpenditure);
                totalOwnInvestment = totalOwnInvestment + parseFloat(el.ownInvestment);
                totalLoanRequired = totalLoanRequired + parseFloat(el.loanRequired);
                totalProjectContribution = totalProjectContribution + parseFloat(el.projectContribution);
                totalLand = totalLand + parseFloat(el.conversionToBigha);
                if (el.farmEquipmentAvailable == "YES") {
                    farmEquipment++;
                }
                if (el.insuranceSupportNeeded == "YES") {
                    insuranceSupport++;
                }
                if (el.shedSupportNeeded == "YES") {
                    shedConstruction++;
                }
                if (el.breedingUnitNeeded == "YES") {
                    breedingUnit++;
                }
                if (el.pondConstructionNeeded == "YES") {
                    pondContruction++;
                }
                if (el.pondRenevationNeeded == "YES") {
                    pondRenovation++;
                }
            });
            let d = {
                totalExpenditure,
                totalOwnInvestment,
                totalProjectContribution,
                totalLoanRequired,
                agriculture,
                livestock,
                fishery,
                vermicompost,
                totalLand,
                farmEquipment,
                insuranceSupport,
                shedConstruction,
                breedingUnit,
                pondRenovation,
                pondContruction,
            };
            ffData[index]['summary'] = d;
            index++;
        }
        let lsMasterData = await this.livelihoodSourceTableService.findAllOrderedForFblp();
        return {
            data: ffData,
            lsMasterData
        };
    }
    async getFblpConsolidatedReport(shgId) {
        const livelihoodData = await this.livelihoodSourceTableService.findAll();
        let ffData = [];
        if (shgId > 0) {
            ffData = await this.fblpTableRepository.find({
                where: {
                    MkTable: {
                        ShgTable: {
                            shgId
                        },
                    }
                },
                relations: {
                    FblpSubTable: {
                        LivelihoodSourceTable: true
                    },
                },
            });
        }
        let data = [];
        let filteredLsData = livelihoodData.filter((el) => el.needInIntervention == "YES");
        for (const lsd of filteredLsData) {
            let totalExpenditure = 0;
            let totalOwnInvestment = 0;
            let totalProjectContribution = 0;
            let totalLoanRequired = 0;
            let count = 0;
            let totalLand = 0;
            let farmEquipment = 0;
            let insuranceSupport = 0;
            let shedConstruction = 0;
            let breedingUnit = 0;
            let pondRenovation = 0;
            let pondContruction = 0;
            let grandTotalLoan = 0;
            for (const fd of ffData) {
                fd.FblpSubTable.map((el, index) => {
                    if (el.LivelihoodSourceTable.lsCode == lsd.lsCode) {
                        count++;
                        totalExpenditure = totalExpenditure + parseFloat(el.estimatedExpenditure);
                        totalOwnInvestment = totalOwnInvestment + parseFloat(el.ownInvestment);
                        totalLoanRequired = totalLoanRequired + parseFloat(el.loanRequired);
                        totalProjectContribution = totalProjectContribution + parseFloat(el.projectContribution);
                        totalLand = totalLand + parseFloat(el.conversionToBigha);
                        if (el.farmEquipmentAvailable == "YES") {
                            farmEquipment++;
                        }
                        if (el.insuranceSupportNeeded == "YES") {
                            insuranceSupport++;
                        }
                        if (el.shedSupportNeeded == "YES") {
                            shedConstruction++;
                        }
                        if (el.breedingUnitNeeded == "YES") {
                            breedingUnit++;
                        }
                        if (el.pondConstructionNeeded == "YES") {
                            pondContruction++;
                        }
                        if (el.pondRenevationNeeded == "YES") {
                            pondRenovation++;
                        }
                    }
                });
            }
            data.push({
                lsCode: lsd.lsCode,
                incomeSourceCode: lsd.incomeActivityCode,
                count: count,
                totalExpenditure,
                totalOwnInvestment,
                totalProjectContribution,
                totalLoanRequired,
                totalLand,
                farmEquipment,
                insuranceSupport,
                shedConstruction,
                breedingUnit,
                pondRenovation,
                pondContruction
            });
        }
        return data;
    }
    async getFblpConsolidatedReportList(shgId, lsCode) {
        return await this.fblpSubTableService.getFblpConsolidatedReportList(shgId, lsCode);
    }
    async getRepeatAndNewFarmer(mkIds) {
        let newFarmer = 0;
        let repeatFarmer = 0;
        for (const mkId of mkIds) {
            let cc = await this.fblpTableRepository.count({
                where: {
                    MkTable: {
                        mkId
                    }
                }
            });
            if (cc == 1) {
                newFarmer++;
            }
            if (cc > 1) {
                repeatFarmer++;
            }
        }
        return {
            newFarmer,
            repeatFarmer
        };
    }
};
exports.FblpTableService = FblpTableService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_2.InjectRepository)(fblp_table_entity_1.FblpTable)),
    __param(2, (0, common_1.Inject)((0, common_1.forwardRef)(() => mk_table_service_1.MkTableService))),
    __param(3, (0, common_1.Inject)((0, common_1.forwardRef)(() => fblp_sub_table_service_1.FblpSubTableService))),
    __param(4, (0, common_1.Inject)((0, common_1.forwardRef)(() => mk_entitlements_table_service_1.MkEntitlementsTableService))),
    __param(5, (0, common_1.Inject)((0, common_1.forwardRef)(() => livelihood_source_table_service_1.LivelihoodSourceTableService))),
    __param(6, (0, common_1.Inject)((0, common_1.forwardRef)(() => sessionmaster_table_service_1.SessionmasterTableService))),
    __metadata("design:paramtypes", [typeorm_1.Repository,
        user_collection_service_1.UserCollectionService,
        mk_table_service_1.MkTableService,
        fblp_sub_table_service_1.FblpSubTableService,
        mk_entitlements_table_service_1.MkEntitlementsTableService,
        livelihood_source_table_service_1.LivelihoodSourceTableService,
        sessionmaster_table_service_1.SessionmasterTableService])
], FblpTableService);
