"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HousingTableService = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const farmers_table_service_1 = require("../farmers-table/farmers-table.service");
const user_collection_service_1 = require("../user-collection/user-collection.service");
const typeorm_2 = require("typeorm");
const housing_table_entity_1 = require("./entities/housing-table.entity");
const moment_1 = __importDefault(require("moment"));
const farmers_table_entity_1 = require("../farmers-table/entities/farmers-table.entity");
const current_stock_table_entity_1 = require("../current-stock/entities/current-stock-table.entity");
const GlobalConstants_1 = require("../helper/GlobalConstants");
const data_entry_table_service_1 = require("../data-entry-table/data-entry-table.service");
let HousingTableService = class HousingTableService {
    constructor(housingTableRepository, dataEntryTableService, userCollectionService, farmersTableService) {
        this.housingTableRepository = housingTableRepository;
        this.dataEntryTableService = dataEntryTableService;
        this.userCollectionService = userCollectionService;
        this.farmersTableService = farmersTableService;
    }
    async create(createHousingTableInput, user) {
        let checkIfExists = await this.checkIfExists(createHousingTableInput);
        if (checkIfExists) {
            return {
                status: false,
                status_text: "Already Exists",
                data: null
            };
        }
        let ht = new housing_table_entity_1.HousingTable();
        let userTable = await this.userCollectionService.getOneByUserId(user.userId);
        let farmerTable = await this.farmersTableService.findOne(createHousingTableInput.farmerId);
        ht.UserTable = userTable;
        ht.FarmersTable = farmerTable;
        ht.typeOfFarm = createHousingTableInput.typeOfFarm;
        ht.farmingArea = createHousingTableInput.farmingArea;
        ht.isAvailable = createHousingTableInput.isAvailable;
        if (createHousingTableInput.isAvailable == "YES") {
            ht.typesOfHouse = createHousingTableInput.typesOfHouse;
            ht.floor = createHousingTableInput.floor;
            ht.hasSufficientFloor = createHousingTableInput.hasSufficientFloor;
            ht.floorSpace = createHousingTableInput.floorSpace;
        }
        ht.hasBoundaryWall = createHousingTableInput.hasBoundaryWall;
        ht.hasDrainageFacility = createHousingTableInput.hasDrainageFacility;
        ht.hasManurePit = createHousingTableInput.hasManurePit;
        if (createHousingTableInput.hasManurePit == "YES") {
            ht.manurePitDetails = createHousingTableInput.manurePitDetails;
        }
        ht.hasCleaningFacility = createHousingTableInput.hasCleaningFacility;
        if (createHousingTableInput.hasCleaningFacility == "YES") {
            ht.frequencyOfCleaning = createHousingTableInput.frequencyOfCleaning;
        }
        ht.hasWaterSource = createHousingTableInput.hasWaterSource;
        if (createHousingTableInput.hasWaterSource == "YES") {
            ht.waterSourceDetails = createHousingTableInput.waterSourceDetails;
        }
        ht.hasUsedDisinfectants = createHousingTableInput.hasUsedDisinfectants;
        if (createHousingTableInput.hasUsedDisinfectants == "YES") {
            ht.disinfectantDetails = createHousingTableInput.disinfectantDetails;
        }
        ht.dateOfEntry = (0, moment_1.default)().format("DD-MM-YYYY");
        let result = await this.housingTableRepository.save(ht);
        this.dataEntryTableService.create({ type: GlobalConstants_1.CREATE, typeOfEntry: GlobalConstants_1.HOUSING_ADD, remarks: "New housing added" }, user);
        return {
            status: true,
            status_text: "Successfully Submitted",
            data: result
        };
    }
    async countAllHousing(farmerId) {
        if (!farmerId) {
            let organized = await this.housingTableRepository.count({
                where: {
                    typeOfFarm: "ORGANIZED"
                }
            });
            let backyard = await this.housingTableRepository.count({
                where: {
                    typeOfFarm: "BACKYARD"
                }
            });
            return {
                organized,
                backyard
            };
        }
        else {
            let ft = await this.farmersTableService.findOneById(farmerId);
            let organized = await this.housingTableRepository.createQueryBuilder("r")
                .where("r.farmerId=:farmerId", { farmerId: farmerId })
                .andWhere("r.typeOfFarm=:typeOfFarm", { typeOfFarm: "ORGANIZED" })
                .getCount();
            let backyard = await this.housingTableRepository.createQueryBuilder("r")
                .where("r.farmerId=:farmerId", { farmerId: farmerId })
                .andWhere("r.typeOfFarm=:typeOfFarm", { typeOfFarm: "BACKYARD" })
                .getCount();
            return {
                organized,
                backyard
            };
        }
    }
    async countAllHousingArea(farmerId) {
        if (!farmerId) {
            let organized = await this.housingTableRepository.createQueryBuilder("r")
                .where("r.typeOfFarm=:typeOfFarm", { typeOfFarm: "ORGANIZED" })
                .select("SUM(r.farmingArea)", "farmingArea")
                .groupBy('r.farmerId')
                .getRawOne();
            let backyard = await this.housingTableRepository.createQueryBuilder("r")
                .where("r.typeOfFarm=:typeOfFarm", { typeOfFarm: "BACKYARD" })
                .select("SUM(r.farmingArea)", "farmingArea")
                .groupBy('r.farmerId')
                .getRawOne();
            return {
                organized: organized ? organized.farmingArea : 0,
                backyard: backyard ? backyard.farmingArea : 0
            };
        }
        else {
            let organized = await this.housingTableRepository.createQueryBuilder("r")
                .where("r.farmerId=:farmerId", { farmerId: farmerId })
                .andWhere("r.typeOfFarm=:typeOfFarm", { typeOfFarm: "ORGANIZED" })
                .select("SUM(r.farmingArea)", "farmingArea")
                .groupBy('r.farmerId')
                .getRawOne();
            console.log("🚀 ~ file: housing-table.service.ts ~ line 166 ~ HousingTableService ~ countAllHousingArea ~ organized", organized);
            let backyard = await this.housingTableRepository.createQueryBuilder("r")
                .where("r.farmerId=:farmerId", { farmerId: farmerId })
                .andWhere("r.typeOfFarm=:typeOfFarm", { typeOfFarm: "BACKYARD" })
                .select("SUM(r.farmingArea)", "farmingArea")
                .groupBy('r.farmerId')
                .getRawOne();
            return {
                organized: organized ? organized.farmingArea : 0,
                backyard: backyard ? backyard.farmingArea : 0
            };
        }
    }
    async countAllHousingAreaDistrictWise(districtId) {
        let organized = await this.housingTableRepository.createQueryBuilder("r")
            .leftJoinAndMapOne("r.FarmersTable", farmers_table_entity_1.FarmersTable, "ml1", "ml1.farmerId=r.farmerId")
            .where("r.typeOfFarm=:typeOfFarm", { typeOfFarm: "ORGANIZED" })
            .andWhere("ml1.districtId=:districtId", { districtId: districtId })
            .select("SUM(r.farmingArea)", "farmingArea")
            .groupBy('r.farmerId')
            .getRawOne();
        let backyard = await this.housingTableRepository.createQueryBuilder("r")
            .leftJoinAndMapOne("r.FarmersTable", farmers_table_entity_1.FarmersTable, "ml1", "ml1.farmerId=r.farmerId")
            .where("r.typeOfFarm=:typeOfFarm", { typeOfFarm: "BACKYARD" })
            .andWhere("ml1.districtId=:districtId", { districtId: districtId })
            .select("SUM(r.farmingArea)", "farmingArea")
            .groupBy('r.farmerId')
            .getRawOne();
        return {
            organized: organized ? organized.farmingArea : 0,
            backyard: backyard ? backyard.farmingArea : 0
        };
    }
    async countAllHousingByDistAndDiv(districtId, subDivisionId) {
        let organized = await this.housingTableRepository.createQueryBuilder("r")
            .andWhere("r.typeOfFarm=:typeOfFarm", { typeOfFarm: "ORGANIZED" })
            .leftJoinAndMapOne("r.FarmersTable", farmers_table_entity_1.FarmersTable, "ml1", "ml1.farmerId=r.farmerId");
        if (subDivisionId) {
            organized = await organized.andWhere("ml1.subDivisionId=:subDivisionId", { subDivisionId: subDivisionId });
        }
        else {
            organized = await organized.andWhere("ml1.districtId=:districtId", { districtId: districtId });
        }
        organized = await organized.getCount();
        let backyard = await this.housingTableRepository.createQueryBuilder("r")
            .leftJoinAndMapOne("r.FarmersTable", farmers_table_entity_1.FarmersTable, "ml1", "ml1.farmerId=r.farmerId")
            .andWhere("r.typeOfFarm=:typeOfFarm", { typeOfFarm: "BACKYARD" });
        if (subDivisionId) {
            backyard = await backyard.andWhere("ml1.subDivisionId=:subDivisionId", { subDivisionId: subDivisionId });
        }
        else {
            backyard = await backyard.andWhere("ml1.districtId=:districtId", { districtId: districtId });
        }
        backyard = await backyard.getCount();
        return {
            organized,
            backyard
        };
    }
    async checkIfExists(createHousingTableInput) {
        let result = await this.housingTableRepository.count({
            where: {
                typeOfFarm: createHousingTableInput.typeOfFarm,
                farmingArea: createHousingTableInput.farmingArea,
                isAvailable: createHousingTableInput.isAvailable,
                typesOfHouse: createHousingTableInput.typesOfHouse,
                floor: createHousingTableInput.floor,
                hasSufficientFloor: createHousingTableInput.hasSufficientFloor,
                floorSpace: createHousingTableInput.floorSpace,
                hasBoundaryWall: createHousingTableInput.hasBoundaryWall,
                hasDrainageFacility: createHousingTableInput.hasDrainageFacility,
                hasManurePit: createHousingTableInput.hasManurePit,
                manurePitDetails: createHousingTableInput.manurePitDetails,
                hasCleaningFacility: createHousingTableInput.hasCleaningFacility,
                hasWaterSource: createHousingTableInput.hasWaterSource,
                waterSourceDetails: createHousingTableInput.waterSourceDetails,
                hasUsedDisinfectants: createHousingTableInput.hasUsedDisinfectants,
                disinfectantDetails: createHousingTableInput.disinfectantDetails,
                frequencyOfCleaning: createHousingTableInput.frequencyOfCleaning
            }
        });
        if (result == 0) {
            return false;
        }
        else {
            return true;
        }
    }
    async findByFarmerId(farmerId) {
        let farmer = await this.farmersTableService.findOne(farmerId);
        return await this.housingTableRepository.find({
            where: {
                FarmersTable: farmer
            },
            relations: ["UserTable"]
        });
    }
    async findAll() {
        return await this.housingTableRepository.find({
            relations: ["UserTable", "FarmersTable"]
        });
    }
    async findOne(housingId) {
        return await this.housingTableRepository.findOne({
            where: {
                housingId
            },
            relations: ["UserTable", "FarmersTable"]
        });
    }
    async findOneByFarmerId(farmerId) {
        let farmerTable = await this.farmersTableService.findOne(farmerId);
        let result = await this.housingTableRepository.createQueryBuilder("r")
            .where("r.farmerId=:farmerId", { farmerId: farmerTable.farmerId })
            .leftJoinAndMapOne("r.FarmersTable", farmers_table_entity_1.FarmersTable, "ml1", "ml1.farmerId=r.farmerId")
            .getMany();
        return result;
    }
    async update(housingId, updateHousingTableInput, user) {
        let ht = await this.housingTableRepository.findOne({
            where: {
                housingId
            }
        });
        if (!ht) {
            return {
                status: false,
                status_text: "Not found",
                data: null
            };
        }
        else {
            let userTable = await this.userCollectionService.getOneByUserId(user.userId);
            let farmerTable = await this.farmersTableService.findOne(updateHousingTableInput.farmerId);
            ht.UserTable = userTable;
            ht.FarmersTable = farmerTable;
            ht.typeOfFarm = updateHousingTableInput.typeOfFarm;
            ht.farmingArea = updateHousingTableInput.farmingArea;
            ht.isAvailable = updateHousingTableInput.isAvailable;
            if (updateHousingTableInput.isAvailable === "YES") {
                ht.typesOfHouse = updateHousingTableInput.typesOfHouse;
                ht.floor = updateHousingTableInput.floor;
                ht.hasSufficientFloor = updateHousingTableInput.hasSufficientFloor;
                ht.floorSpace = updateHousingTableInput.floorSpace;
            }
            ht.hasBoundaryWall = updateHousingTableInput.hasBoundaryWall;
            ht.hasDrainageFacility = updateHousingTableInput.hasDrainageFacility;
            ht.hasManurePit = updateHousingTableInput.hasManurePit;
            if (updateHousingTableInput.hasManurePit === "YES") {
                ht.manurePitDetails = updateHousingTableInput.manurePitDetails;
            }
            ht.hasCleaningFacility = updateHousingTableInput.hasCleaningFacility;
            if (updateHousingTableInput.hasCleaningFacility === "YES") {
                ht.frequencyOfCleaning = updateHousingTableInput.frequencyOfCleaning;
            }
            ht.hasWaterSource = updateHousingTableInput.hasWaterSource;
            if (updateHousingTableInput.hasWaterSource === "YES") {
                ht.waterSourceDetails = updateHousingTableInput.waterSourceDetails;
            }
            ht.hasUsedDisinfectants = updateHousingTableInput.hasUsedDisinfectants;
            if (updateHousingTableInput.hasUsedDisinfectants === "YES") {
                ht.disinfectantDetails = updateHousingTableInput.disinfectantDetails;
            }
            ht.dateOfEntry = (0, moment_1.default)().format("DD-MM-YYYY");
            let result = await this.housingTableRepository.save(ht);
            this.dataEntryTableService.create({ type: GlobalConstants_1.UPDATE, typeOfEntry: GlobalConstants_1.HOUSING_UPDATE, remarks: "housing UPDATED" }, user);
            return {
                status: true,
                status_text: "Successfully Updated",
                data: result
            };
        }
    }
    async remove(housingId, user) {
        let ht = await this.housingTableRepository.findOne({
            where: {
                housingId
            }
        });
        if (!ht) {
            return {
                status: false,
                status_text: "Not found",
                data: null
            };
        }
        else {
            await this.housingTableRepository.remove(ht);
            this.dataEntryTableService.create({ type: GlobalConstants_1.DELETE, typeOfEntry: GlobalConstants_1.HOUSING_DELETED, remarks: "housing DELETED" }, user);
            return {
                status: true,
                status_text: "Successfully Deleted",
                data: null
            };
        }
    }
    async findSingleFarmerReport(districtId, subDivisionId, farmerId) {
        let farmers = await this.farmersTableService.findAll();
        let data = [];
        for (const d of farmers) {
            let d1 = await this.housingTableRepository.createQueryBuilder("r")
                .leftJoinAndMapOne("r.FarmersTable", farmers_table_entity_1.FarmersTable, "ml1", "ml1.farmerId=r.farmerId")
                .leftJoinAndMapOne("r.CurrentStockTable", current_stock_table_entity_1.CurrentStockTable, "ml2", "ml2.stockId=r.stockId")
                .where("ml1.farmerId=:farmerId", { farmerId: d.farmerId })
                .select("ml1.farmerVillage")
                .addSelect("r.farmingArea")
                .addSelect("r.")
                .groupBy("ml1.farmerId")
                .getMany();
            let d2 = {
                FarmersTable: d,
            };
        }
    }
};
HousingTableService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(housing_table_entity_1.HousingTable)),
    __param(2, (0, common_1.Inject)((0, common_1.forwardRef)(() => user_collection_service_1.UserCollectionService))),
    __param(3, (0, common_1.Inject)((0, common_1.forwardRef)(() => farmers_table_service_1.FarmersTableService))),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        data_entry_table_service_1.DataEntryTableService,
        user_collection_service_1.UserCollectionService,
        farmers_table_service_1.FarmersTableService])
], HousingTableService);
exports.HousingTableService = HousingTableService;
