"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.InterventionTableService = void 0;
const common_1 = require("@nestjs/common");
const intervention_table_entity_1 = require("./entities/intervention-table.entity");
const typeorm_1 = require("@nestjs/typeorm");
const typeorm_2 = require("typeorm");
const user_collection_service_1 = require("../user-collection/user-collection.service");
const mk_table_service_1 = require("../mk-table/mk-table.service");
const moment_1 = __importDefault(require("moment"));
const intervention_training_table_service_1 = require("../intervention-training-table/intervention-training-table.service");
const sessionmaster_table_service_1 = require("../sessionmaster-table/sessionmaster-table.service");
const pop_table_service_1 = require("../pop-table/pop-table.service");
let InterventionTableService = exports.InterventionTableService = class InterventionTableService {
    constructor(interventionTableRepository, userCollectionService, mkTableService, interventionTrainingTableService, sessionmasterTableService, popTableService) {
        this.interventionTableRepository = interventionTableRepository;
        this.userCollectionService = userCollectionService;
        this.mkTableService = mkTableService;
        this.interventionTrainingTableService = interventionTrainingTableService;
        this.sessionmasterTableService = sessionmasterTableService;
        this.popTableService = popTableService;
    }
    async create(createInterventionTableInput, user, type) {
        let intervention = new intervention_table_entity_1.InterventionTable();
        let activeSession = await this.sessionmasterTableService.findActiveSession();
        intervention.MkTable = await this.mkTableService.findOne(createInterventionTableInput.mkId);
        intervention.UserTable = await this.userCollectionService.findOneByUsername(user.username);
        intervention.year = parseInt((0, moment_1.default)().format("YYYY"));
        intervention.SessionmasterTable = activeSession;
        intervention.entryDate = (0, moment_1.default)().format("DD-MM-YYYY");
        if (type == "training") {
            intervention.trainingSubmitted = "YES";
            intervention.trainingSubmittedTime = (0, moment_1.default)().format("DD-MM-YYYY hh:mm A");
        }
        if (type == "adoption") {
            intervention.adoptionSubmitted = "YES";
            intervention.adoptionSubmittedTime = (0, moment_1.default)().format("DD-MM-YYYY hh:mm A");
        }
        if (type == "production") {
            intervention.productionSubmitted = "YES";
            intervention.productionSubmittedTime = (0, moment_1.default)().format("DD-MM-YYYY hh:mm A");
        }
        let result = await this.interventionTableRepository.save(intervention);
        return result;
    }
    findAll() {
        return `This action returns all interventionTable`;
    }
    async findOne(interventionId) {
        let result = await this.interventionTableRepository.findOne({
            where: {
                interventionId
            },
            relations: {
                UserTable: true,
                InterventionAdoptionTable: {
                    IncomeSourceTable: true,
                    LivelihoodSourceTable: true
                }
            }
        });
        if (result) {
            let trainings = await this.interventionTrainingTableService.getTrainingsByInterventionId(result.interventionId);
            result['InterventionTrainingTable'] = trainings;
        }
        return result;
    }
    async findOneAndUpdateTime(interventionId, type) {
        let result = await this.interventionTableRepository.findOne({
            where: {
                interventionId
            },
            relations: {
                UserTable: true
            }
        });
        if (result) {
            if (type == "training") {
                result.trainingSubmittedTime = (0, moment_1.default)().format("DD-MM-YYYY hh:mm A");
                result.trainingSubmitted = "YES";
            }
            if (type == "adoption") {
                result.adoptionSubmittedTime = (0, moment_1.default)().format("DD-MM-YYYY hh:mm A");
                result.adoptionSubmitted = "YES";
            }
            if (type == "production") {
                result.productionSubmittedTime = (0, moment_1.default)().format("DD-MM-YYYY hh:mm A");
                result.productionSubmitted = "YES";
            }
            let res = await this.interventionTableRepository.save(result);
            return res;
        }
    }
    async findByMkId(mkId) {
        return await this.interventionTableRepository.find({
            where: {
                MkTable: {
                    mkId
                }
            }
        });
    }
    async getTrainingAndProductionReport(sessionId, blockId, type) {
        let ffData = [];
        if (type == "block") {
            ffData = await this.interventionTableRepository.find({
                where: {
                    MkTable: {
                        VillageTable: {
                            GrampanchayatTable: {
                                BlockTable: {
                                    blockId: blockId
                                }
                            }
                        }
                    },
                    SessionmasterTable: {
                        sessionId
                    }
                },
                relations: {
                    InterventionTrainingTable: {
                        PopTable: true
                    }
                }
            });
        }
        if (type == "organisation") {
            ffData = await this.interventionTableRepository.find({
                where: {
                    SessionmasterTable: {
                        sessionId
                    }
                },
                relations: {
                    InterventionTrainingTable: {
                        PopTable: true
                    }
                }
            });
        }
        let InterventionTrainingTable = [];
        for (const el of ffData) {
            InterventionTrainingTable = [...InterventionTrainingTable, ...el.InterventionTrainingTable];
        }
        let trainingData = [];
        let trainingMasterTable = await this.popTableService.findAll();
        for (const tmt of trainingMasterTable) {
            let ff = InterventionTrainingTable.filter((el) => el.PopTable.popId === tmt.popId && el.training == "YES");
            trainingData.push(Object.assign(Object.assign({}, tmt), { name: tmt.popName, count: ff.length }));
        }
        return {
            trainingData
        };
    }
    update(id, updateInterventionTableInput) {
        return `This action updates a #${id} interventionTable`;
    }
    remove(id) {
        return `This action removes a #${id} interventionTable`;
    }
    async getCountByMonth(month) {
        const startOfMonth = (0, moment_1.default)(month).startOf('month').format('YYYY-MM-DD');
        const endOfMonth = (0, moment_1.default)(month).endOf('month').format('YYYY-MM-DD');
        let result = await this.interventionTableRepository
            .createQueryBuilder('fl')
            .andWhere("STR_TO_DATE(fl.entryDate, '%d-%m-%Y') BETWEEN :startDate AND :endDate", {
            endDate: endOfMonth,
            startDate: startOfMonth,
        })
            .getCount();
        return result;
    }
};
exports.InterventionTableService = InterventionTableService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(intervention_table_entity_1.InterventionTable)),
    __param(2, (0, common_1.Inject)((0, common_1.forwardRef)(() => mk_table_service_1.MkTableService))),
    __param(3, (0, common_1.Inject)((0, common_1.forwardRef)(() => intervention_training_table_service_1.InterventionTrainingTableService))),
    __param(4, (0, common_1.Inject)((0, common_1.forwardRef)(() => sessionmaster_table_service_1.SessionmasterTableService))),
    __param(5, (0, common_1.Inject)((0, common_1.forwardRef)(() => pop_table_service_1.PopTableService))),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        user_collection_service_1.UserCollectionService,
        mk_table_service_1.MkTableService,
        intervention_training_table_service_1.InterventionTrainingTableService,
        sessionmaster_table_service_1.SessionmasterTableService,
        pop_table_service_1.PopTableService])
], InterventionTableService);
