import { CreateMkTableInput } from './dto/create-mk-table.input';
import { UpdateMkTableInput } from './dto/update-mk-table.input';
import { MkTable } from './entities/mk-table.entity';
import { Repository } from 'typeorm';
import { RedisPubSub } from 'graphql-redis-subscriptions';
import { VillageTableService } from 'src/village-table/village-table.service';
import { UserCollectionService } from 'src/user-collection/user-collection.service';
import { ShgTableService } from 'src/shg-table/shg-table.service';
import { MkIrrigationTableService } from 'src/mk-irrigation-table/mk-irrigation-table.service';
import { MkIncomeActivityTableService } from 'src/mk-income-activity-table/mk-income-activity-table.service';
import { MkTypeOfLandTableService } from 'src/mk-type-of-land-table/mk-type-of-land-table.service';
import { MkTypeOfToolsTableService } from 'src/mk-type-of-tools-table/mk-type-of-tools-table.service';
import { VulnerabilityCategoryTableService } from 'src/vulnerability-category-table/vulnerability-category-table.service';
import { ProducerGroupsTableService } from 'src/producer-groups-table/producer-groups-table.service';
import { InterventionTableService } from 'src/intervention-table/intervention-table.service';
import { FblpTableService } from 'src/fblp-table/fblp-table.service';
import { MkEntitlementsTableService } from 'src/mk-entitlements-table/mk-entitlements-table.service';
import { IncomeSourceTableService } from 'src/income-source-table/income-source-table.service';
import { LivelihoodSourceTableService } from 'src/livelihood-source-table/livelihood-source-table.service';
import { SchemeMasterSubTableService } from 'src/scheme-master-sub-table/scheme-master-sub-table.service';
import { VoTableService } from 'src/vo-table/vo-table.service';
import { ImportDataTypeMk } from 'src/types/import-data-type-mk.type';
export declare const NEW_MK_ADD = "NEW_MK_ADD";
export declare class MkTableService {
    private readonly mkTableRepository;
    private pubSub;
    private readonly shgTableService;
    private readonly villageTableService;
    private readonly userCollectionService;
    private readonly vulnerabilityCategoryTableService;
    private readonly mkIrrigationTableService;
    private readonly mkIncomeActivityTableService;
    private readonly schemeMasterSubTableService;
    private readonly mkTypeOfLandTableService;
    private readonly mkTypeOfToolsTableService;
    private readonly producerGroupsTableService;
    private readonly interventionTableService;
    private readonly fblpTableService;
    private readonly mkEntitlementsTableService;
    private readonly incomeSourceTableService;
    private readonly livelihoodSourceTableService;
    private readonly voTableService;
    constructor(mkTableRepository: Repository<MkTable>, pubSub: RedisPubSub, shgTableService: ShgTableService, villageTableService: VillageTableService, userCollectionService: UserCollectionService, vulnerabilityCategoryTableService: VulnerabilityCategoryTableService, mkIrrigationTableService: MkIrrigationTableService, mkIncomeActivityTableService: MkIncomeActivityTableService, schemeMasterSubTableService: SchemeMasterSubTableService, mkTypeOfLandTableService: MkTypeOfLandTableService, mkTypeOfToolsTableService: MkTypeOfToolsTableService, producerGroupsTableService: ProducerGroupsTableService, interventionTableService: InterventionTableService, fblpTableService: FblpTableService, mkEntitlementsTableService: MkEntitlementsTableService, incomeSourceTableService: IncomeSourceTableService, livelihoodSourceTableService: LivelihoodSourceTableService, voTableService: VoTableService);
    create(createMkTableInput: CreateMkTableInput, user: any): Promise<{
        status: boolean;
        status_text: string;
        data?: undefined;
    } | {
        status: boolean;
        status_text: string;
        data: MkTable;
    }>;
    getMkCountByShgId(shgId: number): Promise<{
        count: number;
    }>;
    getMkFblpCountByShgId(shgId: number): Promise<{
        count: number;
        fblpCount: number;
    }>;
    checkIfExists(createMkTableInput: CreateMkTableInput): Promise<boolean>;
    findAll(): Promise<MkTable[]>;
    findAllByShg(shgId: number): Promise<MkTable[]>;
    findAllOneSingle(mkId: number): Promise<MkTable>;
    findOne(mkId: number): Promise<MkTable>;
    findOneSingle(mkId: number): Promise<MkTable>;
    update(mkId: number, updateMkTableInput: UpdateMkTableInput, user: any): Promise<{
        status: boolean;
        status_text: string;
        data: MkTable;
    } | {
        status: boolean;
        status_text: string;
        data?: undefined;
    }>;
    getCount(): Promise<number>;
    updateProducerGroup(mkId: number, isMemberOfPg: string, pgId: number): Promise<{
        status: boolean;
        status_text: string;
        data: MkTable;
    }>;
    remove(mkId: number): Promise<{
        status: boolean;
        status_text: string;
    }>;
    getCountByUsername(username: string): Promise<number>;
    getAllMkPaginated(limit: number, page: number, shgId: number, search: string): Promise<{
        data: any;
        total: any;
    }>;
    getAllMkPaginatedStateDistrictBlock(limit: number, page: number, search: string, stateId: number, districtId: number, blockId: number): Promise<{
        data: any;
        total: any;
        page: number;
    }>;
    getAllMkNonPaginatedStateDistrictBlock(stateId: number, districtId: number, blockId: number): Promise<{
        data: any;
        total: any;
        landTypes: import("../land-type-table/entities/land-type-table.entity").LandTypeTable[];
        lsData: import("../livelihood-source-table/entities/livelihood-source-table.entity").LivelihoodSourceTable[];
    }>;
    getCountByMonth(month: string): Promise<any>;
    getTotalMkReport(type: string, stateId: number, districtId: number, blockId: number): Promise<{
        aplBplData: any;
        vulData: any;
        toolsData: any;
        irrigationData: any;
        primarySource: any;
        secondarySource: any;
        incomeActivities: import("../income-source-table/entities/income-source-table.entity").IncomeSourceTable[];
        lsData: import("../livelihood-source-table/entities/livelihood-source-table.entity").LivelihoodSourceTable[];
        bands: any;
        wealthRanking: any;
        schemeList: import("../scheme-master-sub-table/entities/scheme-master-sub-table.entity").SchemeMasterSubTable[];
    }>;
    calculateAvgIncome(stateId: number, districtId: number, blockId: number): Promise<any>;
    calculateLevelIncome(stateId: number, districtId: number, blockId: number): Promise<any[]>;
    getMkListPaginated(limit: number, page: number): Promise<{
        data: any;
        total: any;
        page: number;
    }>;
    createMkShortExcel(data: ImportDataTypeMk, user: any): Promise<MkTable>;
}
