"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PigInsuranceTableService = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const moment_1 = __importDefault(require("moment"));
const animal_entry_table_service_1 = require("../animal-table/animal-entry-table.service");
const animal_entry_table_entity_1 = require("../animal-table/entities/animal-entry-table.entity");
const farmers_table_entity_1 = require("../farmers-table/entities/farmers-table.entity");
const housing_table_entity_1 = require("../housing-table/entities/housing-table.entity");
const housing_table_service_1 = require("../housing-table/housing-table.service");
const sub_division_table_service_1 = require("../sub-division-table/sub-division-table.service");
const user_collection_service_1 = require("../user-collection/user-collection.service");
const typeorm_2 = require("typeorm");
const pig_insurance_table_entity_1 = require("./entities/pig-insurance-table.entity");
const data_entry_table_service_1 = require("../data-entry-table/data-entry-table.service");
const GlobalConstants_1 = require("../helper/GlobalConstants");
let PigInsuranceTableService = class PigInsuranceTableService {
    constructor(pigInsuranceTableRepository, userCollectionService, dataEntryTableService, housingTableService, animalEntryTableService, subDivisionTableService) {
        this.pigInsuranceTableRepository = pigInsuranceTableRepository;
        this.userCollectionService = userCollectionService;
        this.dataEntryTableService = dataEntryTableService;
        this.housingTableService = housingTableService;
        this.animalEntryTableService = animalEntryTableService;
        this.subDivisionTableService = subDivisionTableService;
    }
    async create(createPigInsuranceTableInput, user) {
        let userTable = await this.userCollectionService.getOneByUserId(user.userId);
        if (Array.isArray(createPigInsuranceTableInput.animalData) && createPigInsuranceTableInput.animalData.length > 0) {
            for (const el of createPigInsuranceTableInput.animalData) {
                let checkIfExists = await this.checkIfExists(el.animalId);
                if (checkIfExists) {
                    return {
                        status: false,
                        status_text: "Already Exists",
                        data: null
                    };
                }
                else {
                    let ht = new pig_insurance_table_entity_1.PigInsuranceTable();
                    let animalTable = await this.animalEntryTableService.findByAnimalId(el.animalId);
                    let housingTable = await this.housingTableService.findOne(createPigInsuranceTableInput.housingId);
                    ht.UserTable = userTable;
                    ht.AnimalTable = animalTable;
                    ht.HousingTable = housingTable;
                    ht.pigInsuredDate = createPigInsuranceTableInput.pigInsuredDate;
                    ht.insuranceCompany = createPigInsuranceTableInput.insuranceCompany;
                    ht.premiumAmount = createPigInsuranceTableInput.premiumAmount;
                    ht.renewalDate = createPigInsuranceTableInput.renewalDate;
                    ht.settlementClaim = createPigInsuranceTableInput.settlementClaim;
                    let result = await this.pigInsuranceTableRepository.save(ht);
                }
            }
            this.dataEntryTableService.create({ type: GlobalConstants_1.CREATE, typeOfEntry: GlobalConstants_1.INSURANCE_ADD, remarks: "New insurance added" }, user);
            return {
                status: true,
                status_text: "Successfully Submitted",
                data: null
            };
        }
        else
            return {
                status: false,
                status_text: "Invalid Data",
                data: null
            };
    }
    async checkIfExists(animalId) {
        let result = await this.pigInsuranceTableRepository.count({
            where: {
                AnimalTable: {
                    animalId
                }
            }
        });
        if (result == 0) {
            return false;
        }
        else {
            return true;
        }
    }
    async findByFarmerId(animalId) {
        return await this.pigInsuranceTableRepository.find({
            where: {
                AnimalTable: {
                    animalId
                }
            },
            relations: {
                UserTable: true,
                AnimalTable: {
                    HousingTable: {
                        FarmersTable: true
                    }
                }
            }
        });
    }
    async findPigInsuranceByHousingd(housingId) {
        return await this.pigInsuranceTableRepository.find({
            where: {
                AnimalTable: {
                    HousingTable: {
                        housingId
                    }
                },
            },
            relations: {
                UserTable: true,
                AnimalTable: {
                    HousingTable: true
                }
            }
        });
    }
    async findAll() {
        return await this.pigInsuranceTableRepository.find({
            relations: {
                UserTable: true,
                AnimalTable: {
                    HousingTable: {
                        FarmersTable: true
                    }
                }
            }
        });
    }
    async findOne(pigInsuranceId) {
        return await this.pigInsuranceTableRepository.findOne({
            where: {
                pigInsuranceId
            },
            relations: {
                UserTable: true,
                AnimalTable: {
                    HousingTable: {
                        FarmersTable: true
                    }
                }
            }
        });
    }
    async findOneByFarmerId(farmerId) {
        let result = await this.pigInsuranceTableRepository.find({
            where: {
                AnimalTable: {
                    HousingTable: {
                        FarmersTable: {
                            farmerId
                        }
                    }
                }
            }
        });
        return result;
    }
    async update(pigInsuranceId, updatePigInsuranceTableInput, user) {
        let ht = await this.pigInsuranceTableRepository.findOne({
            where: {
                pigInsuranceId
            }
        });
        if (!ht) {
            return {
                status: false,
                status_text: "Not found",
                data: null
            };
        }
        else {
            let userTable = await this.userCollectionService.getOneByUserId(user.userId);
            ht.UserTable = userTable;
            ht.pigInsuredDate = updatePigInsuranceTableInput.pigInsuredDate;
            ht.insuranceCompany = updatePigInsuranceTableInput.insuranceCompany;
            ht.premiumAmount = updatePigInsuranceTableInput.premiumAmount;
            ht.renewalDate = updatePigInsuranceTableInput.renewalDate;
            ht.settlementClaim = updatePigInsuranceTableInput.settlementClaim;
            let result = await this.pigInsuranceTableRepository.save(ht);
            this.dataEntryTableService.create({ type: GlobalConstants_1.UPDATE, typeOfEntry: GlobalConstants_1.INSURANCE_UPDATE, remarks: " insurance updated" }, user);
            return {
                status: true,
                status_text: "Successfully Updated",
                data: result
            };
        }
    }
    async remove(pigInsuranceId, user) {
        let ht = await this.pigInsuranceTableRepository.findOne({
            where: {
                pigInsuranceId
            }
        });
        if (!ht) {
            return {
                status: false,
                status_text: "Not found",
                data: null
            };
        }
        else {
            await this.pigInsuranceTableRepository.remove(ht);
            this.dataEntryTableService.create({ type: GlobalConstants_1.DELETE, typeOfEntry: GlobalConstants_1.INSURANCE_DELETE, remarks: " insurance deleted" }, user);
            return {
                status: true,
                status_text: "Successfully Deleted",
                data: null
            };
        }
    }
    async getInsuranceDetailsOfOneAnimal(animalId) {
        let result = await this.pigInsuranceTableRepository.createQueryBuilder("r")
            .leftJoinAndMapOne("r.AnimalTable", animal_entry_table_entity_1.AnimalEntryTable, "ml1", "ml1.animalId=r.animalId")
            .where("r.animalId=:animalId", { animalId: animalId })
            .getMany();
        return result;
    }
    async getInsuranceDetailsDistrictWise(districtId) {
        const subdivisions = await this.subDivisionTableService.fetchSubDivisionByDistrictId(districtId);
        let data = [];
        for (const d of subdivisions) {
            let d1 = await this.pigInsuranceTableRepository.createQueryBuilder("r")
                .leftJoinAndMapOne("r.AnimalTable", animal_entry_table_entity_1.AnimalEntryTable, "ml3", "ml3.animalId=r.animalId")
                .leftJoinAndMapOne("r.HousingTable", housing_table_entity_1.HousingTable, "ml1", "ml1.housingId=ml3.housingId")
                .leftJoinAndMapOne("ml1.FarmersTable", farmers_table_entity_1.FarmersTable, "ml2", "ml2.farmerId=ml1.farmerId")
                .where("ml2.subDivisionId=:subDivisionId", { subDivisionId: d.subDivisionId })
                .select("MIN(ml2.farmerName) as farmerName")
                .addSelect("COUNT(r.pigInsuranceId)", "insuredPigs")
                .groupBy("ml2.subDivisionId")
                .getRawOne();
            console.log('d1', d1);
            let d2 = {
                SubDivisionTable: d,
                farmerName: d1 ? d1.farmerName : "",
                insuredPigs: d1 ? d1.insuredPigs : 0
            };
            data.push(d2);
        }
        return data;
    }
    checkDaysLeft(date) {
        let d = (0, moment_1.default)(date, 'YYYY-MM-DD').diff((0, moment_1.default)(), 'days');
        if (d < 0) {
            return `has expired ${Math.abs(d)} days ago.`;
        }
        else {
            return `is approaching. you have ${d} days left for renewal.`;
        }
    }
    async getExpiredOrNearlyExpiredData(farmerId, days) {
        let today = (0, moment_1.default)().format("YYYY-MM-DD");
        let result = await this.pigInsuranceTableRepository.createQueryBuilder("r")
            .leftJoinAndMapOne("r.AnimalTable", animal_entry_table_entity_1.AnimalEntryTable, "ml3", "ml3.animalId=r.animalId")
            .leftJoinAndMapOne("r.HousingTable", housing_table_entity_1.HousingTable, "ml1", "ml1.housingId=ml3.housingId")
            .leftJoinAndMapOne("ml1.FarmersTable", farmers_table_entity_1.FarmersTable, "ml2", "ml2.farmerId=ml1.farmerId")
            .where("ml2.farmerId=:farmerId", { farmerId: farmerId })
            .andWhere("STR_TO_DATE(r.renewalDate, '%Y-%m-%d') -  STR_TO_DATE(:today, '%Y-%m-%d') < :days", { days: days, today: today })
            .getMany();
        let d = [];
        result.map((el) => {
            let d1 = {
                description: `Renewal date of insurance of Animal#${el.AnimalTable.animalId} ${this.checkDaysLeft(el.renewalDate)}`,
                link: `/admin/farmers/single/details?farmerId=${farmerId}&step=pig_insurance`
            };
            d.push(d1);
        });
        return d;
    }
};
PigInsuranceTableService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(pig_insurance_table_entity_1.PigInsuranceTable)),
    __param(3, (0, common_1.Inject)((0, common_1.forwardRef)(() => housing_table_service_1.HousingTableService))),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        user_collection_service_1.UserCollectionService,
        data_entry_table_service_1.DataEntryTableService,
        housing_table_service_1.HousingTableService,
        animal_entry_table_service_1.AnimalEntryTableService,
        sub_division_table_service_1.SubDivisionTableService])
], PigInsuranceTableService);
exports.PigInsuranceTableService = PigInsuranceTableService;
