"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RationTableService = exports.NEW_RATION_ADD = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const graphql_redis_subscriptions_1 = require("graphql-redis-subscriptions");
const brand_table_service_1 = require("../brand-table/brand-table.service");
const GlobalConstants_1 = require("../helper/GlobalConstants");
const ration_sub_table_service_1 = require("../ration-sub-table/ration-sub-table.service");
const typeorm_2 = require("typeorm");
const ration_table_entity_1 = require("./entities/ration-table.entity");
exports.NEW_RATION_ADD = "NEW_RATION_ADD";
let RationTableService = class RationTableService {
    constructor(rationTableRepository, pubSub, brandTableService, rationSubTableService) {
        this.rationTableRepository = rationTableRepository;
        this.pubSub = pubSub;
        this.brandTableService = brandTableService;
        this.rationSubTableService = rationSubTableService;
    }
    async create(createRationTableInput) {
        let checkIfExists = await this.checkIfExists(createRationTableInput);
        if (checkIfExists) {
            return {
                status: false,
                status_text: "Already Exists",
                data: null
            };
        }
        else {
            let dd = new ration_table_entity_1.RationTable();
            dd.rationName = createRationTableInput.rationName;
            dd.commonName = createRationTableInput.commonName;
            let brand = await this.brandTableService.findOne(createRationTableInput.brandId);
            dd.BrandTable = brand;
            let result = await this.rationTableRepository.save(dd);
            await this.rationSubTableService.create(createRationTableInput.ingredient, result);
            this.pubSub.publish(exports.NEW_RATION_ADD, 1);
            return {
                status: true,
                status_text: "Successfully Created",
                data: result
            };
        }
    }
    async checkIfExists(createRationTableInput) {
        let result = await this.rationTableRepository.count({
            where: {
                rationName: createRationTableInput.rationName,
                BrandTable: {
                    brandId: createRationTableInput.brandId
                }
            }
        });
        if (result == 0) {
            return false;
        }
        else {
            return true;
        }
    }
    async findAll() {
        let result = await this.rationTableRepository.find({
            relations: {
                BrandTable: true,
            }
        });
        let index = 0;
        for (const el of result) {
            result[index]['RationSubTable'] = await this.rationSubTableService.fetchSubTable(el);
            index++;
        }
        return result;
    }
    async findOne(rationId) {
        let result = await this.rationTableRepository.findOne({
            where: {
                rationId
            },
            relations: ["BrandTable"]
        });
        if (result) {
            result['RationSubTable'] = await this.rationSubTableService.fetchSubTable(result);
        }
        return result;
    }
    async update(rationId, updateRationTableInput) {
        let result = await this.rationTableRepository.findOne({
            where: {
                rationId: rationId
            }
        });
        if (result) {
            let brand = await this.brandTableService.findOne(updateRationTableInput.brandId);
            result.BrandTable = brand;
            result.rationName = updateRationTableInput.rationName;
            result.commonName = updateRationTableInput.commonName;
            let rr = await this.rationTableRepository.save(result);
            await this.rationSubTableService.create(updateRationTableInput.ingredient, rr);
            this.pubSub.publish(exports.NEW_RATION_ADD, 1);
            return {
                status: true,
                status_text: "Successfully Updated",
                data: rr
            };
        }
        else {
            return {
                status: false,
                status_text: "Not found",
                data: null
            };
        }
    }
    async remove(rationId) {
        let result = await this.rationTableRepository.findOne({
            where: {
                rationId: rationId
            }
        });
        if (result) {
            await this.rationSubTableService.deleteSpecific(result);
            await this.rationTableRepository.remove(result);
            this.pubSub.publish(exports.NEW_RATION_ADD, 1);
            return {
                status: true,
                status_text: "Successfully Removed",
                data: null
            };
        }
        else {
            return {
                status: false,
                status_text: "Not Found",
                data: null
            };
        }
    }
};
RationTableService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(ration_table_entity_1.RationTable)),
    __param(1, (0, common_1.Inject)(GlobalConstants_1.PUB_SUB)),
    __param(2, (0, common_1.Inject)((0, common_1.forwardRef)(() => brand_table_service_1.BrandTableService))),
    __param(3, (0, common_1.Inject)((0, common_1.forwardRef)(() => ration_sub_table_service_1.RationSubTableService))),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        graphql_redis_subscriptions_1.RedisPubSub,
        brand_table_service_1.BrandTableService,
        ration_sub_table_service_1.RationSubTableService])
], RationTableService);
exports.RationTableService = RationTableService;
