"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SaleEntryTableService = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const user_collection_service_1 = require("../user-collection/user-collection.service");
const typeorm_2 = require("typeorm");
const housing_table_service_1 = require("../housing-table/housing-table.service");
const breed_table_service_1 = require("../breed-table/breed-table.service");
const current_stock_table_service_1 = require("../current-stock/current-stock-table.service");
const stock_balance_table_service_1 = require("../stock-balance/stock-balance-table.service");
const ledger_entry_table_service_1 = require("../ledger-table/ledger-entry-table.service");
const animal_entry_table_service_1 = require("../animal-table/animal-entry-table.service");
const sale_entry_table_entity_1 = require("./entities/sale-entry-table.entity");
const sale_entry_sub_table_service_1 = require("../sale-entry-sub-table/sale-entry-sub-table.service");
const data_entry_table_service_1 = require("../data-entry-table/data-entry-table.service");
const GlobalConstants_1 = require("../helper/GlobalConstants");
let SaleEntryTableService = class SaleEntryTableService {
    constructor(saleEntryTableRepository, dataEntryTableService, userCollectionService, housingTableService, breedTableService, currentStockTableService, stockBalanceTableService, ledgerEntryTableService, animalEntryTableService, saleEntrySubTableService) {
        this.saleEntryTableRepository = saleEntryTableRepository;
        this.dataEntryTableService = dataEntryTableService;
        this.userCollectionService = userCollectionService;
        this.housingTableService = housingTableService;
        this.breedTableService = breedTableService;
        this.currentStockTableService = currentStockTableService;
        this.stockBalanceTableService = stockBalanceTableService;
        this.ledgerEntryTableService = ledgerEntryTableService;
        this.animalEntryTableService = animalEntryTableService;
        this.saleEntrySubTableService = saleEntrySubTableService;
    }
    async create(createSaleEntryTableInput, user) {
        let ht = new sale_entry_table_entity_1.SaleEntryTable();
        let userTable = await this.userCollectionService.getOneByUserId(user.userId);
        if (Array.isArray(createSaleEntryTableInput.animalData) && createSaleEntryTableInput.animalData.length > 0) {
            ht.dateOfEntry = createSaleEntryTableInput.dateOfEntry;
            ht.remarks = createSaleEntryTableInput.remarks;
            ht.animalData = createSaleEntryTableInput.animalData;
            ht.UserTable = userTable;
            let rr = await this.saleEntryTableRepository.save(ht);
            await this.saleEntrySubTableService.create(Object.assign(Object.assign({}, createSaleEntryTableInput), { saleEntryId: rr.saleEntryId }), user);
            this.dataEntryTableService.create({ type: GlobalConstants_1.CREATE, typeOfEntry: GlobalConstants_1.SALE_ADD, remarks: "New sale added" }, user);
            return {
                status: true,
                status_text: "Successfully Submitted.",
                data: rr,
            };
        }
        else {
            return {
                status: false,
                status_text: "Not found.",
                data: null,
            };
        }
    }
    async findAll() {
        return await this.saleEntryTableRepository.find({
            relations: {
                SaleEntrySubTable: {
                    AnimalTable: {
                        HousingTable: {
                            FarmersTable: true
                        }
                    }
                }
            }
        });
    }
    async findOne(saleEntryId) {
        return await this.saleEntryTableRepository.findOne({
            where: {
                saleEntryId,
            },
            relations: {
                SaleEntrySubTable: {
                    AnimalTable: {
                        HousingTable: {
                            FarmersTable: true
                        }
                    }
                }
            }
        });
    }
    async findByFarmerId(farmerId) {
        return await this.saleEntryTableRepository.find({
            where: {
                SaleEntrySubTable: {
                    AnimalTable: {
                        HousingTable: {
                            FarmersTable: {
                                farmerId
                            }
                        }
                    },
                }
            },
            relations: {
                SaleEntrySubTable: {
                    AnimalTable: {
                        HousingTable: {
                            FarmersTable: true
                        }
                    }
                }
            }
        });
    }
    async findSaleEntryByHousingId(housingId) {
        let result = await this.saleEntrySubTableService.findAllSaleEntryByHousingId(housingId);
        return result;
    }
};
SaleEntryTableService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(sale_entry_table_entity_1.SaleEntryTable)),
    __param(2, (0, common_1.Inject)((0, common_1.forwardRef)(() => user_collection_service_1.UserCollectionService))),
    __param(3, (0, common_1.Inject)((0, common_1.forwardRef)(() => housing_table_service_1.HousingTableService))),
    __param(4, (0, common_1.Inject)((0, common_1.forwardRef)(() => breed_table_service_1.BreedTableService))),
    __param(5, (0, common_1.Inject)((0, common_1.forwardRef)(() => current_stock_table_service_1.CurrentStockTableService))),
    __param(6, (0, common_1.Inject)((0, common_1.forwardRef)(() => stock_balance_table_service_1.StockBalanceTableService))),
    __param(7, (0, common_1.Inject)((0, common_1.forwardRef)(() => ledger_entry_table_service_1.LedgerEntryTableService))),
    __param(8, (0, common_1.Inject)((0, common_1.forwardRef)(() => animal_entry_table_service_1.AnimalEntryTableService))),
    __param(9, (0, common_1.Inject)((0, common_1.forwardRef)(() => sale_entry_sub_table_service_1.SaleEntrySubTableService))),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        data_entry_table_service_1.DataEntryTableService,
        user_collection_service_1.UserCollectionService,
        housing_table_service_1.HousingTableService,
        breed_table_service_1.BreedTableService,
        current_stock_table_service_1.CurrentStockTableService,
        stock_balance_table_service_1.StockBalanceTableService,
        ledger_entry_table_service_1.LedgerEntryTableService,
        animal_entry_table_service_1.AnimalEntryTableService,
        sale_entry_sub_table_service_1.SaleEntrySubTableService])
], SaleEntryTableService);
exports.SaleEntryTableService = SaleEntryTableService;
