"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SaleReadyTableService = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const animal_entry_table_service_1 = require("../animal-table/animal-entry-table.service");
const housing_table_service_1 = require("../housing-table/housing-table.service");
const user_collection_service_1 = require("../user-collection/user-collection.service");
const typeorm_2 = require("typeorm");
const sale_ready_table_entity_1 = require("./entities/sale-ready-table.entity");
let SaleReadyTableService = class SaleReadyTableService {
    constructor(saleReadyTableRepository, userCollectionService, housingTableService, animalEntryTableService) {
        this.saleReadyTableRepository = saleReadyTableRepository;
        this.userCollectionService = userCollectionService;
        this.housingTableService = housingTableService;
        this.animalEntryTableService = animalEntryTableService;
    }
    async create(createSaleReadyTableInput, user) {
        let ht = new sale_ready_table_entity_1.SaleReadyTable();
        let userTable = await this.userCollectionService.getOneByUserId(user.userId);
        if (Array.isArray(createSaleReadyTableInput.animalData) && createSaleReadyTableInput.animalData.length > 0) {
            ht.dateOfEntry = createSaleReadyTableInput.dateOfEntry;
            ht.purpose = createSaleReadyTableInput.purpose;
            ht.animalData = createSaleReadyTableInput.animalData;
            ht.UserTable = userTable;
            let rr = await this.saleReadyTableRepository.save(ht);
            return {
                status: true,
                status_text: "Successfully Submitted.",
                data: rr,
            };
        }
        else {
            return {
                status: false,
                status_text: "Not found.",
                data: null,
            };
        }
    }
    async findAll() {
        return await this.saleReadyTableRepository.find({
            relations: {
                AnimalTable: {
                    HousingTable: {
                        FarmersTable: true
                    }
                }
            }
        });
    }
    async findOne(saleReadyId) {
        return await this.saleReadyTableRepository.findOne({
            where: {
                saleReadyId,
            },
            relations: {
                AnimalTable: {
                    HousingTable: {
                        FarmersTable: true
                    }
                }
            }
        });
    }
    async findByFarmerId(farmerId) {
        return await this.saleReadyTableRepository.find({
            where: {
                AnimalTable: {
                    HousingTable: {
                        FarmersTable: {
                            farmerId
                        }
                    }
                },
            },
            relations: {
                AnimalTable: {
                    HousingTable: {
                        FarmersTable: true
                    }
                }
            }
        });
    }
    async findAllSaleReadyByHousingId(housingId) {
        return await this.saleReadyTableRepository.find({
            where: {
                AnimalTable: {
                    HousingTable: {
                        housingId
                    }
                }
            },
            relations: {
                AnimalTable: {
                    HousingTable: {
                        FarmersTable: true
                    }
                }
            }
        });
    }
    async update(id, updateSaleReadyTableInput) {
        return `This action updates a #${id} saleReadyTable`;
    }
    async remove(saleReadyId) {
        let ht = await this.saleReadyTableRepository.findOne({
            where: {
                saleReadyId
            }
        });
        if (!ht) {
            return {
                status: false,
                status_text: "Not found",
                data: null
            };
        }
        else {
            await this.saleReadyTableRepository.remove(ht);
            return {
                status: true,
                status_text: "Successfully Deleted",
                data: null
            };
        }
    }
};
SaleReadyTableService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(sale_ready_table_entity_1.SaleReadyTable)),
    __param(1, (0, common_1.Inject)((0, common_1.forwardRef)(() => user_collection_service_1.UserCollectionService))),
    __param(2, (0, common_1.Inject)((0, common_1.forwardRef)(() => housing_table_service_1.HousingTableService))),
    __param(3, (0, common_1.Inject)((0, common_1.forwardRef)(() => animal_entry_table_service_1.AnimalEntryTableService))),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        user_collection_service_1.UserCollectionService,
        housing_table_service_1.HousingTableService,
        animal_entry_table_service_1.AnimalEntryTableService])
], SaleReadyTableService);
exports.SaleReadyTableService = SaleReadyTableService;
