"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.StockEntryTableService = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const user_collection_service_1 = require("../user-collection/user-collection.service");
const typeorm_2 = require("typeorm");
const moment_1 = __importDefault(require("moment"));
const stock_entry_table_entity_1 = require("./entities/stock-entry-table.entity");
const housing_table_service_1 = require("../housing-table/housing-table.service");
const breed_table_service_1 = require("../breed-table/breed-table.service");
const current_stock_table_service_1 = require("../current-stock/current-stock-table.service");
const stock_balance_table_service_1 = require("../stock-balance/stock-balance-table.service");
const ledger_entry_table_service_1 = require("../ledger-table/ledger-entry-table.service");
const animal_entry_table_service_1 = require("../animal-table/animal-entry-table.service");
const new_enum_1 = require("../enum/new-enum");
const disease_table_service_1 = require("../disease-table/disease-table.service");
const GlobalConstants_1 = require("../helper/GlobalConstants");
const data_entry_table_service_1 = require("../data-entry-table/data-entry-table.service");
let StockEntryTableService = class StockEntryTableService {
    constructor(stockEntryTableRepository, dataEntryTableService, userCollectionService, housingTableService, breedTableService, currentStockTableService, stockBalanceTableService, ledgerEntryTableService, animalEntryTableService, diseaseTableService) {
        this.stockEntryTableRepository = stockEntryTableRepository;
        this.dataEntryTableService = dataEntryTableService;
        this.userCollectionService = userCollectionService;
        this.housingTableService = housingTableService;
        this.breedTableService = breedTableService;
        this.currentStockTableService = currentStockTableService;
        this.stockBalanceTableService = stockBalanceTableService;
        this.ledgerEntryTableService = ledgerEntryTableService;
        this.animalEntryTableService = animalEntryTableService;
        this.diseaseTableService = diseaseTableService;
    }
    async create(createStockEntryTableInput, user) {
        if (createStockEntryTableInput.qtyMale +
            createStockEntryTableInput.qtyFemale ===
            createStockEntryTableInput.qtyTotal &&
            createStockEntryTableInput.avgWeight > 0) {
            let ht = new stock_entry_table_entity_1.StockEntryTable();
            let userTable = await this.userCollectionService.getOneByUserId(user.userId);
            let housingTable = await this.housingTableService.findOne(createStockEntryTableInput.housingId);
            let breedTable = await this.breedTableService.findOne(createStockEntryTableInput.breedId);
            let totalWeight = createStockEntryTableInput.qtyTotal *
                createStockEntryTableInput.avgWeight;
            let temp = Object.assign(Object.assign({}, createStockEntryTableInput), { totalWeight: totalWeight });
            let cs = await this.currentStockTableService.create(temp);
            if (cs.status) {
                ht.UserTable = userTable;
                ht.HousingTable = housingTable;
                ht.BreedTable = breedTable;
                ht.dateOfEntry = createStockEntryTableInput.dateOfEntry;
                ht.entryType = createStockEntryTableInput.entryType;
                ht.ageGroup = createStockEntryTableInput.ageGroup;
                ht.qtyMale = createStockEntryTableInput.qtyMale;
                ht.qtyFemale = createStockEntryTableInput.qtyFemale;
                ht.qtyTotal = createStockEntryTableInput.qtyTotal;
                ht.avgWeight = createStockEntryTableInput.avgWeight;
                ht.remarks = createStockEntryTableInput.remarks;
                let sb = await this.stockBalanceTableService.create(temp);
                if (sb.status) {
                    let result = await this.stockEntryTableRepository.save(ht);
                    let temp = Object.assign(Object.assign({}, createStockEntryTableInput), { age: createStockEntryTableInput.ageGroup, gender: "", earTag: "", weight: createStockEntryTableInput.avgWeight, referenceId: result.stockEntryId });
                    let lg = await this.ledgerEntryTableService.create(temp, user);
                    if (lg.status) {
                        let animal = await this.animalEntryTableService.multipleEntry(temp, user, createStockEntryTableInput.qtyMale, createStockEntryTableInput.qtyFemale, createStockEntryTableInput.qtyTotal, createStockEntryTableInput.sowId, createStockEntryTableInput.boarId, cs.data.stockId);
                        if (animal.status) {
                            this.dataEntryTableService.create({ type: GlobalConstants_1.CREATE, typeOfEntry: GlobalConstants_1.LIVE_STOCK_ADD, remarks: "New stock added" }, user);
                            return {
                                status: true,
                                status_text: "Successfully Submitted",
                                data: result,
                            };
                        }
                    }
                }
            }
        }
        else {
            return {
                status: false,
                status_text: "Invalid Quantity",
                data: null,
            };
        }
    }
    async deceaseEntry(createDeceaseEntryInput, user) {
        let result = await this.animalEntryTableService.findByAnimalId(createDeceaseEntryInput.animalId);
        if (result && result.isActive == 1) {
            let temp = {
                housingId: result.HousingTable.housingId,
                breedId: result.BreedTable.breedId,
                ageGroup: result.age,
                dateOfEntry: createDeceaseEntryInput.dateOfEntry,
                qtyMale: result.gender == "MALE" ? -1 : 0,
                qtyFemale: result.gender == "FEMALE" ? -1 : 0,
                qtyTotal: -1,
                totalWeight: -result.weight,
                entryType: "DECEASE",
                referenceId: result.animalId,
            };
            await this.stockBalanceTableService.create(temp);
            await this.currentStockTableService.create(temp);
            let temp1 = {
                housingId: result.HousingTable.housingId,
                breedId: result.BreedTable.breedId,
                ageGroup: result.age,
                dateOfEntry: createDeceaseEntryInput.dateOfEntry,
                qtyMale: result.gender == "MALE" ? 1 : 0,
                qtyFemale: result.gender == "FEMALE" ? 1 : 0,
                qtyTotal: 1,
                avgWeight: result.weight,
                entryType: "DECEASE",
                referenceId: result.animalId,
            };
            await this.ledgerEntryTableService.create(temp1, user);
            let userTable = await this.userCollectionService.getOneByUserId(user.userId);
            let housingTable = await this.housingTableService.findOne(result.HousingTable.housingId);
            let breedTable = await this.breedTableService.findOne(result.BreedTable.breedId);
            let diseaseTable = null;
            if (createDeceaseEntryInput.diseaseId) {
                diseaseTable = await this.diseaseTableService.findOne(createDeceaseEntryInput.diseaseId);
            }
            let ht = new stock_entry_table_entity_1.StockEntryTable();
            ht.UserTable = userTable;
            ht.HousingTable = housingTable;
            ht.BreedTable = breedTable;
            ht.DiseaseTable = diseaseTable;
            ht.dateOfEntry = createDeceaseEntryInput.dateOfEntry;
            ht.entryType = new_enum_1.StockEntryType.DECEASE;
            ht.ageGroup = result.age;
            (ht.qtyMale = result.gender == "MALE" ? -1 : 0),
                (ht.qtyFemale = result.gender == "FEMALE" ? -1 : 0),
                (ht.qtyTotal = 1),
                (ht.avgWeight = result.weight),
                (ht.remarks = createDeceaseEntryInput.remarks),
                await this.stockEntryTableRepository.save(ht);
            await this.animalEntryTableService.deceaseEntry(result.animalId, 0);
            return {
                status: true,
                status_text: "Successfully Updated",
                data: null,
            };
        }
        else {
            return {
                status: false,
                status_text: "Not found",
                data: null,
            };
        }
    }
    async findByHousingId(housingId) {
        return await this.stockEntryTableRepository.find({
            where: {
                HousingTable: {
                    housingId,
                },
            },
            relations: [
                "UserTable",
                "BreedTable",
                "HousingTable",
                "HousingTable.FarmersTable",
                "DiseaseTable",
            ],
        });
    }
    async findAll() {
        return await this.stockEntryTableRepository.find({
            relations: [
                "UserTable",
                "BreedTable",
                "HousingTable",
                "HousingTable.FarmersTable",
                "DiseaseTable",
            ],
        });
    }
    async findOne(stockEntryId) {
        return await this.stockEntryTableRepository.findOne({
            where: {
                stockEntryId,
            },
            relations: [
                "UserTable",
                "BreedTable",
                "HousingTable",
                "HousingTable.FarmersTable",
                "DiseaseTable",
            ],
        });
    }
    async findMortalityRate(farmerId) {
        let d1 = (0, moment_1.default)().format("YYYY") + "-01-01";
        let d2 = (0, moment_1.default)().format("YYYY") + "-12-31";
        let result = 0;
        if (farmerId != null && farmerId != undefined) {
            let dead = await this.stockEntryTableRepository
                .find({
                select: ["qtyTotal"],
                where: {
                    HousingTable: {
                        FarmersTable: {
                            farmerId,
                        },
                    },
                    entryType: new_enum_1.StockEntryType.DECEASE,
                    dateOfEntry: (0, typeorm_2.Between)(d1, d2),
                },
            })
                .then((res) => res.reduce((prev, curr) => prev + curr.qtyTotal, 0));
            let birth = await this.stockEntryTableRepository
                .find({
                select: ["qtyTotal"],
                where: {
                    HousingTable: {
                        FarmersTable: {
                            farmerId,
                        },
                    },
                    entryType: new_enum_1.StockEntryType.BIRTH,
                    dateOfEntry: (0, typeorm_2.Between)(d1, d2),
                },
            })
                .then((res) => res.reduce((prev, curr) => prev + curr.qtyTotal, 0));
            if (birth > 0)
                result = dead / (birth * 1000);
        }
        else {
            let dead = await this.stockEntryTableRepository
                .find({
                select: ["qtyTotal"],
                where: {
                    entryType: new_enum_1.StockEntryType.DECEASE,
                    dateOfEntry: (0, typeorm_2.Between)(d1, d2),
                },
            })
                .then((res) => res.reduce((prev, curr) => prev + curr.qtyTotal, 0));
            let birth = await this.stockEntryTableRepository
                .find({
                select: ["qtyTotal"],
                where: {
                    entryType: new_enum_1.StockEntryType.BIRTH,
                    dateOfEntry: (0, typeorm_2.Between)(d1, d2),
                },
            })
                .then((res) => res.reduce((prev, curr) => prev + curr.qtyTotal, 0));
            if (birth > 0)
                result = dead / (birth * 1000);
        }
        return {
            status: true,
            data: result.toFixed(3),
        };
    }
    async findMortalityRateWithDisease(farmerId) {
        let d1 = (0, moment_1.default)().format("YYYY") + "-01-01";
        let d2 = (0, moment_1.default)().format("YYYY") + "-12-31";
        let data = [];
        let disease_table = await this.diseaseTableService.findAll();
        if (Array.isArray(disease_table) && disease_table.length > 0) {
            for (const el of disease_table) {
                let result = 0;
                if (farmerId != null && farmerId != undefined) {
                    let dead = await this.stockEntryTableRepository
                        .find({
                        select: ["qtyTotal"],
                        where: {
                            HousingTable: {
                                FarmersTable: {
                                    farmerId,
                                },
                            },
                            DiseaseTable: {
                                diseaseTableId: el.diseaseTableId,
                            },
                            entryType: new_enum_1.StockEntryType.DECEASE,
                            dateOfEntry: (0, typeorm_2.Between)(d1, d2),
                        },
                    })
                        .then((res) => res.reduce((prev, curr) => prev + curr.qtyTotal, 0));
                    let birth = await this.stockEntryTableRepository
                        .find({
                        select: ["qtyTotal"],
                        where: {
                            HousingTable: {
                                FarmersTable: {
                                    farmerId,
                                },
                            },
                            DiseaseTable: {
                                diseaseTableId: el.diseaseTableId,
                            },
                            entryType: new_enum_1.StockEntryType.BIRTH,
                            dateOfEntry: (0, typeorm_2.Between)(d1, d2),
                        },
                    })
                        .then((res) => res.reduce((prev, curr) => prev + curr.qtyTotal, 0));
                    if (birth > 0)
                        result = dead / (birth * 1000);
                }
                else {
                    let dead = await this.stockEntryTableRepository
                        .find({
                        select: ["qtyTotal"],
                        where: {
                            DiseaseTable: {
                                diseaseTableId: el.diseaseTableId,
                            },
                            entryType: new_enum_1.StockEntryType.DECEASE,
                            dateOfEntry: (0, typeorm_2.Between)(d1, d2),
                        },
                    })
                        .then((res) => res.reduce((prev, curr) => prev + curr.qtyTotal, 0));
                    let birth = await this.stockEntryTableRepository
                        .find({
                        select: ["qtyTotal"],
                        where: {
                            DiseaseTable: {
                                diseaseTableId: el.diseaseTableId,
                            },
                            entryType: new_enum_1.StockEntryType.BIRTH,
                            dateOfEntry: (0, typeorm_2.Between)(d1, d2),
                        },
                    })
                        .then((res) => res.reduce((prev, curr) => prev + curr.qtyTotal, 0));
                    if (birth > 0)
                        result = dead / (birth * 1000);
                }
                data.push({
                    diseaseTable: el,
                    mortalityRate: result.toFixed(3),
                });
            }
        }
        return { data };
    }
};
StockEntryTableService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(stock_entry_table_entity_1.StockEntryTable)),
    __param(2, (0, common_1.Inject)((0, common_1.forwardRef)(() => user_collection_service_1.UserCollectionService))),
    __param(3, (0, common_1.Inject)((0, common_1.forwardRef)(() => housing_table_service_1.HousingTableService))),
    __param(4, (0, common_1.Inject)((0, common_1.forwardRef)(() => breed_table_service_1.BreedTableService))),
    __param(5, (0, common_1.Inject)((0, common_1.forwardRef)(() => current_stock_table_service_1.CurrentStockTableService))),
    __param(6, (0, common_1.Inject)((0, common_1.forwardRef)(() => stock_balance_table_service_1.StockBalanceTableService))),
    __param(7, (0, common_1.Inject)((0, common_1.forwardRef)(() => ledger_entry_table_service_1.LedgerEntryTableService))),
    __param(8, (0, common_1.Inject)((0, common_1.forwardRef)(() => animal_entry_table_service_1.AnimalEntryTableService))),
    __param(9, (0, common_1.Inject)((0, common_1.forwardRef)(() => disease_table_service_1.DiseaseTableService))),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        data_entry_table_service_1.DataEntryTableService,
        user_collection_service_1.UserCollectionService,
        housing_table_service_1.HousingTableService,
        breed_table_service_1.BreedTableService,
        current_stock_table_service_1.CurrentStockTableService,
        stock_balance_table_service_1.StockBalanceTableService,
        ledger_entry_table_service_1.LedgerEntryTableService,
        animal_entry_table_service_1.AnimalEntryTableService,
        disease_table_service_1.DiseaseTableService])
], StockEntryTableService);
exports.StockEntryTableService = StockEntryTableService;
