"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SubscriptionTableService = void 0;
const common_1 = require("@nestjs/common");
const subscription_table_entity_1 = require("./entities/subscription-table.entity");
const typeorm_1 = require("typeorm");
const typeorm_2 = require("@nestjs/typeorm");
const course_table_service_1 = require("../course-table/course-table.service");
const user_collection_service_1 = require("../user-collection/user-collection.service");
const moment_1 = __importDefault(require("moment"));
const course_session_table_service_1 = require("../course-session-table/course-session-table.service");
let SubscriptionTableService = exports.SubscriptionTableService = class SubscriptionTableService {
    constructor(subscriptionTableRepository, courseTableService, courseSessionTableService, userCollectionService) {
        this.subscriptionTableRepository = subscriptionTableRepository;
        this.courseTableService = courseTableService;
        this.courseSessionTableService = courseSessionTableService;
        this.userCollectionService = userCollectionService;
    }
    async create(createSubscriptionTableInput) {
        let checkIfAlreadyExists = await this.checkIfAlreadyExists(createSubscriptionTableInput);
        if (checkIfAlreadyExists == 0) {
            let st = new subscription_table_entity_1.SubscriptionTable();
            let courseTable = await this.courseTableService.findOne(createSubscriptionTableInput.courseId);
            let userTable = await this.userCollectionService.findOne(createSubscriptionTableInput.userId);
            let courseSessionTable = await this.courseSessionTableService.findOne(createSubscriptionTableInput.courseSessionId);
            st.CourseTable = courseTable;
            st.UserTable = userTable;
            st.CourseSessionTable = courseSessionTable;
            st.startDate = courseSessionTable.sessionStartingDate;
            st.endDate = courseSessionTable.sessionEndingDate;
            st.entryDate = (0, moment_1.default)().format('DD-MM-YYYY');
            st.status = createSubscriptionTableInput.status;
            let result = await this.subscriptionTableRepository.save(st);
            return {
                status: true,
                status_text: "Successfully Added",
                data: result
            };
        }
        else {
            return {
                status: false,
                status_text: "Already Exists"
            };
        }
    }
    async checkIfAlreadyExists(createSubscriptionTableInput) {
        let result = await this.subscriptionTableRepository.count({
            where: {
                CourseTable: {
                    courseId: createSubscriptionTableInput.courseId
                },
                CourseSessionTable: {
                    courseSessionId: createSubscriptionTableInput.courseSessionId
                },
                UserTable: {
                    id: createSubscriptionTableInput.userId
                },
            }
        });
        return result;
    }
    async findAll() {
        let result = await this.subscriptionTableRepository.find({
            relations: {
                CourseTable: true,
                UserTable: true,
                CourseSessionTable: true
            }
        });
        return result;
    }
    async findAllSubscriptionsByCourseId(courseId) {
        let result = await this.subscriptionTableRepository.find({
            where: {
                CourseTable: {
                    courseId
                }
            },
            relations: {
                CourseTable: true,
                UserTable: true
            }
        });
        return result;
    }
    async findOne(subscriptionId) {
        let result = await this.subscriptionTableRepository.findOne({
            where: {
                subscriptionId
            },
            relations: {
                CourseTable: true,
                UserTable: true
            }
        });
        return result;
    }
    async enrollMyself(courseId, courseSessionId, user) {
        let result = await this.subscriptionTableRepository.findOne({
            where: {
                CourseTable: {
                    courseId
                },
                CourseSessionTable: {
                    courseSessionId
                },
                UserTable: {
                    id: user.id
                },
                status: "YES",
                isComplete: 0
            },
        });
        if (result) {
            return {
                status: false,
                status_text: "Already subscribed"
            };
        }
        else {
            let subT = new subscription_table_entity_1.SubscriptionTable();
            let courseTable = await this.courseTableService.findOne(courseId);
            let userTable = await this.userCollectionService.findOne(user.id);
            let courseSessionTable = await this.courseSessionTableService.findOne(courseSessionId);
            subT.CourseTable = courseTable;
            subT.UserTable = userTable;
            subT.CourseSessionTable = courseSessionTable;
            subT.startDate = courseSessionTable.sessionStartingDate;
            subT.endDate = courseSessionTable.sessionEndingDate;
            subT.entryDate = (0, moment_1.default)().format('DD-MM-YYYY');
            subT.status = "YES";
            subT.isComplete = 0;
            let updatedResult = await this.subscriptionTableRepository.save(subT);
            return {
                status: true,
                status_text: "Successfully Subscribed",
                data: updatedResult
            };
        }
    }
    async markAsComplete(courseId, courseSessionId, user) {
        let result = await this.subscriptionTableRepository.findOne({
            where: {
                CourseTable: {
                    courseId
                },
                CourseSessionTable: {
                    courseSessionId
                },
                UserTable: {
                    id: user.id
                },
                isComplete: 0
            },
        });
        if (result) {
            result.isComplete = 1;
            await this.subscriptionTableRepository.save(result);
            return {
                status: true,
                status_text: "Course is complete"
            };
        }
        else {
            return {
                status: false,
                status_text: "No Subscription found"
            };
        }
    }
    async checkIfAllowed(courseId, userId) {
        let result = await this.subscriptionTableRepository.findOne({
            where: {
                CourseTable: {
                    courseId
                },
                UserTable: {
                    id: userId
                },
            }
        });
        if (result) {
            if ((0, moment_1.default)(result.endDate, "DD-MM-YYYY").isBefore((0, moment_1.default)())) {
                return false;
            }
            if (result.isComplete == 1) {
                return true;
            }
            if (result.isComplete == 0 && result.status == "YES") {
                return true;
            }
            return false;
        }
        else {
            return false;
        }
    }
    divideDateRangeIntoWeeks(startDate, endDate) {
        let start = (0, moment_1.default)(startDate, 'DD-MM-YYYY');
        let end = (0, moment_1.default)(endDate, 'DD-MM-YYYY');
        let totalDays = end.diff(start, 'days');
        let baseWeekLength = Math.floor(totalDays / 4);
        let extraDays = totalDays % 4;
        let weeks = [];
        let currentStart = start.clone();
        for (let i = 0; i < 4; i++) {
            let weekLength = baseWeekLength;
            if (i === 3) {
                weekLength += extraDays;
            }
            let weekEnd = currentStart.clone().add(weekLength, 'days');
            weeks.push({
                weekStart: currentStart.format('DD-MM-YYYY'),
                weekEnd: weekEnd.format('DD-MM-YYYY')
            });
            currentStart = weekEnd.clone().add(1, 'days');
        }
        return weeks;
    }
    getCurrentWeekNumber(startDate, endDate) {
        let weeks = this.divideDateRangeIntoWeeks(startDate, endDate);
        let today = (0, moment_1.default)();
        for (let i = 0; i < weeks.length; i++) {
            if ((0, moment_1.default)(today).isBetween((0, moment_1.default)(weeks[i].weekStart, "DD-MM-YYYY"), (0, moment_1.default)(weeks[i].weekEnd, "DD-MM-YYYY"), null, '[]')) {
                return i + 1;
            }
        }
        return -1;
    }
    async getDetails(courseId, userId) {
        let result = await this.subscriptionTableRepository.findOne({
            where: {
                CourseTable: {
                    courseId
                },
                UserTable: {
                    id: userId
                },
            }
        });
        if (result) {
            console.log(this.divideDateRangeIntoWeeks(result.startDate, result.endDate));
            result.currentWeek = this.getCurrentWeekNumber(result.startDate, result.endDate);
            return result;
        }
        else {
            return false;
        }
    }
    async update(subscriptionId, updateSubscriptionTableInput) {
        let result = await this.findOne(subscriptionId);
        if (result) {
            let courseTable = await this.courseTableService.findOne(updateSubscriptionTableInput.courseId);
            let userTable = await this.userCollectionService.findOne(updateSubscriptionTableInput.userId);
            let courseSessionTable = await this.courseSessionTableService.findOne(updateSubscriptionTableInput.courseSessionId);
            result.CourseTable = courseTable;
            result.UserTable = userTable;
            result.CourseSessionTable = courseSessionTable;
            result.startDate = courseSessionTable.sessionStartingDate;
            result.endDate = courseSessionTable.sessionEndingDate;
            result.entryDate = (0, moment_1.default)().format('DD-MM-YYYY');
            result.status = updateSubscriptionTableInput.status;
            let updatedResult = await this.subscriptionTableRepository.save(result);
            return {
                status: true,
                status_text: "Successfully Updated",
                data: updatedResult
            };
        }
        else {
            return {
                status: false,
                status_text: "Not Found"
            };
        }
    }
    async remove(subscriptionId) {
        let result = await this.findOne(subscriptionId);
        if (result) {
            await this.subscriptionTableRepository.remove(result);
            return {
                status: true,
                status_text: "Successfully Deleted"
            };
        }
        else {
            return {
                status: false,
                status_text: "Not Found"
            };
        }
    }
    async toggleStatusofSubscription(subscriptionId) {
        let res = await this.subscriptionTableRepository.findOne({
            where: {
                subscriptionId
            }
        });
        if (res) {
            let oldStatus = res.status;
            let newStatus = "YES";
            if (oldStatus == "YES") {
                newStatus = "NO";
            }
            if (oldStatus == "NO") {
                newStatus = "YES";
            }
            res.status = newStatus;
            let result = await this.subscriptionTableRepository.save(res);
            return {
                status: true,
                status_text: "Successfully Updated",
                data: result
            };
        }
        else {
            return {
                status: false,
                status_text: "Not found"
            };
        }
    }
};
exports.SubscriptionTableService = SubscriptionTableService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_2.InjectRepository)(subscription_table_entity_1.SubscriptionTable)),
    __param(1, (0, common_1.Inject)((0, common_1.forwardRef)(() => course_table_service_1.CourseTableService))),
    __param(2, (0, common_1.Inject)((0, common_1.forwardRef)(() => course_session_table_service_1.CourseSessionTableService))),
    __param(3, (0, common_1.Inject)((0, common_1.forwardRef)(() => user_collection_service_1.UserCollectionService))),
    __metadata("design:paramtypes", [typeorm_1.Repository,
        course_table_service_1.CourseTableService,
        course_session_table_service_1.CourseSessionTableService,
        user_collection_service_1.UserCollectionService])
], SubscriptionTableService);
