"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SurveyQuestionsTableService = exports.NEW_QUESTION_ADD = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const survey_questions_table_entity_1 = require("./entities/survey-questions-table.entity");
const typeorm_2 = require("typeorm");
const graphql_redis_subscriptions_1 = require("graphql-redis-subscriptions");
const GlobalConstants_1 = require("../helper/GlobalConstants");
const survey_master_table_service_1 = require("../survey-master-table/survey-master-table.service");
exports.NEW_QUESTION_ADD = "NEW_QUESTION_ADD";
let SurveyQuestionsTableService = exports.SurveyQuestionsTableService = class SurveyQuestionsTableService {
    constructor(surveyQuestionsTableRepository, pubSub, surveyMasterTableService) {
        this.surveyQuestionsTableRepository = surveyQuestionsTableRepository;
        this.pubSub = pubSub;
        this.surveyMasterTableService = surveyMasterTableService;
    }
    async create(createSurveyQuestionsTableInput) {
        let checkIfAlreadyExists = await this.checkIfAlreadyExists(createSurveyQuestionsTableInput);
        if (checkIfAlreadyExists == 0) {
            let st = new survey_questions_table_entity_1.SurveyQuestionsTable();
            st.surveyQuestionEn = createSurveyQuestionsTableInput.surveyQuestionEn;
            st.surveyQuestionAs = createSurveyQuestionsTableInput.surveyQuestionAs;
            st.surveySelectable = createSurveyQuestionsTableInput.surveySelectable;
            st.surveyQuestionLabel = createSurveyQuestionsTableInput.surveyQuestionLabel;
            let surveyMaster = await this.surveyMasterTableService.findOne(createSurveyQuestionsTableInput.surveyMasterId);
            st.SurveyMasterTable = surveyMaster;
            st.options = createSurveyQuestionsTableInput.options.map((option) => ({
                optionLabel: option.optionLabel,
                optionText: option.optionText,
                optionValue: option.optionValue,
            }));
            let result = await this.surveyQuestionsTableRepository.save(st);
            this.pubSub.publish(exports.NEW_QUESTION_ADD, 1);
            return {
                status: true,
                status_text: "Successfully Added",
                data: result
            };
        }
        else {
            return {
                status: false,
                status_text: "Already Exists"
            };
        }
    }
    async checkIfAlreadyExists(createSurveyQuestionsTableInput) {
        let result = await this.surveyQuestionsTableRepository.count({
            where: {
                surveyQuestionEn: createSurveyQuestionsTableInput.surveyQuestionEn,
                surveyQuestionAs: createSurveyQuestionsTableInput.surveyQuestionAs,
                surveyQuestionLabel: createSurveyQuestionsTableInput.surveyQuestionLabel,
                SurveyMasterTable: {
                    surveyId: createSurveyQuestionsTableInput.surveyMasterId
                }
            }
        });
        return result;
    }
    async findAll() {
        return await this.surveyQuestionsTableRepository.find({
            relations: {
                SurveyMasterTable: true
            }
        });
    }
    async findOne(surveyQuestionId) {
        return await this.surveyQuestionsTableRepository.findOne({
            where: {
                surveyQuestionId: surveyQuestionId
            },
            relations: {
                SurveyMasterTable: true
            }
        });
    }
    async findOneBySurveyMasterId(surveyMasterId) {
        return await this.surveyQuestionsTableRepository.find({
            where: {
                SurveyMasterTable: {
                    surveyId: surveyMasterId
                }
            },
            relations: {
                SurveyMasterTable: true
            }
        });
    }
    async findOneSurveyByType(type) {
        let result = await this.surveyQuestionsTableRepository.find({
            where: {
                SurveyMasterTable: {
                    type: type
                }
            },
            relations: {
                SurveyMasterTable: true
            }
        });
        let surveryMasterId = 0;
        if (result.length > 0) {
            surveryMasterId = result[0].SurveyMasterTable.surveyId;
        }
        let labelArray = result.map(el => el.surveyQuestionLabel);
        return {
            data: this.shuffleArray(result),
            length: this.findLargest(labelArray),
            surveyMasterId: surveryMasterId
        };
    }
    findLargest(arr) {
        if (arr.length === 0) {
            return null;
        }
        let maxNum = arr[0];
        for (let i = 1; i < arr.length; i++) {
            if (arr[i] > maxNum) {
                maxNum = arr[i];
            }
        }
        return maxNum;
    }
    shuffleArray(array) {
        for (let i = array.length - 1; i > 0; i--) {
            const j = Math.floor(Math.random() * (i + 1));
            [array[i], array[j]] = [array[j], array[i]];
        }
        return array;
    }
    async update(surveyQuestionId, updateSurveyQuestionsTableInput) {
        let result = await this.surveyQuestionsTableRepository.findOne({
            where: {
                surveyQuestionId: surveyQuestionId
            }
        });
        if (result) {
            let st = result;
            st.surveyQuestionEn = updateSurveyQuestionsTableInput.surveyQuestionEn;
            st.surveyQuestionAs = updateSurveyQuestionsTableInput.surveyQuestionAs;
            st.surveySelectable = updateSurveyQuestionsTableInput.surveySelectable;
            st.surveyQuestionLabel = updateSurveyQuestionsTableInput.surveyQuestionLabel;
            let surveyMaster = await this.surveyMasterTableService.findOne(updateSurveyQuestionsTableInput.surveyMasterId);
            st.SurveyMasterTable = surveyMaster;
            st.options = updateSurveyQuestionsTableInput.options.map((option) => ({
                optionLabel: option.optionLabel,
                optionText: option.optionText,
                optionValue: option.optionValue,
            }));
            let result1 = await this.surveyQuestionsTableRepository.save(st);
            this.pubSub.publish(exports.NEW_QUESTION_ADD, 1);
            return {
                status: true,
                status_text: "Successfully Updated",
                data: result1
            };
        }
        else {
            return {
                status: false,
                status_text: "Not Found",
                data: null
            };
        }
    }
    async remove(surveyQuestionId) {
        let result = await this.surveyQuestionsTableRepository.findOne({
            where: {
                surveyQuestionId: surveyQuestionId
            }
        });
        if (result) {
            await this.surveyQuestionsTableRepository.remove(result);
            this.pubSub.publish(exports.NEW_QUESTION_ADD, 1);
            return {
                status: true,
                status_text: "Successfully Removed",
                data: null
            };
        }
        else {
            return {
                status: false,
                status_text: "Not Found",
                data: null
            };
        }
    }
};
exports.SurveyQuestionsTableService = SurveyQuestionsTableService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(survey_questions_table_entity_1.SurveyQuestionsTable)),
    __param(1, (0, common_1.Inject)(GlobalConstants_1.PUB_SUB)),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        graphql_redis_subscriptions_1.RedisPubSub,
        survey_master_table_service_1.SurveyMasterTableService])
], SurveyQuestionsTableService);
