"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TrainingTableService = exports.NEW_TRAINING_ADD = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const graphql_redis_subscriptions_1 = require("graphql-redis-subscriptions");
const GlobalConstants_1 = require("../helper/GlobalConstants");
const typeorm_2 = require("typeorm");
const training_table_entity_1 = require("./entities/training-table.entity");
const pop_table_service_1 = require("../pop-table/pop-table.service");
const income_source_table_service_1 = require("../income-source-table/income-source-table.service");
const livelihood_source_table_service_1 = require("../livelihood-source-table/livelihood-source-table.service");
exports.NEW_TRAINING_ADD = "NEW_TRAINING_ADD";
let TrainingTableService = exports.TrainingTableService = class TrainingTableService {
    constructor(trainingTableRepository, pubSub, popTableService, incomeSourceTableService, livelihoodSourceTableService) {
        this.trainingTableRepository = trainingTableRepository;
        this.pubSub = pubSub;
        this.popTableService = popTableService;
        this.incomeSourceTableService = incomeSourceTableService;
        this.livelihoodSourceTableService = livelihoodSourceTableService;
    }
    async create(createTrainingTableInput) {
        let checkIfAlreadyExists = await this.checkIfAlreadyExists(createTrainingTableInput);
        if (checkIfAlreadyExists == 0) {
            let st = new training_table_entity_1.TrainingTable();
            st.incomeActivityCode = createTrainingTableInput.incomeActivityCode;
            st.lsCode = createTrainingTableInput.lsCode;
            st.popIds = createTrainingTableInput.popIds;
            let result = await this.trainingTableRepository.save(st);
            this.pubSub.publish(exports.NEW_TRAINING_ADD, 1);
            return {
                status: true,
                status_text: "Successfully Added",
                data: result
            };
        }
        else {
            return {
                status: false,
                status_text: "Already Exists"
            };
        }
    }
    async createorElseReturnObject(createTrainingTableInput) {
        let checkIfAlreadyExists = await this.trainingTableRepository.findOne({
            where: {
                incomeActivityCode: createTrainingTableInput.incomeActivityCode.trim(),
                lsCode: createTrainingTableInput.lsCode.trim()
            }
        });
        if (!checkIfAlreadyExists) {
            let st = new training_table_entity_1.TrainingTable();
            st.incomeActivityCode = createTrainingTableInput.incomeActivityCode;
            st.lsCode = createTrainingTableInput.lsCode;
            st.popIds = createTrainingTableInput.popIds;
            let result = await this.trainingTableRepository.save(st);
            return result;
        }
        else {
            return checkIfAlreadyExists;
        }
    }
    async checkIfAlreadyExists(createTrainingTableInput) {
        let result = await this.trainingTableRepository.count({
            where: {
                incomeActivityCode: createTrainingTableInput.incomeActivityCode,
                lsCode: createTrainingTableInput.lsCode,
            }
        });
        return result;
    }
    async getCount() {
        return await this.trainingTableRepository.count();
    }
    async findAll() {
        let result = await this.trainingTableRepository.find();
        let index = 0;
        for (const el of result) {
            result[index]['popIds'] = await this.popTableService.findOneArray(el.popIds);
            index++;
        }
        return result;
    }
    async findAllTrainingForApp() {
        let data = [];
        let incomes = await this.incomeSourceTableService.findAll();
        for (const income of incomes) {
            let lsCodes = await this.livelihoodSourceTableService.findAllByIncomeActivityCode(income.incomeSourceCode);
            let lsData = [];
            for (const lsCode of lsCodes) {
                let trainings = await this.trainingTableRepository.findOne({
                    where: {
                        incomeActivityCode: income.incomeSourceCode,
                        lsCode: lsCode.lsCode
                    }
                });
                if (trainings) {
                    let popTable = await this.popTableService.findOneArray(trainings.popIds);
                    let d1 = {
                        lsCode: lsCode.lsCode,
                        lsName: lsCode.lsName,
                        PopTable: popTable
                    };
                    lsData.push(d1);
                }
            }
            if (lsData.length > 0) {
                let d = {
                    incomeActivityCode: income.incomeSourceCode,
                    incomeActivityName: income.incomeSourceName,
                    lsData
                };
                data.push(d);
            }
        }
        return data;
    }
    async findOne(trainingId) {
        return await this.trainingTableRepository.findOne({
            where: {
                trainingId
            }
        });
    }
    async update(trainingId, updateTrainingTableInput) {
        let result = await this.findOne(trainingId);
        if (result) {
            result.incomeActivityCode = updateTrainingTableInput.incomeActivityCode;
            result.lsCode = updateTrainingTableInput.lsCode;
            result.popIds = updateTrainingTableInput.popIds;
            let updatedResult = await this.trainingTableRepository.save(result);
            this.pubSub.publish(exports.NEW_TRAINING_ADD, 1);
            return {
                status: true,
                status_text: "Successfully Updated",
                data: updatedResult
            };
        }
        else {
            return {
                status: false,
                status_text: "Not Found"
            };
        }
    }
    async remove(trainingId) {
        let result = await this.findOne(trainingId);
        if (result) {
            await this.trainingTableRepository.remove(result);
            this.pubSub.publish(exports.NEW_TRAINING_ADD, 1);
            return {
                status: true,
                status_text: "Successfully Deleted"
            };
        }
        else {
            return {
                status: false,
                status_text: "Not Found"
            };
        }
    }
};
exports.TrainingTableService = TrainingTableService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(training_table_entity_1.TrainingTable)),
    __param(1, (0, common_1.Inject)(GlobalConstants_1.PUB_SUB)),
    __param(2, (0, common_1.Inject)((0, common_1.forwardRef)(() => pop_table_service_1.PopTableService))),
    __param(3, (0, common_1.Inject)((0, common_1.forwardRef)(() => income_source_table_service_1.IncomeSourceTableService))),
    __param(4, (0, common_1.Inject)((0, common_1.forwardRef)(() => livelihood_source_table_service_1.LivelihoodSourceTableService))),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        graphql_redis_subscriptions_1.RedisPubSub,
        pop_table_service_1.PopTableService,
        income_source_table_service_1.IncomeSourceTableService,
        livelihood_source_table_service_1.LivelihoodSourceTableService])
], TrainingTableService);
