import { Connection, EntityManager, Repository } from 'typeorm';
import { CreateUserCollectionInput } from './dto/create-user-collection.input';
import { UpdateUserCollectionInput } from './dto/update-user-collection.input';
import { CreateUserFromSignupInput } from './dto/create-user-from-signup.input';
import { NewUserAddInput } from './dto/add-nerw-user.input';
import { UserTable } from './entities/user-collection.entity';
import { RedisPubSub } from 'graphql-redis-subscriptions';
import { AccessTableService } from 'src/access-table/access-table.service';
import { DesignationTableService } from 'src/designation-table/designation-table.service';
import { OrganisationTableService } from 'src/organisation-table/organisation-table.service';
import { NewRegistrationInput } from './dto/new-registration.input';
export declare const NEW_USER_ADD = "NEW_USER_ADD";
export declare class UserCollectionService {
    private readonly userRepository;
    private pubSub;
    private connection;
    private readonly entityManager;
    private readonly accessTableService;
    private readonly organisationTableService;
    private readonly designationTableService;
    constructor(userRepository: Repository<UserTable>, pubSub: RedisPubSub, connection: Connection, entityManager: EntityManager, accessTableService: AccessTableService, organisationTableService: OrganisationTableService, designationTableService: DesignationTableService);
    create(createUserCollectionDto: CreateUserCollectionInput): Promise<{
        status: boolean;
        staus_text: string;
        data: UserTable;
    } | {
        status: boolean;
        staus_text: string;
        data: Promise<UserTable>;
    }>;
    newRegistrationFromApp(newRegistrationInput: NewRegistrationInput): Promise<{
        status: boolean;
        staus_text: string;
        data: UserTable;
    }>;
    signupUser(createUserFromSignupInput: CreateUserFromSignupInput): Promise<UserTable | {
        email: string;
        password: string;
    }>;
    checkIfExists(username: string): Promise<boolean>;
    checkIfExistsPhone(phone: string, username: string): Promise<boolean>;
    getUserDetailsWithUsername(username: string): Promise<any>;
    getOneByUserId(userId: number): Promise<any>;
    findAll(): Promise<UserTable[]>;
    findOne(id: number): Promise<UserTable>;
    update(id: number, updateUserCollectionDto: UpdateUserCollectionInput): Promise<{
        status: boolean;
        status_text: string;
        data: UserTable;
    }>;
    updatePassword(username: string, oldPassword: string, newPassword: string): Promise<{
        status: boolean;
        status_text: string;
        data: any;
    }>;
    remove(id: number): Promise<void>;
    toggleActiveStatus(id: number): Promise<{
        status: boolean;
        status_text: string;
        data: UserTable;
    }>;
    findOneByUsername(username: string): Promise<UserTable>;
    findOneByUsernameV2(username: string): Promise<{
        isIntegrator: boolean;
        isEd: boolean;
        id: number;
        username: string;
        password: string;
        designation: string;
        name: string;
        email: string;
        phone: string;
        currentClass: string;
        classRollNo: string;
        semester: string;
        role: string;
        isActive: boolean;
        AccessTable: import("../access-table/entities/access-table.entity").AccessTable[];
        SubscriptionTable: import("../subscription-table/entities/subscription-table.entity").SubscriptionTable[];
        integratorId: number;
        DesignationTable: import("../designation-table/entities/designation-table.entity").DesignationTable;
        CourseTable: import("../course-table/entities/course-table.entity").CourseTable[];
        OrganisationTable: import("../organisation-table/entities/organisation-table.entity").OrganisationTable;
        AssessmentAnswerTable: import("../assessment-answer-table/entities/assessment-answer-table.entity").AssessmentAnswerTable[];
        CourseProgressionTable: import("../course-progression-table/entities/course-progression-table.entity").CourseProgressionTable[];
        DiscussionTable: import("../discussion-table/entities/discussion-table.entity").DiscussionTable[];
        createdAt: Date;
        updatedAt: Date;
    }>;
    checkUser(payload: any): Promise<boolean>;
    deleteUser(id: number): Promise<{
        status: boolean;
        status_text: string;
        data: any;
    }>;
    getNewUsernameForUser(queryRunner: any): Promise<any>;
    addNewUser(newUserAddInput: NewUserAddInput): Promise<{
        status: boolean;
        status_text: string;
        data: UserTable;
    } | {
        status: boolean;
        status_text: any;
        data?: undefined;
    }>;
    getAllUsers(limit: number, page: number, search: string): Promise<{
        data: any;
        total: any;
    }>;
    getAllUsersNew(): Promise<any>;
    getUserByUsername(username: string): Promise<UserTable>;
    findAllAdmin(): Promise<UserTable[]>;
    forgotPassword(username: string): Promise<{
        status: boolean;
        status_text: string;
    }>;
    changePassword(username: string, password: string): Promise<{
        status: boolean;
        status_text: string;
        data: any;
    }>;
    changePasswordSecured(user: any, oldPassword: string, newPassword: string): Promise<{
        status: boolean;
        status_text: string;
    }>;
    addIntegratorToUser(id: number, integratorId: number): Promise<{
        status: boolean;
        status_text: string;
    }>;
    checkIfUserIsIntegrator(user: any): Promise<{
        status: boolean;
        data: UserTable[];
    }>;
}
