import { Resolver, Query, Mutation, Args, Int } from '@nestjs/graphql';
import { AccessTableService } from './access-table.service';
import { AccessTable } from './entities/access-table.entity';
import { CreateAccessTableInput } from './dto/create-access-table.input';
import { UpdateAccessTableInput } from './dto/update-access-table.input';
import { AccessTableReturnType } from './types/access-table-return.type';
import { AccessTableType } from './types/access-table.type';
import { Roles } from 'src/user-collection/decorators/roles.decorator';
import { Role } from 'src/user-collection/roles/role.enum';
import { UseGuards } from '@nestjs/common';
import { GqlAuthGuard } from 'src/auth/guards/gql-auth.guard';
import { RolesGuard } from 'src/auth/guards/roles.guard';

@Resolver(() => AccessTable)
export class AccessTableResolver {
  constructor(private readonly accessTableService: AccessTableService) {}

  @Roles(Role.SUPERADMIN)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => AccessTableReturnType)
  createAccessTable(@Args('createAccessTableInput') createAccessTableInput: CreateAccessTableInput) {
    return this.accessTableService.create(createAccessTableInput);
  }

  @Query(() => [AccessTableType])
  findAllAccess() {
    return this.accessTableService.findAll();
  }

  @Query(() => AccessTableType)
  findOneAccess(@Args('accessId', { type: () => Int }) accessId: number) {
    return this.accessTableService.findOne(accessId);
  }

  @Roles(Role.SUPERADMIN)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => AccessTableReturnType)
  updateAccessTable(@Args('updateAccessTableInput') updateAccessTableInput: UpdateAccessTableInput) {
    return this.accessTableService.update(updateAccessTableInput.accessId, updateAccessTableInput);
  }

  @Roles(Role.SUPERADMIN)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => AccessTableReturnType)
  removeAccessTable(@Args('accessId', { type: () => Int }) accessId: number) {
    return this.accessTableService.remove(accessId);
  }


  @Query(() => [AccessTableType])
  getAllAccessParameterByUsername(@Args('username', { type: () => String }) username: string) {
    return this.accessTableService.getAllAccessParameterByUsername(username);
  }
}
