import { Controller, Get, Post, Request, Body, Header, UseInterceptors, UploadedFiles, UploadedFile, UsePipes, ValidationPipe, UseGuards, Req, Param, Res } from '@nestjs/common';
import { AppService } from './app.service';
import { AuthService } from './auth/auth.service';
import { LoginDto } from './user-collection/dto/login-dto';
import { Roles } from './user-collection/decorators/roles.decorator';
import { Role } from './user-collection/roles/role.enum';
import { GqlAuthGuard } from './auth/guards/gql-auth.guard';
import { RolesGuard } from './auth/guards/roles.guard';
import { JwtAuthGuard } from './auth/guards/jwt-auth.guard';
import { ImportDataType } from './types/import-data-type.type';
import moment from 'moment';
import { ImportDataTypePg } from './types/import-data-type-pg.type';
import { ImportDataTypeUser } from './types/import-data-type-user.type';
import { ImportDataTypeMk } from './types/import-data-type-mk.type';
import { FileInterceptor } from '@nestjs/platform-express';
import { UPLOAD_PATH } from './helper/GlobalConstants';
import { join } from 'path';
import { existsSync, mkdirSync, renameSync } from 'fs';
import { multerOptions } from './config/mutler-config';

@Controller()
export class AppController {
  constructor(
    private readonly appService: AppService,
    private readonly authService: AuthService,
    // private readonly mailService: MailService
  ) { }



  // @Get('test-email')
  // async emailTesting() {
  //   return await this.mailService.welcomeEmailTest({
  //     toEmail: 'pranjalkr.saikia@gmail.com',
  //     data: {
  //       name: 'Pranjal',
  //       link: 'https://https://fonts.google.com/specimen/Montserrat'
  //     }
  //   });
  // }


  @Post('login')
  @UsePipes(ValidationPipe)
  @Header('Cache-Control', 'no-cache')
  login(@Body('') loginDto: LoginDto) {
    return this.authService.login(loginDto);
  }

  @Roles(Role.SUPERADMIN)
  @UseGuards(JwtAuthGuard, RolesGuard)
  @Post('upload-bulk-file-shg')
  uploadBulkFileShg(@Body('data') data: ImportDataType[], @Req() req: any) {
    // perform action
    // return this.appService.updateBulkData(data, req.user);

  }


  @Roles(Role.SUPERADMIN)
  @UseGuards(JwtAuthGuard, RolesGuard)
  @Post('upload-bulk-file-user')
  uploadBulkFileUser(@Body('data') data: ImportDataTypeUser[], @Req() req: any) {
    // perform action
    return this.appService.updateBulkDataUser(data, req.user);

  }


  @Roles(Role.SUPERADMIN)
  @UseGuards(JwtAuthGuard, RolesGuard)
  @Post('upload-bulk-file-pg')
  uploadBulkFilePg(@Body('data') data: ImportDataTypePg[], @Req() req: any) {
    // perform action
    // return this.appService.updateBulkDataPg(data, req.user);

  }


  @Roles(Role.SUPERADMIN)
  @UseGuards(JwtAuthGuard, RolesGuard)
  @Post('upload-bulk-file-mk')
  uploadBulkFileMk(@Body('data') data: ImportDataTypeMk[], @Req() req: any) {
    // perform action
    // return this.appService.updateBulkDataMk(data, req.user);

  }


  @Roles(Role.SUPERADMIN, Role.USER, Role.SUPERVISOR)
  @UseGuards(JwtAuthGuard, RolesGuard)
  @Get('get-dashboard-count')
  getDashboardCounts() {
    // perform action
    return this.appService.getDashboardCounts();

  }




  @Roles(Role.SUPERADMIN, Role.USER, Role.ADMIN)
  @UseGuards(JwtAuthGuard, RolesGuard)
  @Get('get-dashboard-count-app')
  getDashboardCountApp(@Req() req: any) {
    console.log("🚀 ~ file: app.controller.ts:68 ~ AppController ~ getDashboardCountApp ~ req:", req.user)
    // perform action
    return this.appService.getDashboardCountForApplication(req?.user?.username);

  }


  @Roles(Role.SUPERADMIN, Role.USER, Role.ADMIN)
  @UseGuards(JwtAuthGuard, RolesGuard)
  @Get('get-data-for-offline-use')
  getOfflineDataForEntry(@Req() req: any) {
    return this.appService.getOfflineDataForEntry(req?.user?.username, req?.user?.role);

  }


  @Roles(Role.SUPERADMIN, Role.USER, Role.ADMIN)
  @UseGuards(JwtAuthGuard, RolesGuard)
  @Get('get-assigned-state-and-all')
  getAssignedStatesAndAll(@Req() req: any) {
    return this.appService.getAssignedStatesAndAll(req?.user?.username, req?.user?.role);

  }


  @Roles(Role.SUPERADMIN, Role.USER, Role.ADMIN)
  @UseGuards(JwtAuthGuard, RolesGuard)
  @Get('get-intervention-data-for-offline-use')
  getInterventionOfflineDataForEntry(@Req() req: any) {
    return this.appService.getInterventionOfflineDataForEntry(req?.user?.username, req?.user?.role);

  }


  @Roles(Role.SUPERADMIN, Role.USER, Role.ADMIN)
  @UseGuards(JwtAuthGuard, RolesGuard)
  @Get('get-fblp-single-report/:mkId')
  async getFblpSingleReport(@Param('mkId') mkId: number) {

    // fetch the latest fblp of this mkId now
    // return await this.appService.getFblpSingleReport(mkId)

  }

  @Roles(Role.SUPERADMIN, Role.USER, Role.ADMIN)
  @UseGuards(JwtAuthGuard, RolesGuard)
  @Get('get-mk-single-report/:mkId')
  async getMkSingleReport(@Param('mkId') mkId: number) {

    // fetch the latest fblp of this mkId now
    // return await this.appService.getMkSingleReport(mkId)

  }


  @Roles(Role.SUPERADMIN, Role.USER, Role.ADMIN)
  @UseGuards(JwtAuthGuard, RolesGuard)
  @Get('get-convergence-report/:sessionId')
  async getConvergenceReport(@Param('sessionId') sessionId: number) {

    // return await this.appService.getConvergenceReport(sessionId)

  }


  @Roles(Role.SUPERADMIN, Role.USER, Role.ADMIN)
  @UseGuards(JwtAuthGuard, RolesGuard)
  @Post('get-convergence-report-integrator/:sessionId')
  async getConvergenceReportIntergrator(@Param('sessionId') sessionId: number, @Body() body: any) {

    // return await this.appService.getConvergenceReportIntegrator(sessionId, body)

  }


  @Roles(Role.SUPERADMIN, Role.USER, Role.ADMIN)
  @UseGuards(JwtAuthGuard, RolesGuard)
  @Post('convergence-list-download')
  async convergenceListDownload(@Body() body: any, @Res() res) {

    let schemeMasterSubId = body.schemeMasterSubId;
    let sessionId = body.sessionId;


    var fileName = `convergence-report-download-${moment().toISOString()}.xlsx`;

    res.setHeader('Content-Type', 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet');
    res.setHeader("Content-Disposition", "attachment; filename=" + fileName);


    var Excel = require('exceljs');
    var workbook = new Excel.Workbook();
    var worksheet = workbook.addWorksheet('MK Table');


    workbook.creator = 'Sesta MIS';
    workbook.lastModifiedBy = 'Sesta MIS';
    workbook.created = new Date();
    workbook.modified = new Date();

    // const data = await this.appService.convergenceListDownload(schemeMasterSubId, sessionId);


    worksheet.columns = [
      { header: 'MK ID', key: 'mkId', width: 10 },
      { header: 'Name', key: 'mkName', width: 32 },
      { header: 'State', key: 'stateName', width: 10 },
      { header: 'District', key: 'districtName', width: 10 },
      { header: 'Block', key: 'blockName', width: 10 },
      { header: 'Grampanchayat', key: 'gpName', width: 10 },
      { header: 'Village', key: 'villageName', width: 10 },

      { header: 'Scheme Name', key: 'schemeName', width: 30 },
      { header: 'Sub Scheme Name', key: 'subSchemeName', width: 30 },
      { header: 'Amount Sanctioned', key: 'amountSanctioned', width: 10 },
      { header: 'Amount Leveraged', key: 'amountLeveraged', width: 10 },
      { header: 'Date of Entry', key: 'date', width: 10 },
      { header: 'Status', key: 'status', width: 10 },

      { header: 'Entered By', key: 'userName', width: 10 },
      { header: 'Entered on', key: 'createdAtReadable', width: 10 },

    ];



    // for (const el of data) {




    //   let rowData = {
    //     ...el,
    //     mkId: el.MkTable.mkId,
    //     mkName: el.MkTable.mkName,
    //     schemeName: el.SchemeMasterSubTable.SchemeMasterTable.schemeMasterName,
    //     subSchemeName: el.SchemeMasterSubTable.schemeMasterSubName,
    //     stateName: el.MkTable.VillageTable.GrampanchayatTable.BlockTable.DistrictTable.StateTable.stateName,
    //     districtName: el.MkTable.VillageTable.GrampanchayatTable.BlockTable.DistrictTable.districtName,
    //     blockName: el.MkTable.VillageTable.GrampanchayatTable.BlockTable.blockName,
    //     gpName: el.MkTable.VillageTable.GrampanchayatTable.gpName,
    //     villageName: el.MkTable.VillageTable.villageName,
    //     userName: el.UserTable.name,
    //     createdAtReadable: moment(el.createdAt).format(
    //       "DD-MM-YYYY hh:mm:ss a"
    //     ),

    //   }




    //   worksheet.addRow({
    //     ...rowData,

    //   });

    // }




    await workbook.xlsx.write(res);

    res.end();

  }


  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(JwtAuthGuard, RolesGuard)
  @Get('get-intervention-report')
  async getInterventionReport(@Req() req: any) {
    let sessionId = req.query.hasOwnProperty('sessionId') ? req.query.sessionId : 0;
    let stateId = req.query.hasOwnProperty('stateId') ? req.query.stateId : 0;
    let districtId = req.query.hasOwnProperty('districtId') ? req.query.districtId : 0;
    let blockId = req.query.hasOwnProperty('blockId') ? req.query.blockId : 0;
    let type = req.query.hasOwnProperty('type') ? req.query.type : "block";

    // fetch the latest fblp of this mkId now
    // return await this.appService.getInterventionReport(sessionId, stateId, districtId, blockId, type)

  }

  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(JwtAuthGuard, RolesGuard)
  @Get('get-activity-participation-list')
  async getActivityParticipationList(@Req() req: any) {
    let sessionId = req.query.hasOwnProperty('sessionId') ? req.query.sessionId : 0;
    let page = req.query.hasOwnProperty('page') ? req.query.page : 1;
    let limit = req.query.hasOwnProperty('limit') ? req.query.limit : 100;



    // fetch the latest fblp of this mkId now
    // return await this.appService.getActivityParticipationList(sessionId, page, limit)

  }



  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(JwtAuthGuard, RolesGuard)
  @Get('get-activity-planning-report')
  async getActivityPlanningReport(@Req() req: any) {
    let sessionId = req.query.hasOwnProperty('sessionId') ? req.query.sessionId : 0;
    let userId = req.query.hasOwnProperty('userId') ? req.query.userId : 1;




    // fetch the latest fblp of this mkId now
    // return await this.appService.getActivityPlanningReport(sessionId, userId)

  }



  @Post('upload-file-to-server')
  @UseGuards(JwtAuthGuard)
  @UseInterceptors(FileInterceptor('receiptFile', multerOptions))
  async updatePaymentReceiptFile(@UploadedFile() file: any, @Req() req: any) {

    let sFolder = join(
      __dirname,
      '..',
      '..',
      UPLOAD_PATH,
      'ASSETS'
    )

    if (!existsSync(sFolder)) {
      mkdirSync(sFolder, { recursive: true })
    }

    let up = join(__dirname, '..', '..', UPLOAD_PATH)

    let prePath = ''

    if (file != undefined && file.filename != undefined) {
      renameSync(`${up}/${file.filename}`, `${sFolder}/${file.filename}`)
      prePath = `/${UPLOAD_PATH}/ASSETS/${file.filename}`
    }

    return prePath;


  }

}
