import { Module } from '@nestjs/common';
import { ServeStaticModule } from '@nestjs/serve-static';
import { AuthModule } from './auth/auth.module';
import { join } from 'path';
import { AppGateway } from './app.gateway';
import { GraphQLModule } from '@nestjs/graphql';
import { ApolloDriver, ApolloDriverConfig } from '@nestjs/apollo';
import { UserCollectionModule } from './user-collection/user-collection.module';
import { AppResolver } from './app.resolver';
import { PubSubModule } from './pubSub.module';
import { AppController } from './app.controller';
import { AppService } from './app.service';
import { TypeOrmModule } from '@nestjs/typeorm';
import { UserTable } from './user-collection/entities/user-collection.entity';
import { NotificationTableModule } from './notification-table/notification-table.module';
import { NotificationTable } from './notification-table/entities/notification-table.entity';
import { OtpTable } from './otp-table/otp-table.entity';
import { AccessTableModule } from './access-table/access-table.module';
import { AccessTable } from './access-table/entities/access-table.entity';
import { ScheduleModule } from '@nestjs/schedule';
import { ConfigModule } from '@nestjs/config';
import { MailModule } from './mail/mail.module';
import { PDFModule } from '@t00nday/nestjs-pdf';
import { DesignationTableModule } from './designation-table/designation-table.module';
import { DesignationTable } from './designation-table/entities/designation-table.entity';
import { CourseTableModule } from './course-table/course-table.module';
import { CourseTable } from './course-table/entities/course-table.entity';
import { CourseUnitTableModule } from './course-unit-table/course-unit-table.module';
import { CourseUnitTable } from './course-unit-table/entities/course-unit-table.entity';
import { FileTable } from './file-table/file-table.entity';
import { FileTableModule } from './file-table/file-table.module';
import { AssessmentMasterTableModule } from './assessment-master-table/assessment-master-table.module';
import { AssessmentQuestionsTableModule } from './assessment-questions-table/assessment-questions-table.module';
import { AssessmentAnswerTableModule } from './assessment-answer-table/assessment-answer-table.module';
import { AssessmentMasterTable } from './assessment-master-table/entities/assessment-master-table.entity';
import { AssessmentQuestionsTable } from './assessment-questions-table/entities/assessment-questions-table.entity';
import { AssessmentAnswerTable } from './assessment-answer-table/entities/assessment-answer-table.entity';

import { SubscriptionTable } from './subscription-table/entities/subscription-table.entity';
import { SubscriptionTableModule } from './subscription-table/subscription-table.module';
import { OrganisationTableModule } from './organisation-table/organisation-table.module';
import { OrganisationTable } from './organisation-table/entities/organisation-table.entity';
import { CourseProgressionTableModule } from './course-progression-table/course-progression-table.module';
import { CourseProgressionTable } from './course-progression-table/entities/course-progression-table.entity';
import { DiscussionTableModule } from './discussion-table/discussion-table.module';
import { DiscussionTable } from './discussion-table/entities/discussion-table.entity';
import { CourseSessionTableModule } from './course-session-table/course-session-table.module';
import { CourseSessionTable } from './course-session-table/entities/course-session-table.entity';




@Module({
  imports: [
    ConfigModule.forRoot(),
    TypeOrmModule.forRoot({
      "type": "mysql",
      "host": "localhost",
      "port": 3306,
      "username": process.env.DB_USERNAME,
      "password": process.env.DB_PASSWORD,
      "database": process.env.DB_NAME,
      "entities": [
        UserTable,
        NotificationTable,
        OtpTable,
        AccessTable,
        DesignationTable,
        CourseTable,
        CourseUnitTable,
        FileTable,
        AssessmentMasterTable,
        AssessmentQuestionsTable,
        AssessmentAnswerTable,
        SubscriptionTable,
        OrganisationTable,
        CourseProgressionTable,
        DiscussionTable,
        CourseSessionTable
      ],
      "synchronize": true
    }),
    GraphQLModule.forRoot<ApolloDriverConfig>({
      autoSchemaFile: 'schema.gql',
      driver: ApolloDriver,
      sortSchema: true,
      playground: true,
      installSubscriptionHandlers: true,

      context: ({ req, connection }) => connection ? { req: connection.context } : { req },
    }),

    
    ServeStaticModule.forRoot({
      rootPath: join(__dirname, '..', 'uploads'),
      exclude: ['/api*'],
      serveRoot: "/uploads",
      serveStaticOptions: {
        setHeaders: (res, path, stat) => {
          res.set('Cross-Origin-Resource-Policy', 'cross-origin');
        }
      }
    }),
    ServeStaticModule.forRoot({
      rootPath: join(__dirname, '..', 'client'),
      exclude: ['/api*', "/uploads*"],
    }),
    PDFModule.register({
      isGlobal: true,
      view: {
        root: join(__dirname, 'templates', 'reports'),
        engine: 'handlebars',
      },
    }),
    ScheduleModule.forRoot(),
    UserCollectionModule,
    AuthModule,
    PubSubModule,
    NotificationTableModule,
    AccessTableModule,
    MailModule,
    DesignationTableModule,
    CourseTableModule,
    CourseUnitTableModule,
    FileTableModule,
    AssessmentMasterTableModule,
    AssessmentQuestionsTableModule,
    AssessmentAnswerTableModule,
    SubscriptionTableModule,
    OrganisationTableModule,
    CourseProgressionTableModule,
    DiscussionTableModule,
    CourseSessionTableModule,
  ],
  controllers: [AppController],
  providers: [AppResolver, AppGateway, AppService],
  exports: [AppGateway]
})
export class AppModule { }
