import { Resolver, Query, Mutation, Args, Int } from '@nestjs/graphql';
import { AssessmentAnswerTableService } from './assessment-answer-table.service';
import { AssessmentAnswerTable } from './entities/assessment-answer-table.entity';
import { CreateAssessmentAnswerTableInput } from './dto/create-assessment-answer-table.input';
import { UpdateAssessmentAnswerTableInput } from './dto/update-assessment-answer-table.input';
import { Roles } from 'src/user-collection/decorators/roles.decorator';
import { Role } from 'src/user-collection/roles/role.enum';
import { UseGuards } from '@nestjs/common';
import { GqlAuthGuard } from 'src/auth/guards/gql-auth.guard';
import { RolesGuard } from 'src/auth/guards/roles.guard';
import { AssessmentAnswersTableReturnType } from './types/assessment-answer-table-return.type';
import { CurrentUser } from 'src/user-collection/decorators/user.decorator';
import { AssessmentAnswersTableType } from './types/assessment-answer-table.type';
import { AssessmentAnswersTablePaginationType } from './types/assessment-answer-table-pagination.type';



@Resolver(() => AssessmentAnswerTable)
export class AssessmentAnswerTableResolver {
  constructor(private readonly assessmentAnswerTableService: AssessmentAnswerTableService) {}

  @Roles(Role.SUPERADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => AssessmentAnswersTableReturnType)
  createAssessmentAnswerTable(@Args('createAssessmentAnswerTableInput') createAssessmentAnswerTableInput: CreateAssessmentAnswerTableInput, @CurrentUser() user: any) {
    return this.assessmentAnswerTableService.create(createAssessmentAnswerTableInput, user);
  }

  @Roles(Role.SUPERADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Query(() => [AssessmentAnswersTableType])
  findAllAssessmentAnswers() {
    return this.assessmentAnswerTableService.findAll();
  }

  @Roles(Role.SUPERADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Query(() => AssessmentAnswersTableType)
  findOneAnswerById(@Args('answerId', { type: () => Int }) answerId: number) {
    return this.assessmentAnswerTableService.findOne(answerId);
  }


  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Query(() => AssessmentAnswersTablePaginationType)
  getListOfAnswersByUser(@Args('limit') limit: number, @Args('page') page: number,@Args('type') type: string, @Args('search') search: string,@CurrentUser() user: any) {
    return this.assessmentAnswerTableService.getListOfAnswersByUser(limit, page,type, search, user);
  }

  @Roles(Role.SUPERADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Query(() => AssessmentAnswersTableType)
  recalculateScoreForShgAndVo(@Args('answerId', { type: () => Int }) answerId: number) {
    return this.assessmentAnswerTableService.recalculateScore(answerId);
  }

  @Roles(Role.SUPERADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Query(() => AssessmentAnswersTableType, { name: 'assessmentAnswerTable' })
  findOneAssessmentAnswer(@Args('answerId', { type: () => Int }) answerId: number) {
    return this.assessmentAnswerTableService.findOne(answerId);
  }

  // @Mutation(() => AssessmentAnswerTable)
  // updateAssessmentAnswerTable(@Args('updateAssessmentAnswerTableInput') updateAssessmentAnswerTableInput: UpdateAssessmentAnswerTableInput) {
  //   return this.assessmentAnswerTableService.update(updateAssessmentAnswerTableInput.id, updateAssessmentAnswerTableInput);
  // }

  // @Roles(Role.SUPERADMIN, Role.USER)
  // @UseGuards(GqlAuthGuard, RolesGuard)
  // @Mutation(() => AssessmentAnswersTableReturnType)
  // removeAssessmentAnswerTable(@Args('answerId', { type: () => Int }) answerId: number) {
  //   return this.assessmentAnswerTableService.remove(answerId);
  // }
}
