import { Resolver, Query, Mutation, Args, Int, Subscription } from '@nestjs/graphql';
import { AssessmentQuestionsTableService } from './assessment-questions-table.service';
import { AssessmentQuestionsTable } from './entities/assessment-questions-table.entity';
import { CreateAssessmentQuestionsTableInput } from './dto/create-assessment-questions-table.input';
import { UpdateAssessmentQuestionsTableInput } from './dto/update-assessment-questions-table.input';
import { Inject, UseGuards } from '@nestjs/common';
import { PUB_SUB } from 'src/helper/GlobalConstants';
import { RedisPubSub } from 'graphql-redis-subscriptions';
import { Roles } from 'src/user-collection/decorators/roles.decorator';
import { Role } from 'src/user-collection/roles/role.enum';
import { GqlAuthGuard } from 'src/auth/guards/gql-auth.guard';
import { RolesGuard } from 'src/auth/guards/roles.guard';
import { AssessmentQuestionsTableReturnType } from './types/assessment-questions-table-return.type';
import { AssessmentQuestionsTableType } from './types/assessment-questions-table.type';
import { AssessmentQuestionsTableReturnAppType } from './types/assessment-questions-table-return-app.type';
import { CountChangedType } from 'src/types/count-changed.type';

export const NEW_QUESTION_ADD = "NEW_QUESTION_ADD"

@Resolver(() => AssessmentQuestionsTable)
export class AssessmentQuestionsTableResolver {
  constructor(private readonly assessmentQuestionsTableService: AssessmentQuestionsTableService,
    @Inject(PUB_SUB) private pubSub: RedisPubSub
  ) {}

  @Roles(Role.SUPERADMIN)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => AssessmentQuestionsTableReturnType)
  createAssessmentQuestionsTable(@Args('createAssessmentQuestionsTableInput') createAssessmentQuestionsTableInput: CreateAssessmentQuestionsTableInput) {
    return this.assessmentQuestionsTableService.create(createAssessmentQuestionsTableInput);
  }

  @Roles(Role.SUPERADMIN)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Query(() => [AssessmentQuestionsTableType])
  findAllAssessmentQuestions() {
    return this.assessmentQuestionsTableService.findAll();
  }

  @Roles(Role.SUPERADMIN)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Query(() => AssessmentQuestionsTableReturnType)
  findOne(@Args('assessmentQuestionId', { type: () => Int }) assessmentQuestionId: number) {
    return this.assessmentQuestionsTableService.findOne(assessmentQuestionId);
  }

  @Roles(Role.SUPERADMIN)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Query(() => [AssessmentQuestionsTableType])
  findOneByAssessmentMasterId(@Args('assessmentId', { type: () => Int }) assessmentId: number) {
    return this.assessmentQuestionsTableService.findOneByAssessmentMasterId(assessmentId);
  }

  @Roles(Role.SUPERADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Query(() => AssessmentQuestionsTableReturnAppType)
  findOneAssessmentByType(@Args('type', { type: () => String }) type: string) {
    return this.assessmentQuestionsTableService.findOneAssessmentByType(type);
  }


  @Roles(Role.SUPERADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Query(() => AssessmentQuestionsTableReturnAppType)
  findOneAssessmentByIdForApp(@Args('assessmentId', { type: () => Int }) assessmentId: number) {
    return this.assessmentQuestionsTableService.findOneAssessmentByIdForApp(assessmentId);
  }


  @Roles(Role.SUPERADMIN)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => AssessmentQuestionsTableReturnType)
  updateAssessmentQuestionsTable(@Args('updateAssessmentQuestionsTableInput') updateAssessmentQuestionsTableInput: UpdateAssessmentQuestionsTableInput) {
    return this.assessmentQuestionsTableService.update(updateAssessmentQuestionsTableInput.assessmentQuestionId, updateAssessmentQuestionsTableInput);
  }

  @Roles(Role.SUPERADMIN)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => AssessmentQuestionsTableReturnType)
  removeAssessmentQuestionsTable(@Args('assessmentQuestionId', { type: () => Int }) assessmentQuestionId: number) {
    return this.assessmentQuestionsTableService.remove(assessmentQuestionId);
  }

  @Subscription(() => CountChangedType, {
    resolve: (payload) => {
      return { countChanged: payload }
    }
  })
  assessmentQuestionsSubscription() {
    return this.pubSub.asyncIterator(NEW_QUESTION_ADD);
  }
}
