import { Injectable } from '@nestjs/common';
import { PassportStrategy } from '@nestjs/passport';
import { ExtractJwt, Strategy } from 'passport-jwt';
import { UserCollectionService } from 'src/user-collection/user-collection.service';
import { jwtConstants } from '../constants';


@Injectable()
export class JwtStrategy extends PassportStrategy(Strategy) {
  constructor(
    private readonly userCollectionService: UserCollectionService,
  ) {
    super({
      jwtFromRequest: ExtractJwt.fromAuthHeaderAsBearerToken(),
      ignoreExpiration: false,
      secretOrKey: jwtConstants.secret,
    });
  }
  

  // for user validation on each request:: Donot change
  async validate(payload: any) {
    let checkuser = await this.userCollectionService.checkUser(payload);
    if (checkuser) {
      return { userId: payload.sub, username: payload.username, ...payload };
    } else {
      return false;
    }
  }
}