import { Resolver, Query, Mutation, Args, Int } from '@nestjs/graphql';
import { CourseProgressionTableService } from './course-progression-table.service';
import { CourseProgressionTable } from './entities/course-progression-table.entity';
import { CreateCourseProgressionTableInput } from './dto/create-course-progression-table.input';
import { UpdateCourseProgressionTableInput } from './dto/update-course-progression-table.input';
import { CurrentUser } from 'src/user-collection/decorators/user.decorator';
import { Roles } from 'src/user-collection/decorators/roles.decorator';
import { Role } from 'src/user-collection/roles/role.enum';
import { GqlAuthGuard } from 'src/auth/guards/gql-auth.guard';
import { RolesGuard } from 'src/auth/guards/roles.guard';
import { UseGuards } from '@nestjs/common';
import { CourseProgressionTableReturnType } from './types/course-progression-table-return.type';
import { CourseProgressionTableType } from './types/course-progression-table.type';

@Resolver(() => CourseProgressionTable)
export class CourseProgressionTableResolver {
  constructor(private readonly courseProgressionTableService: CourseProgressionTableService) {}


  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => CourseProgressionTableReturnType)
  updateCourseProgression(@Args('courseUnitId', { type: () => Int }) courseUnitId: number, @Args('type', { type: () => String }) type: string, @CurrentUser() user: any) {
    return this.courseProgressionTableService.updateCourseProgression(courseUnitId, type, user);
  }

  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Query(() => [CourseProgressionTableType])
  findAllCourseProgression(@Args('courseId', { type: () => Int }) courseId: number) {
    return this.courseProgressionTableService.findAllCourseProgression(courseId);
  }


  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => CourseProgressionTableReturnType)
  toggleStatusOfTable(@Args('courseProgressionId', { type: () => Int }) courseProgressionId: number, @Args('type', { type: () => String }) type: string, @Args('currentValue', { type: () => Int }) currentValue: number) {
    return this.courseProgressionTableService.toggleStatusOfTable(courseProgressionId, type, currentValue);
  }

  // @Mutation(() => CourseProgressionTable)
  // createCourseProgressionTable(@Args('createCourseProgressionTableInput') createCourseProgressionTableInput: CreateCourseProgressionTableInput) {
  //   return this.courseProgressionTableService.create(createCourseProgressionTableInput);
  // }

  // @Query(() => [CourseProgressionTable], { name: 'courseProgressionTable' })
  // findAll() {
  //   return this.courseProgressionTableService.findAll();
  // }

 

  // @Mutation(() => CourseProgressionTable)
  // updateCourseProgressionTable(@Args('updateCourseProgressionTableInput') updateCourseProgressionTableInput: UpdateCourseProgressionTableInput) {
  //   return this.courseProgressionTableService.update(updateCourseProgressionTableInput.id, updateCourseProgressionTableInput);
  // }

  // @Mutation(() => CourseProgressionTable)
  // removeCourseProgressionTable(@Args('id', { type: () => Int }) id: number) {
  //   return this.courseProgressionTableService.remove(id);
  // }
}
