import { Inject, Injectable, forwardRef } from '@nestjs/common';
import { CreateCourseProgressionTableInput } from './dto/create-course-progression-table.input';
import { UpdateCourseProgressionTableInput } from './dto/update-course-progression-table.input';
import { InjectRepository } from '@nestjs/typeorm';
import { CourseProgressionTable } from './entities/course-progression-table.entity';
import { Repository } from 'typeorm';
import { PUB_SUB } from 'src/helper/GlobalConstants';
import { RedisPubSub } from 'graphql-redis-subscriptions';
import { CourseUnitTableService } from 'src/course-unit-table/course-unit-table.service';
import { UserCollectionService } from 'src/user-collection/user-collection.service';
import { AssessmentAnswerTableService } from 'src/assessment-answer-table/assessment-answer-table.service';

@Injectable()
export class CourseProgressionTableService {
  constructor(
    @InjectRepository(CourseProgressionTable)
    private readonly courseProgressionTableRepository: Repository<CourseProgressionTable>,

    @Inject(PUB_SUB) private pubSub: RedisPubSub,



    @Inject(forwardRef(() => CourseUnitTableService))
    private readonly courseUnitTableService: CourseUnitTableService,


    @Inject(forwardRef(() => UserCollectionService))
    private readonly userCollectionService: UserCollectionService,


    @Inject(forwardRef(() => AssessmentAnswerTableService))
    private readonly assessmentAnswerTableService: AssessmentAnswerTableService,

  ) { }


  async updateCourseProgression(courseUnitId: number, type: string, user: any) {
    let result = await this.courseProgressionTableRepository.findOne({
      where: {
        CourseUnitTable: {
          courseUnitId
        },
        UserTable: {
          id: user.id
        },
        type
      }
    })

    if (result) {
      result.isComplete = 1;
      result.canTakeAssesment = 1;

      await this.courseProgressionTableRepository.save(result)

      return {
        status: true,
        status_text: "Successfully Updated"
      }
    } else {
      // create 

      let rs = new CourseProgressionTable();
      rs.CourseUnitTable = await this.courseUnitTableService.findOne(courseUnitId)
      rs.UserTable = await this.userCollectionService.findOne(user.id)
      rs.isComplete = 1;
      rs.canTakeAssesment = 1;
      rs.type = type;

      await this.courseProgressionTableRepository.save(rs);

      return {
        status: true,
        status_text: "Successfully Updated"
      }
    }
  }


  async getProgressionReport(courseUnitId: number, user: any) {
    return await this.courseProgressionTableRepository.findOne({
      where: {
        CourseUnitTable: {
          courseUnitId
        },
        UserTable: {
          id: user.id
        }
      }
    })
  }


  async checkIfAllowedForAssessment(courseUnitId: number, user: any) {
    let result = await this.courseProgressionTableRepository.findOne({
      where: {
        CourseUnitTable: {
          courseUnitId
        },
        UserTable: {
          id: user.id
        }
      }
    })


    if (result) {
      if (result.assesmentStatus == 0) {
        return true;
      } else {
        return false;
      }
    }
    return false;
  }


  async getScoreForAssessment(courseUnitId: number, user: any) {
    let result = await this.courseProgressionTableRepository.findOne({
      where: {
        CourseUnitTable: {
          courseUnitId
        },
        UserTable: {
          id: user.id
        }
      }
    })


    if (result) {
      if (result.assesmentStatus == 1) {
        return result.assesmentScore;
      } else {
        return false;
      }
    }
    return false;
  }


  async updateCourseProgressionAssessment(courseUnitId: number, user: any, assesmentTaken: number, assesmentTakenOn: string, assesmentScore: number, assesmentStatus: number) {
    let result = await this.courseProgressionTableRepository.findOne({
      where: {
        CourseUnitTable: {
          courseUnitId
        },
        UserTable: {
          id: user.id
        }
      }
    })

    if (result) {
      result.assesmentTaken = assesmentTaken;
      result.assesmentTakenOn = assesmentTakenOn;
      result.assesmentScore = assesmentScore;
      result.assesmentStatus = assesmentStatus;

      await this.courseProgressionTableRepository.save(result)

      return true;
    } else {
      return false;
    }
  }


  async findAllCourseProgression(courseId: number) {
    let result = await this.courseProgressionTableRepository.find({
      where: {
        CourseUnitTable: {
          CourseTable: {
            courseId
          }
        }
      },
      relations: {
        CourseUnitTable: true,
        UserTable: true
      }
    })


    let index = 0;
    for (const el of result) {
      result[index]['AssessmentAnswersTable'] = await this.assessmentAnswerTableService.findAllByCourseIdAndUserId(el.CourseUnitTable.courseUnitId, el.UserTable.id)
      index++;
    }


    return result;
  }


  async toggleStatusOfTable(courseProgressionId: number, type: string, currentValue: number) {
    let result = await this.courseProgressionTableRepository.findOne({
      where: {
        courseProgressionId
      }
    })

    if (result) {

      let newValue = 0;

      if (currentValue == 1) {
        newValue = 0;
      }

      if (currentValue == 0) {
        newValue = 1;
      }

      // if isComplete is set to 0
      if (type == "isComplete") {

        if (newValue == 0) {
          // we will unset the following
          result.isComplete = 0;
          result.canTakeAssesment = 0;
          result.assesmentScore = 0;
          result.assesmentTaken = 0;
          result.assesmentTakenOn = null;
          result.assesmentStatus = 0;
        }

        if(newValue == 1){
          result.isComplete = 1;
        }

      }


      if (type == "canTakeAssesment") {
        if (newValue == 0) {
          result.canTakeAssesment = 0;
        }

        if(newValue == 1){
          result.canTakeAssesment = 1;
        }
      }

      if (type == "assesmentTaken") {
        if (newValue == 0) {
          result.assesmentTaken = 0;
        }

        if(newValue == 1){
          result.assesmentTaken = 1;
        }
      }

      if (type == "assesmentStatus") {
        if (newValue == 0) {
          result.assesmentStatus = 0;
        }

        if(newValue == 1){
          result.assesmentStatus = 1;
        }
      }



      await this.courseProgressionTableRepository.save(result);

      return {
        status: true,
        status_text: "Successfully Updated"
      }

    } else {
      return {
        status: false,
        status_text: "Not found"
      }
    }
  }



}
