import { Resolver, Query, Mutation, Args, Int } from '@nestjs/graphql';
import { CourseSessionTableService } from './course-session-table.service';
import { CourseSessionTable } from './entities/course-session-table.entity';
import { CreateCourseSessionTableInput } from './dto/create-course-session-table.input';
import { UpdateCourseSessionTableInput } from './dto/update-course-session-table.input';
import { CourseSessionTableReturnType } from './types/course-session-table-return.type';
import { Roles } from 'src/user-collection/decorators/roles.decorator';
import { Role } from 'src/user-collection/roles/role.enum';
import { UseGuards } from '@nestjs/common';
import { GqlAuthGuard } from 'src/auth/guards/gql-auth.guard';
import { RolesGuard } from 'src/auth/guards/roles.guard';
import { CourseSessionTableType } from './types/course-session-table.type';

@Resolver(() => CourseSessionTable)
export class CourseSessionTableResolver {
  constructor(private readonly courseSessionTableService: CourseSessionTableService) {}

  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)  
  @Mutation(() => CourseSessionTableReturnType)
  createCourseSessionTable(@Args('createCourseSessionTableInput') createCourseSessionTableInput: CreateCourseSessionTableInput) {
    return this.courseSessionTableService.create(createCourseSessionTableInput);
  }

  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)  
  @Query(() => [CourseSessionTableType])
  findAllCourseSession() {
    return this.courseSessionTableService.findAll();
  }

  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)  
  @Query(() => CourseSessionTableType)
  findOneCourseSession(@Args('courseSessionId', { type: () => Int }) courseSessionId: number) {
    return this.courseSessionTableService.findOne(courseSessionId);
  }

  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)  
  @Query(() => [CourseSessionTableType])
  findOneCourseSessionByCourseId(@Args('courseId', { type: () => Int }) courseId: number) {
    return this.courseSessionTableService.findOneByCourseId(courseId);
  }

  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)  
  @Mutation(() => CourseSessionTableReturnType)
  updateCourseSessionTable(@Args('updateCourseSessionTableInput') updateCourseSessionTableInput: UpdateCourseSessionTableInput) {
    return this.courseSessionTableService.update(updateCourseSessionTableInput.courseSessionId, updateCourseSessionTableInput);
  }

  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)  
  @Mutation(() => CourseSessionTableReturnType)
  removeCourseSessionTable(@Args('courseSessionId', { type: () => Int }) courseSessionId: number) {
    return this.courseSessionTableService.remove(courseSessionId);
  }


  @Roles(Role.SUPERADMIN, Role.ADMIN)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => CourseSessionTableReturnType)
  toggleStatusofCourseSession(@Args('courseSessionId', { type: () => Int }) courseSessionId: number) {
    return this.courseSessionTableService.toggleStatusofCourseSession(courseSessionId);
  }
}
