import { AssessmentMasterTable } from 'src/assessment-master-table/entities/assessment-master-table.entity';
import { CourseSessionTable } from 'src/course-session-table/entities/course-session-table.entity';
import { CourseUnitTable } from 'src/course-unit-table/entities/course-unit-table.entity';
import { DiscussionTable } from 'src/discussion-table/entities/discussion-table.entity';
import { SubscriptionTable } from 'src/subscription-table/entities/subscription-table.entity';
import { UserTable } from 'src/user-collection/entities/user-collection.entity';
import { Column, CreateDateColumn, Entity, JoinColumn, ManyToOne, NoNeedToReleaseEntityManagerError, OneToMany, PrimaryGeneratedColumn, UpdateDateColumn } from 'typeorm';

@Entity()
export class CourseTable {
  @PrimaryGeneratedColumn()
  courseId: number;

  @Column({ length: 300 })
  courseName: string;

  @Column({ length: 300, nullable: true })
  courseCode: string;

  @Column({ length: 1200, nullable: true })
  courseDescription: string;

  @Column({ nullable: true })
  courseDuration: number;

  @Column({ length: 50, default: "FREE" })
  courseType: string;

  @Column({ length: 50, default: "NO" })
  hasAssesment: string;

  @Column({ length: 50, default: "NO" })
  isActive: string;


  @Column({ type: "text", nullable: true })
  courseBanner: string;


  @OneToMany(type => CourseUnitTable, pt => pt.CourseTable)
  CourseUnitTable: CourseUnitTable[];

  @OneToMany(type => DiscussionTable, pt => pt.CourseTable)
  DiscussionTable: DiscussionTable[];
  
  @OneToMany(type => SubscriptionTable, pt => pt.CourseTable)
  SubscriptionTable: SubscriptionTable[];

  @OneToMany(type => AssessmentMasterTable, et => et.CourseTable)
  AssessmentMasterTable: AssessmentMasterTable[];

  @OneToMany(type => CourseSessionTable, pt => pt.CourseTable)
  CourseSessionTable: CourseSessionTable[];


  @ManyToOne(type => UserTable, dt => dt.CourseTable, {nullable: true})
  @JoinColumn({ name: "userId" })
  UserTable: UserTable;

  @CreateDateColumn({ type: "timestamp", default: () => "CURRENT_TIMESTAMP(6)" })
  public createdAt: Date;

  @UpdateDateColumn({ type: "timestamp", default: () => "CURRENT_TIMESTAMP(6)", onUpdate: "CURRENT_TIMESTAMP(6)" })
  public updatedAt: Date;
}
