import { Resolver, Query, Mutation, Args, Int, Subscription } from '@nestjs/graphql';
import { CourseUnitTableService } from './course-unit-table.service';
import { CourseUnitTable } from './entities/course-unit-table.entity';
import { CreateCourseUnitTableInput } from './dto/create-course-unit-table.input';
import { UpdateCourseUnitTableInput } from './dto/update-course-unit-table.input';
import { Inject, UseGuards } from '@nestjs/common';
import { PUB_SUB } from 'src/helper/GlobalConstants';
import { RedisPubSub } from 'graphql-redis-subscriptions';
import { Roles } from 'src/user-collection/decorators/roles.decorator';
import { Role } from 'src/user-collection/roles/role.enum';
import { GqlAuthGuard } from 'src/auth/guards/gql-auth.guard';
import { RolesGuard } from 'src/auth/guards/roles.guard';
import { CourseUnitTableReturnType } from './types/course-unit-table-return.type';
import { CourseUnitTableType } from './types/course-unit-table.type';
import { CountChangedType } from 'src/types/count-changed.type';

export const NEW_COURSE_UNIT_ADD = "NEW_COURSE_UNIT_ADD"

@Resolver(() => CourseUnitTable)
export class CourseUnitTableResolver {
  constructor(private readonly courseUnitTableService: CourseUnitTableService,
    @Inject(PUB_SUB) private pubSub: RedisPubSub
    ) {}

  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => CourseUnitTableReturnType)
  createCourseUnitTable(@Args('createCourseUnitTableInput') createCourseUnitTableInput: CreateCourseUnitTableInput) {
    return this.courseUnitTableService.create(createCourseUnitTableInput);
  }

  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Query(() => [CourseUnitTableType])
  findAllCourseUnit() {
    return this.courseUnitTableService.findAll();
  }

  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Query(() => CourseUnitTableType)
  findOneCourseUnit(@Args('courseUnitId', { type: () => Int }) courseUnitId: number) {
    return this.courseUnitTableService.findOne(courseUnitId);
  }

  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Query(() => [CourseUnitTableType])
  findOneCourseUnitByCourseId(@Args('courseId', { type: () => Int }) courseId: number) {
    return this.courseUnitTableService.findOneCourseUnitByCourseId(courseId);
  }

  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => CourseUnitTableReturnType)
  updateCourseUnitTable(@Args('updateCourseUnitTableInput') updateCourseUnitTableInput: UpdateCourseUnitTableInput) {
    return this.courseUnitTableService.update(updateCourseUnitTableInput.courseUnitId, updateCourseUnitTableInput);
  }


  @Roles(Role.SUPERADMIN, Role.ADMIN)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => CourseUnitTableReturnType)
  toggleHasAssesmentofCourseUnit(@Args('courseUnitId', { type: () => Int }) courseUnitId: number) {
    return this.courseUnitTableService.toggleHasAssesmentofCourseUnit(courseUnitId);
  }

  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => CourseUnitTableReturnType)
  removeCourseUnitTable(@Args('courseUnitId', { type: () => Int }) courseUnitId: number) {
    return this.courseUnitTableService.remove(courseUnitId);
  }

  @Subscription(() => CountChangedType, {
    resolve: (payload) => {
      return { countChanged: payload }
    }
  })
  courseUnitSubscription() {
    return this.pubSub.asyncIterator(NEW_COURSE_UNIT_ADD);
  }
}
