import { Inject, Injectable, forwardRef } from '@nestjs/common';
import { CreateCourseUnitTableInput } from './dto/create-course-unit-table.input';
import { UpdateCourseUnitTableInput } from './dto/update-course-unit-table.input';
import { InjectRepository } from '@nestjs/typeorm';
import { CourseUnitTable } from './entities/course-unit-table.entity';
import { Repository } from 'typeorm';
import { PUB_SUB } from 'src/helper/GlobalConstants';
import { RedisPubSub } from 'graphql-redis-subscriptions';
import { CourseTableService } from 'src/course-table/course-table.service';
import { FileTableService } from 'src/file-table/file-table.service';

export const NEW_COURSE_UNIT_ADD = "NEW_COURSE_UNIT_ADD"

@Injectable()
export class CourseUnitTableService {
  constructor(
    @InjectRepository(CourseUnitTable)
    private readonly courseUnitTableRepository: Repository<CourseUnitTable>,

    @Inject(PUB_SUB) private pubSub: RedisPubSub,

    @Inject(forwardRef(() => CourseTableService))
    private readonly courseTableService: CourseTableService,

    @Inject(forwardRef(() => FileTableService))
    private readonly fileTableService: FileTableService,



  ) { }

  async create(createCourseUnitTableInput: CreateCourseUnitTableInput) {
    let checkIfAlreadyExists = await this.checkIfAlreadyExists(createCourseUnitTableInput);

    if (checkIfAlreadyExists == 0) {

      let st = new CourseUnitTable();

      let courseTable = await this.courseTableService.findOne(createCourseUnitTableInput.courseId)
      st.CourseTable = courseTable;

      st.week = createCourseUnitTableInput.week;
      st.courseUnitName = createCourseUnitTableInput.courseUnitName;
      st.courseUnitDescription = createCourseUnitTableInput.courseUnitDescription;
      st.courseUnitDuration = createCourseUnitTableInput.courseUnitDuration;
      //st.numberOfPages = createCourseUnitTableInput.numberOfPages;
      //st.weightOfPercentage = createCourseUnitTableInput.weightOfPercentage;
      //st.courseFile = createCourseUnitTableInput.courseFile;
      st.courseUnitBanner = createCourseUnitTableInput.courseUnitBanner;

      //st.hasAssesment = createCourseUnitTableInput.hasAssesment;
      //st.totalScore = createCourseUnitTableInput.totalScore;
      //st.passingScore = createCourseUnitTableInput.passingScore;
      st.isLink = createCourseUnitTableInput.isLink;
      st.link = createCourseUnitTableInput.link;

      let result = await this.courseUnitTableRepository.save(st);


      // send to subscriber
      this.pubSub.publish(NEW_COURSE_UNIT_ADD, 1);


      return {
        status: true,
        status_text: "Successfully Added",
        data: result
      }

    } else {
      return {
        status: false,
        status_text: "Already Exists"
      }
    }
  }

  async checkIfAlreadyExists(createCourseUnitTableInput: CreateCourseUnitTableInput) {


    let result = await this.courseUnitTableRepository.count({
      where: {
        courseUnitName: createCourseUnitTableInput.courseUnitName,
        CourseTable: {
          courseId: createCourseUnitTableInput.courseId
        },
        week: createCourseUnitTableInput.week
      }
    })

    return result;

  }

  async findAll() {
    return await this.courseUnitTableRepository.find();
  }

  async findOne(courseUnitId: number) {
    let result: any = await this.courseUnitTableRepository.findOne({
      where: {
        courseUnitId
      },
      relations: {
        CourseTable: true
      }
    })

    if (result) {
      // get the file link

      // let url = await this.fileTableService.getUrlOfFileId(result.courseFile);
      // result.courseFileUrl = url;

      let bannerUrl = await this.fileTableService.getUrlOfFileId(result.courseUnitBanner);
      result.courseUnitBannerUrl = bannerUrl;

    }

    return result;
  }


  async findOneCourseUnitByCourseId(courseId: number) {
    let result = await this.courseUnitTableRepository.find({
      where: {
        CourseTable: {
          courseId
        }
      },
      order: {
        week: "ASC"
      }
    })

    let index = 0;
    for (const r of result) {
      // get the file link



      // let url = await this.fileTableService.getUrlOfFileId(r.courseFile);
      // result[index]['courseFileUrl'] = url;

      let bannerurl = await this.fileTableService.getUrlOfFileId(r.courseUnitBanner);
      result[index]['courseUnitBannerUrl'] = bannerurl;
      index++;
    }

    return result;
  }

  async update(courseUnitId: number, updateCourseUnitTableInput: UpdateCourseUnitTableInput) {
    let result = await this.findOne(courseUnitId);

    if (result) {

      let courseTable = await this.courseTableService.findOne(updateCourseUnitTableInput.courseId)
      result.CourseTable = courseTable;

      result.courseUnitName = updateCourseUnitTableInput.courseUnitName;
      result.courseUnitDescription = updateCourseUnitTableInput.courseUnitDescription;
      result.courseUnitDuration = updateCourseUnitTableInput.courseUnitDuration;
      //result.numberOfPages = updateCourseUnitTableInput.numberOfPages;
      //result.weightOfPercentage = updateCourseUnitTableInput.weightOfPercentage;
      //result.courseFile = updateCourseUnitTableInput.courseFile;
      result.courseUnitBanner = updateCourseUnitTableInput.courseUnitBanner;
      result.week = updateCourseUnitTableInput.week;


      //result.hasAssesment = updateCourseUnitTableInput.hasAssesment;
      //result.totalScore = updateCourseUnitTableInput.totalScore;
      //result.passingScore = updateCourseUnitTableInput.passingScore;
      result.isLink = updateCourseUnitTableInput.isLink;
      result.link = updateCourseUnitTableInput.link;

      let updatedResult = await this.courseUnitTableRepository.save(result);


      // send to subscriber
      this.pubSub.publish(NEW_COURSE_UNIT_ADD, 1);


      return {
        status: true,
        status_text: "Successfully Updated",
        data: updatedResult
      }
    } else {
      return {
        status: false,
        status_text: "Not Found"
      }
    }
  }

  async remove(courseUnitId: number) {
    let result = await this.findOne(courseUnitId)

    if (result) {
      await this.courseUnitTableRepository.remove(result);


      // send to subscriber
      this.pubSub.publish(NEW_COURSE_UNIT_ADD, 1);


      return {
        status: true,
        status_text: "Successfully Deleted"
      }
    } else {
      return {
        status: false,
        status_text: "Not Found"
      }
    }
  }


  async toggleHasAssesmentofCourseUnit(courseUnitId: number){
    let res = await this.courseUnitTableRepository.findOne({
      where: {
        courseUnitId
      }
    })


    if (res) {
      // update
      let oldStatus = res.hasAssesment;

      let newStatus = "YES";
      if (oldStatus == "YES") {
        newStatus = "NO";
      }

      if (oldStatus == "NO") {
        newStatus = "YES";
      }

      res.hasAssesment = newStatus;

      let result = await this.courseUnitTableRepository.save(res);


      return {
        status: true,
        status_text: "Successfully Updated",
        data: result
      }

    } else {
      // return error
      return {
        status: false,
        status_text: "Not found"
      }
    }
  }
}
