import { Inject, Injectable } from '@nestjs/common';
import { CreateDesignationTableInput } from './dto/create-designation-table.input';
import { UpdateDesignationTableInput } from './dto/update-designation-table.input';
import { InjectRepository } from '@nestjs/typeorm';
import { DesignationTable } from './entities/designation-table.entity';
import { Repository } from 'typeorm';
import { RedisPubSub } from 'graphql-redis-subscriptions';
import { PUB_SUB } from 'src/helper/GlobalConstants';

export const NEW_DESIGNATION_ADD = "NEW_DESIGNATION_ADD"


@Injectable()
export class DesignationTableService {
  constructor(
    @InjectRepository(DesignationTable)
    private readonly designationTableRepository: Repository<DesignationTable>,

    @Inject(PUB_SUB) private pubSub: RedisPubSub,

  ) { }

  async create(createDesignationTableInput: CreateDesignationTableInput) {
    let checkIfAlreadyExists = await this.checkIfAlreadyExists(createDesignationTableInput);

    if (checkIfAlreadyExists == 0) {

      let st = new DesignationTable();
      st.designationName = createDesignationTableInput.designationName;

      let result = await this.designationTableRepository.save(st);


      // send to subscriber
      this.pubSub.publish(NEW_DESIGNATION_ADD,  1 );


      return {
        status: true,
        status_text: "Successfully Added",
        data: result
      }

    } else {
      return {
        status: false,
        status_text: "Already Exists"
      }
    }
  }

  async checkIfAlreadyExists(createDesignationTableInput: CreateDesignationTableInput) {

    let result = await this.designationTableRepository.count({
      where: {
        designationName: createDesignationTableInput.designationName
      }
    })

    return result;

  }

  async findAll() {
    return await this.designationTableRepository.find();
  }

  async findOne(designationId: number) {
    return await this.designationTableRepository.findOne({
      where: {
        designationId
      }
    })
  }

  async update(designationId: number, updateDesignationTableInput: UpdateDesignationTableInput) {
    let result = await this.findOne(designationId);

    if (result) {

   
      
      result.designationName = updateDesignationTableInput.designationName;

      let updatedResult = await this.designationTableRepository.save(result);


       // send to subscriber
       this.pubSub.publish(NEW_DESIGNATION_ADD,  1 );


      return {
        status: true,
        status_text: "Successfully Updated",
        data: updatedResult
      }
    } else {
      return {
        status: false,
        status_text: "Not Found"
      }
    }
  }

  async remove(designationId: number) {
    let result = await this.findOne(designationId)

    if (result) {
      await this.designationTableRepository.remove(result);


       // send to subscriber
       this.pubSub.publish(NEW_DESIGNATION_ADD,  1 );

       
      return {
        status: true,
        status_text: "Successfully Deleted"
      }
    } else {
      return {
        status: false,
        status_text: "Not Found"
      }
    }
  }
}
