import { Controller, Post, UseInterceptors, UploadedFile, Body, Param, Delete, Get } from '@nestjs/common';
import { FileTableService } from './file-table.service';
import { FileInterceptor } from '@nestjs/platform-express';
import { multerOptions } from 'src/config/mutler-config';
import { UPLOAD_PATH } from 'src/helper/GlobalConstants';
import { join } from 'path';
import { existsSync, mkdirSync, renameSync } from 'fs';


@Controller('file-table')
export class FileTableController {
    constructor(
        private readonly fileTableService: FileTableService,

    ) { }


    @Post('file-upload-new')
    @UseInterceptors(FileInterceptor('uploadFile', multerOptions))
    async uploadFileNew(@UploadedFile() file, @Body() body: any) {


        let sFolder = join(
            __dirname,
            '..',
            '..',
            UPLOAD_PATH,
            body.code,
            body.fileType
        )
        //console.log("🚀 ~ file: leave-table.controller.ts ~ line 27 ~ LeaveTableController ~ createNew ~ sFolder", file)

        if (!existsSync(sFolder)) {
            mkdirSync(sFolder, { recursive: true })
        }

        let up = join(__dirname, '..', '..', UPLOAD_PATH)

        let prePath = ''

        if (file != undefined && file.filename != undefined) {
            renameSync(`${up}/${file.filename}`, `${sFolder}/${file.filename}`)

            prePath = `/${UPLOAD_PATH}/${body.code}/${body.fileType}/${file.filename}`
        }


        return await this.fileTableService.saveFileNew(file, prePath, body.fileType, body.code);
    }


    @Post(':code/:fileType/file-upload')
    @UseInterceptors(FileInterceptor('file', multerOptions))
    async uploadFile(@UploadedFile() file, @Param() params: any) {
        return await this.fileTableService.saveFile(file, params.fileType, params.code);
    }


    @Delete('delete-file/:fileId/:code')
    async deleteFile(@Param() params: any) {
        return await this.fileTableService.deleteFile(params.fileId, params.code);
    }


    @Get(':code/:fileType/get-file-upload')
    async getUploadedFile(@Param() params: any) {
        return await this.fileTableService.getMyFilesByType(params.code, params.fileType);
    }




    @Post('delete-file-uploaded-new')
    async deleteFileWithFileName(@Body() body: any) {
        let fileName = body['fileName'];
        return await this.fileTableService.deleteFileNew(fileName);
    }


    @Post('delete-file-uploaded-new-by-id')
    async deleteFileWithId(@Body() body: any) {
        let fileId = body['fileId'];
        return await this.fileTableService.deleteFileWithId(fileId);
    }

    @Delete(':code/:fileId/delete-file-uploaded')
    async deleteFileWithoutId(@Param() params: any) {
        return await this.fileTableService.deleteFile(params.fileId, params.code);
    }

}
