import { Injectable, Inject, forwardRef } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { FileTable } from './file-table.entity';
import { Repository } from 'typeorm';
import { UPLOAD_PATH } from 'src/helper/GlobalConstants';
import { existsSync, mkdirSync, renameSync } from 'fs'
import { join } from 'path'
import { CourseTableService } from 'src/course-table/course-table.service';
@Injectable()
export class FileTableService {
    constructor(
        @InjectRepository(FileTable)
        private readonly fileTableRepository: Repository<FileTable>,

        @Inject(forwardRef(() => CourseTableService))
        private readonly courseTableService: CourseTableService,
    ) { }


    async checkIfAlreadyExists(fileType: string) {
        return false;
    }


    async saveFileNew(file: any, prePath: string, fileType: string, code: string) {



        let fileTable = new FileTable();
        fileTable.fileName = file.filename;
        fileTable.code = code;
        fileTable.fileType = fileType;
        fileTable.path = prePath;
        fileTable.mimeType = file.mimetype;

        let result =  await this.fileTableRepository.save(fileTable);

        return result.fileId;

    }

    async saveFile(file: any, fileType: string, code: string) {



        let fileTable = new FileTable();
        fileTable.fileName = file.filename;
        fileTable.code = code;
        fileTable.fileType = fileType;
        fileTable.path = `uploads/${file.filename}`;
        fileTable.mimeType = file.mimetype;

        

        let result = await this.fileTableRepository.save(fileTable);


        // we will update if the code is COURSE_BANNER_IMAGES

        if(fileType == "COURSE_BANNER_IMAGES"){
            // update the file id to he course table
            await this.courseTableService.updateCourseBanner(code, result.fileId)
        }


        return result;

    }


    async getMyFilesByType(code: string, fileType: string) {


        let result = await this.fileTableRepository.findOne({
            where: {
                code,
                fileType
            }
        })

        if (result !== null && result !== undefined) {
            return result;
        } else {
            return false;
        }
    }


    async getUrlOfFileId(fileId: string) {


        let result = await this.fileTableRepository.findOne({
            where: {
                fileId
            }
        })

        if (result !== null && result !== undefined) {
            return result.path;
        } else {
            return false;
        }
    }



    async deleteFile(fileId: string, code: string) {

        const fs = require('fs')
        let file = await this.fileTableRepository.findOne({
            where: {
                fileId
            }
        });
        if (file !== null && file !== undefined) {
            const path = require('path');
            let p = path.join(__dirname, '..', '..', file.path);
            // check if the file exists
            let ifExists = fs.existsSync(p);
            if (ifExists) {

                fs.unlinkSync(p);
                // remove from database

                await this.fileTableRepository.remove(file);

                return {
                    status: true,
                    status_text: 'Successfully Deleted'
                };
            } else {
                return {
                    status: true,
                    status_text: 'Successfully Deleted'
                };
            }
        } else {
            return {
                status: true,
                status_text: 'File is already deleted'
            }
        }
    }

    async deleteFileNew(fileName: string) {

        const fs = require('fs')
        let file = await this.fileTableRepository.findOne({
            where: {
                fileName
            }
        });
        if (file !== null && file !== undefined) {
            const path = require('path');
            let p = path.join(__dirname, '..', '..', file.path);
            // check if the file exists
            let ifExists = fs.existsSync(p);
            if (ifExists) {

                fs.unlinkSync(p);
                // remove from database

                await this.fileTableRepository.remove(file);

                return {
                    status: true,
                    status_text: 'Successfully Deleted'
                };
            } else {
                return {
                    status: true,
                    status_text: 'Successfully Deleted'
                };
            }
        } else {
            return {
                status: true,
                status_text: 'File is already deleted'
            }
        }
    }


    async deleteFileWithId(fileId: string) {

        const fs = require('fs')
        let file = await this.fileTableRepository.findOne({
            where: {
                fileId
            }
        });
        if (file !== null && file !== undefined) {
            const path = require('path');
            let p = path.join(__dirname, '..', '..', file.path);
            // check if the file exists
            let ifExists = fs.existsSync(p);
            if (ifExists) {

                fs.unlinkSync(p);
                // remove from database

                await this.fileTableRepository.remove(file);

                return {
                    status: true,
                    status_text: 'Successfully Deleted'
                };
            } else {
                return {
                    status: true,
                    status_text: 'Successfully Deleted'
                };
            }
        } else {
            return {
                status: true,
                status_text: 'File is already deleted'
            }
        }
    }
}
