import { Resolver, Query, Mutation, Args, Int, Subscription } from '@nestjs/graphql';
import { NotificationTableService } from './notification-table.service';
import { NotificationTable } from './entities/notification-table.entity';
import { CreateNotificationTableInput } from './dto/create-notification-table.input';
import { UpdateNotificationTableInput } from './dto/update-notification-table.input';
import { NotificationTableType } from './types/notification-table.type';
import { Roles } from 'src/user-collection/decorators/roles.decorator';
import { Role } from 'src/user-collection/roles/role.enum';
import { Inject, UseGuards } from '@nestjs/common';
import { GqlAuthGuard } from 'src/auth/guards/gql-auth.guard';
import { RolesGuard } from 'src/auth/guards/roles.guard';
import { NotificationTableReturnType } from './types/notification-table-return-type';
import { PUB_SUB } from 'src/helper/GlobalConstants';
import { RedisPubSub } from 'graphql-redis-subscriptions';
import { NewNotificationAdd } from './types/new-notification-add.type';


export const NEW_NOTIFICATION_ADD  = "NEW_NOTIFICATION_ADD"


@Resolver(() => NotificationTable)
export class NotificationTableResolver {
  constructor(private readonly notificationTableService: NotificationTableService,  @Inject(PUB_SUB) private pubSub: RedisPubSub) {}

  @Roles(Role.SUPERADMIN)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => NotificationTableReturnType)
  createNotificationTable(@Args('createNotificationTableInput') createNotificationTableInput: CreateNotificationTableInput) {
    return this.notificationTableService.addNew(createNotificationTableInput);
  }


  @Roles(Role.SUPERADMIN)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Query(() => [NotificationTableType])
  findAllNotifications() {
    return this.notificationTableService.getAllActive();
  }


  

  // @Query(() => NotificationTable, { name: 'notificationTable' })
  // findOne(@Args('id', { type: () => Int }) id: number) {
  //   return this.notificationTableService.findOne(id);
  // }

  @Roles(Role.SUPERADMIN)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => NotificationTableReturnType)
  updateNotificationTable(@Args('updateNotificationTableInput') updateNotificationTableInput: UpdateNotificationTableInput) {
    return this.notificationTableService.updateTable(updateNotificationTableInput.notificationId, updateNotificationTableInput);
  }


  @Roles(Role.SUPERADMIN)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => NotificationTableReturnType)
  removeNotificationTable(@Args('notificationId', { type: () => Int }) notificationId: number) {
    return this.notificationTableService.deleteTable(notificationId);
  }


  @Subscription(() => NewNotificationAdd, {
    resolve: (payload) => {
      return payload
    }
  })
  notificationSubscription() {
    console.log('Async Iterator Called')
    return this.pubSub.asyncIterator(NEW_NOTIFICATION_ADD);
  }

}
