import { Inject, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { RedisPubSub } from 'graphql-redis-subscriptions';
import moment from 'moment';
import { PUB_SUB } from 'src/helper/GlobalConstants';
import { Repository } from 'typeorm';
import { CreateNotificationTableInput } from './dto/create-notification-table.input';
import { UpdateNotificationTableInput } from './dto/update-notification-table.input';
import { NotificationTable } from './entities/notification-table.entity';


export const NEW_NOTIFICATION_ADD  = "NEW_NOTIFICATION_ADD"


@Injectable()
export class NotificationTableService {
  constructor(
    @InjectRepository(NotificationTable)
    private readonly notificationTableRepository: Repository<NotificationTable>,
    @Inject(PUB_SUB) private pubSub: RedisPubSub
) { }

async addNew(createTestTableDto: CreateNotificationTableInput) {

    let checkIfExists = await this.checkIfExists(createTestTableDto);
    if (!checkIfExists) {
        let lt = new NotificationTable();
        lt.title = createTestTableDto.title;
        lt.description = createTestTableDto.description;
        lt.link = createTestTableDto.link;
        lt.date = moment().format('DD-MM-YYYY')
        
        
        lt.status = 1;
       

        let result = await this.notificationTableRepository.save(lt);

       
        this.pubSub.publish(NEW_NOTIFICATION_ADD,  {
            title: result.title,
            message: result.description
        } );
        return {
            status: true,
            status_text: "Successfully created",
            data: result
        }

    } else {
        return {
            status: false,
            status_text: "Already exist"
        }
    }

}

async checkIfExists(createTestTableDto: CreateNotificationTableInput) {
    let result = await this.notificationTableRepository.findOne({
        where: {
            title: createTestTableDto.title,
        }
    })

    if (result !== null && result !== undefined) {
        return true;
    } else {
        return false;
    }
}

async getAll() {
    return await this.notificationTableRepository.find()
}


async getAllActive() {
    let result = await this.notificationTableRepository.find({
        where: {
            status: 1
        },
        order: {
            notificationId: 'DESC'
        }
    })
    
    console.log("🚀 ~ file: notification-table.service.ts ~ line 77 ~ NotificationTableService ~ getAllActive ~ result", result)
    return result
}



async getById(notificationId: number) {
    return await this.notificationTableRepository.findOne({
        where: {
            notificationId
        }
    })
}


async updateTable(notificationId: number, createTestTableDto: UpdateNotificationTableInput) {

    let em = await this.notificationTableRepository.findOne({
        where: {
            notificationId
        }
    })

    if (em != null && em != undefined) {
        em.title = createTestTableDto.title;
        em.description = createTestTableDto.description;
        em.link = createTestTableDto.link;
        
        

        let result = await this.notificationTableRepository.save(em);
        this.pubSub.publish(NEW_NOTIFICATION_ADD, {title: null});
        return {
            status: true,
            status_text: "Updated Successfully",
            data: result
        }
    } else {
        return {
            status: false,
            status_text: "Test not found",
            data: false
        }
    }
}


async toggleServiceStatus(notificationId: number, newStatus: boolean) {


    let em = await this.notificationTableRepository.findOne({
        where: {
            notificationId
        }
    })

    if (em != null && em != undefined) {
        em.status = newStatus ? 1 : 0;
        

        let result = await this.notificationTableRepository.save(em);

        return {
            status: true,
            status_text: "Updated Successfully",
            data: result
        }
    } else {
        return {
            status: false,
            status_text: "Test not found",
            data: false
        }
    }
}


async deleteTable(notificationId: number) {


    let em = await this.notificationTableRepository.findOne({
        where: {
            notificationId
        }
    })

    if (em != null && em != undefined) {
       

    let result = await this.notificationTableRepository.remove(em);
    this.pubSub.publish(NEW_NOTIFICATION_ADD, {title: null});
    return {
        status: true,
        status_text: 'Successfully deleted',
        data: result
    }

}
else {
    return {
        status: false,
        status_text: "not found",
        data: false
    }
}
}
}
