import { Resolver, Query, Mutation, Args, Int, Subscription } from '@nestjs/graphql';
import { OrganisationTableService } from './organisation-table.service';
import { OrganisationTable } from './entities/organisation-table.entity';
import { CreateOrganisationTableInput } from './dto/create-organisation-table.input';
import { UpdateOrganisationTableInput } from './dto/update-organisation-table.input';
import { OrganisationTableReturnType } from './types/organisation-table-return.type';
import { OrganisationTableType } from './types/organisation-table.type';
import { Inject, UseGuards } from '@nestjs/common';
import { GqlAuthGuard } from 'src/auth/guards/gql-auth.guard';
import { RolesGuard } from 'src/auth/guards/roles.guard';
import { Roles } from 'src/user-collection/decorators/roles.decorator';
import { Role } from 'src/user-collection/roles/role.enum';
import { CountChangedType } from 'src/types/count-changed.type';
import { PUB_SUB } from 'src/helper/GlobalConstants';
import { RedisPubSub } from 'graphql-redis-subscriptions';

export const NEW_ORGANISATION_ADD = "NEW_ORGANISATION_ADD"


@Resolver(() => OrganisationTable)
export class OrganisationTableResolver {
  constructor(private readonly organisationTableService: OrganisationTableService, @Inject(PUB_SUB) private pubSub: RedisPubSub,) { }

  @Roles(Role.SUPERADMIN, Role.ADMIN)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => OrganisationTableReturnType)
  createOrganisationTable(@Args('createOrganisationTableInput') createOrganisationTableInput: CreateOrganisationTableInput) {
    return this.organisationTableService.create(createOrganisationTableInput);
  }

  // @Roles(Role.SUPERADMIN, Role.ADMIN)
  // @UseGuards(GqlAuthGuard, RolesGuard)
  @Query(() => [OrganisationTableType])
  findAllOrganisations() {
    return this.organisationTableService.findAll();
  }

  @Roles(Role.SUPERADMIN, Role.ADMIN)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Query(() => OrganisationTableType)
  findOneOrganisation(@Args('organisationId', { type: () => Int }) organisationId: number) {
    return this.organisationTableService.findOne(organisationId);
  }

  @Roles(Role.SUPERADMIN, Role.ADMIN)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => OrganisationTableReturnType)
  updateOrganisationTable(@Args('updateOrganisationTableInput') updateOrganisationTableInput: UpdateOrganisationTableInput) {
    return this.organisationTableService.update(updateOrganisationTableInput.organisationId, updateOrganisationTableInput);
  }

  @Roles(Role.SUPERADMIN, Role.ADMIN)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => OrganisationTableReturnType)
  removeOrganisationTable(@Args('organisationId', { type: () => Int }) organisationId: number) {
    return this.organisationTableService.remove(organisationId);
  }


  @Subscription(() => CountChangedType, {
    resolve: (payload) => {
      return { countChanged: payload }
    }
  })
  organisationSubscription() {
    return this.pubSub.asyncIterator(NEW_ORGANISATION_ADD);
  }
}
