import { Inject, Injectable } from '@nestjs/common';
import { CreateOrganisationTableInput } from './dto/create-organisation-table.input';
import { UpdateOrganisationTableInput } from './dto/update-organisation-table.input';
import { InjectRepository } from '@nestjs/typeorm';
import { OrganisationTable } from './entities/organisation-table.entity';
import { Repository } from 'typeorm';
import { PUB_SUB } from 'src/helper/GlobalConstants';
import { RedisPubSub } from 'graphql-redis-subscriptions';

export const NEW_ORGANISATION_ADD = "NEW_ORGANISATION_ADD"


@Injectable()
export class OrganisationTableService {
  constructor(
    @InjectRepository(OrganisationTable)
    private readonly OrganisationTableRepository: Repository<OrganisationTable>,

    @Inject(PUB_SUB) private pubSub: RedisPubSub,

    

  ) { }

  async create(createOrganisationTableInput: CreateOrganisationTableInput) {
    let checkIfAlreadyExists = await this.checkIfAlreadyExists(createOrganisationTableInput);

    if (checkIfAlreadyExists == 0) {

      let st = new OrganisationTable();
      st.organisationName = createOrganisationTableInput.organisationName;
      st.organisationCode = createOrganisationTableInput.organisationCode;
      st.organisationAddress = createOrganisationTableInput.organisationAddress;
      st.isOwn = createOrganisationTableInput.isOwn;
     

      let result = await this.OrganisationTableRepository.save(st);


      // send to subscriber
      this.pubSub.publish(NEW_ORGANISATION_ADD, 1);


      return {
        status: true,
        status_text: "Successfully Added",
        data: result
      }

    } else {
      return {
        status: false,
        status_text: "Already Exists"
      }
    }
  }

  async checkIfAlreadyExists(createOrganisationTableInput: CreateOrganisationTableInput) {

    let result = await this.OrganisationTableRepository.count({
      where: {
        organisationCode: createOrganisationTableInput.organisationCode,

      }
    })

    return result;

  }

  async findAll() {
    let result = await this.OrganisationTableRepository.find();

   

    return result;

  }



  async findOne(organisationId: number) {
    let result: any = await this.OrganisationTableRepository.findOne({
      where: {
        organisationId
      }
    })



   

    return result;
  }




  async update(organisationId: number, updateOrganisationTableInput: UpdateOrganisationTableInput) {
    let result = await this.findOne(organisationId);

    if (result) {



      result.organisationName = updateOrganisationTableInput.organisationName;
      result.organisationCode = updateOrganisationTableInput.organisationCode;
      result.organisationAddress = updateOrganisationTableInput.organisationAddress;
      result.isOwn = updateOrganisationTableInput.isOwn;
     

      let updatedResult = await this.OrganisationTableRepository.save(result);


      // send to subscriber
      this.pubSub.publish(NEW_ORGANISATION_ADD, 1);


      return {
        status: true,
        status_text: "Successfully Updated",
        data: updatedResult
      }
    } else {
      return {
        status: false,
        status_text: "Not Found"
      }
    }
  }

  async remove(organisationId: number) {
    let result = await this.findOne(organisationId)

    if (result) {
      await this.OrganisationTableRepository.remove(result);


      // send to subscriber
      this.pubSub.publish(NEW_ORGANISATION_ADD, 1);


      return {
        status: true,
        status_text: "Successfully Deleted"
      }
    } else {
      return {
        status: false,
        status_text: "Not Found"
      }
    }
  }




}
