import { Injectable, Inject } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { OtpTable } from './otp-table.entity';
import { Repository } from 'typeorm';
import { OTP_GENERATOR, OTP_SEND } from 'src/helper/GlobalConstants';
import { ThirdPartyCallService } from 'src/third-party-call/third-party-call.service';
import { MailService } from 'src/mail/mail.service';


@Injectable()
export class OtpTableService {
    constructor(
        @InjectRepository(OtpTable)
        private readonly otpTableRepository: Repository<OtpTable>,
        @Inject(OTP_GENERATOR) private otpGenerator: any,
        private readonly mailService: MailService
    ) {

    }

    async generateOtpNow(phone: string, employeeName: string, email: string) {

        let result = await this.getOtpViaPhone(phone);
        let otp = await this.generateOtp();

        if (result === false) {
            // create a new otp & insert
            let otpTable = new OtpTable();
            otpTable.otp = otp;
            otpTable.phone = phone;

            let res = await this.otpTableRepository.save(otpTable);

            // send Email
            this.mailService.forgotPasswordOTp({
                toEmail: email,
                otp,
                name: employeeName
            });



            let firstPart = email.split('@')[0];

            var last = email.substr(firstPart.length - 2);
            var first = email.substring(0, 2);
            return {
                status: true,
                status_text: `An otp has been sent to your email ending with: ${first}***${last}`,
                data: false,
                otp: ''
            }

        } else {
            // create a new otp and update

            result.otp = otp;
            let res = await this.otpTableRepository.save(result);


            // send Email
            this.mailService.forgotPasswordOTp({
                toEmail: email,
                otp,
                name: employeeName
            });




            let firstPart = email.split('@')[0];

            var last = email.substr(firstPart.length - 2);
            var first = email.substring(0, 2);
            return {
                status: true,
                status_text: `An otp has been sent to your email ending with: ${first}*****${last}`,
                data: false,
                otp: ''
            }
        }
    }

    async checkOtp(phone: string, otp: string) {
        let result = await this.otpTableRepository.findOne({
            where: {

                phone,
                otp

            }
        });

        if (result === null || result === undefined) {
            return {
                status: false,
                status_text: 'OTP doesnot match. Try again',
                data: false
            }
        } else {
            return {
                status: true,
                status_text: 'Enter new password now',
                data: false
            }
        }
    }

    async getOtpViaPhone(phone: string) {
        let result = await this.otpTableRepository.findOne({
            where: {
                phone
            }
        });

        if (result === null || result === undefined) {
            return false;
        } else {
            return result;
        }
    }

    async generateOtp() {
        var digits = '0123456789';
        let OTP = '';
        for (let i = 0; i < 6; i++) {
            OTP += digits[Math.floor(Math.random() * 10)];
        }
        return OTP;
        //return this.otpGenerator.generate(6, { digits: true, alphabets: false, upperCase: false, specialChars: false });
    }
}
