import { Resolver, Query, Mutation, Args, Int } from '@nestjs/graphql';
import { SubscriptionTableService } from './subscription-table.service';
import { SubscriptionTable } from './entities/subscription-table.entity';
import { CreateSubscriptionTableInput } from './dto/create-subscription-table.input';
import { UpdateSubscriptionTableInput } from './dto/update-subscription-table.input';
import { Roles } from 'src/user-collection/decorators/roles.decorator';
import { Role } from 'src/user-collection/roles/role.enum';
import { UseGuards } from '@nestjs/common';
import { GqlAuthGuard } from 'src/auth/guards/gql-auth.guard';
import { RolesGuard } from 'src/auth/guards/roles.guard';
import { SubscriptionTableReturnType } from './types/subscription-table-return.type';
import { SubscriptionTableType } from './types/subscription-table.type';
import { CurrentUser } from 'src/user-collection/decorators/user.decorator';

@Resolver(() => SubscriptionTable)
export class SubscriptionTableResolver {
  constructor(private readonly subscriptionTableService: SubscriptionTableService) {}

  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => SubscriptionTableReturnType)
  createSubscriptionTable(@Args('createSubscriptionTableInput') createSubscriptionTableInput: CreateSubscriptionTableInput) {
    return this.subscriptionTableService.create(createSubscriptionTableInput);
  }

  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Query(() => [SubscriptionTableType])
  findAllSubscriptions() {
    return this.subscriptionTableService.findAll();
  }



  @Roles(Role.SUPERADMIN, Role.ADMIN)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => SubscriptionTableReturnType)
  toggleStatusofSubscription(@Args('subscriptionId', { type: () => Int }) subscriptionId: number) {
    return this.subscriptionTableService.toggleStatusofSubscription(subscriptionId);
  }


  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Query(() => [SubscriptionTableType])
  findAllSubscriptionsByCourseId(@Args('courseId', { type: () => Int }) courseId: number) {
    return this.subscriptionTableService.findAllSubscriptionsByCourseId(courseId);
  }

  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Query(() => SubscriptionTableType)
  findOneSubscription(@Args('subscriptionId', { type: () => Int }) subscriptionId: number) {
    return this.subscriptionTableService.findOne(subscriptionId);
  }


  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => SubscriptionTableReturnType)
  enrollMyself(@Args('courseId', { type: () => Int }) courseId: number, @Args('courseSessionId', { type: () => Int }) courseSessionId: number, @CurrentUser() user: any) {
    return this.subscriptionTableService.enrollMyself(courseId, courseSessionId, user);
  }



  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => SubscriptionTableReturnType)
  markAsComplete(@Args('courseId', { type: () => Int }) courseId: number, @Args('courseSessionId', { type: () => Int }) courseSessionId: number, @CurrentUser() user: any) {
    return this.subscriptionTableService.markAsComplete(courseId, courseSessionId, user);
  }

  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => SubscriptionTableReturnType)
  updateSubscriptionTable(@Args('updateSubscriptionTableInput') updateSubscriptionTableInput: UpdateSubscriptionTableInput) {
    return this.subscriptionTableService.update(updateSubscriptionTableInput.subscriptionId, updateSubscriptionTableInput);
  }

  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => SubscriptionTableReturnType)
  removeSubscriptionTable(@Args('subscriptionId', { type: () => Int }) subscriptionId: number) {
    return this.subscriptionTableService.remove(subscriptionId);
  }
}
