import { AccessTable } from "src/access-table/entities/access-table.entity";
import { AssessmentAnswerTable } from "src/assessment-answer-table/entities/assessment-answer-table.entity";
import { CourseProgressionTable } from "src/course-progression-table/entities/course-progression-table.entity";
import { CourseTable } from "src/course-table/entities/course-table.entity";
import { DesignationTable } from "src/designation-table/entities/designation-table.entity";
import { DiscussionTable } from "src/discussion-table/entities/discussion-table.entity";
import { OrganisationTable } from "src/organisation-table/entities/organisation-table.entity";
import { SubscriptionTable } from "src/subscription-table/entities/subscription-table.entity";
import { Entity, PrimaryGeneratedColumn, Column, PrimaryColumn, CreateDateColumn, UpdateDateColumn, OneToMany, JoinColumn, ManyToOne } from "typeorm";



@Entity()
export class UserTable {
    @PrimaryGeneratedColumn()
    id: number;

    @Column()
    username: string;

    @Column()
    password: string;

    @Column({ nullable: true })
    designation: string;

    @Column()
    name: string;

    @Column({ nullable: true })
    email: string;

    @Column()
    phone: string;


    // new fields
    @Column({ nullable: true, length: 200})
    currentClass: string;

    @Column({ nullable: true, length: 200})
    classRollNo: string;

    @Column({ nullable: true, length: 200})
    semester: string;

    // end new fields

    @Column()
    role: string;

    @Column({ default: true })
    isActive: boolean;

    @OneToMany(type => AccessTable, pt => pt.UserTable)
    AccessTable: AccessTable[];

    @OneToMany(type => SubscriptionTable, pt => pt.UserTable)
    SubscriptionTable: SubscriptionTable[];

    @Column({default: null})
    integratorId: number;

    @ManyToOne(type => DesignationTable, dt => dt.UserTable)
    @JoinColumn({ name: "designationId" })
    DesignationTable: DesignationTable;


    @OneToMany(type => CourseTable, et => et.UserTable)
    CourseTable: CourseTable[];


    @ManyToOne(type => OrganisationTable, dt => dt.UserTable)
    @JoinColumn({ name: "organisationId" })
    OrganisationTable: OrganisationTable;

    @OneToMany(type => AssessmentAnswerTable, et => et.UserTable)
    AssessmentAnswerTable: AssessmentAnswerTable[];

    @OneToMany(type => CourseProgressionTable, et => et.UserTable)
    CourseProgressionTable: CourseProgressionTable[];

    @OneToMany(type => DiscussionTable, et => et.UserTable)
    DiscussionTable: DiscussionTable[];

    @CreateDateColumn({ type: "timestamp", default: () => "CURRENT_TIMESTAMP(6)" })
    public createdAt: Date;


    @UpdateDateColumn({ type: "timestamp", default: () => "CURRENT_TIMESTAMP(6)", onUpdate: "CURRENT_TIMESTAMP(6)" })
    public updatedAt: Date;

}

